﻿  ; Load AutoText
  ;
  ; V1.0 - 31/AUG/2021
  ;        20/SEP/2021
  ;        2\/SEP/2021
  ; V1.1 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure load_autotext(force_file$)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading AutoText List... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    
    
    If force_file$=""
      file$=OpenFileRequester("Open AutoText List", "", "BlockList.xml (*.xml)|*.xml", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
        ProcedureReturn #False
      EndIf
    Else
      file$=force_file$
    EndIf      
    
    
    ; Check if .XML file exists
    If IsFileExist(file$,"",".XML file not found.")=#False
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf    
    
    
    time_taken = ElapsedMilliseconds()  
    
    
    ; Load .XML file
    ReadFile(1,Left(file$,Len(file$)-3)+"xml")
    ReadStringFormat(1)
      buffer$=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)  
      filesize_xml=Lof(1)
    CloseFile(1)
    buffer$=ConvertStringToUnix(buffer$)
    buffer$=ReplaceString(buffer$,#LF$,"") ; Have all in one line, like when taken from LibreOffice folder
    number_of_autotexts=CountString(buffer$,"<block-list:block ")
    
    ; Check if .XML has entries
    If number_of_autotexts=0
      MessageRequester("Error", ".XML file has no valid entries.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False      
    EndIf
    
    
    ; Read AutoText entries
    AddWindowTimer(#WINDOW_MAIN,800,800)
    filesize_autotexts=0
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_autotexts)
    ClearGadgetItems(#PANEL_BOX_AUTOTEXT)
    For f=1 To number_of_autotexts
      
      ReDim autotexts(f-1)
      t$=StringField(buffer$,f+1,"<block-list:block ")
      t=FindString(t$,"abbreviated-name=")
      autotexts(f-1)\abbreviated_name.s=ExtractStringFromQuotes(t$, t+Len("abbreviated-name="))
      t=FindString(t$,"package-name=")
      autotexts(f-1)\package_name.s=ExtractStringFromQuotes(t$, t+Len("package-name="))
      t=FindString(t$,"block-list:name=")
      autotexts(f-1)\name.s=ExtractStringFromQuotes(t$, t+Len("block-list:name="))
      
      ; Get the filename of the autotexts xmls
      For ff=Len(file$) To 1 Step -1
        
        t$=Mid(file$,ff,1)
        If t$="/"
          ttt$=autotexts(f-1)\abbreviated_name.s+".xml"
          file2$=Left(file$,ff)+autotexts(f-1)\abbreviated_name.s+"/"
          If FileSize(file2$+ttt$)<=0
            file2$+"content.xml"
          Else
            file2$+ttt$
          EndIf
          Break
        EndIf
        If t$="\"
          ttt$=autotexts(f-1)\abbreviated_name.s+".xml"
          file2$=Left(file$,ff)+autotexts(f-1)\abbreviated_name.s+"\"
          If FileSize(file2$+ttt$)<=0
            file2$+"content.xml"
          Else
            file2$+ttt$            
          EndIf
          Break
        EndIf
        
      Next ff
      
      ; Load the autotexts xmls
      OpenFile(1,file2$)  
        ReadStringFormat(1)
        ttt$=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)
        ttt$=ConvertStringToUnix(ttt$)
        ttt$=ReplaceString(ttt$,#LF$,"")
        ttt$=ReplaceString(ttt$,">",">"+#LF$)
        ttt$=clean_xml_tags(ttt$)
        autotexts(f-1)\rule_text.s=ttt$
        filesize_autotexts+Lof(1)
      CloseFile(1)

        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event=WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
;           update_number_words()
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading AutoText List... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
         If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)           
            ProcedureReturn #False
         EndIf
       EndIf
       
    Next f
    
    RemoveWindowTimer(#WINDOW_MAIN,800)
    SetGadgetText(#TEXT_PANEL_BOX_AUTOTEXT_NUMBER_WORDS,"# AutoTexts:"+Str(number_of_autotexts))
    GadgetFit(#TEXT_PANEL_BOX_AUTOTEXT_NUMBER_WORDS)    
    
    autotext_filename$=file$
    update_filename_autotext()
    
    ; 31/OCT/2017 - Support for "Recent Files" menu entry
    create_load_update_recent_files(file$)
    update_recent_files_menu()

    ; Populate the ListIconGadget
    For f=1 To number_of_autotexts
      ttt$=Str(f)+#LF$+autotexts(f-1)\abbreviated_name.s+#LF$+autotexts(f-1)\package_name.s+#LF$+autotexts(f-1)\name.s
      AddGadgetItem(#PANEL_BOX_AUTOTEXT,-1,ttt$)
      SetGadgetItemColor(#PANEL_BOX_AUTOTEXT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)            
    Next f


    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("BlockList.xml:",filesize_xml,#True)+"+"+ShowFileSizeFormatted("xmls:",filesize_autotexts,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))     
   
  EndProcedure 
  
  
  
  
  ; Updates the filename of BlockList.xml in AutoText
  ;
  ; Used in open and save as
  ;
  ; V1.0 - 21/SEP/2021
  ; V1.1 - 19/DEC/2021
  ;        o Replaced "Filename:" with "File:"
  ; V1.2 - 01/NOV/2023
  ;        o Replaced "File:" with "💾:"
  ; V1.3 - 26/NOV/2023
  ;        o Replaced the disk emoji plain text with an image
  ; V1.4 - 13/JAN/2024
  ;        14/JAN/2024
  ;        o It now shows a folder emoji with the path of the filename
  ;
  Procedure update_filename_autotext()
    
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_AUTOTEXT_FILENAME,""+GetOnlyFilename(autotext_filename$,"n/a"),#True)
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_AUTOTEXT_FOLDER,""+GetOnlyPath(autotext_filename$,"n/a"),#True)

  EndProcedure  
  
  
  
  
  ; Cleans the XML tags
  ;
  ; V1.0 - 22/SEP/2021
  ;
  Procedure.s clean_xml_tags(t$)
    
    ; Check number of lines
    counter=CountString(t$,#LF$)
    
    
    ; No lines, it returns ""
    If counter=0 : ProcedureReturn "" : EndIf
    
    returnstring$=""
    
    For f=1 To counter
      
      word$=StringField(t$,f,#LF$)
      
      If Right(word$,Len("</text:p>"))="</text:p>" Or Right(word$,Len("</text:span>"))="</text:span>"
        word$=ReplaceString(word$,"</text:p>","")
        word$=ReplaceString(word$,"</text:span>","")
;         word$=Mid(word$,FindString(word$,">")+1)+#LF$
        returnstring$+word$+#LF$
      EndIf
      
    Next f
    
    
    ; Remove double #LF$
    returnstring$=ReplaceString(returnstring$,#LF$+#LF$,#LF$)
    
    
    ; Return without XML tags
    ProcedureReturn returnstring$
      
  EndProcedure
  
  
; IDE Options = PureBasic 6.04 LTS (Windows - x64)
; CursorPosition = 176
; FirstLine = 161
; Folding = -
; EnableXP
; DisableDebugger
; CompileSourceDirectory
; EnableUnicode