﻿  ; Check if .prefs exists and is valid, displaying a pop-up warning in #False
  ; Notice to change the version number of the .prefs in both procedures when we change the version number of .prefs
  ; This is a duplicate of the load prefs code, just to be displayed after the GUI is designed
  ; 
  ; V1.0 - 14/DEC/2016
  ;        6/JAN/2017
  ; V1.1 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V1.2 - 06/NOV/2019
  ;        o Fix: Opening more than one instance of PTG says the prefs file is invalid (file not closed)
  ;
  Procedure check_prefs()
    
    ; Check if .PREFS file exists
    If FileSize(main_path$+"settings/"+"ptg3.prefs")<=0
      ProcedureReturn #False
    EndIf    
    
    ; Attempt to load .PREFS file
    ReadFile(1,main_path$+"settings/"+"ptg3.prefs")
    t$=ReadString(1,#PB_UTF8)
    If t$<>prefs_file_version$
      MessageRequester("Error", "The .prefs file is invalid."+#LF$+#LF$+"Maybe it is from a different version of"+#LF$+"Proofing Tool GUI.",#PB_MessageRequester_Error)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, ".prefs file invalid.")
      CloseFile(1)
      ProcedureReturn #False
    EndIf
    CloseFile(1)
    
    ProcedureReturn #True
    
  EndProcedure
  
  
  
  
  ; Loads the preferences
  ; It returns #True = Success or #False = Failed
  ; This is a duplicate of the check prefs code, just to load before the GUI is designed
  ;
  ; V1.0 - 29/SEP/2016
  ; V1.1 - 5/OCT/2016
  ;        o Improved a bit the code.
  ; V1.2 - 14/DEC/2016
  ;        o Updated load/save prefs
  ; V1.3 - 6/JAN/2017
  ;        o Now makes uses of ".AFF Aid" 
  ; V1.4 - 7/FEB/2018
  ;        o Now loads "Number separator chr"
  ; V1.5 - 15/FEB/2018
  ;        20/FEB/2018
  ;        o Improved the window size handling
  ; V1.6 - 9/MAY/2018
  ;        o Coding of "Check for Updates"  
  ; V1.7 - 11/JUL/2018
  ;        o Prefs: Now allows to set colour for "Primary", "Suffix" and "Prefix"   
  ; V1.8 - 05/DEC/2018
  ;        o Preferences now has option for "Both" in Affixes (CIRCUMFIX)
  ; V1.9 - 13/MAR/2019
  ;        o Warn over N words while extracting and counting duplicates on wordlist
  ; V2.0 - 19/JUN/2019
  ;        o Preferences allow to choose the font for the language.
  ; V2.1 - 28/SEP/2019
  ;        o Now loads dictionary_column_sort prefs
  ; V2.2 - 05/MAY/2020
  ;        o Now loads the "Show/Merge/Delete duplicates .dic" prefs
  ; V2.3 - 23/MAY/2020
  ;        29/MAY/2020
  ;        06/JUN/2020
  ;        o Started coding the settings of Extract POS
  ; V2.4 - 18/JUN/2020
  ;        o Loads settings for multiwords
  ; V2.5 - 25/AUG/2021
  ;        o Added spaces on left regarding autocorrect (prefs + core)
  ; V2.6 - 07/JAN/2022
  ;        o Now uses a variable prefs_loaded_or_okay_pressed to know if it defaults the prefs
  ; V2.7 - 26/MAR/2023
  ;        o All the preferences (including "Remember") now include a version ID for the future open/save profile
  ; V2.8 - 05/APR/2023
  ;        o Support for all_synonyms_in_one_line
  ;        
  Procedure load_preferences()
    
    ; Loads Dictionary stats
    If FileSize(main_path$+"settings/"+"dic_stats_remember.prefs")>0
      OpenFile(1,main_path$+"settings/"+"dic_stats_remember.prefs")
        t$=ReadString(1,#PB_UTF8)
        If t$=prefs_file_version$
          dictionary_stats$=ReadString(1,#PB_UTF8)
        EndIf
        ; Work around for version 25 so that we don't need to create all new prefs files
        If t$="25" And Loc(1)<>Eof(1)
          dictionary_stats_last_remember$=ReadString(1,#PB_UTF8)
        ElseIf Val(t$)>Val("25")
          dictionary_stats_last_remember$=ReadString(1,#PB_UTF8)
        EndIf
        If dictionary_stats_last_remember$="" : dictionary_stats_last_remember$="Last Remember:N/A" : EndIf
      CloseFile(1)
    EndIf  

    
    ; Loads LanguageTool Multiwords on Bulk Import
    If FileSize(main_path$+"settings/"+"multiwords.prefs")>=0
      OpenFile(1,main_path$+"settings/"+"multiwords.prefs")  
        t$=ReadString(1,#PB_UTF8)
        If t$=prefs_file_version$
          combobox_multiwords_position=ReadLong(1)
          combobox_multiwords_position_target=ReadLong(1)
          string_multiwords_position_target_other$=ReadString(1,#PB_UTF8)
        EndIf
      CloseFile(1)
    EndIf       
    
    
    ; Loads LanguageTool POS rules
    ; Loads languagetool_pos_rules_language_selected prefs + rules
    If FileSize(main_path$+"settings/"+"extract_pos_settings.prefs")>=0
      OpenFile(1,main_path$+"settings/"+"extract_pos_settings.prefs")  
        t$=ReadString(1,#PB_UTF8)
        If t$=prefs_file_version$
          languagetool_pos_rules_language_selected=ReadLong(1)
        EndIf
      CloseFile(1)
    EndIf       
    If FileSize(main_path$+"settings/"+"extract_pos_lists.prefs")>=0
      ; Attempt to save languagetool_pos_rules_language_selected .PREFS file
      OpenFile(1,main_path$+"settings/"+"extract_pos_lists.prefs")    
        t$=ReadString(1,#PB_UTF8)
        If t$=prefs_file_version$
          languagetool_pos_rules_language_selected_total_number_languages=Val(ReadString(1,#PB_UTF8))
          For f=1 To languagetool_pos_rules_language_selected_total_number_languages          
            languagetool_pos_rules_languages$(f-1)=ReadString(1,#PB_UTF8) ; Languages names in combobox
            counter=Val(ReadString(1,#PB_UTF8))
            languagetool_pos_rules_language_selected_rules_number_max_each(f-1)=counter
            If counter>0
              For ff=1 To counter
                t$=ReadString(1,#PB_UTF8) 
                languagetool_pos_rules_language_selected_rules$(f-1,ff)=ReplaceString(t$,#TAB$,#LF$)
              Next ff
            EndIf
          Next f
        EndIf
      CloseFile(1)    
    EndIf
    
    
    ; Loads dictionary_column_sort prefs
    If FileSize(main_path$+"settings/"+"derivates_sort.prefs")>=0
      OpenFile(1,main_path$+"settings/"+"derivates_sort.prefs")  
        t$=ReadString(1,#PB_UTF8)
        If t$=prefs_file_version$
          dictionary_column_sort=ReadLong(1)
        EndIf
      CloseFile(1)
    EndIf        
    
    
    ; Loads "Show/Merge/Delete duplicates .dic" .PREFS file
    If FileSize(main_path$+"settings/"+"show_merge_del_dupes.prefs")>=0
      OpenFile(1,main_path$+"settings/"+"show_merge_del_dupes.prefs")                  
        t$=ReadString(1,#PB_UTF8)
        If t$=prefs_file_version$
          show_merge_delete_duplicates_dic_checkbox(1)=ReadByte(1)
          show_merge_delete_duplicates_dic_checkbox(2)=ReadByte(1)
          show_merge_delete_duplicates_dic_checkbox(3)=ReadByte(1)
          show_merge_delete_duplicates_dic_checkbox(4)=ReadByte(1)
        EndIf
      CloseFile(1)    
    EndIf
    
    
    ; Check if .PREFS file exists
    If FileSize(main_path$+"settings/"+"ptg3.prefs")<=0
      ProcedureReturn #False
    EndIf    
    
    
    ; Attempt to load .PREFS file
    ReadFile(1,main_path$+"settings/"+"ptg3.prefs")
    t$=ReadString(1,#PB_UTF8)
    If t$<>prefs_file_version$
      CloseFile(1)
      ProcedureReturn #False
    EndIf
    
    
    ; Set resolution
    window_resolution$=ReadString(1,#PB_UTF8)
    desktop_resolution$=ReadString(1,#PB_UTF8)
    window_resolution_max$=ReadString(1,#PB_UTF8)
    
    ; Set number of lines
    number_of_lines=ReadLong(1)
    
    ; Set number of pixel plus
    pixels_plus=ReadLong(1)
    
    ; Set LanguageTool number of lines
    number_of_lines_languagetool=ReadLong(1)
    
    ; Set .AFF aid language
    aff_aid_language$=ReadString(1,#PB_UTF8) ; Now it has the filename
    
    ; Number separator chr
    number_separator_chr$=ReadString(1,#PB_UTF8)
    
    ; Colours for dictionary editor - 11/JUL/2018 + 5/DEC/2018
    affix_colour_primary=ReadLong(1)
    affix_colour_suffix=ReadLong(1)
    affix_colour_prefix=ReadLong(1)
    affix_colour_both=ReadLong(1)
    
    ; Time of last check for updates
    check_for_updates$=ReadString(1,#PB_UTF8)
    last_check_updates_time.q=ReadQuad(1)
    
    ; Ask if abort after N words while extracting and counting duplicates on wordlist
    ask_after_n_words=ReadLong(1)
    
    ; Language font and size
    font_for_language$=ReadString(1,#PB_UTF8)
    font_for_language_size$=ReadString(1,#PB_UTF8)
    
    ; Autocorrect spaces on the left
    autocorrect_spaces_on_left=ReadLong(1)
    
    ; All synonyms in one line
    all_synonyms_in_one_line=ReadLong(1)
    
    ; Close file
    CloseFile(1)
  
    ; Return success
    prefs_loaded_or_okay_pressed=#True
    ProcedureReturn #True
    
  EndProcedure
  
  
  
  
  ; Saves the Preferences, called from the Preferences Editor PLUS after silent automatic checking for updates
  ;
  ; V1.0 - 9/MAY/2018
  ; V1.1 - 11/JUL/2018
  ;        o Prefs: Now allows to set colour for "Primary", "Suffix" and "Prefix"
  ; V1.8 - 05/DEC/2018
  ;        o Preferences now has option for "Both" in Affixes (CIRCUMFIX)
  ; V1.9 - 13/MAR/2019
  ;        o Warn over N words while extracting and counting duplicates on wordlist
  ; V2.0 - 19/JUN/2019
  ;        o Preferences allow to choose the font for the language.
  ; V2.1 - 25/AUG/2021
  ;        o Added spaces on left regarding autocorrect (prefs + core)
  ; V2.2 - 05/APR/2023
  ;        o Support for all_synonyms_in_one_line
  ;
  Procedure save_preferences()
    
    ; Delete old .prefs file    
    If FileSize(main_path$+"settings/"+"ptg3.prefs")>=0
      DeleteFile(main_path$+"settings/"+"ptg3.prefs")
    EndIf     
    
    
    ; Attempt to save .PREFS file
    OpenFile(1,main_path$+"settings/"+"ptg3.prefs")
                  
      ; PREFS version
      WriteString(1,prefs_file_version$+#LF$,#PB_UTF8)

      ; Set resolution
      WriteString(1,window_resolution$+#LF$,#PB_UTF8)
      WriteString(1,desktop_resolution$+#LF$,#PB_UTF8)
      WriteString(1,window_resolution_max$+#LF$,#PB_UTF8)
    
      ; Set number of lines
      WriteLong(1,number_of_lines)
                    
      ; Set number of pixel plus
      WriteLong(1,pixels_plus)
                    
      ; Set LanguageTool number of lines
      WriteLong(1,number_of_lines_languagetool)
                    
      ; Language of .AFF Aid
      WriteString(1,aff_aid_language$+#LF$,#PB_UTF8)
                    
      ; Number separator chr
      WriteString(1,number_separator_chr$+#LF$,#PB_UTF8)
      
      ; Colours for dictionary editor - 11/JUL/2018 + 5/DEC/2018
      WriteLong(1,affix_colour_primary)
      WriteLong(1,affix_colour_suffix)
      WriteLong(1,affix_colour_prefix)
      WriteLong(1,affix_colour_both)
                          
      ; Last time check for updates
      WriteString(1,check_for_updates$+#LF$,#PB_UTF8)
      WriteQuad(1,last_check_updates_time.q)
      
      ; Ask if abort after N words while extracting and counting duplicates on wordlist
      WriteLong(1,ask_after_n_words)
      
      ; Language font and size
      WriteString(1,font_for_language$+#LF$,#PB_UTF8)
      WriteString(1,font_for_language_size$+#LF$,#PB_UTF8)
      
      ; Autocorrect spaces on the left
      WriteLong(1,autocorrect_spaces_on_left)
      
      ; All synonyms in one line
      WriteLong(1,all_synonyms_in_one_line)
      
    CloseFile(1)
    
    
  EndProcedure
  
  
  
  
  ; Now it allows "Custom" ".AFF Aids" that reads from the file: "custom_aff_aid.LANG.txt"+"custom_aff_aid.LANG.png"
  ; Notice that the flags are stored in number 20000+
  ; (30)=EN_GB (BUILT IN)
  ;
  ; V1.0 - 7/JAN/2017
  ; V1.1 - 19/MAR/2017
  ;        o PREFS now accept external files with rules and flags
  ; V1.2 - 21/MAR/2017  
  ;        o If no image is found for a file in the custom Aff aid files, there is no attempt to load/use the flag
  ;
  Procedure load_all_aff_aid()
    
    
    ; Current count 0 = English GB > Image = 30 > filename$="*****"
    aff_aid_language_array$(0)="eng-GB:english:English GB"+#LF$+"30"+#LF$+"*****"
    
    
    ; Get the files and the rules
    If ExamineDirectory(#DIRECTORY,main_path$+"custom_aff_aid","*.txt")
      current_count=1
      While NextDirectoryEntry(#DIRECTORY)
        If DirectoryEntryType(#DIRECTORY)=#PB_DirectoryEntry_File
          
          ; Check if file with rules exist
          If DirectoryEntrySize(#DIRECTORY)>0
            
            ; Load the rules, whose first line is the language name
            ReDim aff_aid_language_array_rules$(current_count)
            OpenFile(1,main_path$+"custom_aff_aid/"+DirectoryEntryName(#DIRECTORY))
              aff_aid_language_array_rules$(current_count)=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)  
            CloseFile(1)

            ; Replace Windows/Mac EOL with Linux ones
            aff_aid_language_array_rules$(current_count)=ReplaceString(aff_aid_language_array_rules$(current_count),#CRLF$,#LF$)
            aff_aid_language_array_rules$(current_count)=ReplaceString(aff_aid_language_array_rules$(current_count),#LFCR$,#LF$)
            
            ; Load the flag image
            If FileSize(main_path$+"custom_aff_aid/"+Left(DirectoryEntryName(#DIRECTORY),Len(DirectoryEntryName(#DIRECTORY))-3)+"png")<>-1
              LoadImage(20000+current_count,main_path$+"custom_aff_aid/"+Left(DirectoryEntryName(#DIRECTORY),Len(DirectoryEntryName(#DIRECTORY))-3)+"png")
            EndIf
            
            ; Create the entry that holds the LANGUAGE_NAME + IMAGE_NUMBER + FILENAME
            ReDim aff_aid_language_array$(current_count)
            aff_aid_language_array$(current_count)=StringField(aff_aid_language_array_rules$(current_count),1,#LF$)+#LF$+Str(20000+current_count)+#LF$+DirectoryEntryName(#DIRECTORY)
            
            ; Increase the current count
            current_count+1
            
          EndIf    
        EndIf
      Wend
      current_count-1 ; To give the exact number of files processed NOT counting with the GB one which is ID = 0
      
      ; Sort the languages
      SortArray(aff_aid_language_array$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
      
    Else
      
      ProcedureReturn #False      
      
    EndIf
    
    
  EndProcedure
  
  
  
  
  ; Set the Preferences to be used all over Proofing Tool GUI
  ;
  ; V1.0 - 28/SEP/2016
  ;        29/SEP/2016
  ; V1.1 - 12/DEC/2016
  ;        13/DEC/2016
  ;        o Added resolution+LanguageTool number of lines between rules to Prefs
  ;        o When the pop-up of the first time settings prefs appears, it shows an information tip.
  ; V1.2 - 14/DEC/2016
  ;        o Updated load/save prefs   
  ;        o Changing the resolution in Prefs remembers the word on the top for each tab
  ; V1.3 - 20/DEC/2016
  ;        o Fix: Exiting the PREFS after changing the resolution no longer always jump to the fist tab
  ; V1.4 - 06/JAN/2017
  ;        o Now makes uses of ".AFF Aid"
  ; V1.5 - 07/JAN/2017
  ;        o Now it allows a "Custom" ".AFF Aid" that reads from the file: "custom_aff_aid.txt" 
  ;        o Added checkboxes to the lines so that they take the real space
  ; V1.6 - 11/MAR/2017
  ;        o Changed the buttons height to 25. 
  ; V1.7 - 19/MAR/2017
  ;        o PREFS now accept external files with rules and flags
  ; V1.8 - 21/MAR/2017  
  ;        o If no image is found for a file in the custom Aff aid files, there is no attempt to load/use the flag
  ; V1.9 - 04/APR/2017
  ;        o Fix: Changing the GUI resolution in the PREFS would reset the .AFF EditorGadget
  ; V2.0 - 26/JAN/2018
  ;        o Now it reds the gadget if invalid minimum number of lines in LanguageTool
  ; V2.1 - 07/FEB/2018
  ;        o ComboBox with higher height (+6 pixels) for better Ubuntu look.
  ;        o Now tip appears all the time
  ;        o New option: "Number separator chr"
  ; V2.2 - 13/FEB/2018
  ;        o Added new resolutions to the Prefs.
  ;        o Use of Plus Pixels in Prefs + GUI
  ; V2.3 - 15/FEB/2018  
  ;        o Improved the window size handling as well as a "Maximise Window (WxH)"
  ; V2.4 - 20/FEB/2018  
  ;        o Fix: Get maximum window on PREFS now works in Linux
  ;        o Added +10 pixels to windows resolutions text to fit in Linux
  ; V2.5 - 20/MAR/2018
  ;        26/MAR/2018
  ;        o Cleaned the code
  ; V2.6 - 28/MAR/2018
  ;        o Support for Hyphenation
  ; V2.7 - 19/APR/2018
  ;        o Fix: Changing resolution in the Preferences would reset each tab filename to "n/a" and erase the Hyphenation rules
  ; V2.8 - 09/MAY/2018
  ;        o Coding of "Check for Updates"
  ; V2.9 - 11/JUL/2018
  ;        o Prefs: Now allows to set colour for "Primary", "Suffix" and "Prefix"   
  ; V3.0 - 25/JUL/2018
  ;        o Cleaned the code by using a function to turn the gadgets background to Red/White on errors.
  ; V3.1 - 05/DEC/2018
  ;        o Preferences now has option for "Both" in Affixes (CIRCUMFIX)
  ; V3.2 - 14/JAN/2019
  ;        o Fix: Combobox icons with colours now appear straight in Linux.
  ;        o If "Maximise window" height <600 pixels, the radio button gets disabled
  ; V3.3 - 13/MAR/2019
  ;        o Warn over N words while extracting and counting duplicates on wordlist  
  ; V3.4 - 14/MAR/2019
  ;        o Fixed and improved the Preferences window
  ; V3.5 - 22/MAR/2019
  ;        o Preferences: Small GUI fix for Linux.
  ; V3.6 - 12/JUN/2019
  ;        o GTK3 Ready
  ; V3.7 - 19/JUN/2019
  ;        o Preferences allow to choose the font for the language.
  ; V3.8 - 21/DEC/2020
  ;        o Prefs: "0" disables above N words warning
  ; V3.9 - 04/APR/2021
  ;        o Cleaned the Prefs code and disabled 1024x resolutions in it
  ;        o Fix font issue in Prefs
  ; V4.0 - 25/AUG/2021
  ;        o Added spaces on left regarding autocorrect (prefs + core)
  ; V4.1 - 03/SEP/2021
  ;        o Rewrote the "Preferences"
  ; V4.2 - 06/DEC/2021
  ;        o Tip on prefs improved and changed position.
  ; V4.3 - 07/JAN/2022
  ;        o Prefs default to full screen resolution and three simple rule for number of lines in full screen.
  ; V4.4 - 12/JAN/2022
  ;        o Fix: When opening Prefs for the first time, the tooltip would disappear while changing window resolutions
  ; V4.5 - 15/JUL/2023
  ;        16/JUL/2023
  ;        o Rewriting of the preferences window
  ;
  Procedure preferences()
    
    ; Get maximum window size from an invisible window
    ExamineDesktops()
    desktop_resolution$=Str(DesktopWidth(0))+"x"+Str(DesktopHeight(0))   
    window_resolution_max_width=GetMaxWindowDesktopWidth()
    window_resolution_max_height=GetMaxWindowDesktopHeight()
    window_resolution_max$=Str(window_resolution_max_width)+"x"+Str(window_resolution_max_height-StatusBarHeight(#WINDOW_MAIN_STATUS_BAR)-MenuHeight())
    
    
    ; Default if prefs not set yet - 7/JAN/2022
    If prefs_loaded_or_okay_pressed=#False
      window_resolution$=window_resolution_max$
;       number_of_lines=(300-30+5+40+get_extra_height(100)+pixels_plus)/21.5 ; Works well with x768 and x1014 (1920x1080)
      number_of_lines=(13*(300-30+5+40+get_extra_height(100)+pixels_plus)-Val(font_for_language_size$)*2)/(GadgetHeight(#PANEL_BOX_DICTIONARY)-Val(font_for_language_size$)*2)
    EndIf
    
    
    ; Open the window
    w=15+244+10-30-15+300-100-150-150-100+(800-640)+640-10+20+20+20+20
    h=50+50+1+2+6+6+5+5+20+10+30+200-50-20-20+100+100+200-30-30-30-40+10-10-5+get_extra_height(100)+10
    If OpenWindow(#WINDOW_PREFERENCES,0,0,w,h,"Preferences",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    DisableWindow(#WINDOW_MAIN,#True)
    
    
    ; Store old variables
    old_window_resolution$=window_resolution$
    old_number_of_lines=number_of_lines
    old_pixels_plus=pixels_plus
    old_number_of_lines_languagetool=number_of_lines_languagetool
    old_aff_aid_language$=aff_aid_language$
    old_number_separator_chr$=number_separator_chr$
    old_check_for_updates$=check_for_updates$
    old_last_check_updates_time.q=last_check_updates_time.q
    top_of_listicongadget_dictionary=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
    top_of_listicongadget_thesaurus=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)
    top_of_listicongadget_autocorrect=GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)
    old_affix_colour_primary=affix_colour_primary
    old_affix_colour_suffix=affix_colour_suffix
    old_affix_colour_prefix=affix_colour_prefix
    old_affix_colour_both=affix_colour_both
    old_ask_after_n_words=ask_after_n_words
    old_font_for_language$=font_for_language$
    old_font_for_language_size$=font_for_language_size$
    old_autocorrect_spaces_on_left=autocorrect_spaces_on_left
    
    
    ; Width of frame gadgets in Prefs
    frame_gadgets_width_prefs=10+200+10+10+15+10+20+20
    
    ; To make it easier to fix the Y coordinate of the warning signs
    adjust_warning_signs_y=5
    
    
    x=5
    y=10
    ListIconGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,x,y,200+20,300-30+5+40+get_extra_height(100)+pixels_plus,"# Visible lines (OS specific)",1000,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection|#PB_ListIcon_CheckBoxes)
    SetGadgetFont(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,FontID(#font_for_language_id))
    HideGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,#True)
    ; Populate the ListIconGadget
    For f=1 To 500
      AddGadgetItem(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,-1,"Line "+Str(f))
    Next f
    HideGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,#False)
  
    
    
    y+GadgetHeight(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+10+5+5
    
    y-pixels_plus
    
    ; SpinGadget with the number of lines
    SpinGadget(#SPIN_WINDOW_PREFERENCES_LINES,x,y,60-8,default_gadget_height_spin,1,100,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_LINES,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5,y+2,100,default_gadget_height_text,"Visible lines")
    SetGadgetTextState(#SPIN_WINDOW_PREFERENCES_LINES, number_of_lines)

    
    ; SpinGadget Pixels Plus
    y+GadgetHeight(#SPIN_WINDOW_PREFERENCES_LINES)+5+5
    SpinGadget(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,x,y,60-8,default_gadget_height_spin,-10,10,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_PLUSPIXELS,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS)+5,y+2,100,default_gadget_height_text,"Plus/Minus Pixels")
    GadgetFit(#TEXT_WINDOW_PREFERENCES_PLUSPIXELS)
    SetGadgetTextState(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS, pixels_plus)
    
    
    ; Window resolution
    x=GadgetX(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+GadgetWidth(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+10+15
    y=GadgetY(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)
    FrameGadget(#FRAME_WINDOW_PREFERENCES_RESOLUTION,x,y,frame_gadgets_width_prefs,(default_gadget_height_radio*3)+(5*3)+10+10+5-(2*2),"Resolution")
    
    y+5+10+5+3
    
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600,x+10,y,100-10-10+10,default_gadget_height_radio,"1280×600")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768,x+10+100,y,100-10-10+10+10+10,default_gadget_height_radio,"1280×768")
    y+default_gadget_height_radio+5-2
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600,x+10,y,100-10-10+10,default_gadget_height_radio,"1440×600")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768,x+10+100,y,100-10-10+10+10+10,default_gadget_height_radio,"1440×768")
    y+default_gadget_height_radio+5-2
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE,x+10,y,100-10-10+100+50,default_gadget_height_radio,"Maximise Window ("+ReplaceString(window_resolution_max$,"x","×")+")")
    
    GadgetFit(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600)
    ImageGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600_WARNING,GadgetX(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600)+GadgetWidth(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600),GadgetY(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600)+adjust_warning_signs_y,ImageWidth(121),ImageHeight(121),0)
    If window_resolution_max_width<1280 Or window_resolution_max_height<600 : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600_WARNING,ImageID(121)) : EndIf

    GadgetFit(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768)
    ImageGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768_WARNING,GadgetX(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768)+GadgetWidth(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768),GadgetY(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768)+adjust_warning_signs_y,ImageWidth(121),ImageHeight(121),0)
    If window_resolution_max_width<1280 Or window_resolution_max_height<768 : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768_WARNING,ImageID(121)) : EndIf
 
    GadgetFit(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600)
    ImageGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600_WARNING,GadgetX(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600)+GadgetWidth(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600),GadgetY(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600)+adjust_warning_signs_y,ImageWidth(121),ImageHeight(121),0)
    If window_resolution_max_width<1440 Or window_resolution_max_height<600 : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600_WARNING,ImageID(121)) : EndIf

    GadgetFit(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768)
    ImageGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768_WARNING,GadgetX(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768)+GadgetWidth(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768),GadgetY(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768)+adjust_warning_signs_y,ImageWidth(121),ImageHeight(121),0)
    If window_resolution_max_width<1440 Or window_resolution_max_height<768 : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768_WARNING,ImageID(121)) : EndIf

    GadgetFit(#OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE)
    If window_resolution_max_height<600 : DisableGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE,#True) : EndIf
    
    window_resolution_selected=-1
    If window_resolution$="1280x600" : window_resolution_selected=#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600 : EndIf
    If window_resolution$="1440x600" : window_resolution_selected=#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600 : EndIf
    If window_resolution$="1280x768" : window_resolution_selected=#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768 : EndIf
    If window_resolution$="1440x768" : window_resolution_selected=#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768 : EndIf
    If window_resolution_selected=-1
      window_resolution$=window_resolution_max$
      window_resolution_selected=#OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE
    EndIf
    SetGadgetState(window_resolution_selected,#True)
      
    
    ; LanguageTool - Minimum lines between rules
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_RESOLUTION)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_RESOLUTION)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_LANGUAGETOOL,x,y,frame_gadgets_width_prefs,default_gadget_height_spin+10+15+5,"LanguageTool")
    y+10+10
    SpinGadget(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,x+10,y,60-8,default_gadget_height_spin,1,50,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_LANGUAGETOOL,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL)+5+5+5,GadgetY(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL)+2,200,default_gadget_height_text,"Minimum lines between rules")
    SetGadgetTextState(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL, number_of_lines_languagetool)
    
    
    ; .AFF Aid combobox
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_LANGUAGETOOL)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_LANGUAGETOOL)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_AFF_AID,x,y,frame_gadgets_width_prefs,default_gadget_height_combobox+10+15+5,".AFF Aid Language")
    y+10+10
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,x+10,y,frame_gadgets_width_prefs-10-10,default_gadget_height_combobox,#PB_ComboBox_Image)
    AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,-1,"n/a")
    current_aff_aid_language=0
    image=0
    For f=0 To ArraySize(aff_aid_language_array$())
      If StringField(aff_aid_language_array$(f),3,#LF$)<>"" : image=Val(StringField(aff_aid_language_array$(f),2,#LF$)) : EndIf
      If IsImage(image)
        AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,-1,StringField(aff_aid_language_array$(f),1,#LF$),ImageID(image))
      Else
        AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,-1,StringField(aff_aid_language_array$(f),1,#LF$))
      EndIf
      If aff_aid_language$=StringField(aff_aid_language_array$(f),3,#LF$) : current_aff_aid_language=f+1 : EndIf  
    Next f
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,current_aff_aid_language)
    
    
    ; Number separator chr
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_AFF_AID)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_AFF_AID)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,x,y,frame_gadgets_width_prefs,default_gadget_height_string+10+15+5,"Number separator character")      
    y+10+10
    StringGadget(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,x+10,y,16+2+10,default_gadget_height_string,number_separator_chr$)
    SetGadgetAttribute(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,#PB_String_MaximumLength,1)
    TextGadget(#TEXT_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,x+10+GadgetWidth(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR)+5,y,200,default_gadget_height_text,"Preview:"+FormatNumber(1000000,0,".",number_separator_chr$))
    GadgetToolTip(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,"One character only. It can be left blank.")
    
    
    ; Warn over N words decoding
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_WARN_OVER_N_WORDS,x,y,frame_gadgets_width_prefs,default_gadget_height_spin+10+15+5,"Warn over N words decoding (RAM/SSD/HDD)")
    y+10+10
    SpinGadget(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS,x+10,y,60-8+30+10,default_gadget_height_spin,0,99999999,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_WARN_OVER_N_WORDS,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)+5+5+5,GadgetY(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)+2,200,default_gadget_height_text,"Words (0 disables)")
    GadgetFit(#TEXT_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)
    ImageGadget(#TEXT_WINDOW_PREFERENCES_WARN_OVER_N_WORDS_WARNING,GadgetX(#TEXT_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)+GadgetWidth(#TEXT_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)+2,adjust_warning_signs_y+GadgetY(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)+2-1,ImageWidth(121),ImageHeight(121),ImageID(121))
    SetGadgetTextState(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS, ask_after_n_words)


    ; Autocorrect - spaces on the left
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_WARN_OVER_N_WORDS)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_AUTOCORRECT,x,y,frame_gadgets_width_prefs,default_gadget_height_spin+10+15+5,"Autocorrect")
    y+10+10
    SpinGadget(#SPIN_WINDOW_PREFERENCES_AUTOCORRECT,x+10,y,60-8,default_gadget_height_spin,1,50,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_AUTOCORRECT,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_AUTOCORRECT)+5+5+5,GadgetY(#SPIN_WINDOW_PREFERENCES_AUTOCORRECT)+2,200,default_gadget_height_text,"Spaces on the left of each line")
    GadgetFit(#TEXT_WINDOW_PREFERENCES_AUTOCORRECT)
    SetGadgetTextState(#SPIN_WINDOW_PREFERENCES_AUTOCORRECT, autocorrect_spaces_on_left)    
    
    
    ; Check for updates
    x=GadgetX(#FRAME_WINDOW_PREFERENCES_RESOLUTION)+GadgetWidth(#FRAME_WINDOW_PREFERENCES_RESOLUTION)+10+15
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_RESOLUTION)
    FrameGadget(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES,x,y,frame_gadgets_width_prefs,(default_gadget_height_radio*4)+(4*5)+5+10+15+5+default_gadget_height_text+default_gadget_height_button-10-(3*2),"Check for Updates")
    y+10+10
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL,x+10,y,100-10-10+10,default_gadget_height_radio,"Manual")
    y+5+default_gadget_height_radio-2
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_DAILY,x+10,y,100-10-10+10,default_gadget_height_radio,"Daily")
    y+5+default_gadget_height_radio-2
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_WEEKLY,x+10,y,100-10-10+10,default_gadget_height_radio,"Weekly")
    y+5+default_gadget_height_radio-2
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY,x+10,y,100-10-10+100+50,default_gadget_height_radio,"Monthly")
    y+2+default_gadget_height_radio
    TextGadget(#TEXT_WINDOW_PREFERENCES_CHECKUPDATES_LASTCHECKED,x+10,y,100-10-10+100+50,default_gadget_height_text,"Last checked:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",last_check_updates_time.q))
    y+2+default_gadget_height_text
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_CHECKUPDATES,x+10,y,90,default_gadget_height_button,"Check")
    If check_for_updates$="manual" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL,#True) : EndIf
    If check_for_updates$="daily" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_DAILY,#True) : EndIf
    If check_for_updates$="weekly" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_WEEKLY,#True) : EndIf
    If check_for_updates$="monthly" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY,#True) : EndIf
    
    
    ; Colours for dictionary editor - 11/JUL/2018
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES)+10
    x=GadgetX(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES)
    FrameGadget(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY,x,y,frame_gadgets_width_prefs,(default_gadget_height_combobox*2)+(5*2)+15+10,"Affix Colours")
    y+10+5+5
    x+10
    y+2
    TextGadget(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,x,y+2,90-15-20,default_gadget_height_text,"Primary ",#PB_Text_Right)
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,x+90+5-15-20,y,50,default_gadget_height_combobox,#PB_ComboBox_Image)
    For f=0 To 7
      AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,-1,"",ImageID(61+f))
    Next f    
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,affix_colour_primary)
    y+25+5
    y+2
    y+4
    TextGadget(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,x,y+2,90-15-20,default_gadget_height_text,"Suffix ",#PB_Text_Right)
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,x+90+5-15-20,y,50,default_gadget_height_combobox,#PB_ComboBox_Image)
    For f=0 To 7
      AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,-1,"",ImageID(61+f))
    Next f    
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,affix_colour_suffix)
    y=GadgetY(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY)
    x=GadgetX(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX)+GadgetWidth(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX)+10+40+5
    TextGadget(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_BOTH,x,y+2,90-15-20,default_gadget_height_text,"Both ",#PB_Text_Right)
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_BOTH,x+90+5-15-20,y,50,default_gadget_height_combobox,#PB_ComboBox_Image)
    For f=0 To 7
      AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_BOTH,-1,"",ImageID(61+f))
    Next f    
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_BOTH,affix_colour_both)    
    x=GadgetX(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX)+GadgetWidth(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX)+10+40+5
    y=GadgetY(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX)
    TextGadget(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,x,y+2,90-15-20,default_gadget_height_text,"Prefix ",#PB_Text_Right)
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,x+90+5-15-20,y,50,default_gadget_height_combobox,#PB_ComboBox_Image)
    For f=0 To 7
      AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,-1,"",ImageID(61+f))
    Next f    
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,affix_colour_prefix)    
    
    x=GadgetX(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY)
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY)+10
    
    
    ; Font for Language
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE,x,y,frame_gadgets_width_prefs,default_gadget_height_string+5+default_gadget_height_string+10+15+5,"Font for language")
    y+10+10  
    StringGadget(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE,x+10,y,100+30+15,default_gadget_height_string,font_for_language$,#PB_String_ReadOnly)
    StringGadget(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_SIZE,GadgetX(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE)+GadgetWidth(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE)+10,y,-3-2+100-40-15,default_gadget_height_string,font_for_language_size$,#PB_String_ReadOnly)
    ButtonImageGadget(#BUTTON_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE,GadgetX(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_SIZE)+GadgetWidth(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_SIZE)+10,y,default_gadget_height_button,default_gadget_height_button,ImageID(2))
    y+default_gadget_height_string+5
    StringGadget(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_TEST,x+10,y,frame_gadgets_width_prefs-10-10,default_gadget_height_string,"")
    SetGadgetFont(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_TEST,FontID(#font_for_language_id))
    GadgetToolTip(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_TEST,"Test the font here")
    
    
    ; Add the SAVE, OK and CANCEL buttons
    y=h-default_gadget_height_button-10
    x=GetCentreX(w,default_gadget_width_button_ok_close_cancel)-10-default_gadget_width_button_ok_close_cancel
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_SAVE,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"Save")
    x+default_gadget_width_button_ok_close_cancel+10
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_OKAY,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"OK")
    x+default_gadget_width_button_ok_close_cancel+10
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_CANCEL,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"Cancel")    
    
    
    ; Display the tip
    x=GadgetX(#BUTTON_WINDOW_PREFERENCES_SAVE)
    y=GadgetY(#BUTTON_WINDOW_PREFERENCES_SAVE)-10-ImageHeight(28)+3
    ImageGadget(#IMAGE_WINDOW_PREFERENCES_TIP,x,y,ImageWidth(28),ImageHeight(28),ImageID(28))
    
    
    ; Set the line selected in the number of lines
    SetGadgetItemState(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,number_of_lines-1,#PB_ListIcon_Selected)
    SetActiveGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)
                       
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_PREFERENCES,#PB_Shortcut_Escape,1000)    
    AddKeyboardShortcut(#WINDOW_PREFERENCES,#PB_Shortcut_Return,1003)  
    
    
    okay=#False
    cancel=#False
    Repeat
          
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu) or <ESC> pressed
      If event=#PB_Event_CloseWindow Or (event=#PB_Event_Menu And EventMenu()=1000)
        cancel=#True     
      EndIf     
      
      
      Select Event 
        Case #PB_Event_Gadget
          Select EventGadget()      
              
              ; Save button
              Case #BUTTON_WINDOW_PREFERENCES_SAVE
                Debug "Pressed SAVE"
                
                flag=#False
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_LINES,1,100,"The number of lines must be in the range 1:100",flag)
                  Continue
                EndIf                   
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,1,50,"The number of lines must be in the range 1:50",flag)
                  Continue
                EndIf                      
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,-10,10,"The number of 'Pixels Plus' must be in the range -10:10",flag)
                  Continue
                EndIf      
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS,0,99999999,"The number of 'Warn Over N Word' must be in the range 0:99999999",flag)
                  Continue
                EndIf                               
                
                save_preferences()                                       
                 
                 
              ; OK button
              Case #BUTTON_WINDOW_PREFERENCES_OKAY
                Debug "Pressed OK"
                flag=#False
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_LINES,1,100,"The number of lines must be in the range 1:100",flag)
                  Continue
                EndIf                   
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,1,50,"The number of lines must be in the range 1:50",flag)
                  Continue
                EndIf                      
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,-10,10,"The number of 'Pixels Plus' must be in the range -10:10",flag)
                  Continue
                EndIf      
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS,0,99999999,"The number of 'Warn Over N Word' must be in the range 1:99999999",flag)
                  Continue
                EndIf    
                If InvalidValue(#SPIN_WINDOW_PREFERENCES_AUTOCORRECT,1,50,"The number of spaces on left for autocorrect must be in the range 1:50",flag)
                  Continue
                EndIf                      
                
                prefs_loaded_or_okay_pressed=#True
                okay=#True
                
                
              ; Cancel button
              Case #BUTTON_WINDOW_PREFERENCES_CANCEL
                Debug "Pressed CANCEL"
                cancel=#True
                
                
              ; Spin Gadget NUMBER OF VISIBLE LINES
              Case #SPIN_WINDOW_PREFERENCES_LINES
                Debug "Pressed SPIN"                  
                  
                number_of_lines=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_LINES,1,100,#False,#False)
                SetGadgetItemState(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,number_of_lines-1,#PB_ListIcon_Selected)
                
                
              ; Double-click
              Case #LISTICONGADGET_WINDOW_PREFERENCES_LINES
                Debug "Double-click on ListIconGadget"
                number_of_lines=GetGadgetState(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+1
                SetGadgetTextState(#SPIN_WINDOW_PREFERENCES_LINES, number_of_lines)

                
              ; Spin Gadget PIXELS PLUS 
              Case #SPIN_WINDOW_PREFERENCES_PLUSPIXELS
                Debug "Pressed SPIN Pixels Plus"                  
            
                pixels_plus=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,-10,10,#False,#False)
                     
                If GetGadgetColor(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,#PB_Gadget_BackColor)=-1

                  ; Resize ListIconGadget
                  ResizeGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,GadgetX(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetY(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetWidth(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),300-30+5+40+get_extra_height(100)+pixels_plus)
                      
                EndIf                      
                         
                
              ; Spin Gadget WARN OVER N WORDS
              Case #SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS
                Debug "Pressed SPIN"                  
                  
                ask_after_n_words=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_WARN_OVER_N_WORDS,0,99999999,#False,#False)
                      
                
              ; Change the font for language
              Case #BUTTON_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE
                Debug "Change the font for language button pressed"
                
                If FontRequester("Arial",10,#PB_FontRequester_Effects)
                  
                  font_for_language$=SelectedFontName()
                  font_for_language_size$=Str(SelectedFontSize())
                  SetGadgetText(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE,font_for_language$)
                  SetGadgetText(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_SIZE,font_for_language_size$)
                  
                  If font_for_language$<>old_font_for_language$ Or font_for_language_size$<>old_font_for_language_size$
                    t=LoadFont(#font_for_language_id_temp,font_for_language$,Val(font_for_language_size$),#PB_Font_HighQuality) ;Load language font - 19/JUN/2019
                    If t=#False : MessageRequester("Error", "Can't load "+font_for_language$+" font.",#PB_MessageRequester_Error) : EndIf          
                  EndIf

                  SetGadgetFont(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,FontID(#font_for_language_id_temp))
                  SetGadgetFont(#STRING_WINDOW_PREFERENCES_FONT_FOR_LANGUAGE_TEST,FontID(#font_for_language_id_temp))
                  
                EndIf
                
                
              ; Resolution of window
              Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600 To #OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768
                 
                Select EventGadget()
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600
                    window_resolution$="1280x600"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600
                    window_resolution$="1440x600"
                                        
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768
                    window_resolution$="1280x768"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768
                    window_resolution$="1440x768"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE
                    window_resolution$=window_resolution_max$
                    
                EndSelect
                
                ; Resize window
                ResizeWindow(#WINDOW_PREFERENCES,WindowX(#WINDOW_PREFERENCES),WindowY(#WINDOW_PREFERENCES),WindowWidth(#WINDOW_PREFERENCES),1+2+6+6+5+5+20+10+30+200-50-20-20+100+100+200-30-30-30-40+10-10-5+get_extra_height(100)+50+50+10)
                
                ; Resize ListIconGadget
                ResizeGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,GadgetX(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetY(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetWidth(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),300-30+5+40+get_extra_height(100)+pixels_plus)
                
                ; Move # of lines
                x=5
                y=10+GadgetHeight(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+10+5+5
                y-pixels_plus
                ; SpinGadget with the number of lines
                ResizeGadget(#SPIN_WINDOW_PREFERENCES_LINES,x,y,60-8,default_gadget_height_spin)
                ResizeGadget(#TEXT_WINDOW_PREFERENCES_LINES,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5,y+2,100,default_gadget_height_text)                
                ; Move Pixels Plus
                ; SpinGadget Pixels Plus
                y+GadgetHeight(#SPIN_WINDOW_PREFERENCES_LINES)+5+5
                ResizeGadget(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,x,y,60-8,default_gadget_height_spin)
                ResizeGadget(#TEXT_WINDOW_PREFERENCES_PLUSPIXELS,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS)+5,y+2,100,default_gadget_height_text)  
                GadgetFit(#TEXT_WINDOW_PREFERENCES_PLUSPIXELS)
                
                ; Move Save, OK and Cancel
                ; Add the SAVE, OK and CANCEL buttons
                y=WindowHeight(#WINDOW_PREFERENCES)-default_gadget_height_button-10
                x=GadgetX(#SPIN_WINDOW_PREFERENCES_LINES)+20+10+5+30+30+ImageWidth(28)/2+10 
                ResizeGadget(#BUTTON_WINDOW_PREFERENCES_SAVE,x,y,90,default_gadget_height_button)              
                ResizeGadget(#BUTTON_WINDOW_PREFERENCES_OKAY,x+100,y,90,default_gadget_height_button)
                ResizeGadget(#BUTTON_WINDOW_PREFERENCES_CANCEL,x+100+100,y,90,default_gadget_height_button)
                
                ; Display the tip
                x=GadgetX(#BUTTON_WINDOW_PREFERENCES_SAVE)
                y=GadgetY(#BUTTON_WINDOW_PREFERENCES_SAVE)-10-ImageHeight(28)+3
                ResizeGadget(#IMAGE_WINDOW_PREFERENCES_TIP,x,y,ImageWidth(28),ImageHeight(28))
                
                
                
              ; Spin Gadget - LanguageTool - Minimum lines between rules
              Case #SPIN_WINDOW_PREFERENCES_LANGUAGETOOL
                Debug "Pressed SPIN"                  
            
                number_of_lines_languagetool=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,1,50,#False,#False)
                                         
                
              ; Change the ComboBox of the .AFF Aid country
              Case #COMBOBOX_WINDOW_PREFERENCES_AFF_AID
                Debug "Changed the .AFF Aid country"
                  
                If GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID)=0
                  aff_aid_language$=""
                Else
                  aff_aid_language$=StringField(aff_aid_language_array$(GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID)-1),3,#LF$)
                EndIf
                                  
                
              ; Change the Number separator chr
              Case #STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR
                Debug "Changed the Number separator chr"
                number_separator_chr$=GetGadgetText(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR)
                SetGadgetText(#TEXT_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,"Preview:"+FormatNumber(1000000,0,".",number_separator_chr$))
                
                
              ; CHECK button
              Case #BUTTON_WINDOW_PREFERENCES_CHECKUPDATES
                check_for_updates(#False)
                last_check_updates_time.q=Date()
                SetGadgetText(#TEXT_WINDOW_PREFERENCES_CHECKUPDATES_LASTCHECKED,"Last checked:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",last_check_updates_time.q))              
               
                
              ; Check for updates
              Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL To #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY
                  
                  Select EventGadget()
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL
                      check_for_updates$="manual"
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_DAILY
                      check_for_updates$="daily"
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_WEEKLY
                      check_for_updates$="weekly"
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY
                      check_for_updates$="monthly"
                      
                  EndSelect 
                  
                  
            ; Affixes colours
            ; Primary
            Case #COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY
              affix_colour_primary=GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY)
              
            ; Suffix                
            Case  #COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX
              affix_colour_suffix=GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX)    
              
            ; Prefix
            Case #COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX
              affix_colour_prefix=GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX) 
                  
            ; Both
            Case #COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_BOTH
              affix_colour_both=GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_BOTH)               
              
              
            Case #SPIN_WINDOW_PREFERENCES_AUTOCORRECT
              Debug "Pressed SPIN Autocorrect left chrs"                  
            
              autocorrect_spaces_on_left=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_AUTOCORRECT,1,50,#False,#False)              
              
          EndSelect  
                  
              
      EndSelect
      
      
    Until okay=#True Or cancel=#True
    
    
    ; Pressed CANCEL
    ; Restore old variables
    If cancel=#True
      
      window_resolution$=old_window_resolution$
      number_of_lines=old_number_of_lines
      pixels_plus=old_pixels_plus
      number_of_lines_languagetool=old_number_of_lines_languagetool    
      aff_aid_language$=old_aff_aid_language$      
      number_separator_chr$=old_number_separator_chr$
      check_for_updates$=old_check_for_updates$
      last_check_updates_time.q=old_last_check_updates_time.q
      affix_colour_primary=old_affix_colour_primary
      affix_colour_suffix=old_affix_colour_suffix
      affix_colour_prefix=old_affix_colour_prefix
      affix_colour_both=old_affix_colour_both
      ask_after_n_words=old_ask_after_n_words
      autocorrect_spaces_on_left=old_autocorrect_spaces_on_left
      
      font_for_language$=old_font_for_language$
      font_for_language_size$=old_font_for_language_size$
      
    EndIf
    
    
    ; Pressed OK
    If okay=#True

      If StringField(old_window_resolution$,1,"x")<>Str(1024+get_extra_width(100)) Or StringField(old_window_resolution$,2,"x")<>Str(600+get_extra_height(100)) Or old_pixels_plus<>pixels_plus Or font_for_language$<>old_font_for_language$ Or font_for_language_size$<>old_font_for_language_size$
        
        If font_for_language$<>old_font_for_language$ Or font_for_language_size$<>old_font_for_language_size$
          t=LoadFont(#font_for_language_id,font_for_language$,Val(font_for_language_size$),#PB_Font_HighQuality) ;Load language font - 19/JUN/2019
          If t=#False : MessageRequester("Error", "Can't load "+font_for_language$+" font.",#PB_MessageRequester_Error) : EndIf          
        EndIf
        
        current_tab=GetGadgetState(#PANEL_MAIN)
        t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
        t1$=GetGadgetText(#EDITORGADGET_PANEL_HYPHENATOR_DIC)
        t2$=GetGadgetText(#STRING_PANEL_HYPHENATOR_WORD)
        t3$=""
        If CountGadgetItems(#LISTICONGADGET_PANEL_HYPHENATOR_DIC)>0
          For f=1-1 To CountGadgetItems(#LISTICONGADGET_PANEL_HYPHENATOR_DIC)-1
            t3$+GetGadgetItemText(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,f,0)+#LF$
            t3$+GetGadgetItemText(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,f,1)+#LF$
            t3$+GetGadgetItemText(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,f,2)+#LF$
            t3$+Chr(9)
          Next f
        EndIf             
        CloseWindow(#WINDOW_MAIN)        
        draw_GUI()
        SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,t$)
        SetGadgetText(#EDITORGADGET_PANEL_HYPHENATOR_DIC,t1$)
        SetGadgetText(#STRING_PANEL_HYPHENATOR_WORD,t2$)
        If t3$<>""
          For f=1 To CountString(t3$,Chr(9))
            AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(t3$,f,Chr(9)))
          Next f
        EndIf
        update_number_words_for_hyphenation()
        SetGadgetState(#PANEL_MAIN,current_tab) ; Fixed on 20/DEC/2016
        BindGadgetEvent(#PANEL_BOX_DICTIONARY_VERTICAL_BAR, @global_update_virtual_vertical_scroll_bar_pre()) ; Dictionary
        BindGadgetEvent(#PANEL_BOX_SYNONYMS_VERTICAL_BAR, @global_update_virtual_vertical_scroll_bar_pre())    ; Thesaurs
        BindGadgetEvent(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR, @global_update_virtual_vertical_scroll_bar_pre()) ; Hyphenation
        BindGadgetEvent(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR, @global_update_virtual_vertical_scroll_bar_pre()) ; Autocorrect
        update_filename_dictionary()
        update_filename_synonyms()
        update_filename_hyphenation()
        update_filename_autocorrect_list()
      EndIf

      ; Change the paging of the vertical scroll bars
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)
      SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)
      SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)      
      SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)        

      ; Reset dictionary
      selected_dictionary_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
      global_update_virtual_vertical_scroll_bar(-1)

      ; Reset thesaurus
      selected_thesaurus_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
      global_update_virtual_vertical_scroll_bar(-1)
      
      ; Reset hyphenation
      selected_hyphenation_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
      global_update_virtual_vertical_scroll_bar(-1)
      
      ; Reset autocorrect
      selected_autocorrect_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
      global_update_virtual_vertical_scroll_bar(-1)                
      
      ; Place back the first item on top of the ListIconGadget
      SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,top_of_listicongadget_dictionary)
      SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,top_of_listicongadget_thesaurus)
      SetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,top_of_listicongadget_autocorrect)
      global_update_virtual_vertical_scroll_bar(-1)
      
      ; Reset the .AFF Aid current combobox position if changed the country
      If aff_aid_language$<>old_aff_aid_language$ : aff_aid_language_current_combobox_entry=0 : EndIf
      
    EndIf
      
        
    ; Close the Preferences window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    If IsWindow(#WINDOW_PREFERENCES) : CloseWindow(#WINDOW_PREFERENCES) : EndIf ; Close it only, if the changing of window resolution didn't do it
    
  EndProcedure

; IDE Options = PureBasic 6.03 LTS (Windows - x64)
; CursorPosition = 938
; FirstLine = 931
; Folding = -
; EnableXP
; DisableDebugger
; CompileSourceDirectory
; EnableUnicode