﻿  ; Load the words from a .DIC file (Dictionary)
  ;
  ; V1.0 - 5/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 5/JUN/2013
  ;        o Coded the function
  ; V1.2 - 7/JUN/2013
  ;        o It now loads the .AFF file and checks if it is in UTF-8,
  ;          showing a warning if not.
  ; V1.3 - 8/JUN/2013
  ;        <ESC> now aborts loading
  ; V1.4 - 20/JUN/2013
  ;        o A HUGE speed gain thanks to Ashley from the PureBasic Forum
  ; V1.5 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.6 - 26/JUN/2013
  ;        o Changed the non UTF-8 warning to a better one
  ;        o Changed the WindowTimer code To make it work better (faster) in Linux.
  ; V1.7 - 8/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.8 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.9 - 7/FEB/2014
  ;        o Code improved
  ; V2.0 - 27/FEB/2014
  ;        o Removed the file buffer
  ;        o Array used instead of listicongadget
  ;        o Fix: The progress bar now works in Linux
  ; V2.1 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1    
  ; V2.2 - 21/MAR/2014  
  ;        o Resets progress bar when task completes  
  ; V2.3 - 16/NOV/2014    
  ;        o Simplified and optimised the code thanks to Bernd from the PureBasic forum
  ; V2.4 - 21/NOV/2014      
  ;        o Converts .DIC words separated from tags by a space (ex: Galician) asking first.
  ; V2.5 - 3/DEC/2014        
  ;        o Fixed a bug of adding multiple tabs while opening a .DIC with spaces instead of tabs
  ; V2.6 - 22/APR/2015
  ;        o In the requesters complaining about "UTF-8" it now says "UTF-8 without BOM"
  ; V2.7 - 30/OCT/2015
  ;        o Replaced the "UTF-8 W/O BOM" with "UTF-8 WITHOUT BOM" to make it more understandable.
  ; V2.8 - 23/MAR/2016
  ;        o Used the flag #PB_ByteLength in PeekS to avoid trash at the end of the .AFF file when it is loaded
  ; V2.9 - 29/AUG/2016
  ;        o Simplified the .AFF loading
  ; V3.0 - 22/DEC/2016
  ;        o Started implementing the multiselection.
  ; V3.1 - 7/JUN/2017
  ;        o undo/redo support  
  ; V3.2 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry 
  ; V3.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V3.4 - 28/MAR/2018
  ;        o Convert .AFF + .DIC (hyphenation) to #LF$
  ; V3.5 - 06/FEB/2019
  ;        o Fix: Removed flag "=" to be recognised as morphological information on dictionary load
  ; V3.6 - 30/SEP/2019
  ;        o Replaced "UTF-8 WITHOUT BOM" with just "UTF-8".
  ; V3.7 - 10/FEB/2020
  ;        o Now reads BOM files (3-FEB added) properly without the warning visible on Autocorrect.
  ; V3.8 - 21/MAR/2020
  ;        o Better warning if no "SET UTF-8" is found in the .AFF while loading a .DIC
  ; V3.9 - 06/JAN/2021
  ;        o Options that use file open show on the status bar the filesize after "OK."
  ; V4.0 - 02/JUL/2021
  ;        o Now the sizes of load/save main files for each tab shows the files label. Ex: "(aff:1 MB)+(dic: 10MB)"
  ; V4.1 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure load_dictionary(force_file$)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading Dictionary... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    
    ; Confirm if to load a new Dictionary if the file in memory has been changed
    If dictionary_changed=#True
      If MessageRequester("Warning!","The Dictionary has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf    
    
    
    If force_file$=""
      file$=OpenFileRequester("Open Dictionary", "", "Dictionary (*.dic)|*.dic", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
        ProcedureReturn #False
      EndIf
    Else
      file$=force_file$
    EndIf      
    
    
    ; Check if .AFF file exists
    If FileSize(Left(file$,Len(file$)-3)+"aff") <= 0
      MessageRequester("Error", ".AFF file not found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf    
    
    
    ; Check if .DIC file exists
    If FileSize(file$) <= 0
      MessageRequester("Error", ".DIC file not found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf    
        
    
    ; Check If .AFF is in UTF-8 format (we presume that the .DIC is in the same format as the .AFF)
    ReadFile(1,Left(file$,Len(file$)-3)+"aff")
    ReadStringFormat(1)
    t=#False
    Repeat
      text$=UCase(ReadString(1,#PB_UTF8))
      If Left(text$,3)="SET" : t=#True :EndIf
    Until Eof(1) Or t=#True
    CloseFile(1)
    If Mid(text$,5)<>"UTF-8"
      If MessageRequester("Warning!","Dictionary is not in "+Chr(34)+"UTF-8-BOM"+Chr(34)+" format."+#LF$+#LF$+"The .AFF must start with:"+#LF$+Chr(34)+"SET UTF-8"+Chr(34)+#LF$+#LF$+"It may lead to corrupt results."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        ProcedureReturn #False
      EndIf
    EndIf    
    
    
    dictionary_disable_undo_redo() ; 7/JUN/2017
    
    
    selected_dictionary_word=-1 ; Reset the current word selected
    dictionary_changed=#False ; Dictionary set to unchanged
    ; Load .AFF file
    SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,"") ; Clear the .AFF text gadget   
    time_taken = ElapsedMilliseconds()
    ReadFile(1,Left(file$,Len(file$)-3)+"aff")
    ReadStringFormat(1)
      buffer$=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)  
      filesize_aff=Lof(1)
    CloseFile(1)
    buffer$=ConvertStringToUnix(buffer$)
    SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,buffer$)
             
    
    ; Read .DIC file
    If number_of_words>0 : ClearGadgetItems(#PANEL_BOX_DICTIONARY) : EndIf ; To remove the current words from the gadgetlist    
    convert_space_to_tab=-1 ;Convert space to tab (Galician dictionary) -1 = Didn't ask | 0 = Asked and NO | 1 = Asked and YES
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    ReadFile(1,File$)  
      ReadStringFormat(1)
      number_of_words=0
      Dim words$(0)
      Dim words_selected(0)
      number_of_words_total_selected=0
      text$=ReadString(1,#PB_UTF8) ;Just to read the header of the .DIC which contains the number of words
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,Lof(1))
      Repeat
        word$=ReadString(1,#PB_UTF8)
        ; Ask here to convert space to tab *START*
        If FindString(word$,#TAB$)<>0 : convert_space_to_tab=0 : EndIf ; 3/DEC/2014 - Fixed a bug of adding multiple tabs while opening a .DIC with spaces instead of tabs
; Below removed flag "=" to be recognised as morphological information - 6/FEB/2019
;         If FindString(word$,#TAB$)=0 And (FindString(word$,"[")<>0 Or FindString(word$,"]")<>0 Or FindString(word$,",")<>0 Or FindString(word$,"=")<>0 Or FindString(word$,":")<>0) And convert_space_to_tab=-1
        If FindString(word$,#TAB$)=0 And (FindString(word$,"[")<>0 Or FindString(word$,"]")<>0 Or FindString(word$,",")<>0 Or FindString(word$,":")<>0) And convert_space_to_tab=-1  
          If MessageRequester("Warning!","This .DIC seems to separate Words from Tags using a space."+#LF$+#LF$+"Use first space as a separator?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            convert_space_to_tab=1 ; YES
          Else
            convert_space_to_tab=0 ; NO
          EndIf      
        EndIf
        ; Ask here to convert space to tab *END*
        If convert_space_to_tab=1
          ReplaceString(word$," ",#LF$,#PB_String_InPlace,1,1)
        Else
          ReplaceString(word$,#TAB$,#LF$,#PB_String_InPlace,1,1)
        EndIf
        ReDim words$(number_of_words)
        ReDim words_selected(number_of_words)
        words$(number_of_words)=word$
        number_of_words+1
;        AddGadgetItem(#PANEL_BOX_DICTIONARY,-1,word$)
        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event=WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          update_number_words()
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading Dictionary... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,Loc(1),#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
         If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)           
            ProcedureReturn #False
         EndIf
        EndIf

      Until Eof(1)
    filesize_dic=Lof(1)
    CloseFile(1)  
    update_number_words()
    RemoveWindowTimer(#WINDOW_MAIN,800)
    dictionary_words_filename$=file$
    update_filename_dictionary()
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 21/MAR/2014
      
    ; New code to handle the vertical scroll bar
    SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(-1)
    
    ; 31/OCT/2017 - Support for "Recent Files" menu entry
    create_load_update_recent_files(file$)
    update_recent_files_menu()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("aff:",filesize_aff,#True)+"+"+ShowFileSizeFormatted("dic:",filesize_dic,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
  EndProcedure
  
  
  
  
  ; Save/Save As the words to a .DIC file (Dictionary) + .AFF file
  ;
  ; V1.0 - 5/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 16/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 20/JUN/2013
  ;        o HUGE speed gain saving the .AFF file and removed the first phase
  ;          that would first convert the listbox to an array and then save
  ;          the array.
  ; V1.3 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.4 - 26/JUN/2013
  ;        o Changed the WindowTimer code to make it work better (faster) in Linux.
  ; V1.5 - 11/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.6 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.7 - 27/FEB/2014
  ;        o Removed the file buffer
  ;        o It now works with V3+ words$() 
  ; V1.8 - 28/FEB/2014
  ;        o Changed the WindowTimer code To make it work better (faster) in Linux.
  ;        o Fix: The progress bar now works in Linux
  ; V1.9 - 27/MAR/2014  
  ;        o Resets progress bar when task completes   
  ; V2.0 - 3/DEC/2014
  ;        o Simplified and optimised the save dictionary code thanks to Bernd from the PureBasic forum.
  ; V2.1 - 11/SEP/2016
  ;        o Now it saves Dictionary + Thesaurus + Autocorrect with #LF$ instead of #CRLF for Linux compatibility
  ; V2.2 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry
  ; V2.3 - 01/DEC/2017
  ;        o Message Requesters now have a sign.
  ; V2.4 - 11/APR/2019
  ;        o Saving the dictionary also shows "Saving .aff file..."
  ; V2.5 - 03/FEB/2020
  ;        o Dictionary now has a BOM
  ; V2.6 - 02/DEC/2020
  ;        o Options that use file save/extract shown on the status bar show the filesize after "Okay."
  ; V2.7 - 02/JUL/2021
  ;        o Now the sizes of load/save main files for each tab shows the files label. Ex: "(aff:1 MB)+(dic: 10MB)"  
  ; V2.8 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure save_saveas_dictionary(save, saveas)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Dictionary... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) 
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    
    t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
    If t$=""
      MessageRequester("Warning!","The .AFF file is empty.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf    
 
    If save=1
      file$=dictionary_words_filename$
      If file$="" : saveas=1 : EndIf
    EndIf
    
    If saveas=1
      file$=SaveFileRequesterEnhanced("Save Dictionary","", "Dictionary (*.dic)|*.dic",".dic")
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf
    EndIf

    If FileSize(file$)>0 : DeleteFile(file$) : EndIf ; Delete .DIC file
    If FileSize(Left(file$,Len(file$)-4)+".aff")>0 : DeleteFile(Left(file$,Len(file$)-4)+".aff") : EndIf ; Delete the .AFF file
    
    
    ; Save the .AFF file, using UTF-8 encoding
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving .aff file...")
    t$=ConvertStringToUnix(GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF))
    OpenFile(1,Left(file$,Len(file$)-4)+".aff")
      WriteStringFormat(1,#PB_UTF8)
      WriteString(1,t$,#PB_UTF8)
      filesize_aff=Lof(1)
    CloseFile(1)
              

    ; Saving the .DIC file
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Dictionary... <ESC> to abort.")
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,File$)
      WriteStringFormat(1,#PB_UTF8)
      WriteString(1,Str(number_of_words)+#LF$,#PB_UTF8) ; Write the # of words  
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
      time_taken=ElapsedMilliseconds()
      For f=0 To number_of_words-1
        text$=words$(f)
        ReplaceString(text$,#LF$,Chr(9),#PB_String_InPlace,1)
        WriteString(1,text$+#LF$,#PB_UTF8)
                      
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Dictionary... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        

      Next f
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      filesize_dic=Lof(1)
      CloseFile(1)
      
      ; Dictionary set to unchanged
      dictionary_changed=#False
      update_number_words()
      
      ; 31/OCT/2017 - Support for "Recent Files" menu entry
      create_load_update_recent_files(file$)
      update_recent_files_menu()      
      
      dictionary_words_filename$=file$
      update_filename_dictionary()
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("aff:",filesize_aff,#True)+"+"+ShowFileSizeFormatted("dic:",filesize_dic,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
  EndProcedure
  
  
  
  
  ; Updates the number of words in the Dictionary
  ; Useful in open, add, delete
  ;
  ; V1.0 - 5/JUN/2013
  ; V1.1 - 21/JUN/2013
  ;        o Improved the code thanks to a tip from Bernd (PureBasic forum)
  ; V1.2 - 16/JUL/2013
  ;        o Now places a "*" if the file has been changed  
  ; V1.3 - 7/FEB/2014
  ;        o Code improved
  ; V1.4 - 13/MAR/2014
  ;        o Like suggested by Filiep, it now shows: "Word: [211/588557]"
  ; V1.5 - 25/OCT/2017
  ;        o Formats the total number of items to be better seen.
  ; V1.6 - 20/JUL/2018
  ;        o Now the changes allow a colour asterisk in red
  ; V1.7 - 23/JUL/2018
  ;        o Fix: Asterisks in red no longer flash
  ; V1.8 - 27/JUL/2018
  ;        28/JUL/2018
  ;        o Fix: Reduced asterisks vanishing if scrolling holding down the arrow keys  
  ; V1.9 - 19/SEP/2018
  ;        o Floppy icon on tabs (blue/red)
  ;
  Procedure update_number_words()
    
    ; Black/Red floppy
    If dictionary_changed=#False And GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK)<>""
      SetGadgetItemImage(#PANEL_MAIN,0,ImageID(71))
    EndIf
    If dictionary_changed=#True And GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK)<>"*"
      SetGadgetItemImage(#PANEL_MAIN,0,ImageID(72))
    EndIf    
    
    t$=""
    If dictionary_changed=#True : t$="*" : EndIf        
    If selected_dictionary_word=-1
      t1$="# Words:"+FormatNumber(number_of_words,0,".",number_separator_chr$)
    Else
      t1$="Word: ["+FormatNumber(selected_dictionary_word+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$)+"]"
    EndIf
    
    If t1$<>GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS)
      gadget=#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS
      gadget2=#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK
      w=GadgetWidth(gadget)
      SetGadgetText(gadget,t1$)
      GadgetFit(gadget)
      If GadgetX(gadget2)<GadgetX(gadget)+GadgetWidth(gadget) Or GadgetX(gadget2)>=GadgetX(gadget)+GadgetWidth(gadget)+4
        ResizeGadget(gadget2,GadgetX(gadget)+GadgetWidth(gadget),GadgetY(gadget),16,16)                        
        GadgetRefresh(gadget2)  
      EndIf
    EndIf   
    
    If GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK)<>t$
      SetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK,t$)
    EndIf
    
  EndProcedure  
  
  
  
  
  ; Updates the filename of the words in the Dictionary
  ; Used in open and save as
  ;
  ; V1.0 - 11/JUN/2013
  ; V1.1 - 21/JUN/2013
  ;        o Improved the code thanks to a tip from Bernd (PureBasic forum)
  ; V1.2 - 11/FEB/2016
  ;        o Simplified the code
  ; V1.3 - 25/FEB/2020
  ;        o Fix: Long filenames now fit in the GUI
  ; V1.4 - 26/FEB/2020
  ;        o Cleaned the code that updates filenames in the GUI  
  ; V1.5 - 19/DEC/2021
  ;        o Replaced "Filename:" with "File:"
  ; V1.6 - 01/NOV/2023
  ;        o Replaced "File:" with "💾:"
  ; V1.7 - 26/NOV/2023
  ;        o Replaced the disk emoji plain text with an image
  ; V1.8 - 13/JAN/2024
  ;        14/JAN/2024
  ;        o It now shows a folder emoji with the path of the filename
  ;
  Procedure update_filename_dictionary()

    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_DICTIONARY_FILENAME,""+GetOnlyFilename(dictionary_words_filename$,"n/a"),#True)
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_DICTIONARY_FOLDER,""+GetOnlyPath(dictionary_words_filename$,"n/a"),#True)
  
  EndProcedure  
  
  
  
  
  ; Sort the Dictionary
  ;
  ; V1.0 - 12/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 24/MAY/2013
  ;        o Coded the complete procedure
  ; V1.2 - 20/JUN/2013
  ;        o A HUGE speed gain thanks to Ashley from the PureBasic Forum
  ; V1.3 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.4 - 26/JUN/2013
  ;        o Changed the WindowTimer code to make it work better (faster) in Linux.
  ; V1.5 - 12/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.6 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.7 - 27/FEB/2014
  ;        o It now works with V3+ words$()
  ; V1.8 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1  
  ; V1.9 - 21/MAR/2014  
  ;        o Resets progress bar when task completes  
  ; V2.0 - 22/DEC/2016
  ;        25/DEC/2016
  ;        o Started implementing the multiselection.
  ; V2.1 - 07/JUN/2017
  ;        o undo/redo support
  ; V2.2 - 01/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V2.3 - 03/APR/2018
  ;        o Support for hyphenation
  ; V2.4 - 13/NOV/2019
  ;        o Support for Latin sorting
  ; V2.5 - 30/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure sort_dictionary()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting Dictionary...")
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn 0
    EndIf
    
    dictionary_store_undo(#True) ; 7/JUN/2017
    
    ; Show the progress bar
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_words*3) 
    
    ; Dictionary set to changed
    selected_dictionary_word=-1 ; Reset the current word selected
    dictionary_changed=#True
    update_number_words()
        
    time_taken=ElapsedMilliseconds()
    AddWindowTimer(#WINDOW_MAIN,800,800)
    
    
    ; Remove accents from entries
    ReDim sort_arrays_in_latin(number_of_words-1)
    For f=0 To number_of_words-1
      
      sort_arrays_in_latin(f)\in_latin=RemoveAccentsString(words$(f))
      sort_arrays_in_latin(f)\in_ascii=words$(f)
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f
    
    
    ; Sort the array and delete the list 
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting... <ESC> to abort. Found:"+FormatNumber(number_of_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    SortStructuredArray(sort_arrays_in_latin(),#PB_Sort_Ascending,OffsetOf(sort_arrays_in_latin\in_latin.s),TypeOf(sort_arrays_in_latin\in_latin.s))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,number_of_words*2,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting... <ESC> to abort. Found:"+FormatNumber(number_of_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
    
    ; Populate the list with the sorted words
    For f=0 To number_of_words-1
      
      words$(f)=sort_arrays_in_latin(f)\in_ascii
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,(number_of_words*2)+f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating with accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f    
    
    
    ; Populate the ListIconGadget
    ClearGadgetItems(#PANEL_BOX_DICTIONARY)       
    SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(-1)
    
    
    ; Clear progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) 

    
    ; No selected items
    update_number_words_selected_none()
    
    
    ; Hyphenation support - Populate the ListIconGadget - 3/APR/2018
    selected_hyphenation_word=-1 ; Reset the current word selected
    update_number_hyphenations()
    ClearGadgetItems(#LISTICONGADGET_PANEL_HYPHENATOR)       
    SetGadgetState(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(2)    
    
    RemoveWindowTimer(#WINDOW_MAIN,800)         
    
  EndProcedure  
  
  
  
  
  ; Delete words in the Dictionary
  ;
  ; V1.0 - 12/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 14/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.3 - 28/FEB/2014
  ;        o It now works with V3+ words$() 
  ; V1.4 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1    
  ;          (for DELETE, only if the number of words become 0)
  ; V1.5 - 27/MAR/2014
  ;        o It now allows deleting words not currently visible in the listicongadget
  ; V1.6 - 1/AUG/2014
  ;        o It now shows the words to be deleted inside quotation marks
  ; V1.7 - 23/DEC/2016
  ;        o Started implementing the multiselection.
  ; V1.8 - 7/JUN/2017
  ;        o undo/redo support
  ; V1.9 - 6/NOV/2017
  ;        o Now recognises being called from advanced find  
  ; V2.0 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V2.1 - 3/APR/2018
  ;        o Support for hyphenation
  ; V2.2 - 18/NOV/2018
  ;        o Replaced some more "words" with "dictionary words"
  ;
  Procedure delete_word(is_called_from_advance_find)
    
    If number_of_words=0 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    t=selected_dictionary_word
    If number_of_words>0 And t=-1 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No dictionary word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    If number_of_words>0 And t<>-1      
      text$=words$(t)
      text$=ReplaceString(text$,#LF$," ")
      If MessageRequester("Warning!","Really delete dictionary word?"+#LF$+Chr(34)+text$+Chr(34),#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
;        RemoveGadgetItem(#PANEL_BOX_DICTIONARY,GetGadgetState(#PANEL_BOX_DICTIONARY))        
;        t1=GetGadgetState(#PANEL_BOX_DICTIONARY)+GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)        
        flag=#False 
        
        dictionary_store_undo(#False) ; 7/JUN/2017
        
        ; Only one word in the dictionary
        If number_of_words=1 And flag=#False
          ReDim words$(0)
          ReDim words_selected(0)
          words_selected(0)=#False
          flag=#True
        EndIf
        
        ; The word to delete is the last
        If number_of_words=t+1 And flag=#False
          ReDim words$(ArraySize(words$())-1)
          ReDim words_selected(ArraySize(words_selected())-1)
          flag=#True
        EndIf
        
        ; The word to delete is inside the list, not the first nor the last item
        If flag=#False
          For f=t+1 To number_of_words-1
            words$(f-1)=words$(f)
            words_selected(f-1)=words_selected(f)
          Next f
          ReDim words$(ArraySize(words$())-1)          
          ReDim words_selected(ArraySize(words_selected())-1)   
          flag=#True
        EndIf
        
        number_of_words-1
        update_number_words_selected_count()
        SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        selected_dictionary_word=-1 ; Reset the current word selected
        
        dictionary_changed=#True
        update_number_words()
        global_update_virtual_vertical_scroll_bar(-1)
        
        
        ; Hyphenation support - 3/APR/2018
        SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        selected_hyphenation_word=-1 ; Reset the current word selected      
        update_number_hyphenations()
        global_update_virtual_vertical_scroll_bar(2)
        
        ProcedureReturn #True
        
      EndIf
    EndIf
    
    ProcedureReturn #False
    
  EndProcedure    
  
  
  
  
  ; Erase words in the Dictionary
  ;
  ; V1.0 - 13/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 13/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 14/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ;        o After a purge, the filename is reset.
  ; V1.3 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ;        o Checks if there have been changes, and ask before purging
  ; V1.4 - 27/FEB/2014
  ;        o It now works with V3+ words$()
  ; V1.5 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1    
  ; V1.6 - 21/MAR/2014  
  ;        o Resets progress bar when task completes   
  ; V1.7 - 23/MAR/2014  
  ;        o Fix: Purge Dictionary didn't reset the vertical scroll bar
  ; V1.8 - 22/DEC/2016
  ;        24/DEC/2016
  ;        o Started implementing the multiselection.
  ; V1.9 - 7/JUN/2017
  ;        o undo/redo support
  ; V2.0 - 1/DEC/2017
  ;        o Message Requesters now have a sign.
  ; V2.1 - 18/NOV/2018
  ;        o Replaced some more "words" with "dictionary words"
  ;
  Procedure erase_dictionary()
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf
    
    ; Confirm if to erase the Dictionary if the file in memory has been changed
    If dictionary_changed=#True
      If MessageRequester("Warning!","The Dictionary has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf     
    
    If number_of_words>0
      If MessageRequester("Warning!","Really delete all dictionary words?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        
        dictionary_store_undo(#True) ; 7/JUN/2017
        
        ; Reset total number of selected words
        update_number_words_selected_none()        
        
        FUCKTIME=ElapsedMilliseconds()
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Erasing...")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        number_of_words=0
        ClearGadgetItems(#PANEL_BOX_DICTIONARY)
        Dim words$(0)
        Dim words_selected(0)
        number_of_words_total_selected=0
        update_number_words()
        dictionary_words_filename$=""
        update_filename_dictionary()
        
        selected_dictionary_word=-1 ; Reset the current word selected
        dictionary_changed=#False ; Dictionary set to unchanged
        update_number_words()
        global_update_virtual_vertical_scroll_bar(-1)
        Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
        
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,100,#PB_StatusBar_Raised,0,100)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,100) ; Reset progress bar - 21/MAR/2014
                
        
      EndIf
    EndIf
        
  EndProcedure    
  
 
  
  
  ; ADD/EDIT WORD
  ; If add=1 and edit=0 then it ADDS
  ; If add=0 and edit=1 then it EDITS
  ;
  ; V1.0 - 13/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 15/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 16/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed  
  ; V1.3 - 7/AUG/2013
  ;        8/AUG/2013
  ;        o Initial support for derived suggestions
  ; V1.4 - 10/AUG/2013
  ;        o Added to the list box the field "Type"
  ; V1.5 - 11/AUG/2013
  ;        o Started coding the PFX and SFX
  ; V1.6 - 14/AUG/2013
  ;        o Added the Suffixes code
  ; V1.7 - 15/AUG/2013  
  ;        o The derived words listbox now has a column with the code of each word
  ; V1.8 - 16/AUG/2013
  ;        o The search for Suffixes and Prefixes now give priority to "N" instead of "Y"
  ; v1.9 - 17/AUG/2013
  ;        o Coded a display button to find the affix/prefix in the .AFF
  ;        o Added a position column
  ; V2.0 - 23/AUG/2013
  ;        o Added a WindowTimer to avoid the window to stop responding
  ; V2.1 - 24/AUG/2013
  ;        o Optimised the code a little.
  ;        o (fix) In case letters are typed during a refresh, to refresh again
  ; V2.2 - 30/AUG/2013
  ;        o Coded the DISPLAY function, thanks to Ash and to a forum member
  ; V2.3 - 2/SEP/2013
  ;        o Coded the double-clicking in derived words to move to the .AFF position
  ; V2.4 - 31/OCT/2013
  ;        o First I allowed only one SFX or PFX by code
  ;        o Now it allows both - en_GB "O" code is used by SFX and PFX
  ; V2.5 - 11/FEB/2014
  ;        o Ubuntu/Linux users can close the window in its gadget
  ; V2.6 - 13/FEB/2014
  ;        o Check if <ESC> was pressed
  ; V2.7 - 27/FEB/2014
  ;        o It now works with V3+ words$()
  ; V2.8 - 21/MAR/2014 
  ;        o Improved: While editing a word, the cursor focus is now in the "Word:" textbox.  
  ; V2.9 - 24/MAR/2014 
  ;        o Improved the code to work better with V3+ (selected word not visible in listicongadget)
  ; V3.0 - 18/APR/2014
  ;        o Added the shortcut <ENTER>
  ;        o If the word/flag has a space at the beginning or end, the text box becomes red
  ; V3.1 - 20/APR/2014  
  ;        o It now searches for two spaces in the word, making the text box become red.
  ; V3.2 - 8/JUL/2014
  ;        10/JUL/2014
  ;        o Replaced "Flags" with "POStag".
  ; V3.3 - 11/JUL/2014  
  ;        o Replaced "POStags" with "Tags" as suggested by Filiep.
  ;        o Checks if word ends with "/" or has more than one "/" (invalid).
  ;        o Moved part of the code to extract the derivates, in order to be used by the new menu.
  ; V3.4 - 12/JUL/2014    
  ;        o Fix Regression from build 56: sometimes while writing too fast, the listicongadget wasn't refreshed.
  ;        o Added and coded the "Rule" column
  ; V3.5 - 13/JUL/2014
  ;        o Fix: It no longer shows "Primary" for invalid words (blank OR starting with a "/")
  ; V3.6 - 14/AUG/2014
  ;        o Increased the width of the editor/listicon gadgets, plus the "rules" field.
  ; V3.7 - 5/NOV/2014
  ;        o Now it only sets the "changed" flag, if the word/tags are really changed.
  ; V3.8 - 15/NOV/2014
  ;        o It now accepts: \/ are escaped "/"  
  ; V3.9 - 17/JAN/2015
  ;        o Ultrafast searching for rules positions  
  ; V4.0 - 29/OCT/2015
  ;        o Increased the width from 640 to 800 to show more content.
  ; V4.1 - 2/NOV/2015
  ;        o Replaced the listicongadget field "Position" with "Code Position".
  ; V4.2 - 9/NOV/2015
  ;        o Simplified the code a bit
  ; V4.3 - 20/JAN/2016
  ;        o "Display" button now disabled if nothing is selected in the ListIconGadget
  ; V4.4 - 5/OCT/2016
  ;        o Pop-up menu in words editor to copy the selected line word into the clipboard
  ; V4.5 - 15/DEC/2016
  ;        o Dictionary editor now has a larger window
  ; V4.6 - 23/DEC/2016
  ;        o Started implementing the multiselection.
  ; V4.7 - 6/JAN/2017
  ;        o Now makes uses of ".AFF Aid"
  ; V4.8 - 11/MAR/2017
  ;        o Changed the buttons height to 25.   
  ; V4.9 - 4/APR/2017
  ;        o Support via coloured symbol for NOSUGGEST in words editor 
  ; V5.0 - 7/JUN/2017
  ;        o undo/redo support
  ; V5.1 - 23/JUL/2017
  ;        o F1 (help) now works in the edit/new items windows    
  ; V5.2 - 21/NOV/2017
  ;        o Added "Missing Codes:" to the Dictionary editor.
  ; V5.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V5.4 - 7/FEB/2018
  ;        o ComboBox with higher height (+6 pixels) for better Ubuntu look.
  ; V5.5 - 14/FEB/2018
  ;        o Now enlarges according to window size definition
  ; V5.6 - 3/APR/2018
  ;        o Support for hyphenation
  ; V5.7 - 13/APR/2018
  ;        o Support for compressed AF
  ; V5.8 - 18/MAY/2018
  ;        o Taboo's warning sign is now an image and is stored in a long instead of string for speed up.
  ; V5.9 - 23/MAY/2018
  ;        o Combobox of ".AFF Aid" wider for bigger windows.
  ; V6.0 - 14/JUN/2018
  ;        o Image for NEEDAFFIX and FORBIDDENWORD
  ; V6.1 - 30/JUN/2018
  ;        o Fix: NEEDAFFIX and FORBIDDENWORD variables reset at start of ADD/EDIT
  ; V6.2 - 5/JUL/2018
  ;        o Dictionary editor: Code (flag) is now an emoji: 🏁
  ;        o Taboo image now has a tip
  ; V6.3 - 7/JUL/2018
  ;        o Replaced the word "Tags" with "Postags"
  ; V6.4 - 8/JUL/2018
  ;        o Missing flags is now written in red.
  ;        o Colours for "Suffixes" (#GreenDark) and "Prefixes" (#Blue)
  ; V6.5 - 9/JUL/2018
  ;        o Better check if word is valid before exiting.
  ; V6.6 - 7/SEP/2018
  ;        o Pop-up menus now use the flag: #PB_Menu_ModernLook (Windows only)
  ; V6.7 - 24/OCT/2018
  ;        o Edit/New Dictionary word window now shows "FLAG CHR", "FLAG NUM" OR "FLAG LONG"
  ; V6.8 - 26/OCT/2018
  ;        o Edit/New Dictionary word nows shows "CIRCUMFIX"
  ; V6.9 - 29/OCT/2018
  ;        o Coded the "CIRCUMFIX" flag.
  ; V7.0 - 18/NOV/2018
  ;        o Replaced some more "words" with "dictionary words"
  ; V7.1 - 06/FEB/2019
  ;        o Dictionary editor now has in the pop-up menu "Copy word & rule".
  ; V7.2 - 09/FEB/2019
  ;        o Timer decoding added here and removed from the decoding Procedure
  ; V7.3 - 15/FEB/2019
  ;        o Change background colour to yellow if word is commented (starts with "/")
  ; V7.4 - 19/MAR/2019
  ;         o Added to dictionary editor pop-up menu:
  ;           1) "Copy all words"
  ;           2) "Copy all words & rules"
  ; V7.5 - 01/APR/2019
  ;        o Added support for "KEEPCASE"
  ;        o In the dictionary editor, increased the delay before refreshing the derivates while typing
  ; V7.6 - 11/APR/2019
  ;        o Added a refresh button
  ;        o Allows to refresh decoding during processing, and OKAY and CANCEL
  ; V7.7 - 11/JUN/2019
  ;        o GTK3 Ready
  ; V7.8 - 20/JUN/2019
  ;        o Preferences allow to choose the font for the language.
  ; V7.9 - 03/JUL/2019
  ;        o Added 10 pixels to each tab of the ListIconGadget to compensate if one increases the font size
  ; V8.0 - 11/JUL/2019  
  ;        o Coded "Remove flag from textbox"
  ; V8.1 - 16/JUL/2019 
  ;        o Coding of COMPLEXPREFIXES
  ; V8.2 - 27/SEP/2019
  ;        28/SEP/2019
  ;        29/SEP/2019
  ;        o Allow to sort by column
  ; V8.3 - 30/SEP/2019
  ;        o Replaced "Flag" symbol with text
  ; V8.4 - 03/FEB/2020
  ;        04/FEB/2020
  ;        o Dictionary Editor: Coding of Shantanu's summary
  ; V8.5 - 22/MAR/2020
  ;        o Dictionary Editor: New "Match" gadgets
  ; V8.6 - 06/APR/2020
  ;        10/APR/2020
  ;        o Fixing CIRCUMFIX (Brandon)
  ; V8.7 - 17/MAY/2020
  ;        o Yellow flag in Dictionary editor if input is a rem
  ;        o Fix: Dictionary Editor words starting with / (comments) now accept spaces at end
  ; V8.8 - 06/NOV/2020
  ;        o Dictionary editor: New flags+subflags text and StringGadgets
  ; V8.9 - 16/AUG/2022
  ;        o New popup menu option "Copy flag rule" in dictionary editor
  ; V9.0 - 17/JUL/2023
  ;        o Dictionary words editor no longer enables "Display" button if word=primary
  ; V9.1 - 19/JUL/2023
  ;        o Dictionary words editor, improved GUI
  ;
  Procedure add_edit_word(add,edit)

    If number_of_words=0 And edit=#True
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    t=selected_dictionary_word
    If t=-1 And edit=#True
      MessageRequester("Warning!","No dictionary word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
     
    t$="Add a word" ;add=#True
    If edit=#True : t$="Edit a word" : EndIf
    w=get_extra_width(100)+(1024-800)+(800-640)+640-10
    h=get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120+10
    If OpenWindow(#WINDOW_ADD_EDIT,0,0,w,h,t$,#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_ADD_EDIT,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)
    
    ; Word field
    TextGadget(#TEXT_WINDOW_ADD_EDIT_WORD,0+6,0+6+2+1,80,default_gadget_height_text,"Word:")
    t=selected_dictionary_word
    t$=""
    If t<>-1 And edit=#True
      t$=StringField(words$(t),1,#LF$)
    EndIf
    StringGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD,3+0+6+100-15-15-20,0+6,320-10-20-40+20,default_gadget_height_string,t$)
    SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
    old_word$=t$ ; 5/NOV/2014
    SetGadgetFont(#TEXTBOX_WINDOW_ADD_EDIT_WORD,FontID(#font_for_language_id))
    
    ; Refresh button
    ButtonImageGadget(#BUTTON_REFRESH_DERIVATES,-3+GadgetX(#TEXTBOX_WINDOW_ADD_EDIT_WORD)+GadgetWidth(#TEXTBOX_WINDOW_ADD_EDIT_WORD)+16-4-2,GadgetY(#TEXTBOX_WINDOW_ADD_EDIT_WORD)-1,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(108))    
    GadgetToolTip(#BUTTON_REFRESH_DERIVATES,"Refresh")
    
    ; NOSUGGEST fields
    TextGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO,-4+GadgetX(#BUTTON_REFRESH_DERIVATES)+GadgetWidth(#BUTTON_REFRESH_DERIVATES)+5+8+3-3,GadgetY(#BUTTON_REFRESH_DERIVATES)+2,40,default_gadget_height_text,"Taboo:")    
    ImageGadget(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,GadgetX(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO)+GadgetWidth(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO)+5-3,GadgetY(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO)-3-1,24,20,ImageID(49))
    GadgetToolTip(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,"NOSUGGEST Flag")
    
    ; POSTAGS field
    TextGadget(#TEXT_WINDOW_ADD_EDIT_FLAG,0+6,5+2+4+0+6+2+1+20+5+2+5,80,default_gadget_height_text,"Postags:")
    t$=""
    If t<>-1 And edit=#True
      t1=FindString(words$(t),#LF$)
      If t1>0 : t$=StringField(words$(t),2,#LF$) : EndIf
    EndIf    
    StringGadget(#EDITOR_WINDOW_ADD_EDIT_TAG,3+0+6+100-15-15-20,5+2+4+0+6+2+1+20+5+5,600-20-40-20-10-20-40+20-100,default_gadget_height_string,t$)      
    old_tags$=t$ ; 5/NOV/2014
    SetGadgetFont(#EDITOR_WINDOW_ADD_EDIT_TAG,FontID(#font_for_language_id))
    
    ; Add the OK and CANCEL buttons
    ButtonGadget(#BUTTON_ADD_EDIT_OKAY,5-20-20+0+6+100-15,5+5+2+2+2+4+30+30+3+5,90,default_gadget_height_button,"OK")
    ButtonGadget(#BUTTON_ADD_EDIT_CANCEL,5-20-20+0+6+100-15+100,5+5+2+2+2+4+30+30+3+5,90,default_gadget_height_button,"Cancel")
    
    ; Derived gadget listbox
    ListIconGadget(#PANEL_BOX_DERIVED_WORDS,10+40+5,5+5-2+2+4+200-40-50-20-20+60-20+20+20-40-5+5,get_extra_width(100)+3+(1024-800)+(800-640)+10+320+100-30-50+5+155+20+20,-10+60+100+20+5+40-5+get_extra_height(20)-5,"Derived words",10-10+320+100-30-50+5-5-100-20-10-10-10-10-10-10+10,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,1,"Type",150-20-20-10-10-10-5+10)    
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,2,"Flag",150-20-20-10-10-10-5-5-5-2-5+10+10)        
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,3,"Flag Position",10+15+150-20-20-10-10-10-5-2+5+10)        
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,4,"Rule",get_extra_width(15)+20+15+5+15+150-20-20-10+10+10+10+10+5+10+100-5-5-5-10+20+40+40+10)
    ButtonGadget(#PANEL_BOX_DERIVED_WORDS_DISPLAY,10+40+5+10-5-3,2+200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2-2+60+get_extra_height(20),90,default_gadget_height_button,"Display")       
;     TextGadget(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,10+100+20+30+5,200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2+60+get_extra_height(20),400-200,default_gadget_height_text,"# Derived Words:0")
    SetGadgetFont(#PANEL_BOX_DERIVED_WORDS,FontID(#font_for_language_id))
    
    ; The new "Match" gadgets
;     y=GadgetY(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS)
;     x=GadgetX(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS)+GadgetWidth(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS)
    y=GadgetY(#PANEL_BOX_DERIVED_WORDS_DISPLAY)
    x=GadgetX(#PANEL_BOX_DERIVED_WORDS_DISPLAY)+GadgetWidth(#PANEL_BOX_DERIVED_WORDS_DISPLAY)+30+30
    
    TextGadget(#TEXT_BOX_DERIVED_WORDS_WORD,x,y+3,90-60+10,18,"Word:",#PB_Text_Right)
    StringGadget(#STRING_BOX_DERIVED_WORDS_WORD,GadgetX(#TEXT_BOX_DERIVED_WORDS_WORD)+GadgetWidth(#TEXT_BOX_DERIVED_WORDS_WORD)+10,y,90+60+15+20+30+30,default_gadget_height_string,"")
    SetGadgetFont(#STRING_BOX_DERIVED_WORDS_WORD,FontID(#font_for_language_id))
    ButtonGadget(#BUTTON_BOX_DERIVED_WORDS_WORD_MATCH,GadgetX(#STRING_BOX_DERIVED_WORDS_WORD)+GadgetWidth(#STRING_BOX_DERIVED_WORDS_WORD)+10,y,90,default_gadget_height_button,"Match")
    TextGadget(#TEXT_BOX_DERIVED_WORDS_WORD_MATCH,GadgetX(#BUTTON_BOX_DERIVED_WORDS_WORD_MATCH)+GadgetWidth(#BUTTON_BOX_DERIVED_WORDS_WORD_MATCH)+10,y,90,default_gadget_height_text,"")
    
    ; AFF Text + Editor Gadget
    TextGadget(#TEXT_WINDOW_ADD_EDIT_AFF,5+0+6,5+2+2+get_extra_height(20)+2+200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2+20+5+2+2+60,60-15,default_gadget_height_text,".AFF:")
    EditorGadget(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD,5+0+6+2+1+20+5+10+10,5+2+get_extra_height(20)+2+200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2+20+5+2+2+60,-200-200-200+get_extra_width(60)+(1024-800)+(800-640)+15+200+20+30+30+10+3+100+100+100-50,-5+get_extra_height(80)+100+20+60-4-default_gadget_height_button-5-5-2-5,#PB_Editor_ReadOnly)    
    SetGadgetText(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD,derived_aff_nospaces$)
    SetGadgetFont(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD,FontID(#font_for_language_id))
    
    ; Sorting Derived words by column
    x=GadgetX(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)
    y=GadgetY(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)+GadgetHeight(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)+5-2-2-4+5
    x+2+2+2+2
    y+2+2+2+2+2+2+3+1
    width=100-10-5-5-5+5*6
    OptionGadget(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO1,x,y,width,default_gadget_height_radio,"Default")
    GadgetFit(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO1)
    x+GadgetWidth(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO1)+15
    OptionGadget(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO2,x,y,width,default_gadget_height_radio,"Type + Flag")
    GadgetFit(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO2)
    x+GadgetWidth(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO2)+15
    OptionGadget(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO3,x,y,width,default_gadget_height_radio,"Flag + Rule")
    GadgetFit(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO3)
    x+GadgetWidth(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO3)+15
    OptionGadget(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO4,x,y,width,default_gadget_height_radio,"Flag Pos + Rule")
    GadgetFit(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO4)
    x+GadgetWidth(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO4)+15
    OptionGadget(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO5,x,y,width,default_gadget_height_radio,"Rule + Flag")    
    GadgetFit(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO5)
    x+GadgetWidth(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO5)+10
    x+5
    y-3-1
    ButtonGadget(#BUTTON_DERIVED_WORDS_SORT_SORT,x,y,90,default_gadget_height_button,"Sort")
    x+100+10+15-15-3-2+10
    ButtonGadget(#BUTTON_DERIVED_WORDS_SORT_REMEMBER,x,y,90,default_gadget_height_button,"Remember")    
    x=GadgetX(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)
    y=GadgetY(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)+GadgetHeight(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)+5-2-2-4+5
    FrameGadget(#FRAME_BUTTON_DERIVED_WORDS_SORT,x,y,GadgetX(#BUTTON_DERIVED_WORDS_SORT_REMEMBER)+GadgetWidth(#BUTTON_DERIVED_WORDS_SORT_REMEMBER)+10-x,30+4+2+2+4+4+1+4,"")
    SetGadgetState(#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO1+dictionary_column_sort-1,#True)
    
    ; Summary + Top Level tags + Nested tags
    x=GadgetX(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)+GadgetWidth(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)+15
    y=GadgetY(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)
    y+5
    TextGadget(#LISTICON_DICTIONARY_SUMMARY,x,y,100,default_gadget_height_text,"Summary:")
    GadgetFit(#LISTICON_DICTIONARY_SUMMARY)
    x+GadgetWidth(#LISTICON_DICTIONARY_SUMMARY)+15
    y-5
    ListIconGadget(#LISTICON_DICTIONARY_SUMMARY_TOP_LEVEL_TAGS,x,y,250-5+get_extra_width(20),GadgetHeight(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD),"Top level tags",240-5-5+get_extra_width(20))
    SetGadgetFont(#LISTICON_DICTIONARY_SUMMARY_TOP_LEVEL_TAGS,FontID(#font_for_language_id))
    x+GadgetWidth(#LISTICON_DICTIONARY_SUMMARY_TOP_LEVEL_TAGS)+15
    ListIconGadget(#LISTICON_DICTIONARY_SUMMARY_NESTED_TAGS,x,y,250-5+get_extra_width(20),GadgetHeight(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD),"Nested tags",240-5-5+get_extra_width(20))
    SetGadgetFont(#LISTICON_DICTIONARY_SUMMARY_NESTED_TAGS,FontID(#font_for_language_id))
    
    ; AFF Aid
    x=GadgetX(#EDITOR_WINDOW_ADD_EDIT_TAG)+GadgetWidth(#EDITOR_WINDOW_ADD_EDIT_TAG)+10+20
    y=5    
    t$="n/a"
    For f=0 To ArraySize(aff_aid_language_array$())
      If StringField(aff_aid_language_array$(f),3,#LF$)<>"" : image=Val(StringField(aff_aid_language_array$(f),2,#LF$)) : EndIf
      If aff_aid_language$=StringField(aff_aid_language_array$(f),3,#LF$)
        current_aff_aid_language=f
        t$=StringField(aff_aid_language_array$(f),1,#LF$)
        Break
      EndIf  
    Next f    
    TextGadget(#TEXT_WINDOW_ADD_EDIT_AFF_AID,x,y,100+200+150+100-10-3,default_gadget_height_text,".AFF Aid "+"("+t$+"):")
    y+16+2+1+1
    ComboBoxGadget(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,x,y,get_extra_width(100)+100+200+150+100-10-3,default_gadget_height_combobox,#PB_ComboBox_Image)
    populate_combobox_aff_aid(image,current_aff_aid_language)
    SetGadgetState(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,aff_aid_language_current_combobox_entry)
    SetGadgetFont(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,FontID(#font_for_language_id))
        
    ; Missing Codes
    TextGadget(#TEXT_MISSING_CODES,GadgetX(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID),1+1+1+GadgetY(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+GadgetHeight(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+5+3,100-40+10+10+10+10-20-10-6,default_gadget_height_text,"⚠ 🏁:")
    GadgetFit(#TEXT_MISSING_CODES)
    SetGadgetColor(#TEXT_MISSING_CODES,#PB_Gadget_FrontColor,#Red)
    StringGadget(#STRING_MISSING_CODES,-2-2+4+GadgetX(#TEXT_MISSING_CODES)+GadgetWidth(#TEXT_MISSING_CODES)+5,-1+1+GadgetY(#TEXT_MISSING_CODES)-2,-50+100+150-30-30-15-3+get_extra_width(20),default_gadget_height_string,"",#PB_String_ReadOnly)
    SetGadgetColor(#STRING_MISSING_CODES,#PB_Gadget_BackColor,-1)
    SetGadgetFont(#STRING_MISSING_CODES,FontID(#font_for_language_id))
    
    ; Missing Subcodes
    TextGadget(#TEXT_MISSING_CODES_SUB,GadgetX(#STRING_MISSING_CODES)+GadgetWidth(#STRING_MISSING_CODES)+3+3,1+1+1+GadgetY(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+GadgetHeight(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+5+3,100-40+10+10+10+10-20-10-6,default_gadget_height_text,"⚠ 🏁🏁:")    
    GadgetFit(#TEXT_MISSING_CODES_SUB)
    SetGadgetColor(#TEXT_MISSING_CODES_SUB,#PB_Gadget_FrontColor,#Red)
    StringGadget(#STRING_MISSING_CODES_SUB,2-2-2+4+GadgetX(#TEXT_MISSING_CODES_SUB)+GadgetWidth(#TEXT_MISSING_CODES_SUB)+5,-1+1+GadgetY(#TEXT_MISSING_CODES_SUB)-2,-50+100+150-30-30-15-3+get_extra_width(20),default_gadget_height_string,"",#PB_String_ReadOnly)
    SetGadgetColor(#STRING_MISSING_CODES_SUB,#PB_Gadget_BackColor,-1)
    SetGadgetFont(#STRING_MISSING_CODES_SUB,FontID(#font_for_language_id))
    
    ; KEEPCASE + NEEDAFFIX + FORBIDDENWORD + CIRCUMFIX + COMPLEXPREFIXES
    ImageGadget(#IMAGE_KEEPCASE,2+GadgetX(#STRING_MISSING_CODES_SUB)+GadgetWidth(#STRING_MISSING_CODES_SUB)+5,GadgetY(#STRING_MISSING_CODES_SUB),53,23,ImageID(80))
    GadgetToolTip(#IMAGE_KEEPCASE,"KEEPCASE Flag")    
    ImageGadget(#IMAGE_NEEDAFFIX,GadgetX(#IMAGE_KEEPCASE)+GadgetWidth(#IMAGE_KEEPCASE)+5,GadgetY(#IMAGE_KEEPCASE),53,23,ImageID(56))
    GadgetToolTip(#IMAGE_NEEDAFFIX,"NEEDAFFIX Flag")
    ImageGadget(#IMAGE_FORBIDDENWORD,GadgetX(#IMAGE_NEEDAFFIX)+GadgetWidth(#IMAGE_NEEDAFFIX)+5,GadgetY(#IMAGE_NEEDAFFIX),53,23,ImageID(57))
    GadgetToolTip(#IMAGE_FORBIDDENWORD,"FORBIDDENWORD Flag")      
    ImageGadget(#IMAGE_COMPLEXPREFIXES,GadgetX(#IMAGE_FORBIDDENWORD)+GadgetWidth(#IMAGE_FORBIDDENWORD)+5,GadgetY(#IMAGE_FORBIDDENWORD),53,23,ImageID(86))
    GadgetToolTip(#IMAGE_COMPLEXPREFIXES,"COMPLEXPREFIXES Flag")
    ImageGadget(#IMAGE_CIRCUMFIX,GadgetX(#IMAGE_COMPLEXPREFIXES)+GadgetWidth(#IMAGE_COMPLEXPREFIXES)+5,GadgetY(#IMAGE_COMPLEXPREFIXES),53,23,ImageID(78))
    GadgetToolTip(#IMAGE_CIRCUMFIX,"CIRCUMFIX Flag")
    keepcaseword_code=#False
    needaffixword_code=#False
    needaffixword_code_inside=#False
    forbiddenword_code=#False
    circumfix_code=#False
    circumfix_code2=#False ; set for primary word, so if there is at least one CIRCUMFIX it will turn on the image gadget
    
    ; FLAG CHR, NUM, LONG
    ImageGadget(#IMAGE_CHR_NUM_LONG,GadgetX(#IMAGE_CIRCUMFIX)+GadgetWidth(#IMAGE_CIRCUMFIX)+5,GadgetY(#IMAGE_CIRCUMFIX),53,23,0)
    
    ; TIP: COMPLEX PREFIXES are not supported yet! - 12/APR/2022
    TextGadget(#TEXT_COMPLEX_PREFIXES_NOT_SUPPORTED_YET,GadgetX(#IMAGE_KEEPCASE),2+GadgetY(#IMAGE_KEEPCASE)+GadgetHeight(#IMAGE_KEEPCASE)+3,200,default_gadget_height_text,"⚠ COMPLEX PREFIXES not supported yet!")
    SetGadgetColor(#TEXT_COMPLEX_PREFIXES_NOT_SUPPORTED_YET,#PB_Gadget_FrontColor,#Red)
    SetGadgetFont(#TEXT_COMPLEX_PREFIXES_NOT_SUPPORTED_YET,FontID(#font_for_language_id))
    GadgetFit(#TEXT_COMPLEX_PREFIXES_NOT_SUPPORTED_YET)
    
    Select flag_char
      Case 1
        SetGadgetState(#IMAGE_CHR_NUM_LONG,ImageID(74))
        GadgetToolTip(#IMAGE_CHR_NUM_LONG,"FLAG chr")
        
      Case 2
        SetGadgetState(#IMAGE_CHR_NUM_LONG,ImageID(76))
        GadgetToolTip(#IMAGE_CHR_NUM_LONG,"FLAG long")        
        
      Case 3
        SetGadgetState(#IMAGE_CHR_NUM_LONG,ImageID(75))
        GadgetToolTip(#IMAGE_CHR_NUM_LONG,"FLAG num")                
        
    EndSelect
        
        
    ; Wait for the user to press the OK or CANCEL button
    flag=#False ;to be used if text box with words is changed. #False=unchanged, #True=changed
    time=ElapsedMilliseconds()
    
    ;In case we are editing a word, it shows instantely the derived words
    If edit=#True
      flag=#True
      time=time-300-10
    EndIf
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_Escape,1000)    
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_Return,1003)    
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_F1,1004)    

    ; Create Popup menus
    CreatePopupImageMenu(#POPUPMENU_WINDOW_ADD_EDIT,#PB_Menu_ModernLook)
    MenuItem(#POPUPMENU_WINDOW_ADD_EDIT_COPY,"Copy word")   
    MenuItem(#POPUPMENU_WINDOW_ADD_EDIT_COPY_ALL,"Copy all words")
    MenuItem(#POPUPMENU_WINDOW_ADD_EDIT_COPY_PLUS_RULE,"Copy word && rule")     
    MenuItem(#POPUPMENU_WINDOW_ADD_EDIT_COPY_PLUS_RULE_ALL,"Copy all words && rules") 
    MenuItem(#POPUPMENU_WINDOW_ADD_EDIT_COPY_FLAG_RULE,"Copy flag rule") 
    MenuBar()
    MenuItem(#POPUPMENU_WINDOW_ADD_EDIT_REMOVE_FLAG_TEXTBOX,"Remove 🏁 from textbox")
    
    
    okay=#False
    cancel=#False
; *** CHECK THE CHRS **** START ****
; t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)    
; For f=1 To Len(t$)
;   Debug Mid(t$,f,1)+","+Asc(Mid(t$,f,1))
; Next f    
; *** CHECK THE CHRS **** END ****    
    BindGadgetEvent(#TEXTBOX_WINDOW_ADD_EDIT_WORD, @process_pfx_sfx_update_word(),#PB_EventType_Change) ; Current word
    BindGadgetEvent(#BUTTON_ADD_EDIT_OKAY, @process_pfx_sfx_update_word_okay(),#PB_EventType_LeftClick) ; OK
    BindGadgetEvent(#BUTTON_ADD_EDIT_CANCEL, @process_pfx_sfx_update_word_cancel(),#PB_EventType_LeftClick) ; Cancel
    BindEvent(#PB_Event_Menu, @process_pfx_sfx_update_word_cancel(), #WINDOW_ADD_EDIT,1000) ; <ESC>
    Repeat
      
      
      ; Change background colour to #YellowDark if word is commented (starts with "/")
      t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
      If Left(t$,1)="/" And GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>#YellowDark
        SetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor,#YellowDark)  
        nosuggestword_colour=#Yellow
      EndIf
      
      ; Change background of word string gadget if a space is found at the start or end, or two spaces in a row (invalid word)  
      t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
      If t$<>"" And (((Left(t$,1)=" " Or Right(t$,1)=" " Or FindString(t$,"  ")<>0) Or (Right(t$,1)="/" And Right(t$,2)<>"\/") Or (Right(t$,1)="/" And Right(t$,3)<>"\//")))
        If  Left(t$,1)<>"/" And GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>#Red: SetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor,#Red) : EndIf
      Else
        If Left(t$,1)<>"/" And GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>-1: SetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor,-1) : EndIf
      EndIf    
      
      ; Change background of flag string gadget if a space is found at the start or end (invalid word)  
      t$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
      If t$<>"" And (Left(t$,1)=" " Or Right(t$,1)=" " Or FindString(t$,"  ")<>0)
        If GetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor)<>#Red: SetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor,#Red) : EndIf
      Else
        If GetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor)<>-1: SetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor,-1) : EndIf
      EndIf             
      
      
      ; Disable/Enable the "Display" button according to selected/unselected items in the ListIconGadget
      disable_button_display=#False
      If GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=-1 Or GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=0 : disable_button_display=#True : EndIf
      DisableGadget(#PANEL_BOX_DERIVED_WORDS_DISPLAY,disable_button_display)
      
      
      ; NOSUGGEST support - 4/APR/2017
      If GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)=#Red
        nosuggestword_colour=#Black
      EndIf
      If (GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>-1 Or GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)="") And GetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST)<>ImageID(49) And GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>#YellowDark
        nosuggestword_colour=#Black
        SetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,ImageID(49))        
      EndIf
      If nosuggestword_colour=#Green And GetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST)<>ImageID(50)
        SetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,ImageID(50))
      EndIf
      If nosuggestword_colour=#Red And GetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST)<>ImageID(51)
        SetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,ImageID(51))
      EndIf
      If nosuggestword_colour=#Yellow And GetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST)<>ImageID(96)
        SetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,ImageID(96))
      EndIf      
      
      ; KEEPCASE support - 1/MAY/2019
      If keepcaseword_code=#True And GetGadgetState(#IMAGE_KEEPCASE)=ImageID(80) : SetGadgetState(#IMAGE_KEEPCASE,ImageID(81)) : EndIf
      If keepcaseword_code=#False And GetGadgetState(#IMAGE_KEEPCASE)=ImageID(81) : SetGadgetState(#IMAGE_KEEPCASE,ImageID(80)) : EndIf
      
      ; NEEDAFFIX support - 14/JUN/2018
      If (needaffixword_code=#True Or needaffixword_code_inside=#True) And GetGadgetState(#IMAGE_NEEDAFFIX)=ImageID(56) : SetGadgetState(#IMAGE_NEEDAFFIX,ImageID(54)) : EndIf
      If needaffixword_code=#False And needaffixword_code_inside=#False And GetGadgetState(#IMAGE_NEEDAFFIX)=ImageID(54) : SetGadgetState(#IMAGE_NEEDAFFIX,ImageID(56)) : EndIf
      
      ; FORBIDDENWORD support - 14/JUN/2018
      If forbiddenword_code=#True And GetGadgetState(#IMAGE_FORBIDDENWORD)=ImageID(57) : SetGadgetState(#IMAGE_FORBIDDENWORD,ImageID(55)) : EndIf
      If forbiddenword_code=#False And GetGadgetState(#IMAGE_FORBIDDENWORD)=ImageID(55) : SetGadgetState(#IMAGE_FORBIDDENWORD,ImageID(57)) : EndIf
      
      ; COMPLEXPREFIXES support - 16/JUL/2019
      If complexprefixes_code=#True And GetGadgetState(#IMAGE_COMPLEXPREFIXES)=ImageID(86) : SetGadgetState(#IMAGE_COMPLEXPREFIXES,ImageID(85)) : EndIf
      If complexprefixes_code=#False And GetGadgetState(#IMAGE_COMPLEXPREFIXES)=ImageID(85) : SetGadgetState(#IMAGE_COMPLEXPREFIXES,ImageID(86)) : EndIf    
      
      ; CIRCUMFIX support - 29/OCT/2018 + 6/APR/2020
      If (circumfix_code2=#True Or needcircumfix_code_inside=#True) And GetGadgetState(#IMAGE_CIRCUMFIX)=ImageID(78) : SetGadgetState(#IMAGE_CIRCUMFIX,ImageID(77)) : EndIf
      If (circumfix_code2=#False And needcircumfix_code_inside=#False) And GetGadgetState(#IMAGE_CIRCUMFIX)=ImageID(77) : SetGadgetState(#IMAGE_CIRCUMFIX,ImageID(78)) : EndIf
      
      
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf     
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf        
      
      
      ; Check if <ENTER> was pressed
      If event=#PB_Event_Menu And EventMenu()=1003
        Debug "OK was pressed"
        PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #BUTTON_ADD_EDIT_OKAY)
      EndIf      
      
      
      ; Shortcut for F1 (help)
      If event=#PB_Event_Menu And EventMenu()=1004
        html_help("")
      EndIf       
      
      
      ; POP-UP MENU
      If EventGadget()=#PANEL_BOX_DERIVED_WORDS And EventType()=#PB_EventType_RightClick
        DisplayPopupMenu(#POPUPMENU_WINDOW_ADD_EDIT, WindowID(#WINDOW_ADD_EDIT))
      EndIf  
      
      Select event
  
      Case #PB_Event_Menu
      
        Select EventMenu()  ; To see which popup menu has been selected

          Case #POPUPMENU_WINDOW_ADD_EDIT_COPY            
            clipboard_copy_words(#False)
            
            
          Case #POPUPMENU_WINDOW_ADD_EDIT_COPY_PLUS_RULE
            clipboard_copy_words_and_rules(#False)    
            
            
          Case #POPUPMENU_WINDOW_ADD_EDIT_COPY_ALL  
            clipboard_copy_words(#True)
            
            
          Case #POPUPMENU_WINDOW_ADD_EDIT_COPY_PLUS_RULE_ALL
            clipboard_copy_words_and_rules(#True)   
            
            
          Case #POPUPMENU_WINDOW_ADD_EDIT_COPY_FLAG_RULE
            clipboard_copy_flag_rule()
            
            
            
          Case #POPUPMENU_WINDOW_ADD_EDIT_REMOVE_FLAG_TEXTBOX
            If remove_flag_from_textbox()=#True
              PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #BUTTON_REFRESH_DERIVATES)
            EndIf
            
            
        EndSelect    
      EndSelect        
            
      
      Select Event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
           ; Change the word
           Case #TEXTBOX_WINDOW_ADD_EDIT_WORD
             If EventType()=#PB_EventType_Change
                flag=#True            
                time-1000 ; To refresh the word in real time (300 would be the exact number, but used 1000 to be 100% sure it will activate the refresh in real time)
             EndIf  
             
           ; Refresh  
           Case #BUTTON_REFRESH_DERIVATES
             flag=#True            
             time-1000 ; To refresh the word in real time (300 would be the exact number, but used 1000 to be 100% sure it will activate the refresh in real time)
             
           ; Double-click in a line   
           Case #PANEL_BOX_DERIVED_WORDS
             If EventType()=#PB_EventType_LeftDoubleClick
               PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #PANEL_BOX_DERIVED_WORDS_DISPLAY)
             EndIf
             
           ; Select DISPLAY
           Case #PANEL_BOX_DERIVED_WORDS_DISPLAY             
             If CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)<>0 And GetGadgetState(#PANEL_BOX_DERIVED_WORDS)<>-1
               t=Val(GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,GetGadgetState(#PANEL_BOX_DERIVED_WORDS),3))-1 ;Get the position of the condition in the .AFF file
               Debug "Position"+Str(t)
               SetPos(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD,t,1)
             EndIf                                   
             
           ; MATCH button   
           Case #BUTTON_BOX_DERIVED_WORDS_WORD_MATCH
             counter=CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)
             t$=GetGadgetText(#STRING_BOX_DERIVED_WORDS_WORD)
             If counter=0 Or t$=""
               MessageRequester("Warning!","No matches to find.",#PB_MessageRequester_Warning) 
             Else
               SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,#PB_All,#PB_Gadget_FrontColor,#Black,0)
               number_of_matches=0
               For f=0 To counter-1
                 If Left(GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,f,0),Len(t$))=t$
                   SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,f,#PB_Gadget_FrontColor,#Red,0)
                   number_of_matches+1
                 EndIf
               Next f
               SetGadgetText(#TEXT_BOX_DERIVED_WORDS_WORD_MATCH,"("+Str(number_of_matches)+")")
               GadgetFit(#TEXT_BOX_DERIVED_WORDS_WORD_MATCH)
             EndIf
             
           ; SORT Radio buttons
           Case #RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO1 To #RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO5
             dictionary_column_sort=EventGadget()-#RADIO_BUTTON_DERIVED_WORDS_SORT_RADIO1+1
             
           ; Press SORT
           Case #BUTTON_DERIVED_WORDS_SORT_SORT
             counter=CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)
             If counter=0
                MessageRequester("Warning!","No derived words found.",#PB_MessageRequester_Warning)               
             Else
                sort_and_display_derivates(counter)
             EndIf
             
           ; Press REMEMBER
           Case #BUTTON_DERIVED_WORDS_SORT_REMEMBER           
            ; Delete old .prefs file    
            If FileSize(main_path$+"settings/"+"derivates_sort.prefs")>=0
              DeleteFile(main_path$+"settings/"+"derivates_sort.prefs")
            EndIf               
            
            ; Attempt to save dictionary_column_sort .PREFS file
            OpenFile(1,main_path$+"settings/"+"derivates_sort.prefs")  
              WriteString(1,prefs_file_version$+#LF$,#PB_UTF8)
              WriteLong(1,dictionary_column_sort)                                
            CloseFile(1)
             
           ; Press OK
           Case #BUTTON_ADD_EDIT_OKAY
             okay=#True
             t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)                               
             If (t$="" Or t$=" ") And okay=#True
               MessageRequester("Warning!","No dictionary word text.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
               okay=#False
             EndIf                   
;              If ((Right(t$,1)="/" And Right(t$,2)<>"\/") Or (Right(t$,1)="/" And Right(t$,3)<>"\//")) And okay=#True            
;                MessageRequester("Warning!","Invalid word.",#PB_MessageRequester_Warning)
;                SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
;                okay=#False
;              EndIf             
             If (CountString(t$,"  ") Or Left(t$,1)=" " Or Right(t$,1)=" ") And GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>#YellowDark And okay=#True
               MessageRequester("Warning!","Invalid spaces in dictionary word.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
               okay=#False
             EndIf
             t$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG) 
             If Len(t$)<>0 And (CountString(t$,"  ") Or Left(t$,1)=" " Or Right(t$,1)=" ") And okay=#True
               MessageRequester("Warning!","Tags incorrect (spaces).",#PB_MessageRequester_Warning)
               SetActiveGadget(#EDITOR_WINDOW_ADD_EDIT_TAG)
               okay=#False          
             EndIf
             If GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)=#Red And okay=#True
               MessageRequester("Warning!","Invalid dictionary word.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
               okay=#False
             EndIf              
             
           ; Press CANCEL  
           Case #BUTTON_ADD_EDIT_CANCEL
             cancel=#True             
             
          EndSelect
       EndSelect                 
            
 
      
      ;Update derived words
      ; Fix: 13/JUL/2014 - It no longer shows "Primary" for invalid words (blank OR starting with a "/")
      If ElapsedMilliseconds()>time+600 And flag=#True
        circumfix_code2=#False ; set for primary word, so if there is at least one CIRCUMFIX it will turn on the image gadget
        ClearGadgetItems(#PANEL_BOX_DERIVED_WORDS)
        colour_line_in_affixes_listicongadget=0
        If GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)<>"" And Left(GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD),1)<>"/"
          AddWindowTimer(#WINDOW_ADD_EDIT,800,800)
          DisableGadget(#BUTTON_REFRESH_DERIVATES,#True)
          Repeat
            ; Notice that the 4-5 lines below are based on the above ones
            circumfix_code2=#False ; set for primary word, so if there is at least one CIRCUMFIX it will turn on the image gadget
            ClearGadgetItems(#PANEL_BOX_DERIVED_WORDS)
            colour_line_in_affixes_listicongadget=0            
            dictionary_force_refresh=0
            If GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)<>"" And Left(GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD),1)<>"/" : process_pfx_sfx(1,GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD),#False,#False,#False,"","","",#False,#False) : EndIf
          Until dictionary_force_refresh=0 Or dictionary_force_refresh=2 Or dictionary_force_refresh=3
          DisableGadget(#BUTTON_REFRESH_DERIVATES,#False)
          RemoveWindowTimer(#WINDOW_ADD_EDIT,800)
          
          ; Sort in ListIconGadget
          counter=CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)
          If counter>0 And dictionary_column_sort<>1 : sort_and_display_derivates(counter) : EndIf       
          
          ; Show top level flags
          If counter>0 : process_top_level_flags(counter) : EndIf  
          
        EndIf  
;         SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)))
        SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
        flag=#False  
        time=ElapsedMilliseconds()
        If dictionary_force_refresh=2 : PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #BUTTON_ADD_EDIT_OKAY) : EndIf
        If dictionary_force_refresh=3 : PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #BUTTON_ADD_EDIT_CANCEL) : EndIf
      EndIf      
      
      
    Until okay=#True Or cancel=#True
    UnbindGadgetEvent(#TEXTBOX_WINDOW_ADD_EDIT_WORD, @process_pfx_sfx_update_word(),#PB_EventType_Change) ; Current word
    UnbindGadgetEvent(#BUTTON_ADD_EDIT_OKAY, @process_pfx_sfx_update_word_okay(),#PB_EventType_LeftClick) ; OK
    UnbindGadgetEvent(#BUTTON_ADD_EDIT_CANCEL, @process_pfx_sfx_update_word_cancel(),#PB_EventType_LeftClick) ; Cancel
    UnbindEvent(#PB_Event_Menu, @process_pfx_sfx_update_word_cancel(), #WINDOW_ADD_EDIT,1000) ; <ESC>
    
    
    
    ; Reset the force refresh flag, so that it doesn't affect other derivates operations in PTG
    dictionary_force_refresh=0
    
    ; OK button pressed - Get the current combobox position
    If okay=#True
      aff_aid_language_current_combobox_entry=GetGadgetState(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)
    EndIf
    
    ; OK button pressed. It will now add or update a synonym and its meanings
    If okay=#True
      t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
      a$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
      If a$<>"" : t$+#LF$+a$ : EndIf
      
      If old_word$<>GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD) Or old_tags$<>GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
        dictionary_store_undo(#False) ; 7/JUN/2017
      EndIf
      
      If add=#True
        
        ReDim words$(number_of_words)        
        ReDim words_selected(number_of_words)        
        words$(number_of_words)=t$
        number_of_words+1
        SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        update_number_words()
        global_update_virtual_vertical_scroll_bar(-1)  
        
        ; Hyphenation support - 3/APR/2018
        SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        update_number_hyphenations()
        global_update_virtual_vertical_scroll_bar(2)
  
      EndIf
      
      If edit=#True
        
        t=selected_dictionary_word         
        words$(t)=t$
        global_update_virtual_vertical_scroll_bar(-1)
        global_update_virtual_vertical_scroll_bar(2) ; Hyphenation support - 3/APR/2018
        
      EndIf
      
    EndIf
    
    ; OK pressed and change the dictionary_changed variable (ONLY IF WORD/TAGS WERE CHANGED - 5/NOV/2014)
    If okay=#True
      If old_word$<>GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD) Or old_tags$<>GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
        dictionary_changed=#True
                
        
      EndIf 
      update_number_words()
      update_number_hyphenations() ; Hyphenation support - 3/APR/2018
    EndIf    
    
    update_vertical_scroll_bar_dictionary_current_item()
    update_vertical_scroll_bar_hyphenation_current_item() ; Hyphenation support - 3/APR/2018
    
    ; Close the Add/Edit window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_ADD_EDIT) 
    
  EndProcedure 
  
  
  
  
  ; To be used in the dictionary editor to copy one or all words into the clipboard
  ;
  ; Variable ALL can be #False (one) or #True (all)
  ;
  ; V1.0 - 19/MAR/2019
  ;
  Procedure clipboard_copy_words(all)
    
    If CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    If GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=-1
      MessageRequester("Warning!","No dictionary word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    pos_line_start=GetGadgetState(#PANEL_BOX_DERIVED_WORDS)
    If all=#True : pos_line_start=0 : EndIf
    pos_line_end=CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)-1
    new_line$=""
    If all=#False : pos_line_end=pos_line_start : EndIf
    If all=#True And pos_line_start<>pos_line_end : new_line$=#LF$ : EndIf
    
    t$=""
    For f=pos_line_start To pos_line_end
      t$+GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,f,0)+new_line$
    Next f
    
    SetClipboardText(t$)
    
    
  EndProcedure
  
  
  
  
  ; To be used in the dictionary editor to copy one or all words plus rules into the clipboard
  ;
  ; Variable ALL can be #False (one) or #True (all)
  ;
  ; V1.0 - 19/MAR/2019
  ;
  Procedure clipboard_copy_words_and_rules(all)
    
    If CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    If GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=-1
      MessageRequester("Warning!","No dictionary word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    pos_line_start=GetGadgetState(#PANEL_BOX_DERIVED_WORDS)
    If all=#True : pos_line_start=0 : EndIf
    pos_line_end=CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)-1
    new_line$=""
    If all=#False : pos_line_end=pos_line_start : EndIf
    If all=#True And pos_line_start<>pos_line_end : new_line$=#LF$ : EndIf
    
    t$=""
    For f=pos_line_start To pos_line_end
      
      t1$=""
      For ff=0 To 4
        If ff=1 : t1$+Chr(34)+GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,0,0)+Chr(34)+"," : EndIf
        If ff<>3 : t1$+Chr(34)+GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,f,ff)+Chr(34) : EndIf
        If ff<>4 And ff<>3 : t1$+"," : EndIf
      Next ff
      t1$=ReplaceString(t1$," "+Chr(34),Chr(34))
      t1$=ReplaceString(t1$,Chr(34)+" ",Chr(34))

      t$+t1$+new_line$
      
    Next f
    
    
    SetClipboardText(t$)
    
    
  EndProcedure
  
  
  
  
  ; To copy the rule of a flag
  ;
  ; V1.0 - 16/AUG/2022
  ;
  Procedure clipboard_copy_flag_rule()
    
    If CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    If GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=-1
      MessageRequester("Warning!","No dictionary word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    pos_line_start=GetGadgetState(#PANEL_BOX_DERIVED_WORDS)
    
    t$+Chr(34)+GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,pos_line_start,4)+Chr(34)
    
    SetClipboardText(t$)
    
  EndProcedure
  
  
  
  
  ; To be used in the dictionary editor to remove flags from the textbox
  ;
  ; V1.0 - 11/JUL/2019
  ;
  Procedure remove_flag_from_textbox()
    
    If CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    If GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=-1
      MessageRequester("Warning!","No dictionary word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    If GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=0
      MessageRequester("Warning!","Primary word doesn't have a flag.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    flag_to_remove$=GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,GetGadgetState(#PANEL_BOX_DERIVED_WORDS),2)
    Debug "Flag to remove:"+flag_to_remove$
    
    
    word$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
    t$=word$
    t$=ReplaceString(t$,"\/",";") ; Support escaped
    t=FindString(t$,"/")
    t$=ReplaceString(t$,";","/") ; Support escaped
    tf=t ; To be used inside the loops to avoid repeating FINDSTRING(T$,"/")
    t2$=t$ : If t>1 : t2$=Left(t$,t-1) : EndIf ; t2$ holds the word before the "/" to simplify the decoding
    If t2$="" : ProcedureReturn #False : EndIf      ; empty word
    
    
    If Right(t$,1)<>"/" And tf>0
       word_codes$=Mid(t$,t+1) ; Get the codes after each word 11/AUG/2013 - Removed the len for extra speed - 22/OCT/2015

       ; For utf-8/chr,long AND num
       len_word_codes=Len(word_codes$)  
       If flag_char=2
          len_word_codes=Round(Len(word_codes$)/2.0,#PB_Round_Up)
       EndIf
       If flag_char=3
          len_word_codes=CountString(word_codes$,",")+1
       EndIf         
       
       Debug "word_codes$:"+word_codes$       
       Debug "len_word_codes:"+Str(len_word_codes)
       
    EndIf
    
    
    ; No flags
    If len_word_codes=0 : ProcedureReturn #False : EndIf
    
    
    new_flags$=""
    
    flag=#False
    For f=1 To len_word_codes                           
      
      faster_word_codes$=get_faster_word_codes(word_codes$,f)
      If faster_word_codes$=flag_to_remove$
        flag=#True
      Else
        new_flags$+faster_word_codes$
        If flag_char=3 : new_flags$+"," : EndIf
      EndIf
      
    Next f
    
    ; No flag found
    If flag=#False
      MessageRequester("Warning!","Flag not found in textbox.",#PB_MessageRequester_Warning)
      ProcedureReturn #False      
    EndIf
    
    ; Remove last "," from flag num
    If flag_char=3 And Right(new_flags$,1)=","
      new_flags$=Left(new_flags$,Len(new_flags$)-1)
    EndIf
    
    ; Replace word in textbox with new one with the flag removed
    If new_flags$<>""
      SetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD,t2$+"/"+new_flags$)      
    Else
      SetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD,t2$)
    EndIf
    
    ; Change done to the textbox
    ProcedureReturn #True
    
  EndProcedure
  
  

  
  ; This procedure does all the decoding work, for PTG 4
  ;
  ; V1.0 - 12/NOV/2023
  ;
  Procedure process_primary_pfx_sfx_both_ptg4(word$,operation_id)
    
    ; Below for the processing arrays
    processed_counter_ptg4_primary=0
    processed_counter_ptg4_sfx=0
    processed_counter_ptg4_pfx=0
    processed_counter_ptg4_both=0
    
    Dim processed_primary_ptg4(0)
    process_array_primary_ptg4()
    
    Dim processed_sfx_ptg4(0)
    process_array_sfx_ptg4()
    
    Dim processed_pfx_ptg4(0)
    process_array_pfx_ptg4()
    
    Dim processed_both_ptg4(0)
    process_array_both_ptg4()

    
    
    ; At the end we have the operation_id processing
    Select operation_id
        
      ; Export  
;       Case XXXXXXXXXXXX
;         For f=1 To processed_counter_ptg4_primary
;         For f=1 To processed_counter_ptg4_sfx
;         For f=1 To processed_counter_ptg4_pfx
;         For f=1 To processed_counter_ptg4_both
;           
;         copy_to_cache_plain_words() ; for merging of flags
;         copy_to_cache_structured_words()
          
    EndSelect
    
    
    
  EndProcedure
  
  
  
  
  ; Procedure for decoding primary
  ;
  ; V1.0 - 12/NOV/2023
  ;
  Procedure process_array_primary_ptg4()
  EndProcedure
  
  
  
  
  ; Procedure for decoding sfx
  ;
  ; V1.0 - 12/NOV/2023
  ;
  Procedure process_array_sfx_ptg4()
  EndProcedure
  
  
  
  
  ; Procedure for decoding both
  ;
  ; V1.0 - 12/NOV/2023
  ;
  Procedure process_array_pfx_ptg4()
  EndProcedure
  
  
  
  
  ; Procedure for decoding primary
  ;
  ; V1.0 - 12/NOV/2023
  ;
  Procedure process_array_both_ptg4()
  EndProcedure

  
  
  
  ; This procedure now processes the prefixes and suffixes.
  ; Values for variable: type_of_processing_of_pfx_sfx
  ;
  ;  1 = add/edit word / Hyphenation
  ;  2 = Extract wordlist
  ;  3 = Count wordlist
  ;  4 = Find duplicates
  ;  5 = Statistics
  ;  6 = Statistics (adds words pop-up options to the EditorGadget - Show)
  ;  7 = Statistics (adds words pop-up options to the EditorGadget - Extract)
  ;100 = "Crunch words with flags"
  ;
  ; According to the value of the variable, for example, if =1 it adds directly in the listicongadget of the add/edit words procedure
  ;  If =2 it saves directly to HDD
  ;  If =3 it just counts
  ;
  ; Notice that the function returns the number of words found, but this is just useful for =2 and =3
  ; in order to add to the status bar
  ; 
  ; The value of WORD$ is the current word from the array, with all the codes
  ;
  ; The CSV is either #false=export as .txt or #true=export as .csv
  ; The COMPOUNDS if #True means to export compound words
  ;
  ; recursivity=0/1 -> In case it is called a rule from inside a rule
  ;
  ; languagetool$: "" or "*" if exporting LanguageTool compounds
  ;
  ; morphologic_part$: Used in show duplicates
  ;
  ;
  ; V1.0 - 11/JUL/2014  
  ; V1.1 - 12/JUL/2014    
  ;        o Optimised the code a bit in the part that searches for Y or N as the header of the rules set
  ;          ( "N" is first searched and only if NOT found, it searches for "Y")
  ;        o Get the number of lines per PFX/SFX is now ultra-fast - just 2 lines instead of 10
  ;        o Added and coded the "Rule" column
  ; V1.2 - 13/JUL/2014      
  ;        o Optimised the bottleneck: Now around 7x faster with the new code
  ; V1.3 - 15/JUL/2014  
  ;        o Replaced the rules text " - " with ": " for the add/edit dictionary
  ; V1.4 - 16/JUL/2014
  ;        o Added CSV
  ; V1.5 - 17/JUL/2014
  ;        o In CSV it now has an extra field with the primary word and the separator is now ";" instead of ",".
  ; V1.6 - 31/JUL/2014  
  ;        o Optimised the code slightly
  ; V1.7 - 6/AUG/2014
  ;        o Coded the "Show Duplicates"
  ; V1.8 - 15/NOV/2014
  ;        o It now accepts: \/ are escaped "/"
  ;          It converts "\/" to ";" and then tries to find a "/" and then it converts ";" to "/"
  ; V1.9 - 8/DEC/2014
  ;        o It now combines PREFIXES against PRIMARY+SUFFIXES
  ; V2.0 - 10/DEC/2014
  ;        o Tried to optimise slightly by only getting the variable value once of Mid(word_codes$,f,1)
  ; V2.1 - 15/DEC/2014
  ;        o Coded statistics
  ; V2.2 - 17/JAN/2015
  ;        o Ultrafast searching for rules positions
  ; V2.3 - 5/MAR/2015
  ;        o Coded "Extract Compounds .txt".
  ; V2.4 - 22/OCT/2015
  ;        o Replaced some string operations with the command StringField for extra speed.
  ;        o Replaced some IFs with SELECT+CASE for extra speed.
  ; V2.5 - 18/JAN/2016
  ;        o Decoding algorithm improved with minor speed gain (it now uses chr(1) INSTEAD of ",," in the ultrafast array
  ; V2.6 - 10/APR/2016
  ;        o Fix: While editing words, more than one prefix would show an incorrect "Code Position"
  ; V2.7 - 13/MAR/2017
  ;        o Added decoding support for (no recursivity yet): 1) FLAG long  2) FLAG num
  ; V2.8 - 28/MAR/2017
  ;        o Added recursivity to dictionary word decoding.
  ; V2.9 - 30/MAR/2017
  ;        o Improved recursivity to dictionary word decoding.  
  ; V3.0 - 31/MAR/2017
  ;        o Improved dictionary word decoding (recursivity)
  ; V3.1 - 1/APR/2017
  ;        o Improved dictionary word decoding (recursivity)  
  ; V3.2 - 4/APR/2017
  ;        o Support via coloured symbol for NOSUGGEST in words editor    
  ;        o Improved: If a code isn't found in the .aff it no longer exits the decoding function
  ; V3.3 - 16/APR/2017
  ;        o Fixed another prefix bug with twofold
  ; V3.4 - 28/APR/2017
  ;        o Support for extracting wordlist compounds for LanguageTool and check for duplicates makes use of morphological information
  ; V3.5 - 21/NOV/2017
  ;        o Added "Missing Codes:" to the Dictionary editor.
  ; V3.6 - 28/NOV/2017
  ;        o Fix: NOSUGGEST flag no longer appears in the not found codes.
  ; V3.7 - 3/DEC/2017
  ;        o Fix: Decoding of rules with a 0 (pt_PT)
  ; V3.8 - 1/APR/2018
  ;        o Hyphenation support
  ; V3.9 - 13/APR/2018
  ;        o Support for compressed AF  
  ; V4.0 - 16/APR/2018
  ;        o NEEDAFFIX support - 16/APR/2018
  ; V4.1 - 27/APR/2018
  ;        o Major Speed up of dictionary processing
  ; V4.2 - 10/MAY/2018
  ;        o Fix: Coded "SFX N" rules, by not applying prefixes to it.
  ; V4.3 - 15/MAY/2018
  ;        o Speed up in dictionary processing by searching first for "SFX Y" instead of "SFX N"
  ; V4.4 - 18/MAY/2018
  ;        o Taboo's warning sign is now an image and is stored in a long instead of string for speed up. 
  ;        o New strategy for extra speed decoding the dictionary, but it takes more RAM.
  ; V4.5 - 23/MAY/2018
  ;        o Fix: Decoding of prefixes wasn't working for some suffixes under certain conditions (due to the new "SFX N" code)
  ; V4.6 - 7/JUN/2018
  ;        o Cleaned the decoding of prefixes and suffixes
  ; V4.7 - 26/OCT/2018
  ;        o Edit/New Dictionary word now supports two instances of "0" in the rules
  ;        o Fix: replaced regarding prefixes conditions_per_line_sfx$(4) with conditions_per_line_pfx$(4)
  ; V4.8 - 29/OCT/2018
  ;        30/OCT/2018
  ;        01/DEC/2018
  ;        o Coded the "CIRCUMFIX" flag.
  ; V4.9 - 14/JAN/2019
  ;        o Improved CIRCUMFIX to avoid decoding words without codes using recursivity
  ; V5.0 - 20/JAN/2019
  ;        21/JAN/2019
  ;        o CIRCUMFIX finally coded.
  ; V5.1 - 08/FEB/2019
  ;        o Minor speed up decoding affixes.
  ; V5.2 - 09/FEB/2019
  ;        10/FEB/2019
  ;        11/FEB/2019
  ;        o Minor speed gain in affix decoding by moving the refresh timer to outside the rules loops
  ;        o Minor speed up decoding affixes
  ; V5.3 - 27/FEB/2019
  ;        28/FEB/2019
  ;        07/MAR/2019
  ;        o Minor speed up decoding affixes
  ; V5.4 - 11/MAR/2019
  ;        o Fixed decoding of prefixes using twofold if there were flags of suffixes after flags of prefixes
  ; V5.5 - 16/MAR/2019
  ;        o Export wordlist now exports in UNIX format with BOM
  ; V5.6 - 18/MAR/2019
  ;        o Extracting as CSV the rules number have zeros before to make sorting easier in Calc/Excel.
  ; V5.7 - 01/APR/2019
  ;        o Added support for "KEEPCASE"
  ; V5.8 - 02/APR/2019
  ;        o Refreshing derivates in the Dictionary Editor now updates the number found during processing  
  ; V5.9 - 11/APR/2019
  ;        o Allows to refresh decoding during processing, and OKAY and CANCEL
  ; V6.0 - 4/MAY/2019
  ;        o Hyphenation now uses colours in the flags (Prefix, Suffix, Both)
  ; V6.1 - 02/JUL/2019
  ;        03/JUL/2019
  ;        o Fixing of CIRCUMFIX flag.
  ; V6.2 - 10/JUL/2019 
  ;        o Cleaned the code regarding the CIRCUMFIX flag.
  ; V6.3 - 26/SEP/2019  
  ;        o Checks inside a rule if it has the NEEDAFFIX FLAG
  ; V6.4 - 27/SEP/2019    
  ;        o Allow to sort by column
  ; V6.5 - 09/OCT/2019
  ;        o Coding the extra options for the statistics (Shantanu Oak)
  ; V6.6 - 15/OCT/2019
  ;        16/OCT/2019
  ;        o Estatistics: Extract with flags option (Shantanu Oak)
  ; V6.7 - 04/FEB/2020
  ;        05/FEB/2020
  ;        o Dictionary Editor: Coding of Shantanu's summary
  ; V6.8 - 17/MAR/2020
  ;        o Fixed the NEEDAFFIX flag
  ; V6.9 - 24/MAR/2020
  ;        25/MAR/2020
  ;        26/MAR/2020
  ;        05/APR/2020
  ;        06/APR/2020
  ;        07/APR/2020
  ;        08/APR/2020
  ;        10/APR/2020
  ; V7.0 - 11/APR/2020
  ;        13/APR/2020
  ;        14/APR/2020
  ;        15/APR/2020
  ;        16/APR/2020
  ;        17/APR/2020
  ;        18/APR/2020
  ;        19/APR/2020
  ;        20/APR/2020
  ;        21/APR/2020
  ;        26/APR/2020
  ;        27/APR/2020
  ;        28/APR/2020
  ;        29/APR/2020
  ;        30/APR/2020
  ;        01/MAY/2020
  ;        02/MAY/2020
  ;        09/MAY/2020
  ;        10/MAY/2020
  ;        11/MAY/2020
  ;        12/MAY/2020
  ;        13/MAY/2020
  ;        14/MAY/2020
  ;        15/MAY/2020
  ;        o Major cleanup of the process pfx/sfx procedure
  ; V7.1 - 11/NOV/2020
  ;        o Coded twofold missing flags
  ; V7.2 - 15/NOV/2020
  ;        o Coding of "Crunch words with flags".
  ; V7.3 - 07/FEB/2021
  ;        o Extract wordlist as CSV now has a "Root_word" field.
  ; V7.4 - 09/FEB/2021
  ;        10/FEB/2021
  ;        o Fix: Root word in exporting as .CSV
  ;
  Procedure process_pfx_sfx(type_of_processing_of_pfx_sfx,word$,csv,compounds,recursivity,languagetool$,morphologic_part$,lines_part$,hyphenate,AF_compressed_decoding_done)

    ; Minor speed up by grouping the check for recursivity=#False
    If recursivity=#False
      
      ; NUMBER DERIVATES FOUND for real time update - 2/APR/2019
      number_of_derivates_found=0
      
      ; NOSUGGEST support - 4/APR/2017
      nosuggestword_colour=#Green
      
      
      ; KEEPCASE support - 1/MAY/2019
      keepcaseword_code=#False
      
      
      ; NEEDAFFIX support - 16/APR/2018
      needaffixword_code=#False
      
      ; NEEDAFFIX INSIDE support - 17/MAR/2020
      needaffixword_code_inside=#False
      
    
      ; FORBIDDENWORD support - 14/JUN/2018
      forbiddenword_code=#False
  
    
      ; CIRCUMFIX support - 29/OCT/2018 + 30/OCT/2018 + 1/DEC/2018 + 2/JUL/2019 + 25/MAR/2020 + 26/MAR/2020
      circumfix_processing$=""
      circumfix_code=#False ; REMOVE AFTER TOP & DOWN ARE CODED - 14/MAY/2020
      needcircumfix_code_inside_up=#False ; 14/MAY/2020
      needcircumfix_code_inside_down=#False ; 14/MAY/2020
      circumfix_code2=#False ; 2/JUL/2019
      circumfix_twofold_counter=0 ; 5/APR/2020
      ; Lines below to fix massive duplicates on CIRCUMFIX - 25/MAR/2020
      needcircumfix_code_inside=#False
      
      circumfix_processing$="" ; for adding left and right affixes
      circumfix_primary$="" ; Stores the primary word for fixing the export as CSV CIRCUMFIX
      circumfix_position=0 ; To grab the position of the flag
      circumfix_ff=0       ; To grab the position of the rule inside the flag
      Dim circumfix_conditions_per_line_both$(5) ; to store the conditions of the prefixes and affixes
      word_original$=""
      faster_word_codes$=""
      Dim conditions_per_line_pfx$(5)
      Dim conditions_per_line_sfx$(5)
      pfx=0 ; Brandon fix to display prefixes with CIRCUMFIX - 28/APR/2020
      sfx=0 ; Brandon fix to display prefixes with CIRCUMFIX - 28/APR/2020      
      one_two_three_brandon_fix=0 ; Brandon fix to display "Both" - 2/MAY/2020
      
      
      ; MISSING CODES - 21/NOV/2017
      missing_codes$=""
      
      ; MISSING CODES - 11/NOV/2020
      missing_codes_sub$=""            
      
      ; SHANTANU SUPPORT FOR SUMMARY - 4/FEB/2020
      top_level_tags$=""
      nested_tags$=""      
      If type_of_processing_of_pfx_sfx=1 And hyphenate=#False
        ClearGadgetItems(#LISTICON_DICTIONARY_SUMMARY_TOP_LEVEL_TAGS)
        ClearGadgetItems(#LISTICON_DICTIONARY_SUMMARY_NESTED_TAGS)
      EndIf  
        
      
      ; Variables below to handle the combination of prefixes with suffixes+primary
      ; 8/DEC/2014
      Dim primary_suffixes$(0) ; Array that stores the words primary+suffixes
      Dim primary_suffixes_sfx_N(0) ; Array that stores the words primary+suffixes ;If =1 it doesn't apply PREFIXES SFX N
      flag_sfx_no=0 ; To not derivates prefixes for given word - 10/MAY/2018: 0=No,1=Yes  
      total_primary_suffixes=0 ; total of primary+suffixes
      Dim prefixes_codes$(0)   ; Array that stores the CODES of the prefixes
      Dim prefixes_codes_position(0); Array that stores the POSITIONS of the CODES of the prefixes
      Dim prefixes_codes_position_in_array(0) ; to store the position of prefix in the array, not in memory
      total_prefixes_codes=0 ; Total of prefixes CODES
      
      
      ; To use in ProcedureReturn
      total_words_found=0
            
    EndIf
    
    
    
; REM DEBUGGING OF CIRCUMFIX DUPLICATES
If recursivity=#False
  Debug "DEBUGGING OF CIRCUMFIX DUPLICATES *START*"
  Debug "word$:"+word$
  Debug "word_codes$:"+word_codes$
  Debug "len_word_codes:"+Str(len_word_codes)
  Debug "word_original$:"+word_original$
  Debug "circumfix_primary$:"+circumfix_primary$
  Debug "faster_word_codes$:"+faster_word_codes$
  Debug "circumfix:"+Str(circumfix)
  Debug "a$:"+a$
  Debug "b$:"+b$
  Debug "total_prefixes_codes:"+Str(total_prefixes_codes)
  Debug "number_of_rules:"+Str(number_of_rules)  
  Debug "total_primary_suffixes:"+Str(total_primary_suffixes)
  Debug "flag_sfx_no:"+Str(flag_sfx_no)
  Debug "total_words_found:"+Str(total_words_found)  
  Debug "conditions_per_line_pfx$(1):"+conditions_per_line_pfx$(1)
  Debug "conditions_per_line_pfx$(2):"+conditions_per_line_pfx$(2)
  Debug "conditions_per_line_pfx$(3):"+conditions_per_line_pfx$(3)
  Debug "conditions_per_line_pfx$(4):"+conditions_per_line_pfx$(4)
  Debug "conditions_per_line_pfx$(5):"+conditions_per_line_pfx$(5)
  Debug "circumfix_position:"+Str(circumfix_position)
  Debug "position1:"+Str(position1)
  Debug "one_two_three_brandon_fix:"+Str(one_two_three_brandon_fix)  
  Debug "t2$:"+t2$
  Debug "needcircumfix_code_inside:"+Str(needcircumfix_code_inside)
  Debug "t$:"+t$
  Debug "primary_suffixes$(0):"+primary_suffixes$(0)
  Debug "primary_suffixes_sfx_N(0):"+Str(primary_suffixes_sfx_N(0))
  Debug "pfx:"+Str(pfx)
  Debug "sfx:"+Str(sfx)
  Debug "position2:"+Str(position2)
  
  
  Debug "conditions_per_line_sfx$(1):"+conditions_per_line_sfx$(1)
  Debug "conditions_per_line_sfx$(2):"+conditions_per_line_sfx$(2)
  Debug "conditions_per_line_sfx$(3):"+conditions_per_line_sfx$(3)
  Debug "conditions_per_line_sfx$(4):"+conditions_per_line_sfx$(4)
  Debug "conditions_per_line_sfx$(5):"+conditions_per_line_sfx$(5)
  Debug "DEBUGGING OF CIRCUMFIX DUPLICATES *END"
EndIf



    ; COMPRESSED AF - 13/APR/2018
    If AF_compressed_number>0 And AF_compressed_decoding_done=#False
      If Val(StringField(word$,2,"/"))>0 And Val(StringField(word$,2,"/"))<=AF_compressed_number
        word$=StringField(word$,1,"/")+"/"+AF_compressed_array$(Val(StringField(word$,2,"/")))
      EndIf
      AF_compressed_decoding_done=#True
    EndIf
    
    
    
    ; Get the word PLUS escaped
    t$=word$
    t$=ReplaceString(t$,"\/",";") ; Support escaped - 15/NOV/2014
    t=FindString(t$,"/")
    t$=ReplaceString(t$,";","/") ; Support escaped - 15/NOV/2014
    tf=t ; 31/JUL/2014 - To be used inside the loops to avoid repeating FINDSTRING(T$,"/")
    t2$=t$ : If t>1 : t2$=Left(t$,t-1) : EndIf ; t2$ holds the word before the "/" to simplify the decoding
    If t2$="" : ProcedureReturn 0 : EndIf      ; empty word
    
    

    ; Root word - 9/FEB/2021
    If recursivity=#False
      root_word$=t2$
    EndIf
    
    
    ; Get the number of flags to apply to word
    If Right(t$,1)<>"/" And tf>0
      
       word_codes$=Mid(t$,t+1) ; Get the codes after each word 11/AUG/2013 - Removed the len for extra speed - 22/OCT/2015
       
       ; Count number of flags
       len_word_codes=count_flags_inside(word_codes$)

    EndIf
    
    
    
    ; KEEPCASE support - 1/MAY/2019
    ; NEEDAFFIX support - 16/APR/2018
    ; CIRCUMFIX support - 29/OCT/2018
    ; Check all word codes to look for the KEEPCASE, NEEDAFIX and CIRCUMFIX
    If ((keepcaseword=#True And keepcaseword_code=#False) Or (needaffixword=#True And needaffixword_code=#False) Or (circumfix=#True And circumfix_code=#False)); And recursivity=#False
    
      ; REMEMBER ORIGINAL WORD for CIRCUMFIX fix when clearing the CIRCUMFIX variable
      If recursivity=#False
        word_original$=t2$      
      EndIf      
          
      
      For f=1 To len_word_codes                      
        
        ; 10/FEB/2019    
        faster_word_codes$=get_faster_word_codes(word_codes$,f)
        
        
        ; KEEPCASE support - 1/MAY/2019
        If faster_word_codes$=keepcaseword$
          keepcaseword_code=#True
          Continue
        EndIf           
        
            
        ; NEEDAFFIX support - 16/APR/2018 + 10/FEB/2019 + 28/FEB/2019
        If faster_word_codes$=needaffixword$
          needaffixword_code=#True
          Continue
        EndIf   
        
        
        ; CIRCUMFIX support - 29/OCT/2018 + 10/FEB/2019 + 28/FEB/2019 + 11/MAY/2020
        If faster_word_codes$=circumfix$ And fuk=999 ; Remmed since main word can't have a CIRCUMFIX flag - Mauro - 11/MAY/2020
          circumfix_code=#True
          circumfix_code2=#True ; set for primary word, so if there is at least one CIRCUMFIX it will turn on the image gadget
          Continue
        EndIf           
        
      Next f
            
    EndIf
    
    
 Debug "word$:"+word$  

        
If recursivity=#False  
  
  
        circumfix_primary$=t2$ ; Stores the primary word for fixing the export as CSV CIRCUMFIX
        
     
        Select type_of_processing_of_pfx_sfx
         Case 1        
           If hyphenate=#False
             number_of_derivates_found+1
;              SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found))
             SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
             AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,t2$+#LF$+"Primary")
             SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,0,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_primary),1)
             add_derivate_to_sort_array(number_of_derivates_found)
           Else
             AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,t2$+#LF$+"")
             SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,0,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_primary),1)             
           EndIf
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no 
          total_primary_suffixes+1                            
          total_words_found+1
          
         Case 2        
          If csv=#False And compounds=#False : WriteString(1,t2$+#LF$,#PB_UTF8) : EndIf
          If csv=#False And compounds=#True And FindString(t2$,"-") : WriteString(1,t2$+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
          If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(t2$,1,"/")+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+Chr(34)+"Primary"+Chr(34)+";"+Chr(34)+Chr(34)+";"+Chr(34)+Chr(34)+";"+Chr(34)+Chr(34)+#LF$,#PB_UTF8) : EndIf          
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no          
          total_primary_suffixes+1                  
          If compounds=#False : total_words_found+1 : EndIf
          
         Case 3 
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no          
          total_primary_suffixes+1                            
          total_words_found+1
          
         Case 4
          ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
          total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=t2$+morphologic_part$+lines_part$
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no           
          total_primary_suffixes+1                            
          total_words_found+1
          
         Case 5 
          If csv=#False : process_statistics(t2$): EndIf    
          total_words_found+1

          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no 
          total_primary_suffixes+1                  
          
         Case 6,7
        
          statistics_flag=#False
          ttt_statistics$=t2$
          ttt_statistics_flag$=" (Primary)"
          If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
          If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
          If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
          If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(t2$,1) : statistics_flag=#True : EndIf
          If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
          If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
          If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
          If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
          If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf

          If statistics_flag=#True And type_of_processing_of_pfx_sfx=6 : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf
          If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no          
          total_primary_suffixes+1                           
            
        EndSelect
        
        
EndIf        
        
            
        If Right(t$,1)<>"/" And tf>0      
          
Debug "flag_char:"+Str(flag_char)
Debug "word_codes$:"+word_codes$
Debug "len_word_codes:"+Str(len_word_codes)



          For f=1 To len_word_codes ;Len(word_codes$)                        
            
            
            ; Moved here on 9/FEB/2019  
            ; Fix to window stop responding if we click on it while in the loop
            If type_of_processing_of_pfx_sfx=1 
              event = WindowEvent()              
              If event=#PB_Event_Timer And EventTimer()=800  : EndIf                          
              
              If dictionary_force_refresh<>0 : ProcedureReturn 0 : EndIf

            EndIf
            
            
            
            flag_sfx_no=0 ; To not derivates prefixes for given word - 10/MAY/2018: 0=No,1=Yes                        
            
            ; 10/FEB/2019    
            faster_word_codes$=get_faster_word_codes(word_codes$,f)
            
Debug "faster_word_codes$:"+faster_word_codes$          


            ; NOSUGGEST support - 4/APR/2017 + 10/FEB/2019 + 28/FEB/2019
            If faster_word_codes$=nosuggestword$
              nosuggestword_colour=#Red
              Continue
            EndIf
            
            
            ; KEEPCASE support - 1/MAY/2019
            If faster_word_codes$=keepcaseword$
              keepcaseword_code=#True
              Continue
            EndIf            
            
            
            ; FORBIDDENWORD support - 4/APR/2017 + 10/FEB/2019 + 28/FEB/2019
            If faster_word_codes$=forbiddenword$
              forbiddenword_code=#True              
              Continue
            EndIf
            
            
            ; CIRCUMFIX support - 29/OCT/2018 + 10/FEB/2019 + 28/FEB/2019
            If faster_word_codes$=circumfix$
;               circumfix_code=#True 
               ; Remmed line above and now it sets the variables below - 19/APR/2020
              needcircumfix_code_inside=#True
              circumfix_code2=#True
              Continue
            EndIf            
            

            
            ; 17/JAN/2015 - Optimised the code the the ultrafast technique
            ; 15/MAY/2018 - Code optimised: "Y" is first searched and only if NOT found, it searches for "N"
            ; 07/MAR/2019 - Removed the space after "Y" and "N", for extra speed finding the flags
            pfx=0 ;To have one routine instead of two, because it can't be a PFX and a SFX at the same time - 31/OCT/2013: IT CAN NOW BE A PREFIX AND SUFFIX AT THE SAME TIME
            sfx=0 ;To have one routine instead of two, because it can't be a PFX and a SFX at the same time - 31/OCT/2013: IT CAN NOW BE A PREFIX AND SUFFIX AT THE SAME TIME
            position1=0 ; PFX
            position2=0 ; SFX               
            t1=FindString(ultrafast_aff_search$,"PFX "+faster_word_codes$+" Y")              
            If t1<>0 : pfx=1 : position1=rules_of_words_total_direct_memory_access(t1) : EndIf  
            t1=FindString(ultrafast_aff_search$,"SFX "+faster_word_codes$+" Y") ; Now gives priority to Y - 15/MAY/2018
            If t1=0
              t1=FindString(ultrafast_aff_search$,"SFX "+faster_word_codes$+" N")
              If t1<>0: flag_sfx_no=1 : EndIf              
            EndIf                                  
            If t1<>0 : sfx=1 : position2=rules_of_words_total_direct_memory_access(t1) : EndIf                                              
            ; Line below REMed 4/APR/2017 - Improved: If a code isn't found in the .aff it no longer exits the decoding function
;             If pfx=0 And sfx=0 : Debug "not found" : Break : EndIf ;Nothing found in the .AFF file
            ;ultrafast technique, get the real position from the ultrafast string if position1>0 or position2>0
            ; If recursivity=#False - 11/NOV/2020
            If recursivity=#False And pfx=0 And sfx=0 And faster_word_codes$<>nosuggestword$ And faster_word_codes$<>needaffixword$ And faster_word_codes$<>forbiddenword$ And faster_word_codes$<>circumfix$ And faster_word_codes$<>keepcaseword$ : missing_codes$+faster_word_codes$ : EndIf
            
            
            ; Twofold flags missing - 11/NOV/2020
            ; If recursivity=#True
            If recursivity=#True And pfx=0 And sfx=0 And faster_word_codes$<>nosuggestword$ And faster_word_codes$<>needaffixword$ And faster_word_codes$<>forbiddenword$ And faster_word_codes$<>circumfix$ And faster_word_codes$<>keepcaseword$
              If flag_char=1 Or flag_char=2
                If Right(missing_codes_sub$,Len(faster_word_codes$))<>faster_word_codes$
                  missing_codes_sub$+faster_word_codes$
                EndIf
              Else
                If Right(missing_codes_sub$,Len(faster_word_codes$))<>","+faster_word_codes$
                  missing_codes_sub$+","+faster_word_codes$
                EndIf
              EndIf
            EndIf
            
            
            If recursivity=#False
              top_level_tags$=faster_word_codes$
            EndIf
            
            If recursivity=#True
              nested_tags$=Chr(9)+faster_word_codes$
            EndIf            
            
            
            
; ADD PREFIXES HERE NEW CODE             
                   
            ; 8/DEC/2014
            ; Get the PFX codes and add them + the position to an array,
            ; for at the end combine PFX with PRIMARY+SFXs
            If pfx=1 And (needcircumfix_code_inside_up=#False And needcircumfix_code_inside_down=#False) And recursivity=#False; Brandon Fix - 6/APR/2020 + 17/APR/2020 + 1/MAY/2020
              
              
              ; 8/DEC/2014
;               Debug "total_prefixes_codes:"+Str(total_prefixes_codes)+":"+Mid(word_codes$,f,1)
              ReDim prefixes_codes$(total_prefixes_codes)              
              ReDim prefixes_codes_position(total_prefixes_codes)       
              ReDim prefixes_codes_position_in_array(total_prefixes_codes)
              prefixes_codes$(total_prefixes_codes)=faster_word_codes$
              prefixes_codes_position(total_prefixes_codes)=rules_of_words_position(position1)
              prefixes_codes_position_in_array(total_prefixes_codes)=position1
              total_prefixes_codes+1
              
            EndIf
    
            
            
            ; Get the number of lines per SFX - 31/OCT/2013
            ; Now ultra-fast - just 2 lines instead of 10 - 12/JUL/2014
            If sfx=1 ; And flag_sfx_no<=1
              
;               If flag_sfx_no=1 : flag_sfx_no=2 : EndIf
                            
              number_of_rules=rules_of_words_total(position2)
              
;  Debug "The SFX has # of meanings:"+a$
              If number_of_rules=0 : Break : EndIf ;To check for a possible zero number of conditions
              
          
              For ff=1 To number_of_rules
                
                
                ; Moved here also on 11/APR/2019
                ; Fix to window stop responding if we click on it while in the loop
                If type_of_processing_of_pfx_sfx=1 
                  event = WindowEvent()              
                  If event=#PB_Event_Timer And EventTimer()=800  : EndIf                          
              
                  If dictionary_force_refresh<>0 : ProcedureReturn 0 : EndIf
  
                EndIf              
                
                                 
                ; This is each group of text with the conditions, up to five
                ; Moved to this position on 24/AUG/2013 since the guide says that using Dim on an existing array, resets it,
                ; which is faster than reseting it with a FOR loop.
                ; Now around 7x faster with the new code - 13/JUL/2014 
                ; Now ultrafast with arrays - 27/APR/2018
                b$=rules_of_words_each$(position2,ff)
                
Debug "NUMBER OF RULES OF SUFFIX + B$:"+Str(number_of_rules)+" - "+b$+" <==================================================="
If recursivity=1 : circumfix_twofold_counter=1 : EndIf

                ; Line below detects local CIRCUMFIX flag - 20/JAN/2019
                needcircumfix_code_inside=is_circumfix(StringField(StringField(b$,4," "),2,"/"),recursivity)
                needcircumfix_code_inside=#False
Debug "KAR!!!!!!!!"+ "recursivity:"+Str(recursivity)+"  circumfix_twofold_counter:"+Str(circumfix_twofold_counter)                
                
                
                ; Lines below to detect/fix the local NEEDAFFIX flag - 17/MAR/2020
                needaffixword_code_inside=is_needaffix(b$)
                If needaffixword_code=#False : needaffixword_code=needaffixword_code_inside : EndIf

                
                ; Replaced the FOR loop using MID$ and FINDSTRING$ with STRINGFIELD for extra speed - 22/OCT/2015
                conditions_per_line_sfx$(3)=StringField(b$,3," ")
                conditions_per_line_sfx$(4)=StringField(b$,4," ")
                If conditions_per_line_sfx$(4)="0" : conditions_per_line_sfx$(4)="" : EndIf 
                ; Line below fixes two instances of 0 - 26/OCT/2018
                If Left(conditions_per_line_sfx$(4),2)="0/" : conditions_per_line_sfx$(4)=Mid(conditions_per_line_sfx$(4),2) : EndIf 
                conditions_per_line_sfx$(5)=StringField(b$,5," ")

                
; Line below fixes Brandon CIRCUMFIX - 10/APR/2020                  
; If needcircumfix_code_inside=#True And StringField(conditions_per_line_sfx$(4),2,"/")=circumfix$ And recursivity=#True : Continue : EndIf
; If  (needcircumfix_code_inside_up=#True Or needcircumfix_code_inside_down=#True) Or (StringField(conditions_per_line_sfx$(4),2,"/")=circumfix$ And circumfix$<>"") : Continue : EndIf                
     
;******************************                  
; PLACE HERE THE CODE TO ADD ITEMS TO THE LISTBOX
; EXAMPLE: SFX S y ies [^aeiou]y
;           1  2 3  4      5
; EXAMPLE: SFX p ão ões ão
;           1  2  3  4   5
;                 
;                 Debug "t2$:"+t2$ 
;                 Debug "word$:"+word$
              
                ; Add text after the word with exceptions in ^[] - 14/AUG/2013
                ;Debug t$+"->"+conditions_per_line_sfx$(5)
                If validate_sfx(t2$,conditions_per_line_sfx$(5))=#True   
                  
                  ; Tried to fix the issue of the last chr of a word appearing as a zero
                  ; 20/JAN/2019
                  zeroed$=""
                  If conditions_per_line_sfx$(3)="0" : conditions_per_line_sfx$(3)="" : zeroed$="0" : EndIf
             
                  If needcircumfix_code_inside_up=#False And needcircumfix_code_inside_down=#False ; And circumfix_twofold_counter=0 ;Or (circumfix_code=#True And circumfix_code2=#True)  ; Brandon fix - added: "(circumfix_code=#True And circumfix_code2=#True)" - 3/JUL/2019    
If needaffixword_code_inside=#False ; needaffixword_code_inside=#False - 17/MAR/2020
;                 If circumfix_code=#False

                  Select type_of_processing_of_pfx_sfx
                    Case 1                         
                       If hyphenate=#False
                        number_of_derivates_found+1
;                         SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found))
                        SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
                        colour_line_in_affixes_listicongadget+1
                        AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+#LF$+"Suffix"+#LF$+faster_word_codes$+#LF$+Str(rules_of_words_position(position2))+#LF$+Str(ff)+": "+conditions_per_line_sfx$(3)+zeroed$+" "+conditions_per_line_sfx$(4)+" "+conditions_per_line_sfx$(5))
                        SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_suffix),1)
                        add_derivate_to_sort_array(number_of_derivates_found)
                      Else  
                        AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+#LF$+faster_word_codes$)
                        colour_line_in_affixes_listicongadget+1
                        SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_suffix),1)  
                      EndIf
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")                   
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                                        
                      total_words_found+1

                      
                     Case 2 
                      If csv=#False And compounds=#False : WriteString(1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+#LF$,#PB_UTF8) : EndIf
                      If csv=#False And compounds=#True And FindString(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),"-") : WriteString(1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
                      If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+Chr(34)+"Suffix"+Chr(34)+";"+Chr(34)+faster_word_codes$+Chr(34)+";"+Chr(34)+Str(rules_of_words_position(position2))+Chr(34)+";"+Chr(34)+add_zeros(Str(ff))+": "+conditions_per_line_sfx$(3)+zeroed$+" "+conditions_per_line_sfx$(4)+ReplaceString(" "+conditions_per_line_sfx$(5)+Chr(34)," "+Chr(34),Chr(34))+#LF$,#PB_UTF8) : EndIf                          
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                       
                      If compounds=#False : total_words_found+1 : EndIf
                      
                     Case 3 
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                                             
                      total_words_found+1
                      
                     Case 4
                      ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                      total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+morphologic_part$+lines_part$
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                          
                      total_words_found+1
                      
                     Case 5 
                      If csv=#False : process_statistics(StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")) : EndIf
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                       
                      total_words_found+1
                      
                      
                     Case 6,7
                      
                      statistics_flag=#False
                      ttt_statistics$=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ttt_statistics_flag$=" Flag:"+faster_word_codes$+" (Primary:"+t2$+")"
                      If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
                      If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(t2$,1) : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf
          
                      If statistics_flag=#True And type_of_processing_of_pfx_sfx=6  : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf
                      If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                                          
                      
                    Case 100
                      procedure_returned_crunch_word$=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ProcedureReturn #True
                      
                  EndSelect
EndIf ; needaffixword_code_inside=#False - 17/MAR/2020                  
                 EndIf 
                 
Debug "READY TO CIRCUMFIX"
Debug "READY TO CIRCUMFIX - needcircumfix_code_inside:"+Str(needcircumfix_code_inside)
Debug "READY TO CIRCUMFIX - needcircumfix_code_inside_up:"+Str(needcircumfix_code_inside_up)
Debug "READY TO CIRCUMFIX - needcircumfix_code_inside_down:"+Str(needcircumfix_code_inside_down)
Debug "READY TO CIRCUMFIX - faster_word_codes$:"+faster_word_codes$
Debug "READY TO CIRCUMFIX - recursivity:"+Str(recursivity)
Debug "READY TO CIRCUMFIX - one_two_three_brandon_fix:"+Str(one_two_three_brandon_fix)
Debug "READY TO CIRCUMFIX - sfx:"+Str(sfx)
If fuk=999 ; disabled temporarily
                 If (needcircumfix_code_inside_up=#True Or needcircumfix_code_inside_down=#True) And faster_word_codes$<>circumfix$ ;And recursivity=#False And one_two_three_brandon_fix=0 and sfx=1
;                  If needcircumfix_code_inside=#True And faster_word_codes$<>circumfix$ And recursivity=#False And one_two_three_brandon_fix=0 And sfx=1
                   
                    t2$=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                    circumfix_processing$=t2$
                    circumfix_position=rules_of_words_position(position2)
                    circumfix_ff=ff
                    circumfix_conditions_per_line_both$(1)=ReplaceString(conditions_per_line_sfx$(1)," ","")
                    circumfix_conditions_per_line_both$(2)=ReplaceString(conditions_per_line_sfx$(2)," ","")
                    circumfix_conditions_per_line_both$(3)=ReplaceString(conditions_per_line_sfx$(3)," ","")+zeroed$
                    circumfix_conditions_per_line_both$(4)=ReplaceString(conditions_per_line_sfx$(4)," ","")
                    circumfix_conditions_per_line_both$(5)=ReplaceString(conditions_per_line_sfx$(5)," ","")
                    circumfix_primary$=circumfix_processing$ ; 2/JUL/2019 - Brandon Fix                    
                    one_two_three_brandon_fix=1 ; Brandon fix: if >1 it adds the words to the listicongadget - 2/JUL/2019
                    Debug "ONE - circumfix_processing$ SFX:"+circumfix_processing$
                    
                 EndIf 
EndIf                 
                 
                 
Debug  "WITH CIRCUMFIX:"+circumfix_processing$+"/"+StringField(conditions_per_line_sfx$(4),2,"/")     
Debug  "WITHOUT CIRCUMFIX:"+Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4)
Debug "circumfix_primary$:"+circumfix_primary$
Debug "circumfix_processing$:"+circumfix_processing$
Debug "t2$:"+t2$
circumfix_primary$="" ; Fixed it today, 15/MAY/2020 - 2:20pm
Debug  "WITH AND WITHOUT CIRCUMFIX"
                  ; recursivity            
                  ; Line below is new ultrafast way of checking for flags inside flags - 14/APR/2020
                  If FindString(conditions_per_line_sfx$(4),"/") And recursivity=#False
                    
                    ; Line below means that a SFX has already been added, so that it recognises "Both" as output
;                     If needcircumfix_code_inside_up=#True Or needcircumfix_code_inside_down=#True : circumfix_twofold_counter=#True : EndIf
                    circumfix_twofold_counter=#True
                    
                    process_pfx_sfx(type_of_processing_of_pfx_sfx,Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),csv,compounds,#True,languagetool$,morphologic_part$,lines_part$,hyphenate,AF_compressed_decoding_done)                    
                    circumfix_processing$=""
                    word$=word_original$
                    t2$=word_original$
                      
                    recursivity=#False
                    circumfix_twofold_counter=#False

;                     circumfix_twofold_counter=#False                        
                      
                  EndIf   
                  
                  

                EndIf
                    
                    
;******************************
                                    
                Next ff                
             EndIf                   
             
             
;******************************             
;******************************
; PLACE HERE THE PFX WITH CIRCUMFIX - 21/JAN/2019 ***START*** BELOW
             
            ; Get the number of lines per PFX - 21/JAN/2019
            ; Now ultra-fast - just 2 lines instead of 10 - 12/JUL/2014
            If pfx=1 ;And recursivity=#False ; Brandon fix - 1/MAY/2020
              
                            
              number_of_rules=rules_of_words_total(position1)
              
;  Debug "The SFX has # of meanings:"+a$
              If number_of_rules=0 : Break : EndIf ;To check for a possible zero number of conditions
                            
              
              For ff=1 To number_of_rules
                
                
                ; Moved here also on 11/APR/2019
                ; Fix to window stop responding if we click on it while in the loop
                If type_of_processing_of_pfx_sfx=1 
                  event = WindowEvent()              
                  If event=#PB_Event_Timer And EventTimer()=800  : EndIf                          
              
                  If dictionary_force_refresh<>0 : ProcedureReturn 0 : EndIf
  
                EndIf                   
                
                
                ; This is each group of text with the conditions, up to five
                ; Moved to this position on 24/AUG/2013 since the guide says that using Dim on an existing array, resets it,
                ; which is faster than reseting it with a FOR loop.
                ; Now around 7x faster with the new code - 13/JUL/2014 
                ; Now ultrafast with arrays - 27/APR/2018
                b$=rules_of_words_each$(position1,ff)
                
Debug "NUMBER OF RULES OF PREFIX + B$:"+Str(number_of_rules)+" - "+b$+" <==================================================="     
If recursivity=1 : circumfix_twofold_counter=1 : EndIf

                ; Line below detects local CIRCUMFIX flag - 20/JAN/2019
                needcircumfix_code_inside=is_circumfix(StringField(StringField(b$,4," "),2,"/"),recursivity)
                needcircumfix_code_inside=#False


;                 needaffixword_code=is_needaffix(b$) ; Detect local NEEDAFFIX flag - 26/SEP/2019
                ; Lines below to detect/fix the local NEEDAFFIX flag - 17/MAR/2020
;                 If recursivity=#False : needaffixword_code_inside=is_needaffix(b$) : EndIf
                needaffixword_code_inside=is_needaffix(b$)
                If needaffixword_code=#False : needaffixword_code=needaffixword_code_inside : EndIf                
                
; Line below removed in Brandon example, since it didn't decode the "bark/abe" - Added again - fix must be somewhere here
;                 If circumfix_code=#False : Break : EndIf ; No CIRCUMFIX flag in Prefixes
; If needcircumfix_code_inside_up=#False And needcircumfix_code_inside_down=#False : Continue : EndIf ; No CIRCUMFIX flag in Prefixes ; Brandon fix - 3/JUL/2019 + 16/APR/2020             
; Line above replaced Break with Continue - 21/APR/2020

                  ; Removed the two lines below because they aren't used in practice                                  
                  ; Replaced the FOR loop using MID$ and FINDSTRING$ with STRINGFIELD for extra speed - 22/OCT/2015
                  conditions_per_line_pfx$(3)=StringField(b$,3," ")
                  conditions_per_line_pfx$(4)=StringField(b$,4," ")
                  If conditions_per_line_pfx$(4)="0" : conditions_per_line_pfx$(4)="" : EndIf 
                  ; Line below fixes two instances of 0 - 26/OCT/2018
                  If Left(conditions_per_line_pfx$(4),2)="0/" : conditions_per_line_pfx$(4)=Mid(conditions_per_line_pfx$(4),2) : EndIf 
                  conditions_per_line_pfx$(5)=StringField(b$,5," ")
                  
                  
; Line below fixes Brandon CIRCUMFIX - 10/APR/2020                  
; If needcircumfix_code_inside=#True And StringField(conditions_per_line_pfx$(4),2,"/")=circumfix$  And recursivity=#False: Continue : EndIf
; Remmed line above - 21/APR/2020


;******************************                  
; PLACE HERE THE CODE TO ADD ITEMS TO THE LISTBOX
; EXAMPLE: SFX S y ies [^aeiou]y
;           1  2 3  4      5
; EXAMPLE: SFX p ão ões ão
;           1  2  3  4   5
;
                  
; t2$ used to appear in this position in the former code (before CIRCUMFIX)
                  
                  
Debug "CIRCUMFIX TWO - INSIDE"                                 
Debug "CIRCUMFIX TWO - t2$:"+t2$
Debug "CIRCUMFIX TWO - circumfix_primary$:"+circumfix_primary$
Debug "CIRCUMFIX TWO - circumfix_twofold_counter:"+Str(circumfix_twofold_counter)
Debug "CIRCUMFIX TWO - INSIDE needcircumfix_code_inside_up:"+Str(needcircumfix_code_inside_up)
Debug "CIRCUMFIX TWO - INSIDE needcircumfix_code_inside_down:"+Str(needcircumfix_code_inside_down)


; Remmed line below on 15/MAY/2020
;                   t2$=circumfix_primary$ ; Brandon fix - 2/JUL/2019

                  If validate_pfx(t2$,conditions_per_line_pfx$(5))=#True And circumfix_twofold_counter=#True And needcircumfix_code_inside_up=needcircumfix_code_inside_down
Debug "CIRCUMFIX TWO - INSIDE THE VALIDATE PFX"                     
Brandon_Fix_circumfix_both_t2$=t2$
circumfix_twofold_counter=#False
                    
                    If conditions_per_line_pfx$(3)<>"0"
                      
                        t2$=StringField(conditions_per_line_pfx$(4)+Right(circumfix_primary$,Len(circumfix_primary$)-Len(conditions_per_line_pfx$(3))),1,"/")+Mid(circumfix_primary$,Len(conditions_per_line_pfx$(3))+1,Len(circumfix_primary$)) ;+t2$  ; 2/JUL/2019 - Brandon Fix
;                         t2$=StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+circumfix_primary$ ;+t2$  ; 2/JUL/2019 - Brandon Fix 
                        circumfix_processing$=t2$  
                        circumfix_position=rules_of_words_position(position1)
                        circumfix_ff=ff
                        circumfix_conditions_per_line_both$(1)=ReplaceString(conditions_per_line_pfx$(1)," ","")
                        circumfix_conditions_per_line_both$(2)=ReplaceString(conditions_per_line_pfx$(2)," ","")
                        circumfix_conditions_per_line_both$(3)=ReplaceString(conditions_per_line_pfx$(3)," ","")
                        circumfix_conditions_per_line_both$(4)=ReplaceString(conditions_per_line_pfx$(4)," ","")
                        circumfix_conditions_per_line_both$(5)=ReplaceString(conditions_per_line_pfx$(5)," ","")                 
                        one_two_three_brandon_fix=2 ; Brandon fix: if >1 it adds the words to the listicongadget - 2/JUL/2019
                        Debug "TWO - circumfix_processing$:"+circumfix_processing$
                     
                    Else                        
                      
; Two lines below to attempt to fix Brandon's issue - 26/MAR/2020                      
circumfix_primary_fix_brandon_20200326$=circumfix_primary$                      
If needcircumfix_code_inside=#False : circumfix_primary_fix_brandon_20200326$="" : EndIf                         
                      conditions_per_line_pfx$(3)=""
                      zeroed$="0"
                      
                      ; Line below to attempt to fix Brandon's issue - 26/MAR/2020
                      t2$=StringField(conditions_per_line_pfx$(4)+circumfix_primary$,1,"/")+circumfix_primary_fix_brandon_20200326$
                        ; Rem line below remmed to fix Brandon's issue - 26/MAR/2020
;                         t2$=StringField(conditions_per_line_pfx$(4)+circumfix_primary$,1,"/")+circumfix_primary$ ;+t2$  ; 2/JUL/2019 - Brandon Fix 
                        circumfix_processing$=t2$
                        circumfix_position=rules_of_words_position(position1)
                        circumfix_ff=ff
                        circumfix_conditions_per_line_both$(1)=ReplaceString(conditions_per_line_pfx$(1)," ","")
                        circumfix_conditions_per_line_both$(2)=ReplaceString(conditions_per_line_pfx$(2)," ","")
                        circumfix_conditions_per_line_both$(3)=ReplaceString(conditions_per_line_pfx$(3)," ","")+zeroed$
                        circumfix_conditions_per_line_both$(4)=ReplaceString(conditions_per_line_pfx$(4)," ","")
                        circumfix_conditions_per_line_both$(5)=ReplaceString(conditions_per_line_pfx$(5)," ","")
                        one_two_three_brandon_fix=2 ; Brandon fix: if >1 it adds the words to the listicongadget - 2/JUL/2019
                        Debug "THREE - circumfix_processing$:"+circumfix_processing$
                        
                        
                      
                    EndIf
                    

If needaffixword_code_inside=#False ; needaffixword_code_inside=#False - 17/MAR/2020

                  Select type_of_processing_of_pfx_sfx
                     Case 1   
                       If hyphenate=#False
                        number_of_derivates_found+1
;                         SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found)) 
                        SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
                        colour_line_in_affixes_listicongadget+1
                        AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,circumfix_processing$+Brandon_Fix_circumfix_both_t2$+#LF$+"Both"+#LF$+faster_word_codes$+#LF$+Str(circumfix_position)+#LF$+Str(circumfix_ff)+": "+circumfix_conditions_per_line_both$(3)+" "+circumfix_conditions_per_line_both$(4)+" "+circumfix_conditions_per_line_both$(5))
                        SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_both),1)
                        add_derivate_to_sort_array(number_of_derivates_found)
                      Else  
                        AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,circumfix_processing$+Brandon_Fix_circumfix_both_t2$+#LF$+faster_word_codes$)
                        colour_line_in_affixes_listicongadget+1
                        SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_both),1)  
                      EndIf
                      total_words_found+1

                      
                     Case 2 
                      If csv=#False And compounds=#False : WriteString(1,circumfix_processing$+Brandon_Fix_circumfix_both_t2$+#LF$,#PB_UTF8) : EndIf
                      If csv=#False And compounds=#True And FindString(circumfix_processing$+Brandon_Fix_circumfix_both_t2$,"-") : WriteString(1,circumfix_processing$+Brandon_Fix_circumfix_both_t2$+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
                      If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(circumfix_processing$+Brandon_Fix_circumfix_both_t2$,1,"/")+Chr(34)+";"+Chr(34)+circumfix_primary$+Chr(34)+";"+Chr(34)+"Both"+Chr(34)+";"+Chr(34)+faster_word_codes$+Chr(34)+";"+Chr(34)+Str(circumfix_position)+Chr(34)+";"+Chr(34)+add_zeros(Str(circumfix_ff))+": "+circumfix_conditions_per_line_both$(3)+" "+circumfix_conditions_per_line_both$(4)+ReplaceString(" "+circumfix_conditions_per_line_both$(5)+Chr(34)," "+Chr(34),Chr(34))+#LF$,#PB_UTF8) : EndIf                          
                      If compounds=#False : total_words_found+1 : EndIf
                      
                     Case 3 
                      total_words_found+1
                      
                     Case 4
                      total_words_found+1
                      
                     Case 5 
                      If csv=#False : process_statistics(circumfix_processing$+Brandon_Fix_circumfix_both_t2$) : EndIf
                      total_words_found+1
                      
                     Case 6,7
                       
                      statistics_flag=#False
                      ttt_statistics$=circumfix_processing$+Brandon_Fix_circumfix_both_t2$
                      ttt_statistics_flag$=" Flag:"+faster_word_codes$+" (Primary:"+t2$+")"
                      If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
                      If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(t2$,1) : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf
          
                      If statistics_flag=#True And type_of_processing_of_pfx_sfx=6 : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf                       
                      If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf
                      
                  EndSelect
                  
                  
                  
;                   circumfix_processing$=""
;                   word$=word_original$
;                   t2$=word_original$
;                   If recursivity=#True : ProcedureReturn 0 : EndIf
                  
                  
EndIf ; needaffixword_code_inside=#False - 17/MAR/2020 

circumfix_twofold_counter=#False
Debug "RIGHT AFTER THE BOTH *START*"
Debug "t2$:"+t2$
Debug "word_original$:"+word_original$
Debug "RIGHT AFTER THE BOTH *END*"


                    ; Line below fixes the triple processing of Prefixes in CIRCUMFIX - Brandon fix - 10/APR/2020
;                     If needcircumfix_code_inside=#True : t2$=word_original$ : EndIf                    
;                     t2$=word_original$ ; Taken from the line above - 15/MAY/2020
t2$=Brandon_Fix_circumfix_both_t2$ ; The fix? Brandon - 15/MAY/2020

                    ; recursivity            
                    ; Line below is new ultrafast way of checking for flags inside flags - 14/APR/2020
                    If FindString(conditions_per_line_sfx$(4),"/") And recursivity=#False
                      
                      ; Line below fixes the triple processing of Prefixes in CIRCUMFIX - Brandon fix + Mauro fix (rule that calls resets circumfix flag inside) - 10/APR/2020
;                       If needcircumfix_code_inside=#True : circumfix_twofold_counter=#True : EndIf  
                      circumfix_twofold_counter=#True

                      
                      process_pfx_sfx(type_of_processing_of_pfx_sfx,circumfix_processing$+"/"+StringField(conditions_per_line_sfx$(4),2,"/"),csv,compounds,#True,languagetool$,morphologic_part$,lines_part$,hyphenate,AF_compressed_decoding_done)                    
                      circumfix_processing$=""
                      word$=word_original$
                      t2$=word_original$
                      recursivity=#False
                      circumfix_twofold_counter=#False

                      ; Line below fixes the triple processing of Prefixes in CIRCUMFIX - Brandon fix + Mauro fix (rule that calls resets circumfix flag inside) - 10/APR/2020
;                       circumfix_twofold_counter=#False ; With recursivity=#False
                      
                    EndIf  
                    
Debug "TRYING TO FIX THE FU *START*"  
Debug "circumfix_processing$:"+circumfix_processing$
Debug "word$:"+word$
Debug "t2$:"+t2$
Debug "word_original$:"+word_original$
; circumfix_processing$=""            
; word$=""
; t2$=""
; word_original$=""
Debug "TRYING TO FIX THE FU *END*"
                EndIf
                 
;******************************
                                    
;                 Next ff                
;              EndIf                   
         
;******************************           
;******************************
; PLACE HERE THE PFX WITH CIRCUMFIX - 21/JAN/2019 ***END*** ABOVE     
             
          
        ; CIRCUMFIX  
If fuk=999
          If (needcircumfix_code_inside=#True Or circumfix_code2=#True) And faster_word_codes$<>circumfix$ And circumfix_processing$<>"" And (StringField(circumfix_conditions_per_line_both$(4),2,"/")<>circumfix$ And  circumfix$<>"")  And one_two_three_brandon_fix>1
one_two_three_brandon_fix=0 ; Brandon fix: if >1 it adds the words to the listicongadget - 2/JUL/2019  
If needaffixword_code_inside=#False ; needaffixword_code_inside=#False - 17/MAR/2020

                  Select type_of_processing_of_pfx_sfx
                     Case 1   
                       If hyphenate=#False
                        number_of_derivates_found+1
;                         SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found)) 
                        SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
                        colour_line_in_affixes_listicongadget+1
                        AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,circumfix_processing$+#LF$+"Both"+#LF$+faster_word_codes$+#LF$+Str(circumfix_position)+#LF$+Str(circumfix_ff)+": "+circumfix_conditions_per_line_both$(3)+" "+circumfix_conditions_per_line_both$(4)+" "+circumfix_conditions_per_line_both$(5))
                        SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_both),1)
                        add_derivate_to_sort_array(number_of_derivates_found)
                      Else  
                        AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,circumfix_processing$+#LF$+faster_word_codes$)
                        colour_line_in_affixes_listicongadget+1
                        SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_both),1)  
                      EndIf
                      total_words_found+1

                      
                     Case 2 
                      If csv=#False And compounds=#False : WriteString(1,circumfix_processing$+#LF$,#PB_UTF8) : EndIf
                      If csv=#False And compounds=#True And FindString(circumfix_processing$,"-") : WriteString(1,circumfix_processing$+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
                      If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(circumfix_processing$,1,"/")+Chr(34)+";"+Chr(34)+circumfix_primary$+Chr(34)+";"+Chr(34)+"Both"+Chr(34)+";"+Chr(34)+faster_word_codes$+Chr(34)+";"+Chr(34)+Str(circumfix_position)+Chr(34)+";"+Chr(34)+add_zeros(Str(circumfix_ff))+": "+circumfix_conditions_per_line_both$(3)+" "+circumfix_conditions_per_line_both$(4)+ReplaceString(" "+circumfix_conditions_per_line_both$(5)+Chr(34)," "+Chr(34),Chr(34))+#LF$,#PB_UTF8) : EndIf                          
                      If compounds=#False : total_words_found+1 : EndIf
                      
                     Case 3 
                      total_words_found+1
                      
                     Case 4
                      total_words_found+1
                      
                     Case 5 
                      If csv=#False : process_statistics(circumfix_processing$) : EndIf
                      total_words_found+1
                      
                     Case 6,7
                       
                      statistics_flag=#False
                      ttt_statistics$=circumfix_processing$
                      ttt_statistics_flag$=" Flag:"+faster_word_codes$+" (Primary:"+t2$+")"
                      If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
                      If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(t2$,1) : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
                      If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf
          
                      If statistics_flag=#True And type_of_processing_of_pfx_sfx=6 : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf                       
                      If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf
                      
                  EndSelect
                  
;                   circumfix_processing$=""
;                   word$=word_original$
;                   t2$=word_original$
;                   If recursivity=#True : ProcedureReturn 0 : EndIf
                  
                  
EndIf ; needaffixword_code_inside=#False - 17/MAR/2020 
EndIf            
EndIf ; FUK=9999




                Next ff                
                
                
             EndIf  
             
             
;              If recursivity=#True : ProcedureReturn 0 : EndIf ; Brandon Fix - 2/JUL/2019
             
              Next f
              
              If recursivity=#True : ProcedureReturn 0 : EndIf ; Brandon Fix - 3/JUL/2019
              
            EndIf

          
          
; ADD PREFIXES HERE NEW CODE             
            

          ; Get the number of lines per PFX - 31/OCT/2013
          ; Now ultra-fast - just 2 lines instead of 10 - 12/JUL/2014
          ; 8/DEC/2014 - New code to combine PFX with PRIMARY+SFXs
          ; 11/MAR/2019 - Fixed decoding of prefixes using twofold if there were flags of suffixes after flags of prefixes
; If circumfix_code=#True Or circumfix_code2=#True : total_prefixes_codes=0 : EndIf ; Brandon fix- 3/JUL/2019
; If (circumfix_code=#False And circumfix_code2=#False) Or (circumfix_code=#True And circumfix_code2=#True)  ; Brandon fix - added: "(circumfix_code=#True And circumfix_code2=#True)" - 3/JUL/2019


          If total_prefixes_codes>0 And total_primary_suffixes>0 And recursivity=#False

            For f=0 To ArraySize(prefixes_codes_position()) ;total_prefixes_codes-1 ; 16/APR/2017 - <- Fix "twofold_Mauro 5.dic" <- Prefixes
              For f_primary_sfx=0 To total_primary_suffixes-1
                
                
                number_of_rules=rules_of_words_total(prefixes_codes_position_in_array(f))
;  Debug "The PFX has # of meanings:"+a$
                If number_of_rules=0 : Break : EndIf ;To check for a possible zero number of conditions
 
             
                For ff=1 To number_of_rules
                  
                  
                  ; Moved here also on 11/APR/2019
                  ; Fix to window stop responding if we click on it while in the loop
                  If type_of_processing_of_pfx_sfx=1 
                    event = WindowEvent()              
                    If event=#PB_Event_Timer And EventTimer()=800  : EndIf                          
              
                    If dictionary_force_refresh<>0 : ProcedureReturn 0 : EndIf
  
                  EndIf                   
                  
                  
                  ; This is each group of text with the conditions, up to five
                  ; Moved to this position on 24/AUG/2013 since the guide says that using Dim on an existing array, resets it,
                  ; which is faster than reseting it with a FOR loop.
                  ; Now around 7x faster with the new code - 13/JUL/2014
                  ; Now ultrafast with arrays - 27/APR/2018
                  b$=rules_of_words_each$(prefixes_codes_position_in_array(f),ff)
     
                  
                  ; Replaced the FOR loop using MID$ and FINDSTRING$ with STRINGFIELD for extra speed - 22/OCT/2015
                  conditions_per_line_pfx$(3)=StringField(b$,3," ")
                  conditions_per_line_pfx$(4)=StringField(b$,4," ")
                  If conditions_per_line_pfx$(4)="0" : conditions_per_line_pfx$(4)="" : EndIf 
                  ; Line below fixes two instances of 0 - 26/OCT/2018
                  If Left(conditions_per_line_pfx$(4),2)="0/" : conditions_per_line_pfx$(4)=Mid(conditions_per_line_pfx$(4),2) : EndIf 
                  conditions_per_line_pfx$(5)=StringField(b$,5," ")

                  ; Brandon fix for "Prefix3" and "Prefix4"- 11/MAY/2020
                  conditions_per_line_pfx3_brandon_fix$=conditions_per_line_pfx$(3)
                 

;******************************                  
; PLACE HERE THE CODE TO ADD ITEMS TO THE LISTBOX
; EXAMPLE: SFX S y ies [^aeiou]y
;           1  2 3  4      5
; EXAMPLE: SFX p ão ões ão
;           1  2  3  4   5
;
                  ; 8/DEC/2014
                  ; Get the primary+suffixes words from the stored array
                  t2$=primary_suffixes$(f_primary_sfx)
         
                  
                  ; Add text after the word with exceptions in ^[] - 17/AUG/2013
                  ;Debug t$+"->"+conditions_per_line_pfx$(5)
                 If primary_suffixes_sfx_N(f_primary_sfx)=0 ; 10/MAY/2018 - To comply with SFX N
                  If validate_pfx(t2$,conditions_per_line_pfx$(5))=#True


;                     If recursivity=#True : activate_circumfix_prefix=#True : EndIf ; Brandon fix - 19/APR/2020
                    
                    If conditions_per_line_pfx$(3)<>"0"
                      
                      ; Brandon fix - 10/APR/2020 - Below remmed and placed another line below it
;                       If (circumfix_code=#False And circumfix_code2=#False)  Or (circumfix_code=#True And circumfix_code2=#True And FindString(conditions_per_line_pfx$(4),"/")=0)  ; Brandon fix - The finding of / cancels the derivates- 3/JUL/2019
                      If needcircumfix_code_inside=#False And circumfix_code2=#False
                      
If needaffixword_code_inside=#False ; needaffixword_code_inside=#False - 17/MAR/2020
                        
                      Select type_of_processing_of_pfx_sfx
                       Case 1                      
                        If hyphenate=#False 
                          number_of_derivates_found+1
;                           SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found))
                          SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
                          colour_line_in_affixes_listicongadget+1
                          AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+#LF$+"Prefix"+#LF$+prefixes_codes$(f)+#LF$+Str(prefixes_codes_position(f))+#LF$+Str(ff)+": "+conditions_per_line_pfx$(3)+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5))
                          SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)
                          add_derivate_to_sort_array(number_of_derivates_found)
                        Else
                          AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+#LF$+prefixes_codes$(f))
                          colour_line_in_affixes_listicongadget+1
                          SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)
                        EndIf
                        total_words_found+1
                      
                       Case 2 
                        If csv=#False And compounds=#False: WriteString(1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+#LF$,#PB_UTF8) : EndIf
                        If csv=#False And compounds=#True And FindString(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),"-") : WriteString(1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
                        If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+"Prefix"+Chr(34)+";"+Chr(34)+prefixes_codes$(f)+Chr(34)+";"+Chr(34)+Str(prefixes_codes_position(f))+Chr(34)+";"+Chr(34)+add_zeros(Str(ff))+Chr(34)+";"+Chr(34)+": "+conditions_per_line_pfx$(3)+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5)+Chr(34)+#LF$,#PB_UTF8) : EndIf                          
                        If compounds=#False : total_words_found+1 : EndIf
                        
                       Case 3                       
                        total_words_found+1
                        
                       Case 4 
                        ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                        total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+morphologic_part$+lines_part$
                        total_words_found+1
                        
                       Case 5
                        If csv=#False : process_statistics(StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")) : EndIf
                        total_words_found+1
                        
                       Case 6,7
                         
                        statistics_flag=#False
                        ttt_statistics$=StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")
                        ttt_statistics_flag$=" Flag:"+faster_word_codes$+" (Primary:"+t2$+")"
                        If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
                        If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(t2$,1) : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf
          
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=6 : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf                         
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf
                        
                      EndSelect
                      
EndIf ; needaffixword_code_inside=#False - 17/MAR/2020
                      
                     EndIf
                     If circumfix_code=#True And fuk=999 ; strange why it got activated before the fuk=999 - 21/JAN/2019
                        t2$=StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+t2$
                        circumfix_processing$=t2$   
;                         total_prefixes_codes=0
                        Debug "TWO TWO"
                     EndIf
                     
                    Else                       
                      
                      conditions_per_line_pfx$(3)=""
                      
                     ; Brandon fix - 10/APR/2020 - Below remmed and placed another line below it 
;                      If (circumfix_code=#False And circumfix_code2=#False)  Or (circumfix_code=#True And circumfix_code2=#True And FindString(conditions_per_line_pfx$(4),"/")=0)  ; Brandon fix - The finding of / cancels the derivates- 3/JUL/2019
                     If needcircumfix_code_inside=#False And circumfix_code2=#False
 
If needaffixword_code_inside=#False ; needaffixword_code_inside=#False - 17/MAR/2020
                       
                      Select type_of_processing_of_pfx_sfx
                       Case 1                      
                        If hyphenate=#False 
                          number_of_derivates_found+1
;                           SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found))
                          SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
                          colour_line_in_affixes_listicongadget+1
                          AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+#LF$+"Prefix"+#LF$+prefixes_codes$(f)+#LF$+Str(prefixes_codes_position(f))+#LF$+Str(ff)+": "+"0"+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5))
                          SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)    
                          add_derivate_to_sort_array(number_of_derivates_found)
                        Else
                          AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+#LF$+prefixes_codes$(f))
                          colour_line_in_affixes_listicongadget+1
                          SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)    
                        EndIf
                        total_words_found+1
                        
                       Case 2
                        If csv=#False And compounds=#False : WriteString(1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+#LF$,#PB_UTF8) : EndIf
                        If csv=#False And compounds=#True And FindString(conditions_per_line_pfx$(4)+t2$,"-") : WriteString(1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
                        If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+Chr(34)+"Prefix"+Chr(34)+";"+Chr(34)+prefixes_codes$(f)+Chr(34)+";"+Chr(34)+Str(prefixes_codes_position(f))+Chr(34)+";"+Chr(34)+add_zeros(Str(ff))+": "+"0"+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5)+Chr(34)+#LF$,#PB_UTF8) : EndIf
                        If compounds=#False : total_words_found+1 : EndIf
                        
                       Case 3 
                        total_words_found+1
                        
                       Case 4                       
                        ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                        total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+morphologic_part$+lines_part$
                        total_words_found+1
                        
                       Case 5                      
                        If csv=#False : process_statistics(StringField(conditions_per_line_pfx$(4)+t2$,1,"/")) : EndIf
                        total_words_found+1
                        
                       Case 6,7

                        statistics_flag=#False
                        ttt_statistics$=StringField(conditions_per_line_pfx$(4)+t2$,1,"/")
                        ttt_statistics_flag$=" Flag:"+faster_word_codes$+" (Primary:"+t2$+")"
                        If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
                        If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(t2$,1) : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf
          
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=6 : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf 
                        
                      EndSelect
                      
EndIf ; needaffixword_code_inside=#False - 17/MAR/2020
                      
                     EndIf 
                     If circumfix_code=#True And fuk=999 ; strange why it got activated before the fuk=999 - 21/JAN/2019
                        t2$=StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+t2$
                        circumfix_processing$=t2$
;                         total_prefixes_codes=0
                        Debug "THREE THREE"
                     EndIf                     
                     
                      
                    EndIf
                   
                    
                    ; 1/APR/2017 - <- Fix "twofold_Mauro 4.dic" <- Prefixes
                    ; 16/APR/2017 - <- Fix "twofold_Mauro 5.dic" <- Prefixes - (nothing changed here, but above)
                    If recursivity=#True
                        If total_prefixes_codes>0 : total_prefixes_codes-1 : EndIf                                            
                    EndIf              
        
                    ; recursivity          MOVED TO END

                    conditions_per_line_pfx3$=conditions_per_line_pfx3_brandon_fix$ ; =conditions_per_line_pfx$(3)
                    conditions_per_line_pfx4$=conditions_per_line_pfx$(4)
                    conditions_per_line_pfx5$=conditions_per_line_pfx$(5)
                    activate_circumfix_prefix_word$=StringField(conditions_per_line_pfx$(4)+"/",1,"/")+Right(t2$,Len(t2$)+1-Len(conditions_per_line_pfx3$))
                    activate_circumfix_prefix_t2$=t2$                    
                    
                    
                    
Debug "TRYING TO FIX THE PREFIX4 DECODING *START*"
Debug "pfx:"+Str(pfx)
Debug "conditions_per_line_pfx3$:"+conditions_per_line_pfx3$
Debug "conditions_per_line_pfx4$:"+conditions_per_line_pfx4$
Debug "activate_circumfix_prefix_word$:"+activate_circumfix_prefix_word$
Debug "activate_circumfix_prefix_t2$:"+activate_circumfix_prefix_t2$
Debug "***"
Debug "NOT FIND STRING: conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-1-Len(conditions_per_line_pfx3$)):"+conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-1-Len(conditions_per_line_pfx3$))
Debug "conditions_per_line_pfx4$+word_original$:"+conditions_per_line_pfx4$+word_original$+"="+"activate_circumfix_prefix_word$:"+activate_circumfix_prefix_word$
Debug "TRYING TO FIX THE PREFIX4 DECODING *END*"
                    
                    

                    ; ************** PLACE HERE THE CIRCUMFIX PREFIX CODE ***START*** - 19/APR/2020
; remmed below on 26/APR/2020 to try to find a better condition
; If recursivity=#False And total_prefixes_codes=1 And (circumfix_code=#True Or needcircumfix_code_inside=#True) And Not FindString(conditions_per_line_pfx4$,"/") And Not FindString(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),"/") And activate_circumfix_prefix=#True
; If  pfx=1  And Not FindString(conditions_per_line_pfx4$,"/") And Not FindString(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-1-Len(conditions_per_line_pfx3$)),"/") And conditions_per_line_pfx4$+word_original$=activate_circumfix_prefix_word$
If Not FindString(conditions_per_line_pfx4$,"/") And conditions_per_line_pfx4$+word_original$=activate_circumfix_prefix_word$
Debug "I AM INSIDE THE PFX3 AND PFX4 * *  * * * *"
                    If conditions_per_line_pfx3$<>"0"
                     
If needaffixword_code_inside=#False ; needaffixword_code_inside=#False - 17/MAR/2020
                        
                      Select type_of_processing_of_pfx_sfx
                       Case 1                      
                        If hyphenate=#False 
                          number_of_derivates_found+1
;                           SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found))
                          SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
                          colour_line_in_affixes_listicongadget+1
                          AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")+#LF$+"Prefix"+#LF$+prefixes_codes$(f)+#LF$+Str(prefixes_codes_position(f))+#LF$+Str(ff)+": "+conditions_per_line_pfx3$+" "+conditions_per_line_pfx4$+" "+conditions_per_line_pfx5$)
                          SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)
                          add_derivate_to_sort_array(number_of_derivates_found)
                        Else
                          AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")+#LF$+prefixes_codes$(f))
                          colour_line_in_affixes_listicongadget+1
                          SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)
                        EndIf
                        total_words_found+1
                        
                       Case 2 
                        If csv=#False And compounds=#False: WriteString(1,StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")+#LF$,#PB_UTF8) : EndIf
                        If csv=#False And compounds=#True And FindString(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),"-") : WriteString(1,StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
                        If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")+Chr(34)+";"+Chr(34)+activate_circumfix_prefix_t2$+Chr(34)+";"+"Prefix"+Chr(34)+";"+Chr(34)+prefixes_codes$(f)+Chr(34)+";"+Chr(34)+Str(prefixes_codes_position(f))+Chr(34)+";"+Chr(34)+add_zeros(Str(ff))+Chr(34)+";"+Chr(34)+": "+conditions_per_line_pfx3$+" "+conditions_per_line_pfx4$+" "+conditions_per_line_pfx5$+Chr(34)+#LF$,#PB_UTF8) : EndIf                          
                        If compounds=#False : total_words_found+1 : EndIf
                        
                       Case 3                       
                        total_words_found+1
                        
                       Case 4 
                        ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                        total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")+morphologic_part$+lines_part$
                        total_words_found+1
                        
                       Case 5
                        If csv=#False : process_statistics(StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")) : EndIf
                        total_words_found+1
                        
                       Case 6,7
                         
                        statistics_flag=#False
                        ttt_statistics$=StringField(conditions_per_line_pfx4$+Right(activate_circumfix_prefix_t2$,Len(activate_circumfix_prefix_t2$)-Len(conditions_per_line_pfx3$)),1,"/")
                        ttt_statistics_flag$=" Flag:"+faster_word_codes$+" (Primary:"+activate_circumfix_prefix_t2$+")"
                        If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
                        If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(activate_circumfix_prefix_t2$,1) : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf
          
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=6 : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf                         
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf
                        
                      EndSelect
                      
EndIf ; needaffixword_code_inside=#False - 17/MAR/2020
                      
                     
                    Else                        
                      conditions_per_line_pfx3$=""
                      
If needaffixword_code_inside=#False ; needaffixword_code_inside=#False - 17/MAR/2020
                       
                      Select type_of_processing_of_pfx_sfx
                       Case 1                      
                        If hyphenate=#False 
                          number_of_derivates_found+1
;                           SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(number_of_derivates_found))
                          SetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,-1,"Derived words ("+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS))+")",0)
                          colour_line_in_affixes_listicongadget+1
                          AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")+#LF$+"Prefix"+#LF$+prefixes_codes$(f)+#LF$+Str(prefixes_codes_position(f))+#LF$+Str(ff)+": "+"0"+" "+conditions_per_line_pfx4$+" "+conditions_per_line_pfx5$)
                          SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)    
                          add_derivate_to_sort_array(number_of_derivates_found)
                        Else
                          AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")+#LF$+prefixes_codes$(f))
                          colour_line_in_affixes_listicongadget+1
                          SetGadgetItemColor(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)    
                        EndIf
                        total_words_found+1
                        
                       Case 2
                        If csv=#False And compounds=#False : WriteString(1,StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")+#LF$,#PB_UTF8) : EndIf
                        If csv=#False And compounds=#True And FindString(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,"-") : WriteString(1,StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")+languagetool$+#LF$,#PB_UTF8) : total_words_found+1 : EndIf
                        If csv=#True : WriteString(1,Chr(34)+root_word$+Chr(34)+";"+Chr(34)+StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")+Chr(34)+";"+Chr(34)+activate_circumfix_prefix_t2$+Chr(34)+";"+Chr(34)+"Prefix"+Chr(34)+";"+Chr(34)+prefixes_codes$(f)+Chr(34)+";"+Chr(34)+Str(prefixes_codes_position(f))+Chr(34)+";"+Chr(34)+add_zeros(Str(ff))+": "+"0"+" "+conditions_per_line_pfx4$+" "+conditions_per_line_pfx5$+Chr(34)+#LF$,#PB_UTF8) : EndIf
                        If compounds=#False : total_words_found+1 : EndIf
                        
                       Case 3 
                        total_words_found+1
                        
                       Case 4                       
                        ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                        total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")+morphologic_part$+lines_part$
                        total_words_found+1
                        
                       Case 5                      
                        If csv=#False : process_statistics(StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")) : EndIf
                        total_words_found+1
                        
                       Case 6,7

                        statistics_flag=#False
                        ttt_statistics$=StringField(conditions_per_line_pfx4$+activate_circumfix_prefix_t2$,1,"/")
                        ttt_statistics_flag$=" Flag:"+faster_word_codes$+" (Primary:"+activate_circumfix_prefix_t2$+")"
                        If type_of_word_selected_statistics_extract_with_flags=#False : ttt_statistics_flag$="" : EndIf
                        If type_of_word_selected_statistics=0 And FindString(ttt_statistics$,"'")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=1 And FindString(ttt_statistics$,"-")>0 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=2 And UCase(Left(ttt_statistics$,1))=Left(activate_circumfix_prefix_t2$,1) : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=3 And Len(ttt_statistics$)=1 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=4 And Len(ttt_statistics$)=2 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=5 And Len(ttt_statistics$)=3 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=6 And Len(ttt_statistics$)=4 : statistics_flag=#True : EndIf
                        If type_of_word_selected_statistics=7 And Len(ttt_statistics$)=5 : statistics_flag=#True : EndIf
          
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=6 : AddLineEditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,ttt_statistics$+ttt_statistics_flag$) : EndIf
                        If statistics_flag=#True And type_of_processing_of_pfx_sfx=7 : WriteString(1,ttt_statistics$+ttt_statistics_flag$+#LF$,#PB_UTF8) : EndIf 
                        
                      EndSelect
                      
EndIf ; needaffixword_code_inside=#False - 17/MAR/2020
                      

                 
                    EndIf

EndIf
                    ; ************** PLACE HERE THE CIRCUMFIX PREFIX CODE ***END*** - 19/APR/2020

                    
                    ; recursivity          ; MOVED TO END

                    ; Line below is new ultrafast way of checking for flags inside flags - 14/APR/2020 + 15/APR/2020
                    If FindString(conditions_per_line_pfx$(4),"/") And recursivity=#False ; Not (circumfix_code=#True Or needcircumfix_code_inside=#True) And recursivity=#False

;                       conditions_per_line_pfx3$=conditions_per_line_pfx$(3)
;                       conditions_per_line_pfx4$=conditions_per_line_pfx$(4)
;                       conditions_per_line_pfx5$=conditions_per_line_pfx$(5)
;                       activate_circumfix_prefix_word$=conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3)))
;                       activate_circumfix_prefix_t2$=t2$
                      
                      ; Line below fixes the triple processing of Prefixes in CIRCUMFIX - Brandon fix + Mauro fix (rule that calls resets circumfix flag inside) - 10/APR/2020
;                       If needcircumfix_code_inside=#True : circumfix_twofold_counter=#True : EndIf 
                      circumfix_twofold_counter=#True
                      
                      ; Brandon fix - 11/MAY/2020
                      conditions_per_line_pfx$(3)=conditions_per_line_pfx3_brandon_fix$

                      process_pfx_sfx(type_of_processing_of_pfx_sfx,conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))-1),csv,compounds,#True,languagetool$,morphologic_part$,lines_part$,hyphenate,AF_compressed_decoding_done)                      
                      recursivity=#False
                      circumfix_twofold_counter=#False
                    ; Line below fixes the triple processing of Prefixes in CIRCUMFIX - Brandon fix + Mauro fix (rule that calls resets circumfix flag inside) - 10/APR/2020
;                     circumfix_twofold_counter=#False ; With recursivity=#False
                      
                    EndIf                      
;                     If recursivity=#True And f=len_word_codes : ProcedureReturn total_words_found : EndIf ;It must always return a number, not #TRUE nor #FALSE


                  EndIf                     
                  
                EndIf
;******************************
                  
                Next ff
              Next f_primary_sfx
             Next f
          EndIf 
; EndIf                  
          
          Debug "CIRCUMFIX:"+t2$+" circumfix_code:"+Str(circumfix_code)+" circumfix_processing$=t2$:"+circumfix_processing$
          
;  Debug "Refresh derived words"
          
          ; CIRCUMFIX WAS HERE BUT NOT WORKING, MOVED IT TO ABOVE

          
        ; Missing Codes - 21/NOV/2017
        If IsGadget(#STRING_MISSING_CODES)
          SetGadgetText(#STRING_MISSING_CODES,missing_codes$)
          If missing_codes$=""
            SetGadgetColor(#STRING_MISSING_CODES,#PB_Gadget_BackColor,-1)
          Else
            SetGadgetColor(#STRING_MISSING_CODES,#PB_Gadget_BackColor,#Red)
          EndIf
        EndIf
        
        
        ; Missing Codes Twofold - 11/NOV/2020
        If IsGadget(#STRING_MISSING_CODES_SUB)
          SetGadgetText(#STRING_MISSING_CODES_SUB,Trim(missing_codes_sub$,","))
          If missing_codes_sub$=""
            SetGadgetColor(#STRING_MISSING_CODES_SUB,#PB_Gadget_BackColor,-1)
          Else
            SetGadgetColor(#STRING_MISSING_CODES_SUB,#PB_Gadget_BackColor,#Red)
          EndIf
        EndIf        
        

        ProcedureReturn total_words_found ;It must always return a number, not #TRUE nor #FALSE
        
  EndProcedure
  
  
  
  
  ; So that if a word is changed while processing the derivates, it exits the decoding and refreshes the decoding
  ;
  ; V1.0 - 11/APR/2019
  ;
  Procedure process_pfx_sfx_update_word()
    
    dictionary_force_refresh=1
    
  EndProcedure
  
  
  
  
  ; So that if OK is pressed while processing the derivates, it exits the decoding
  ;
  ; V1.0 - 11/APR/2019
  ;
  Procedure process_pfx_sfx_update_word_okay()
    
    dictionary_force_refresh=2
    
  EndProcedure
  
  
  
  
  ; So that if CANCEL is pressed while processing the derivates, it exits the decoding
  ;
  ; V1.0 - 11/APR/2019
  ;
  Procedure process_pfx_sfx_update_word_cancel()
    
    dictionary_force_refresh=3
    
  EndProcedure
  
  
  
  
  ; Validate Suffix to check if letters match/unmatch 
  ; validate_sfx(t$,conditions_per_line_sfx$(5))
  ;
  ; V1.0 - 14/AUG/2013
  ; V1.1 - 18/AUG/2013
  ;        o Fixed the procedure
  ; V1.2 - 26/AUG/2013
  ;        o Major speed up decoding suffixes
  ; V1.3 - 29/SEP/2013
  ;        o Improved the code to detect a "." in the rule
  ; V1.4 - 30/SEP/2013
  ;        o Improved the decoding of codes for multiple conditions
  ; V1.5 - 13/JUL/2014
  ;        o Optimised the code
  ; V1.6 - 15/JUL/2014  
  ;        o Rewrote most of the code to fix more complex rules: beast/J -> rule #22 -> [^aeio]e[nrt]
  ; V1.7 - 16/JAN/2016
  ;        o Decoding algorithm improved with minor speed gain
  ; V1.8 - 27/JAN/2019
  ;        28/JAN/2019
  ;        31/JAN/2019
  ;        o Decoding suffixes and prefixes using the RegularExpression library.
  ;
  Procedure validate_sfx(a$,b$)  
Debug "SFX a$:"+a$+ " b$:"+b$+" conditions_per_line_sfx$(3):"+conditions_per_line_sfx$(3)

    ; Check if the rule ends with "."
    If b$="." : ProcedureReturn #True : EndIf
   
    ; Check if characters to replace don't match the original word - faster technique - 16/01/2016
    If conditions_per_line_sfx$(3)<>"0" And Right(a$,Len(conditions_per_line_sfx$(3)))<>conditions_per_line_sfx$(3) : ProcedureReturn #False : EndIf
  
    ; Try to match regular expression - 27/JAN/2019 + 28/JAN/2019 + 31/JAN/2019
    CreateRegularExpression(#RegularExpression,b$+"$",#PB_RegularExpression_DotAll)
    match=MatchRegularExpression(#RegularExpression,a$)
    FreeRegularExpression(#RegularExpression)
    ProcedureReturn match
    
  EndProcedure    
 
  
  
  
  ; Validate Prefix to check if letters match/unmatch 
  ; validate_pfx(t$,conditions_per_line_pfx$(5))
  ;
  ; V1.0 - 17/AUG/2013
  ; V1.1 - 18/AUG/2013
  ;        o Fixed the procedure  
  ; V1.2 - 29/SEP/2013
  ;        o Improved the code to detect a "." in the rule
  ; V1.3 - 30/SEP/2013
  ;        o Improved the decoding of codes for multiple conditions
  ; V1.4 - 13/JUL/2014
  ;        o Optimised the code
  ; V1.5 - 16/JAN/2016
  ;        o Decoding algorithm improved with minor speed gain
  ; V1.6 - 27/JAN/2019
  ;        28/JAN/2019
  ;        31/JAN/2019
  ;        o Decoding suffixes and prefixes using the RegularExpression library.
  ;
  Procedure validate_pfx(a$,b$)
 Debug "PFX a$:"+a$+ " b$:"+b$+" conditions_per_line_pfx$(3):"+conditions_per_line_pfx$(3)  
 
    ;Check if the rule ends with "."
    If b$="." : ProcedureReturn #True : EndIf
    
    ; Check if characters to replace don't match the original word - faster technique - 16/01/2016
    If conditions_per_line_pfx$(3)<>"0" And Left(a$,Len(conditions_per_line_pfx$(3)))<>conditions_per_line_pfx$(3) : ProcedureReturn #False : EndIf
  
    ; Try to match regular expression - 27/JAN/2019 + 28/JAN/2019 + 31/JAN/2019
    CreateRegularExpression(#RegularExpression,"^"+b$,#PB_RegularExpression_DotAll)
    match=MatchRegularExpression(#RegularExpression,a$)
    FreeRegularExpression(#RegularExpression)
    ProcedureReturn match
            
  EndProcedure  
  
  
  
  
  ; To add each derivate processed into the sort array to implement Shantanu's suggestion
  ;
  ; V1.0 - 28/SEP/2019
  ; V1.1 - 30/SEP/2019
  ;        o Special sorting suggested by Shantanu
  ; V1.2 - 05/FEB/2020
  ;
  Procedure add_derivate_to_sort_array(current_number_of_derivates_found)
    
    ; Array starts at position 0
    current_number_of_derivates_found-1
    
    ; Resize the array
    ReDim dictionary_column_sort_line(current_number_of_derivates_found)
    
    ; Add the values from the ListIconGadget    
    dictionary_column_sort_line(current_number_of_derivates_found)\position_in_listicongadget.l=current_number_of_derivates_found
    dictionary_column_sort_line(current_number_of_derivates_found)\word.s=GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,current_number_of_derivates_found,0)
    dictionary_column_sort_line(current_number_of_derivates_found)\type.s=GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,current_number_of_derivates_found,1)
    dictionary_column_sort_line(current_number_of_derivates_found)\type_colour.l=GetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,current_number_of_derivates_found,#PB_Gadget_FrontColor,1)
    dictionary_column_sort_line(current_number_of_derivates_found)\flag.s=GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,current_number_of_derivates_found,2) 
    dictionary_column_sort_line(current_number_of_derivates_found)\flag_position.l=Val(GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,current_number_of_derivates_found,3))
    dictionary_column_sort_line(current_number_of_derivates_found)\flag_position_string.s=Right("0000000000"+GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,current_number_of_derivates_found,3),8)
    t$=GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,current_number_of_derivates_found,4)
    dictionary_column_sort_line(current_number_of_derivates_found)\rule.s=Right("0000000000"+StringField(t$,1,":"),8)+":"+StringField(t$,2,":")
    dictionary_column_sort_line(current_number_of_derivates_found)\flag_top_nested.s=top_level_tags$+nested_tags$
    
    
  EndProcedure
  
  
  
  
  ; This procedure sorts, clears the listicongadget and displays the derivates
  ;
  ; V1.0 - 28/SEP/2019
  ; V1.1 - 30/SEP/2019
  ;        o Special sorting suggested by Shantanu
  ; V1.2 - 30/SEP/2019
  ;        o Sorting columns no case
  ;
  Procedure sort_and_display_derivates(current_number_of_derivates_found)
    
    ; Array starts at position 0
    current_number_of_derivates_found-1    
    
    
    ; Create the extra structured array field suggested by Shantanu
    ; There is a loop for each value, for extra speed by not putting a case inside each item
    If dictionary_column_sort<>1       
      
      Select dictionary_column_sort
          
        Case 2
          For f=0 To current_number_of_derivates_found
            dictionary_column_sort_line(f)\special_sort_shantanu.s=dictionary_column_sort_line(f)\type.s+Chr(9)+dictionary_column_sort_line(f)\flag.s
          Next f
          
        Case 3
          For f=0 To current_number_of_derivates_found
            dictionary_column_sort_line(f)\special_sort_shantanu.s=dictionary_column_sort_line(f)\flag.s+Chr(9)+dictionary_column_sort_line(f)\rule.s
          Next f            
          
        Case 4
          For f=0 To current_number_of_derivates_found
            dictionary_column_sort_line(f)\special_sort_shantanu.s=dictionary_column_sort_line(f)\flag_position_string.s+Chr(9)+dictionary_column_sort_line(f)\rule.s
          Next f          
          
        Case 5
          For f=0 To current_number_of_derivates_found
            dictionary_column_sort_line(f)\special_sort_shantanu.s=dictionary_column_sort_line(f)\rule.s+Chr(9)+dictionary_column_sort_line(f)\flag.s
          Next f
          
      EndSelect    
        
    EndIf
       
    
    ; Sort structured array
    If dictionary_column_sort=1
      SortStructuredArray(dictionary_column_sort_line(),#PB_Sort_Ascending,OffsetOf(dictionary_column_sort_line\position_in_listicongadget.l),TypeOf(dictionary_column_sort_line\position_in_listicongadget.l))
    Else
      SortStructuredArray(dictionary_column_sort_line(),#PB_Sort_Ascending,OffsetOf(dictionary_column_sort_line\special_sort_shantanu.s),TypeOf(dictionary_column_sort_line\special_sort_shantanu.s))
    EndIf
    
    
    ; Clear the ListIconGadget
    ClearGadgetItems(#PANEL_BOX_DERIVED_WORDS)
    
    
    ; Repopulate the ListIconGadget
    For f=0 To current_number_of_derivates_found
      
      t$=""
      t$+dictionary_column_sort_line(f)\word.s
      t$+#LF$
      t$+dictionary_column_sort_line(f)\type.s
      t$+#LF$
      t$+dictionary_column_sort_line(f)\flag.s
      t$+#LF$
      t1$=Str(dictionary_column_sort_line(f)\flag_position.l)
      If t1$="0" : t1$="" : EndIf
      t$+t1$
      t$+#LF$
      t1$=dictionary_column_sort_line(f)\rule.s
      t1$=Str(Val(StringField(t1$,1,":")))+":"+StringField(t1$,2,":")
      If t1$="0:" : t1$="" : EndIf ; Primary word, no rule position
      t$+t1$

      AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,t$)
      SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,f,#PB_Gadget_FrontColor,dictionary_column_sort_line(f)\type_colour.l,1)
    
    Next f
      
      
  EndProcedure
  
  
  
  
  ; To count and add the count of the numbers of equal flags to the ListIconGadget (Shantanu)
  ;
  ; V1.0 - 04/FEB/2020
  ;        05/FEB/2020
  ; 
  Procedure process_top_level_flags(current_number_of_derivates_found)
    
    ; Add primary
    AddGadgetItem(#LISTICON_DICTIONARY_SUMMARY_TOP_LEVEL_TAGS,-1,"Primary 1")
    
    ; Check if only primary
    If current_number_of_derivates_found=1 : ProcedureReturn : EndIf
    
    ; Array starts at position 0
    current_number_of_derivates_found-1 
    
    ; Array to store flags
    Dim top_level_tags$(current_number_of_derivates_found-1)
    
    ; Copy from structured to counting array
    For f=0 To current_number_of_derivates_found-1
      top_level_tags$(f)=dictionary_column_sort_line(f+1)\flag_top_nested.s
;       Debug Str(f)+"->"+dictionary_column_sort_line(f+1)\flag_top_nested.s
    Next f          
    
    ; Sort the counting array
    SortArray(top_level_tags$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
    
    ; Count and add flags in the ListIconGadget      
    For f=0 To current_number_of_derivates_found-1
      t$=top_level_tags$(f)
      occurrences=0
      
      For ff=f To current_number_of_derivates_found-1        
                
        
        If t$=top_level_tags$(ff)     
          occurrences+1          
        Else
          Break
        EndIf
                             
      Next ff
      
      f=ff-1
 
      
      ; Add duplicate word to listicongadget
      If occurrences>0
        If CountString(t$,Chr(9))=0
          AddGadgetItem(#LISTICON_DICTIONARY_SUMMARY_TOP_LEVEL_TAGS,-1,t$+" tag "+Str(occurrences))
        Else
          AddGadgetItem(#LISTICON_DICTIONARY_SUMMARY_NESTED_TAGS,-1,StringField(t$,1,Chr(9))+" tag — "+StringField(t$,2,Chr(9))+" tag "+Str(occurrences))
        EndIf
      EndIf
      
    Next f   
    
  EndProcedure
  
  
  
  
  ; To always have the current word highlighted in the listicongadget
  ;
  ; V1.0 - 5/MAR/2014
  ;
  Procedure update_vertical_scroll_bar_dictionary_current_item()  
    
    If selected_dictionary_word=-1 And GetGadgetState(#PANEL_BOX_DICTIONARY)=-1 : ProcedureReturn #False : EndIf
  
    selected_dictionary_word=GetGadgetState(#PANEL_BOX_DICTIONARY)+GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)   
    t=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
    t1=selected_dictionary_word-t
    
  EndProcedure
  
  
  

; To extract the wordlist to HDD
; Please notice that the process_pfx_sfx procedure will save directly to HDD
; Variable EXTENSION:
;                    0=.TXT
;                    1=.CSV
;                    2=COMPOUNDS .TXT
;
; V1.0 - 11/JUL/2014
; V1.1 - 12/JUL/2014
;        o The tags part of the word wasn't removed
; V1.2 - 16/JUL/2014
;        o Added CSV
; V1.3 - 17/JUL/2014
;        o In CSV it now has an extra field with the primary word and the separator is now ";" instead of ",".
; V1.4 - 31/JUL/2014
;        o The option to extract the wordlist now has TXT and CSV separately to be compatible with MacOS X.
; V1.5 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.6 - 5/MAR/2015
;        o Coded "Extract Compounds .txt".
; V1.7 - 28/APR/2016
;        o Support for LanguageTool
; V1.8 - 25/OCT/2017
;        o Formats the total number of items to be better seen.  
; V1.9 - 1/DEC/2017
;        o Message Requesters now have a sign.
; V2.0 - 15/FEB/2019
;        o Fix: No longer decodes commented dictionary words (starting with "/")
; V2.1 - 08/MAR/2019
;        13/MAR/2019
;        o Warn over N words while extracting and counting duplicates on wordlist
; V2.2 - 16/MAR/2019
;        o Export wordlist now exports in UNIX format with BOM
; V2.3 - 20/NOV/2020
;        02/DEC/2020
;        o Extracting wordlists now shows the filesize of the target file.  
; V2.4 - 21/DEC/2020
;        o Prefs: "0" disables above N words warning
; V2.5 - 07/FEB/2021
;        o Extract wordlist as CSV now has a "Root_word" field.
; V2.6 - 23/NOV/2021
;        o Started showing the time taken in timely operations.
; V2.7 - 19/SEP/2023
;        o Complex Prefixes not supported warning in extract wordlist
;
Procedure extract_wordlist(extension)
    Debug "Pressed EXTRACT WORDLIST"
    
    languagetool$=""
    If extension=6
      languagetool$="*"  ; Extract compounds in LanguageTool format
      extension=2
    EndIf
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting wordlist... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
    

    ; Complex Prefixes not supported warning
    If complexprefixes_code=#True
      MessageRequester("Warning!","COMPLEX PREFIXES not supported yet!",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
      
       
    t$="Text Documents (*.txt)|*.txt"
    If extension=1 : t$="Text CSV (*.csv)|*.csv" : EndIf
    tt$="Extract wordlist to file"
    If extension=2 : tt$="Extract Compounds to file" : EndIf
    file$=SaveFileRequester(tt$,"",t$,0)
    If file$=""
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    If (extension=0 Or extension=2) And LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
    If extension=1 And LCase(Right(file$,4))<>".csv" : file$+".csv": EndIf
    If extension=1 : csv=#True : EndIf
    If extension=2 : compounds=#True : EndIf
    
    
    t=ReadFile(1,file$)
    If t<>0
      CloseFile(1)
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    DeleteFile(file$) ; Delete .TXT file
    
    
    ; Saving the .TXT file
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,file$)    
      WriteStringFormat(1,#PB_UTF8)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
      time_taken=ElapsedMilliseconds()
      total_number_of_processed_words=0
      
      ; Write CSV header
      If csv=#True
        WriteString(1,Chr(34)+"Root_word"+Chr(34)+";"+Chr(34)+"Word"+Chr(34)+";"+Chr(34)+"Primary"+Chr(34)+";"+Chr(34)+"Type"+Chr(34)+";"+Chr(34)+"Code"+Chr(34)+";"+Chr(34)+"Position"+Chr(34)+";"+Chr(34)+"Rule"+Chr(34)+#LF$,#PB_UTF8)
      EndIf
      
      ask=#False
      For f=0 To number_of_words-1
        text$=StringField(words$(f),1,#LF$)
             
        ; Ask to abort if over N words
        If ask=#False And total_number_of_processed_words>=ask_after_n_words And ask_after_n_words<>0
          ask=#True
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Wordlist... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
          If MessageRequester("Warning!","Over "+FormatNumber(ask_after_n_words,0,".",number_separator_chr$)+" words have been decoded."+#LF$+#LF$+"This represents a huge amount of data in the HDD"+#LF$+"that can fill it up and crash the computer."+#LF$+#LF$+"Abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf     
        
        If Left(text$,1)<>"/" : total_number_of_processed_words+process_pfx_sfx(2,text$,csv,compounds,#False,languagetool$,"","",#False,#False) : EndIf
                      
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Wordlist... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" "+ShowFileSizeFormatted("",Lof(1),#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        

      Next f
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Wordlist... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" ("+StrF(Lof(1)/1024/1024,1)+" MB)"+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      filesize_lof=Lof(1)
      CloseFile(1)
      
      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. Words Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" "+ShowFileSizeFormatted("",filesize_lof,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
     
    
EndProcedure




; To count the wordlist
; Please notice that the process_pfx_sfx procedure will return the number of words decoded (total)
;
; V1.0 - 11/JUL/2014
; V1.1 - 12/JUL/2014
;        o The tags part of the word wasn't removed
; V1.2 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.3 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.4 - 1/DEC/2017
;        o Message Requesters now have a sign.  
; V1.5 - 15/MAY/2018
;        o Fix: Count words now separates thousands with the preferences character.
; V1.6 - 15/FEB/2019
;        o Fix: No longer decodes commented dictionary words (starting with "/")
; V1.7 - 23/NOV/2021
;        o Started showing the time taken in timely operations.
;
Procedure count_wordlist()
    Debug "Pressed COUNT WORDLIST"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Counting wordlist... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
        
            
    ; Counting wordlist
    AddWindowTimer(#WINDOW_MAIN,800,800)    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time_taken=ElapsedMilliseconds()
    total_number_of_processed_words=0
      
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
        
      If Left(text$,1)<>"/" : total_number_of_processed_words+process_pfx_sfx(3,text$,#False,#False,#False,"","","",#False,#False) : EndIf
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Counting Wordlist... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_words,1))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
        EndIf
      EndIf
        

      Next f
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      
      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. Words Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
     
    
EndProcedure




; To find duplicates in the wordlist
;
; V1.0 -  6/AUG/2014
; V1.1 - 29/AUG/2014
;        o Now it gives an error if one attempts to export a blank list
;        o Fix: The number of words text no longer cuts the "Export" button while hidding and then showing PTG.
; V1.2 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.3 -  7/AUG/2015
;        o If "Show Duplicates" finds nothing, it opens a requester saying so.
; V1.4 - 11/MAR/2017
;        o Changed the buttons height to 25. 
; V1.5 - 28/APR/2017
;        o Support for morphological data
; V1.6 - 29/APR/2017
;        o Fix: Check for duplicates improved (morphologic data)
; V1.7 - 20/MAY/2017
;        o Now the check for duplicates in dictionary, also shows the lines numbers.
; V1.8 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.9 - 1/DEC/2017
;        o Message Requesters now have a sign.  
; V2.0 - 15/FEB/2019
;        o Fix: No longer decodes commented dictionary words (starting with "/")
; V2.1 - 08/MAR/2019
;        13/MAR/2019
;        o Warn over N words while extracting and counting duplicates on wordlist
; V2.2 - 15/MAR/2019
;        o "Show duplicates wordlist" now clearer to read and exports in UNIX format with BOM
; V2.3 - 06/JUN/2019
;        o GTK3 Ready
; V2.4 - 20/JUN/2019
;        o Preferences allow to choose the font for the language. 
; V2.5- 16/MAY/2020
;        o Now also shows the total of words in the wordlist.
; V2.6 - 12/OCT/2020
;        o Adding of F1 to open the help in the various windows.
; V2.7 - 21/DEC/2020
;        o Prefs: "0" disables above N words warning
; V2.8 - 23/NOV/2021
;        o Started showing the time taken in timely operations.
;
Procedure show_duplicates_wordlist()
  
    Debug "Pressed SHOW DUPLICATES"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
    
    
            
    ; Step 1 of 2 (extracting wordlist)
    AddWindowTimer(#WINDOW_MAIN,800,800)
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time_taken=ElapsedMilliseconds()
    total_number_of_processed_words=0
    Dim total_number_of_processed_words_array$(0)
    
    
    ask=#False
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
      morphologic_part$=#LF$+StringField(words$(f),2,#LF$)
      lines_part$=#LF$+Str(f+1)
      
      ; Ask to abort if over N words
      If ask=#False And total_number_of_processed_words>=ask_after_n_words And ask_after_n_words<>0
        ask=#True
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting..."+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" <ESC> to abort.")
        If MessageRequester("Warning!","Over "+FormatNumber(ask_after_n_words,0,".",number_separator_chr$)+" words have been decoded."+#LF$+#LF$+"This represents a huge amount of data in RAM"+#LF$+"that can fill it up and crash the computer."+#LF$+#LF$+"Abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf        
      
      If Left(text$,1)<>"/" : total_number_of_processed_words+process_pfx_sfx(4,text$,#False,#False,#False,"",morphologic_part$,lines_part$,#False,#False) : EndIf
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting..."+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        
      
    Next f
    
    
    ; Sort the array    
    SortArray(total_number_of_processed_words_array$(),#PB_Sort_Ascending)    
    
    
    ; Step 2 of 2 (comparing wordlist)    
    
    ; First open a hidden window
    If OpenWindow(#WINDOW_SHOW_DUPLICATES,0,0,400+20-5+250+10,4-100+200-50-20-20+100+100+200-30-30-30+10-20-10+20+5+10,"Show Duplicates in wordlist",#PB_Window_Invisible|#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    ListIconGadget(#LISTICON_WINDOW_SHOW_DUPLICATES,5,5,250+400+6+4,200+100-50+10,"Word",100+100,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_DUPLICATES,1,"Occurrences",100)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_DUPLICATES,2,"Lines",250)
    SetGadgetFont(#LISTICON_WINDOW_SHOW_DUPLICATES,FontID(#font_for_language_id))
    ButtonGadget(#BUTTON_WINDOW_SHOW_EXPORT,5+50+50+50+0+50,200+100-50+10+10+10+10+5,100,default_gadget_height_button,"Export")
    ButtonGadget(#BUTTON_WINDOW_SHOW_OKAY,5+50+50+50+50+0+100+10,200+100-50+10+10+10+10+5,100-10-10,default_gadget_height_button,"Close")
    
    ; Progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing... <ESC> to abort.")    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_number_of_processed_words)     
        
        
    For f=0 To total_number_of_processed_words-1
;       t$=total_number_of_processed_words_array$(f)
      t$=StringField(total_number_of_processed_words_array$(f),1,#LF$)+#LF$+StringField(total_number_of_processed_words_array$(f),2,#LF$)
      occurrences=0
      lines$=""
      
      For ff=f To total_number_of_processed_words-1            
                
        
        If t$=StringField(total_number_of_processed_words_array$(ff),1,#LF$)+#LF$+StringField(total_number_of_processed_words_array$(ff),2,#LF$) ;=total_number_of_processed_words_array$(ff)          
          occurrences+1          
          If occurrences>1 : lines$+" + " : EndIf
          lines$+FormatNumber(Val(StringField(total_number_of_processed_words_array$(ff),3,#LF$)),0,".",number_separator_chr$)
        Else
          Break
        EndIf
                             
      Next ff
      
      f=ff-1
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
         StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing..."+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))    
         StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          CloseWindow(#WINDOW_SHOW_DUPLICATES)
          ProcedureReturn #False
        EndIf
      EndIf      
      
      
      ; Add duplicate word to listicongadget
      If occurrences>1
          AddGadgetItem(#LISTICON_WINDOW_SHOW_DUPLICATES,-1,StringField(t$,1,#LF$)+#LF$+FormatNumber(occurrences,0,".",number_separator_chr$)+#LF$+lines$)
      EndIf
      
    Next f    
    
    
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
   
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
    
    ; No duplicates found, exit the Procedure - 7/AUG/2015
    If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)=0
      MessageRequester("Warning!","No duplicates found.",#PB_MessageRequester_Warning)
      DisableWindow(#WINDOW_MAIN,#False)
      CloseWindow(#WINDOW_SHOW_DUPLICATES)                
      ProcedureReturn
    EndIf    
    
    
    HideWindow(#WINDOW_SHOW_DUPLICATES,#False)
    DisableWindow(#WINDOW_MAIN,#True)
    SetActiveWindow(#WINDOW_SHOW_DUPLICATES)
    
    ; Add the number of words found
    TextGadget(#TEXT_WINDOW_SHOW_DUPLICATES_FOUND,5+5,200+100-50+10+10,200-20,default_gadget_height_text,"Found:"+FormatNumber(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES),0,".",number_separator_chr$)+"/"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
    GadgetFit(#TEXT_WINDOW_SHOW_DUPLICATES_FOUND)
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_DUPLICATES,#PB_Shortcut_Escape,1000)
    AddKeyboardShortcut(#WINDOW_SHOW_DUPLICATES,#PB_Shortcut_F1,1020)
    
    
    cancel=#False
    Repeat
      
      event=WaitWindowEvent()
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And EventMenu()=1020
        Debug "F1 pressed"
        html_help("#5.2.5.9.show_duplicates_wordlist")
      EndIf
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
      
      Select event 
        Case #PB_Event_Gadget
         Select EventGadget()
           Case #BUTTON_WINDOW_SHOW_OKAY
             cancel=#True
             
           Case #BUTTON_WINDOW_SHOW_EXPORT
             If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)>0
               flag=#False
               file$=SaveFileRequester("Export Duplicates to file","","Text Documents (*.txt)|*.txt",0)
               If file$<>""
                 If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
                 t=ReadFile(1,file$)
                 If t<>0
                   CloseFile(1)
                   If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                       DeleteFile(file$) ; Delete .TXT file                 
                   Else
                       flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                   EndIf  
                 EndIf    
                 
                 If flag=#False
                    ; Saving the .TXT file
                    AddWindowTimer(#WINDOW_MAIN,800,800)
                    OpenFile(1,file$)  
                    WriteStringFormat(1,#PB_UTF8)
                    
                    WriteString(1,"DUPLICATES FOUND:"+FormatNumber(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES),0,".",number_separator_chr$)+#LF$,#PB_UTF8)
                    WriteString(1,""+#LF$,#PB_UTF8)
                    
                    For f=1 To CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)
                      WriteString(1,GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,0)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1)+"): "+GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,2)+#LF$,#PB_UTF8)
                      
                      ; Fix to window stop responding if we click on it while exporting
                      event = WindowEvent()
                      If event=#PB_Event_Timer And EventTimer()=800 : EndIf 
                
                    Next f
                    CloseFile(1)
                    RemoveWindowTimer(#WINDOW_MAIN,800)          
                 EndIf
                 
               EndIf
               
             EndIf
         EndSelect
         
      EndSelect   
      
    Until cancel=#True
    
    ; Close the Duplicates window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_SHOW_DUPLICATES)    
    
EndProcedure




; Process the Statistics
;
; V1.0 - 15/DEC/2014
; V1.1 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.2 - 10/APR/2015
;        o Pie chart support
; V1.3 - 13/APR/2015
;        o Improved the pie chart with a message in case it gets above 100% + small fix
; V1.4 - 11/MAR/2017
;        o Changed the buttons height to 25. 
; V1.5 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.6 - 1/DEC/2017
;        o Message Requesters now have a sign.
; V1.7 - 12/JUL/2018
;        o Statistics now allow to calculate the number of new words since the last mentioned number.
; V1.8 - 15/JUL/2018
;        o Statistics now export in Linux format and also the extra information.
; V1.9 - 16/JUL/2018
;        20/JUL/2018
;        o Statistics now autoupdate the number of new words when changing the reference value.
; V2.0 - 29/NOV/2018
;        o In the "Statistics" the total number of words is now a StringGadget (can copy its contents)
; V2.1 - 03/DEC/2018
;        o Added two "Copy" (to clipboard) buttons in statistics
; V2.2 - 15/FEB/2019
;        o Fix: No longer decodes commented dictionary words (starting with "/")
; V2.3 - 05/JUN/2019
;        o Statistics now use two decimal houses in the %
; V2.4 - 12/JUN/2019
;        o GTK3 Ready
; V2.5 - 09/SEP/2019
;        o Statistics now removes ".00" for % with round values.
; V2.6 - 06/OCT/2019
;        08/OCT/2019
;        10/OCT/2019
;        o Coding the extra options for the statistics (Shantanu Oak)
; V2.7 - 15/OCT/2019
;        o Estatistics: Extract with flags option (Shantanu Oak)
; V2.8 - 15/FEB/2020
;        o All file operations support BOM.
; V2.9 - 12/OCT/2020
;        o Adding of F1 to open the help in the various windows.
; V3.0 - 23/NOV/2021
;        o Started showing the time taken in timely operations.
; V3.1 - 05/JUN/2022
;        o Statistics now GTK3 ready
;        o Coded statistics by letter
; V3.2 - 05/FEB/2023
;        o Added info icon to "Show Statistics" total of new words string gadget
; V3.3 - 14/APR/2023
;        o Optimised the statistics processing.
; V3.4 - 17/APR/2023
;        o Improved GUI of dictionary statistics and added the "Remember" date.
; V3.5 - 6/NOV/2023
;          1) Fixed: Remember would not update the Remember value
;          2) New button: "Copy ∑ New"
;
Procedure show_statistics()
    Debug "Pressed STATISTICS"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Statistics... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
    
    
    ; Reset the statistics variables
    statistics_words_with_apos=0
    statistics_words_with_hyphen=0
    statistics_words_with_first_letter_uppercase=0
    Dim statistics_words_chr1to5(5)
    statistics_current_word=0
    Dim statistics_word.statistics_word(number_of_words-1)
    Dim statistics_word_per_character(26)
    
    
    ; Processing wordlist
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time_taken=ElapsedMilliseconds()
    total_number_of_processed_words=0
      
      
    For f=0 To number_of_words-1
      
      statistics_current_word=f
      
      text$=StringField(words$(f),1,#LF$)
        
      If Left(text$,1)<>"/" : total_number_of_processed_words+process_pfx_sfx(5,text$,#False,#False,#False,"","","",#False,#False) : EndIf
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Statistics... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_words,1))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
               
    
    w=10+640-10-10-20-10+3+200+400+40+15-3-2
    h=3-30-8+70+3+70-100+200-50-20-20+100+100+200-30-30-30+10-20-10+30+10+10+10+3+20+20+10
    
    ; Open window
    If OpenWindow(#WINDOW_SHOW_STATISTICS,0,0,w,h,"Show Statistics",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    ListIconGadget(#LISTICON_WINDOW_SHOW_STATISTICS,5,5,10+400-50-50+3+5,200+100-50+10,"Statistics (Words)",100+100-50,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS,1,"Occurrences",100)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS,2,"%",40+3+5+10)    
    SetGadgetFont(#LISTICON_WINDOW_SHOW_STATISTICS,FontID(#font_for_language_id))
    CanvasGadget(#CANVAS_WINDOW_SHOW_STATISTICS,10+4+5+400-50-50+10,5,250+5+5,250+5+5+1,#PB_Canvas_Border)
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_EXPORT,10+default_gadget_width_button+10,-5+h-10-default_gadget_height_button,default_gadget_width_button,default_gadget_height_button,"Export")
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_OKAY,(w/2)-(default_gadget_width_button/2),h-10-default_gadget_height_button,default_gadget_width_button,default_gadget_height_button,"Close")
    
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"With ':"+#LF$+FormatNumber(statistics_words_with_apos,0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_with_apos*100)/total_number_of_processed_words,2,#True),ImageID(14))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"With Hyphen:"+#LF$+FormatNumber(statistics_words_with_hyphen,0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_with_hyphen*100)/total_number_of_processed_words,2,#True),ImageID(15))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"Starting uppercase:"+#LF$+FormatNumber(statistics_words_with_first_letter_uppercase,0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_with_first_letter_uppercase*100)/total_number_of_processed_words,2,#True),ImageID(16))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"1 characters long:"+#LF$+FormatNumber(statistics_words_chr1to5(1),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_chr1to5(1)*100)/total_number_of_processed_words,2,#True),ImageID(17))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"2 characters long:"+#LF$+FormatNumber(statistics_words_chr1to5(2),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_chr1to5(2)*100)/total_number_of_processed_words,2,#True),ImageID(18))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"3 characters long:"+#LF$+FormatNumber(statistics_words_chr1to5(3),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_chr1to5(3)*100)/total_number_of_processed_words,2,#True),ImageID(19))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"4 characters long:"+#LF$+FormatNumber(statistics_words_chr1to5(4),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_chr1to5(4)*100)/total_number_of_processed_words,2,#True),ImageID(20))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"5 characters long:"+#LF$+FormatNumber(statistics_words_chr1to5(5),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_words_chr1to5(5)*100)/total_number_of_processed_words,2,#True),ImageID(21))

    ProgressBarGadget(#PROGRESSBAR_WINDOW_SHOW_STATISTICS,-8-100-40-30-20-25-30+GadgetWidth(#CANVAS_WINDOW_SHOW_STATISTICS)+5+20+20+20+60-18+20+10+100+20-10+150-20-5+10+20+10,GadgetY(#CANVAS_WINDOW_SHOW_STATISTICS)+GadgetHeight(#CANVAS_WINDOW_SHOW_STATISTICS)+5,default_gadget_width_progress,default_gadget_height_small_progress,0,number_of_words)
    
    x=GadgetX(#PROGRESSBAR_WINDOW_SHOW_STATISTICS)
    y=GadgetY(#PROGRESSBAR_WINDOW_SHOW_STATISTICS)+GadgetHeight(#PROGRESSBAR_WINDOW_SHOW_STATISTICS)+1
    
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_WINDOW_SHOW_STATISTICS_TIME_TAKEN,x,y,100,default_gadget_height_text,"")  
    
    y+default_gadget_height_text+1
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_WINDOW_SHOW_STATISTICS,x,y,100,default_gadget_height_text,"")
    
    
    
    ; Draw a black circle + %s in the canvas
    Dim colour_degree(7)
    colour_degree(0)=RGB(163,73,164)
    colour_degree(1)=RGB(63,72,204)
    colour_degree(2)=RGB(0,162,232)
    colour_degree(3)=RGB(34,177,76)
    colour_degree(4)=RGB(255,242,0)
    colour_degree(5)=RGB(255,127,39)
    colour_degree(6)=RGB(237,28,36)
    colour_degree(7)=RGB(136,0,21)
    
    StartDrawing(CanvasOutput(#CANVAS_WINDOW_SHOW_STATISTICS))
      DrawingMode(#PB_2DDrawing_Gradient)
      BackColor(#White)
      FrontColor(#Black)
      ConicalGradient(125,125,0)

;       GradientColor(0.00,#Black)
;       GradientColor(1.00-0.0001,#Black)
;       Circle(125,125,110)

      sum.f=0.0
      current_degree_position.f=0.00
      For f=0 To 7
        current_degree_percentage.f=ValF(GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,f,2))/100.0       
        sum.f+current_degree_percentage.f
        If current_degree_percentage.f>0.00
          GradientColor(current_degree_position.f,colour_degree(f))
          current_degree_position.f+current_degree_percentage.f
          GradientColor(current_degree_position.f-0.00001,colour_degree(f))
        EndIf
      Next f
      If current_degree_position.f<>1.00
        GradientColor(current_degree_position.f,#Black)
        GradientColor(1.00-0.01,#Black)
      EndIf
      Circle(125,125,110,#White)   
    
      DrawingMode(#PB_2DDrawing_Default)
    
      ; Check if sum.f>100.0%
      If sum.f>100.0 : DrawText(5,250+5+5-10-10-2,"NOTE: >100%",#Black,#White) : EndIf
      
    StopDrawing()
    
    
    
      ; Create Popup menus
    CreatePopupImageMenu(#POPUPMENU_WINDOW_SHOW_STATISTICS,#PB_Menu_ModernLook)
    MenuItem(#POPUPMENU_WINDOW_SHOW_STATISTICS_SHOW,"Show")   
    MenuItem(#POPUPMENU_WINDOW_SHOW_STATISTICS_EXTRACT,"Extract")
    MenuItem(#POPUPMENU_WINDOW_SHOW_STATISTICS_EXTRACT_WITH_FLAGS,"Extract with Flags")
    
    DisableWindow(#WINDOW_MAIN,#True)
    SetActiveWindow(#WINDOW_SHOW_STATISTICS)
    
    
    ; The statistics per chr
    x=GadgetX(#CANVAS_WINDOW_SHOW_STATISTICS)+GadgetWidth(#CANVAS_WINDOW_SHOW_STATISTICS)+10
    y=GadgetY(#CANVAS_WINDOW_SHOW_STATISTICS)
    ListIconGadget(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1,x,y,10+400-50-50+3+5,200+100-50+10+30+30+30,"Statistics (Letters)",100+100-50,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1,1,"Occurrences",100)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1,2,"%",40+3+5+10)
    SetGadgetFont(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1,FontID(#font_for_language_id))
    x=GadgetX(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1)+GadgetWidth(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1)+10
    ListIconGadget(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR2,x,y,10+400-50-50+3+5,200+100-50+10+30+30+30,"Statistics (Letters)",100+100-50,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR2,1,"Occurrences",100)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR2,2,"%",40+3+5+10)  
    SetGadgetFont(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR2,FontID(#font_for_language_id))
    
    ; Add the information regarding the letters
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1,-1,"#/other"+#LF$+FormatNumber(statistics_word_per_character(0),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_word_per_character(0)*100)/total_number_of_processed_words,2,#True))
    For f=1 To 13
      AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1,-1,Chr(Asc("a")+f-1)+#LF$+FormatNumber(statistics_word_per_character(f),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_word_per_character(f)*100)/total_number_of_processed_words,2,#True))
    Next f
    For f=14 To 26
      AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR2,-1,Chr(Asc("a")+f-1)+#LF$+FormatNumber(statistics_word_per_character(f),0,".",number_separator_chr$)+#LF$+FloatToString((statistics_word_per_character(f)*100)/total_number_of_processed_words,2,#True))
    Next f
    SetGadgetItemColor(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR1,#PB_All,#PB_Gadget_BackColor,#BlueLight,0)
    SetGadgetItemColor(#LISTICON_WINDOW_SHOW_STATISTICS_PER_CHR2,#PB_All,#PB_Gadget_BackColor,#BlueLight,0)
    
    
    ; Add the number of words found
    x=10
    y=GadgetY(#LISTICON_WINDOW_SHOW_STATISTICS)+GadgetHeight(#LISTICON_WINDOW_SHOW_STATISTICS)+5
    TextGadget(#TEXT_WINDOW_SHOW_STATISTICS_WORDLIST,x,y,default_gadget_width_button,default_gadget_height_text,"Current wordlist:",#PB_Text_Right)
    x+default_gadget_width_button+10
    StringGadget(#STRING_WINDOW_SHOW_STATISTICS_WORDLIST,x,y,100+15,default_gadget_height_string,"",#PB_String_ReadOnly)
    SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_WORDLIST,FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
    x+GadgetWidth(#STRING_WINDOW_SHOW_STATISTICS_WORDLIST)+10
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_WORDLIST_COPY,x,y,default_gadget_width_button,default_gadget_height_button,"Copy")

    
    x=GadgetX(#STRING_WINDOW_SHOW_STATISTICS_WORDLIST)-10-default_gadget_width_button
    y+default_gadget_height_string+10

    
    x=10
    y+10+3
    
    ; COPY button
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_COPY,x,y,default_gadget_width_button,default_gadget_height_button,"∑ Copy →")
    
    ; Remember string
    x+default_gadget_width_button+10
    StringGadget(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER,x,y,100+15,default_gadget_height_string,dictionary_stats$,#PB_String_Numeric)
    
    ; REMEMBER
    x+100+10+15
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_REMEMBER,x,y,default_gadget_width_button,default_gadget_height_button,"Remember")
    
    ; FORGET
    x+default_gadget_width_button+10
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_FORGET,x,y,default_gadget_width_button,default_gadget_height_button,"Forget")    
    
    ; TOTAL NEW
    x=GadgetX(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)
    y+default_gadget_height_string+5+5
    StringGadget(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,x,y,100+50+10+10+100+5+3+1+15,default_gadget_height_string,"",#PB_String_ReadOnly)
    words_remembered=Val(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER))
    If words_remembered>=0 And GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>""
      SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"∑ New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+" / Date:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
    EndIf
    x+GadgetWidth(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)+10
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_TOTAL_NEW_COPY,x,y,default_gadget_width_button,default_gadget_height_button,"Copy")
    x=GadgetX(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)-10-ImageWidth(110)
    y=GadgetY(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)+2
    ImageGadget(#IMAGE_WINDOW_SHOW_STATISTICS_TOTAL_NEW,x,y,ImageWidth(110),ImageHeight(110),ImageID(110))
    x=GadgetXplusGadgetWidth(#BUTTON_WINDOW_SHOW_STATISTICS_TOTAL_NEW_COPY)+10
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_TOTAL_NEW_COPY_TOTAL_NEW,x,y-2,default_gadget_width_button,default_gadget_height_button,"Copy ∑ New")
    
    
    ; REMEMBER DATE
    x=GadgetX(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)
    y+default_gadget_height_string+5+5
    StringGadget(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_REMEMBER,x,y,100+50+10+10+100+5+3+1+15,default_gadget_height_string,dictionary_stats_last_remember$,#PB_String_ReadOnly)
    
    ; REMEMBER DATE COPY
    x+GadgetWidth(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_REMEMBER)+10
    ButtonGadget(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_REMEMBER_COPY,x,y,default_gadget_width_button,default_gadget_height_button,"Copy")
    
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_STATISTICS,#PB_Shortcut_Escape,1000)
    AddKeyboardShortcut(#WINDOW_SHOW_STATISTICS,#PB_Shortcut_F1,1020)
 
    cancel=#False
    Repeat
      
      event=WaitWindowEvent()
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And EventMenu()=1020
        Debug "F1 pressed"
        html_help("#5.2.5.10.statistics")
      EndIf
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
      
      ; Pop-up menu right-click
      ; POP-UP MENU
      If EventGadget()=#LISTICON_WINDOW_SHOW_STATISTICS And EventType()=#PB_EventType_RightClick
        DisplayPopupMenu(#POPUPMENU_WINDOW_SHOW_STATISTICS, WindowID(#WINDOW_SHOW_STATISTICS))
      EndIf        
      
      ; Double-click on ListIconGadget
      If EventGadget()=#LISTICON_WINDOW_SHOW_STATISTICS And EventType()=#PB_EventType_LeftDoubleClick
        statistics_popup_menu_show()
      EndIf        
      
      
      Select event 
          
        Case #PB_Event_Menu
          Select EventMenu()
            Case #POPUPMENU_WINDOW_SHOW_STATISTICS_SHOW
              statistics_popup_menu_show()
              
            Case #POPUPMENU_WINDOW_SHOW_STATISTICS_EXTRACT
              type_of_word_selected_statistics_extract_with_flags=#False
              statistics_popup_menu_extract()
              
            Case #POPUPMENU_WINDOW_SHOW_STATISTICS_EXTRACT_WITH_FLAGS
              type_of_word_selected_statistics_extract_with_flags=#True
              statistics_popup_menu_extract()              
              
          EndSelect
          
        Case #PB_Event_Gadget
         Select EventGadget()
           Case #BUTTON_WINDOW_SHOW_STATISTICS_OKAY
             cancel=#True
             
           Case #BUTTON_WINDOW_SHOW_STATISTICS_EXPORT
             flag=#False
             file$=SaveFileRequester("Export Statistics to file","","Text Documents (*.txt)|*.txt",0)
             If file$<>""
               If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
               t=ReadFile(1,file$)
               If t<>0
                 CloseFile(1)
                 If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                     DeleteFile(file$) ; Delete .TXT file                 
                 Else
                     flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                 EndIf  
               EndIf    
                 
               If flag=#False
                  ; Saving the .TXT file
                  OpenFile(1,file$)
                  WriteStringFormat(1,#PB_UTF8)
                    
                  WriteString(1,"Total wordlist:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+#LF$,#PB_UTF8)
                  WriteString(1,""+#LF$)
                  WriteString(1,"WORDS:"+#LF$)
                  WriteString(1,"            With ':"+FormatNumber(statistics_words_with_apos,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,0,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"       With Hyphen:"+FormatNumber(statistics_words_with_hyphen,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,1,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"Starting uppercase:"+FormatNumber(statistics_words_with_first_letter_uppercase,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,2,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1," 1 characters long:"+FormatNumber(statistics_words_chr1to5(1),0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,3,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1," 2 characters long:"+FormatNumber(statistics_words_chr1to5(2),0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,4,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1," 3 characters long:"+FormatNumber(statistics_words_chr1to5(3),0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,5,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1," 4 characters long:"+FormatNumber(statistics_words_chr1to5(4),0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,6,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1," 5 characters long:"+FormatNumber(statistics_words_chr1to5(5),0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,7,2)+"%)"+#LF$,#PB_UTF8)    
                  WriteString(1,""+#LF$)
                  ref_value$="N/A"
                  If GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>"" : ref_value$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER) : EndIf
                  WriteString(1,"Reference Value:"+#LF$+ref_value$+#LF$,#PB_UTF8)
                  new_words$="N/A"
                  If GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)<>"" : new_words$=Mid(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW),5) : EndIf
                  WriteString(1,"New Words:"+#LF$+new_words$+#LF$,#PB_UTF8)
                  WriteString(1,""+#LF$)
                  WriteString(1,"LETTERS:"+#LF$)
                  WriteString(1,"#/other:"+FormatNumber(statistics_word_per_character(0),0,".",number_separator_chr$)+" ("+FloatToString((statistics_word_per_character(0)*100)/total_number_of_processed_words,2,#True)+"%)"+#LF$,#PB_UTF8)
                  For f=1 To 26
                    WriteString(1,"      "+Chr(Asc("a")+f-1)+":"+FormatNumber(statistics_word_per_character(f),0,".",number_separator_chr$)+" ("+FloatToString((statistics_word_per_character(f)*100)/total_number_of_processed_words,2,#True)+"%)"+#LF$,#PB_UTF8)
                  Next f
                  
                  CloseFile(1)
                  
               EndIf
                 
             EndIf
             
             
           ; Manually change number of reference value
           Case #STRING_WINDOW_SHOW_STATISTICS_REMEMBER
             If EventType()=#PB_EventType_Change
               
                ; Update new words
                words_remembered=Val(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER))
                If words_remembered>=0 And GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>""
                  SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+" / Date:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
                EndIf         
                If GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)="" : SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"") : EndIf
                
             EndIf
             
           ; Copy 
           Case #BUTTON_WINDOW_SHOW_STATISTICS_COPY
             SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER,Str(total_number_of_processed_words))
             
             ; Update new words
             words_remembered=Val(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER))
             If words_remembered>=0 And GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>""
               SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+" / Date:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
             EndIf                  
             
             
           ; Delete old remember.prefs file     
           Case #BUTTON_WINDOW_SHOW_STATISTICS_FORGET
             
              If FileSize(main_path$+"settings/"+"dic_stats_remember.prefs")>=0
                DeleteFile(main_path$+"settings/"+"dic_stats_remember.prefs")              
              EndIf  
            
              SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER,"")
              SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"")
              
              
           ; Remember the previous number of words
           Case #BUTTON_WINDOW_SHOW_STATISTICS_REMEMBER 
             t$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)
             If t$=""
               MessageRequester("Warning!","Remember gadget is blank.",#PB_MessageRequester_Warning)
               
             Else
               
                ; Delete old dic_stats_remember.prefs file    
                If FileSize(main_path$+"settings/"+"dic_stats_remember.prefs")>=0
                  DeleteFile(main_path$+"settings/"+"dic_stats_remember.prefs")
                EndIf                
                
                ; Attempt to save dic_stats_remember.PREFS file
                OpenFile(1,main_path$+"settings/"+"dic_stats_remember.prefs")  
                  WriteString(1,prefs_file_version$+#LF$,#PB_UTF8)
                  WriteString(1,t$+#LF$,#PB_UTF8)
                  dictionary_stats_last_remember$="Last Remember:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date())
                  SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_REMEMBER,dictionary_stats_last_remember$)
                  WriteString(1,dictionary_stats_last_remember$+#LF$,#PB_UTF8)
                CloseFile(1)
                
                ; Update new words
                words_remembered=Val(t$)
                SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+" / Date:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
                dictionary_stats$=t$
                
             EndIf
             
             
           Case #BUTTON_WINDOW_SHOW_STATISTICS_WORDLIST_COPY
             t$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_WORDLIST)
             SetClipboardText(t$)
             
             
           Case #BUTTON_WINDOW_SHOW_STATISTICS_TOTAL_NEW_COPY
             t$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)
             If t$=""
               MessageRequester("Warning!","Nothing to copy to clipboard.",#PB_MessageRequester_Warning)
             Else
               SetClipboardText(t$)
             EndIf
             
             
           Case #BUTTON_WINDOW_SHOW_STATISTICS_TOTAL_NEW_COPY_TOTAL_NEW
             t$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)
             If t$=""
               MessageRequester("Warning!","Nothing to copy to clipboard.",#PB_MessageRequester_Warning)
             Else
               SetClipboardText(Str(total_number_of_processed_words-words_remembered))
             EndIf
             
             
           Case #STRING_WINDOW_SHOW_STATISTICS_TOTAL_REMEMBER_COPY
             t$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_REMEMBER)
             If t$=""
               MessageRequester("Warning!","Nothing to copy to clipboard.",#PB_MessageRequester_Warning)
             Else
               SetClipboardText(t$)
             EndIf
             
         EndSelect
         
      EndSelect   
      
    Until cancel=#True
    
    ; Close the Duplicates window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_SHOW_STATISTICS)    
           
EndProcedure




; This procedure is called from inside the process_pfx_sfx code, in order to increase the different type of statistics found
;
; V1.0 - 15/DEC/2014
; V1.1 - 18/JUL/2018
;        o Optimised slightly the statistics processing regarding uppercase words.
; V1.2 - 06/OCT/2019
;        o Coding the extra options for the statistics (Shantanu Oak)
; V1.3 - 05/JUN/2022
;        06/JUN/2022
;        o Coded statistics by letter
;        o Fix: Statistics, When a word started with a symbols was marked as uppercase
; V1.4 - 27/JUN/2022
;        o Speeded up the processing of letters and lenght in the "Show Statistics"
; V1.5 - 14/APR/2023
;        o Optimised the statistics processing.
;
Procedure process_statistics(t$)
  
  len_of_word=Len(t$) ; For extra speed to avoid in each condition to check the len - 27/JUN/2022
  
  ; Check for apostrophe
  If FindString(t$,"'")>0
    statistics_words_with_apos+1
    statistics_word(statistics_current_word)\statistics_words_with_apos_counter+1
  EndIf
  
  ; Check for hyphen
  If FindString(t$,"-")>0
    statistics_words_with_hyphen+1
    statistics_word(statistics_current_word)\statistics_words_with_hyphen_counter+1
  EndIf
  
  ; Check for uppercase
  t1$=Left(t$,1)
  If t1$<>LCase(t1$)
    statistics_words_with_first_letter_uppercase+1
    statistics_word(statistics_current_word)\statistics_words_with_first_letter_uppercase_counter+1
  EndIf
  
  ; Check for length of word: 1 to 5 chrs
  If len_of_word<=5
    statistics_words_chr1to5(len_of_word)+1
    statistics_word(statistics_current_word)\statistics_words_chr_counter1to5=len_of_word
  EndIf

  ; Check for letter/number/other at start of word
  ; Speeded up the code below - 27/JUN/2022 + 14/APR/2023
  ; It removes the accent without verifying if ucase != lcase since only a few words are symbols at start, saving time
  chr_position_in_array=0
  chr_first_letter$=RemoveAccentsString(Left(t$,1))
  asc_first_letter=Asc(chr_first_letter$)
  ; Asc("a")=97
  ; Asc("z")=122
  If asc_first_letter>=97 And asc_first_letter<=122
    chr_position_in_array=asc_first_letter-97+1
  EndIf
  statistics_word_per_character(chr_position_in_array)+1 
  
EndProcedure




; Shows the words based on the selected criteria, such as hyphen, 1 chr, 2 chrs, etc
;
; V1.0 - 08/OCT/2019
;        09/OCT/2019
;        11/OCT/2019
; V1.1 - 15/OCT/2019
;        o Estatistics: Extract with flags option (Shantanu Oak)
; V1.2 - 21/OCT/2019
;        o Improved selecting the words using cursors plus check if clicked in the same word.
; V1.3 - 06/NOV/2019
;        o Extra options for the statistics now has a larger width window+gadgets
; V1.4 - 21/NOV/2019
;        o Titlebar of Estatistics: Extract with flags improved showing the criterium.
; V1.5 - 05/JUN/2022
;        o Statistics now GTK3 ready
;
Procedure statistics_popup_menu_show()
  
  
    If GetGadgetState(#LISTICON_WINDOW_SHOW_STATISTICS)=-1
      MessageRequester("Warning!","No dictionary word type selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    t$=GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,GetGadgetState(#LISTICON_WINDOW_SHOW_STATISTICS),0)
    t$=Left(t$,Len(t$)-1)
    
    w=20+150+20+20+20-5+60+60+320+100-30-20-20+20+20+20+30
    h=3+200-50-20-20+100+100+25+10+20+2+5+16+4
    
    If OpenWindow(#WINDOW_SHOW_STATISTICS_POPUP_MENU,0,0,w,h,"Show words based on selected criteria: "+t$,#PB_Window_WindowCentered,WindowID(#WINDOW_SHOW_STATISTICS))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    DisableWindow(#WINDOW_SHOW_STATISTICS,#True)
    ListIconGadget(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU,10,15+-80-100+200-40-50-20-20+60-20+20+20+25-2,-40+20+20+20+60+60+320+100-30-50+5+20+20-60,2-2+100+20+5+10+20-3+100+80-15,"Matches",100+10+20+20+20+30+5,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU,1,"Position",100+10+5)
    AddGadgetColumn(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU,2,"Occurences",20+40+3+5+10+10) 
    SetGadgetFont(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU,FontID(#font_for_language_id))
    ButtonGadget(#BUTTON_SHOW_STATISTICS_POPUP_MENU_EXTRACT,10,GadgetY(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)+GadgetHeight(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)+5,100+10+10+10,default_gadget_height_button,"Extract")
    ButtonGadget(#BUTTON_SHOW_STATISTICS_POPUP_MENU_EXTRACT_WITH_FLAGS,10+GadgetWidth(#BUTTON_SHOW_STATISTICS_POPUP_MENU_EXTRACT)+10,GadgetY(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)+GadgetHeight(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)+5,100+10+10+10,default_gadget_height_button,"Extract with Flags")
    ButtonGadget(#BUTTON_SHOW_STATISTICS_POPUP_MENU_CLOSE,GadgetX(#BUTTON_SHOW_STATISTICS_POPUP_MENU_EXTRACT_WITH_FLAGS)+10+GadgetWidth(#BUTTON_SHOW_STATISTICS_POPUP_MENU_EXTRACT)+10+10,h-10-default_gadget_height_button,default_gadget_width_button,default_gadget_height_button,"Close")
    TextGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_TEXT,GadgetX(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)+GadgetWidth(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)+5,GadgetY(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU),100,default_gadget_height_text,"Preview")
    EditorGadget(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,GadgetX(#EDITOR_SHOW_STATISTICS_POPUP_MENU_TEXT),GadgetY(#EDITOR_SHOW_STATISTICS_POPUP_MENU_TEXT)+GadgetHeight(#EDITOR_SHOW_STATISTICS_POPUP_MENU_TEXT)+5,20+30+3+5+15+20+100+60+40,-2-3+-20+60+60+200,#PB_Editor_ReadOnly)
    SetGadgetFont(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,FontID(#font_for_language_id))
    ProgressBarGadget(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU,-30-20-25-30+GadgetWidth(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR)+5+20+20+20+60-18+20+10+100+20-10+150-20-5+10+20+10,-3+200-50-20-20+100+100-30+2+25+10+20,default_gadget_width_progress,default_gadget_height_small_progress,0,number_of_words)
    
    x=GadgetX(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU)
    y=GadgetY(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU)+GadgetHeight(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU)+1
    
    ; Time processing
    TextGadget(#TEXT_SHOW_STATISTICS_POPUP_MENU_TIME_TAKEN,x,y,100,default_gadget_height_text,"")
    
    y+default_gadget_height_text+2
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_SHOW_STATISTICS_POPUP_MENU,x,y,100,default_gadget_height_text,"")
    
    
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_STATISTICS_POPUP_MENU,#PB_Shortcut_Escape,1000)    
    
    
    type_of_word_selected_statistics=GetGadgetState(#LISTICON_WINDOW_SHOW_STATISTICS) ; To make it also work on the decoding procedure
    
    SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU,"<ESC> to abort.")
    
    AddWindowTimer(#WINDOW_SHOW_STATISTICS_POPUP_MENU,800,800)
    time_taken=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    For f=0 To number_of_words-1
  
      text$=StringField(words$(f),1,#LF$)
      
      counter=0
      
      Select type_of_word_selected_statistics
          
        Case 0          
          counter=statistics_word(f)\statistics_words_with_apos_counter
          
        Case 1
          counter=statistics_word(f)\statistics_words_with_hyphen_counter
          
        Case 2
          counter=statistics_word(f)\statistics_words_with_first_letter_uppercase_counter
          
        Case 3 To 7
          counter=statistics_word(f)\statistics_words_chr_counter1to5
          
      EndSelect
      
      If counter>0
        AddGadgetItem(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU,-1,text$+#LF$+Str(f+1)+#LF$+Str(counter))
      EndIf
      
      
      ; Fix To window stop responding If we click on it While in the loop, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_words,1))
        GadgetFit(#TEXT_SHOW_STATISTICS_POPUP_MENU_TIME_TAKEN)
        SetGadgetState(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU,f)
      EndIf
      
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          SetGadgetState(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU,0)
          Break
        EndIf
      EndIf
      
      
   Next f
   Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
   SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
   RemoveWindowTimer(#WINDOW_SHOW_STATISTICS_POPUP_MENU,800)
   SetGadgetState(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU,0)
   SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU,"")
   
   
   statistics_selected_word=-1
   
   
   cancel=#False
   okay=#False
   Repeat
     
     event=WaitWindowEvent()
     
     ; Pressed the close window gadget (Ubuntu)
     If event=#PB_Event_CloseWindow
       cancel=#True     
     EndIf
     
     
     ; Check if <ESC> was pressed
     If event=#PB_Event_Menu And EventMenu()=1000
       Debug "<ESC> was pressed"
       cancel=#True
     EndIf     
     
     
     Select Event
      Case #PB_Event_Gadget
        Select EventGadget()
            
          Case #BUTTON_SHOW_STATISTICS_POPUP_MENU_EXTRACT
            type_of_word_selected_statistics_extract_with_flags=#False
            statistics_popup_menu_extract()
            
          Case #BUTTON_SHOW_STATISTICS_POPUP_MENU_EXTRACT_WITH_FLAGS
            type_of_word_selected_statistics_extract_with_flags=#True
            statistics_popup_menu_extract()            
            
          Case #BUTTON_SHOW_STATISTICS_POPUP_MENU_CLOSE
          
            okay=#True
            
          Case #LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU 
            If (EventType()=#PB_EventType_LeftClick Or EventType()=#PB_EventType_Change) And GetGadgetState(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)<>-1 And GetGadgetState(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)<>statistics_selected_word
              Debug "Left-Click on the ListIconGadget"
              statistics_selected_word=GetGadgetState(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU)
              SetGadgetText(#EDITOR_SHOW_STATISTICS_POPUP_MENU_EDITOR,"")
              f=Val(GetGadgetItemText(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU,GetGadgetState(#LISTICONGADGET_SHOW_STATISTICS_POPUP_MENU),1))-1
              text$=StringField(words$(f),1,#LF$)
              If Left(text$,1)<>"/"
                type_of_word_selected_statistics_extract_with_flags=#True
                process_pfx_sfx(6,text$,#False,#False,#False,"","","",#False,#False)
              EndIf
            EndIf
            
        EndSelect
        
     EndSelect
     
   Until okay=#True Or cancel=#True
   
   CloseWindow(#WINDOW_SHOW_STATISTICS_POPUP_MENU)
   DisableWindow(#WINDOW_SHOW_STATISTICS,#False)
   
   
   
EndProcedure




; Exports the words based on the selected criteria, such as hyphen, 1 chr, 2 chrs, etc
;
; V1.0 - 09/OCT/2019
;        11/OCT/2019
; V1.1 - 21/DEC/2020
;        o Prefs: "0" disables above N words warning
;
Procedure statistics_popup_menu_extract()
  
  
    If GetGadgetState(#LISTICON_WINDOW_SHOW_STATISTICS)=-1
      MessageRequester("Warning!","No dictionary word type selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    If Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,GetGadgetState(#LISTICON_WINDOW_SHOW_STATISTICS),1))=0
      MessageRequester("Warning!","No dictionary words of that type.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    ; Ask to abort if over N words
    If Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,GetGadgetState(#LISTICON_WINDOW_SHOW_STATISTICS),1))>=ask_after_n_words And ask_after_n_words<>0     
      If MessageRequester("Warning!","Over "+FormatNumber(ask_after_n_words,0,".",number_separator_chr$)+" words have been decoded."+#LF$+#LF$+"This represents a huge amount of data in the HDD"+#LF$+"that can fill it up and crash the computer."+#LF$+#LF$+"Abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes     
        ProcedureReturn #False
      EndIf
    EndIf       
    
    
    type_of_word_selected_statistics=GetGadgetState(#LISTICON_WINDOW_SHOW_STATISTICS) ; To make it also work on the decoding procedure
    
    window_run=1 ; Statistics main window
    If IsWindow(#WINDOW_SHOW_STATISTICS_POPUP_MENU) : window_run=2 : EndIf ; Window is the show one
    
    t$="Text Documents (*.txt)|*.txt"
    tt$="Extract wordlist to file"
    file$=SaveFileRequester(tt$,"",t$,0)
    If file$=""
      ProcedureReturn #False
    EndIf
    
    
    ; Append file extension
    If LCase(Right(file$,4))<>".txt" : file$+".txt" : EndIf
    
    
    t=ReadFile(1,file$)
    If t<>0
      CloseFile(1)
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        ProcedureReturn #False
      EndIf      
    EndIf    
    DeleteFile(file$) ; Delete .TXT file    
    
    
    If window_run=1
      AddWindowTimer(#WINDOW_SHOW_STATISTICS,800,800)
    Else
      AddWindowTimer(#WINDOW_SHOW_STATISTICS_POPUP_MENU,800,800)
    EndIf
    time_taken=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()    
    If window_run=1
      SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_STATISTICS,0)
      SetGadgetText(#TEXT_WINDOW_SHOW_STATISTICS,"<ESC> to abort.")
    Else
      SetGadgetState(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU,0)
      SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU,"<ESC> to abort.")      
    EndIf
    OpenFile(1,file$)    
    WriteStringFormat(1,#PB_UTF8)    
    For f=0 To number_of_words-1
      
      text$=StringField(words$(f),1,#LF$)
      
      counter=0
      
      Select type_of_word_selected_statistics
          
        Case 0          
          counter=statistics_word(f)\statistics_words_with_apos_counter
          
        Case 1
          counter=statistics_word(f)\statistics_words_with_hyphen_counter
          
        Case 2
          counter=statistics_word(f)\statistics_words_with_first_letter_uppercase_counter
          
        Case 3 To 7
          counter=statistics_word(f)\statistics_words_chr_counter1to5

      EndSelect
      
      
      If counter>0
        If Left(text$,1)<>"/" : process_pfx_sfx(7,text$,#False,#False,#False,"","","",#False,#False) : EndIf
      EndIf
      
      
      ; Fix To window stop responding If we click on it While in the loop, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        If window_run=1
          SetGadgetText(#TEXT_WINDOW_SHOW_STATISTICS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_STATISTICS,f)
        Else
          SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          SetGadgetState(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU,f)
        EndIf
      EndIf
      
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          Break
        EndIf
      EndIf      
      
      
   Next f
   CloseFile(1)
   If window_run=1
      SetGadgetText(#TEXT_WINDOW_SHOW_STATISTICS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_STATISTICS,0)
      SetGadgetText(#TEXT_WINDOW_SHOW_STATISTICS,"")
   Else
      SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      SetGadgetState(#PROGRESSBAR_SHOW_STATISTICS_POPUP_MENU,0)
      SetGadgetText(#TEXT_SHOW_STATISTICS_POPUP_MENU,"")
   EndIf   
   Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)   
   If window_run=1
     RemoveWindowTimer(#WINDOW_SHOW_STATISTICS,800)
   Else
     RemoveWindowTimer(#WINDOW_SHOW_STATISTICS_POPUP_MENU,800)
   EndIf
   
EndProcedure




; Fix spaces in dictionary
;
; V1.0 - 16/FEB/2016
; V1.1 - 7/JUN/2017
;        o undo/redo support
; V1.2 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure fix_spaces_wordlist()
    Debug "Pressed FIX SPACES DICTIONARY"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fix Invalid Spaces... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; Processing wordlist - STEP 1 of 2
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
      
    
    word_fixes=0
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        If Left(text$,1)=" " : text$=Mid(text$,2) : EndIf
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And Left(text$,1)<>" " And Right(text$,1)<>" "
      
      ; Abort if an empty word is found
      If text$=""
        Debug "NULL WORD FOUND!"
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Null word at Line:"+Str(f+1)+"! Aborted.")
        RemoveWindowTimer(#WINDOW_MAIN,800)
        ProcedureReturn #False        
      EndIf
      
      
      ; Increment fixes
      If text$<>StringField(words$(f),1,#LF$)
        word_fixes+1        
      EndIf      
      
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Wordlist 1/2..."+Str(f)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; No fixes found
    If word_fixes=0
      MessageRequester("Warning!","No spaces to fix found!",#PB_MessageRequester_Warning)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR,0,"OK.")
      RemoveWindowTimer(#WINDOW_MAIN,800)
      ProcedureReturn #False      
    EndIf
    
    
    dictionary_store_undo(#True) ; 7/JUN/2017
    
    
    ; Fixing spaces - STEP 2 of 2    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2...0 <ESC> To abort.")
    
    word_fixes=0
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        If Left(text$,1)=" " : text$=Mid(text$,2) : EndIf
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And Left(text$,1)<>" " And Right(text$,1)<>" "
      
      ; Increment fixes + fix directly in the array
      If text$<>StringField(words$(f),1,#LF$)
        word_fixes+1
        dictionary_changed=#True
        t$=text$
        If FindString(words$(f),#LF$)>0
          t$+#LF$+StringField(words$(f),2,#LF$)
        EndIf
        words$(f)=t$
      EndIf

      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2..."+Str(f)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          global_update_virtual_vertical_scroll_bar(-1)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f    
   
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixed words:"+Str(word_fixes))  
    global_update_virtual_vertical_scroll_bar(-1)
    
        
EndProcedure




; This procedure removed the redundant code of decoding .AFF files and also has an ultrafast technique
; for finding the position of the rules, by storing in a string:
; "rule1,,position1,,rule2,,position2,," - Searching in a very small array is much faster than searching in a big .AFF
;
; Now the position is the array number - 27/APR/2018
;
; 7/MAR/2019: "rule1+chr(1)+rule2+chr(1) (...)" -> "PFX A Y"+chr(1)+"SFX B N"+chr(1) (...)
;
; New technique for "AFF Validate" that stores "PFX"/"SUF" + " " + "Y"/"N" + "FLAG" + "LINE_IN_AFF" + "NUMBER_RULES_IF_FLAG"  + chr(1) - 7/MAR/2020 + 10/MAR/2020
; Also, 7-MAR/2020 for the AFF VALIDATE:
;    ultrafast_aff_search_pfx_sfx_line_pfx_counter = number of prefixes found
;    ultrafast_aff_search_pfx_sfx_line_sfx_counter = number of suffixes found
;
; Also, ultrafast_aff_search_pfx_sfx_line_check_for_dupes$(ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter-1)
; stores the flags to search for dupes in the "AFF Validate" - 9/MAR/2020
;
; V1.0 - 17/JAN/2015
; V1.1 - 22/APR/2015
;        o Added the text while optimising the .AFF: "Optimising .AFF decode..."
;        o Optimised slightly the optimising of the .AFF
; V1.2 - 23/APR/2015
;        o Now there is a warning if we attempt to use (not supported) an .AFF with:
;          1) FLAG long  2) FLAG num
; V1.3 - 9/JUN/2015
;        o Added compatibility with:
;          1) FLAG long  2) FLAG num
;        o Optimised the decode of the .AFF
; V1.4 - 18/JAN/2016
;        o Decoding algorithm improved with minor speed gain (it now uses chr(1) INSTEAD of ",," in the ultrafast array
; V1.5 - 21/JAN/2016
;        o Simplified the code a bit
; V1.6 - 10/APR/2016
;        o Fix: Incorrect removing of spaces if spaces equals two while optimising the .AFF in the cache
;           While FindString(derived_aff_nospaces$,"  ")
;             derived_aff_nospaces$=RemoveString(derived_aff_nospaces$,"  ") >>>> derived_aff_nospaces$=ReplaceString(derived_aff_nospaces$,"  "," ") 
;           Wend
; V1.7 - 5/OCT/2016
;        o Fix: While opening a dictionary and after exiting the editor, the page length would be lost since
;               I was using the same variable for two different parts of the code.
; V1.8 - 20/MAR/2017
;        o Optimising the .AFF now shows a progress bar
;        o Simplified the .AFF optimising code
;        o Major speed gain in the .AFF optimising code (gl_ES)
; V1.9 - 4/APR/2017
;        o Support via coloured symbol for NOSUGGEST in words editor
; V2.0 - 30/MAR/2018
;        o Major speed gain in "Optimising .AFF decode cache" - up to ~30% faster
; V2.1 - 13/APR/2018
;        o Support for compressed AF
; V2.2 - 16/APR/2018
;        o NEEDAFFIX support - 16/APR/2018  
; V2.3 - 27/APR/2018
;        o Major Speed up of dictionary processing
; V2.4 - 18/MAY/2018
;        o New strategy for extra speed decoding the dictionary, but it takes more RAM.
; V2.5 - 14/JUN/2018
;        o Add/Edit dictionary: Image for NEEDAFFIX and FORBIDDENWORD
; V2.6 - 29/OCT/2018
;        o Coded the "CIRCUMFIX" flag.
; V2.7 - 07/MAR/2019
;        o Now the number of rules per affix is not stored in the string for smaller string and faster access and decoding
; V2.8 - 01/APR/2019
;        o Added support for "KEEPCASE"
; V2.9 - 16/JUL/2019
;        o Added support for "COMPLEXPREFIXES"
; V3.0 - 07/MAR/2020
;        09/MAR/2020
;        10/MAR/2020
;        o Added support for AFF VALIDATE
; V3.1 - 30/NOV/2021
;        o Started showing the time taken in timely operations.
; V3.2 - 14/APR/2022
;        o Optimised the .AFF decode cache
; V3.3 - 28/DEC/2022
;        o Processing the faster version of the .aff now shows %
; V3.4 - 29/DEC/2022
;        o Simplified code by using the common function: TrimStringAll()
;
Procedure faster_searchable_version_of_aff()
  
  t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
  If derived_aff$<>t$
        
    ; Support for three types of flags
    flag_char=1 ; normal 1 chr
    If FindString(t$,"FLAG long") : flag_char=2 : EndIf ; 2 chrs
    If FindString(t$,"FLAG num") : flag_char=3 : EndIf ; numbers variable chrs  
    
    time_taken=ElapsedMilliseconds()   
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Optimising .AFF decode cache...")
    
    derived_aff$=t$
    derived_aff_nospaces$=t$
    ReplaceString(derived_aff_nospaces$,Chr(9)," ",#PB_String_InPlace) ; replace tabs with spaces - 13/MAR/2017
    
    ; Remove the repeated spaces from the .aff using the function TrimStringAll() - 29/DEC/2022
    derived_aff_nospaces$=TrimStringAll(derived_aff_nospaces$)
    derived_aff_nospaces_no_eol$=derived_aff_nospaces$+#CRLF$
    derived_aff_nospaces_no_eol$=ConvertStringToUnix(derived_aff_nospaces_no_eol$) ; To convert to Linux Line endings    
    
    ; Convert the rules into the ultrafast search string - 17/JAN/2015 + EXTRA SPEED on 22/APR/2015
    ultrafast_aff_search$=""  
    ultrafast_aff_search_pfx_sfx_line$=""
    ultrafast_aff_search_pfx_sfx_line_pfx_counter=0
    ultrafast_aff_search_pfx_sfx_line_sfx_counter=0
    ultrafast_aff_search_pfx_sfx_line_check_for_dupes$(0)
    t$=derived_aff_nospaces_no_eol$
    t_char$=t$ ; to be used in the decoding of flags while calling the procedure for faster access
    t1=1       ;Variable that stores the position of the #LF$ 
    
    
    ; Counts the total of "PFX" and "SFX" to abort if they are all processed
    sum_total_pfx_and_sfx=CountString(t$,"PFX")+CountString(t$,"SFX")
    sum_pfx_and_sfx=0 ; The current sum in the loop to abort when it matches the total
    t1=1 ; current position
    number_of_LFs=CountString(t$,#LF$) ; total number of fields to parse
    
    ; Compressed AF
    AF_compressed_number=0
    Dim AF_compressed_array$(0)
    
    
    ; Ultrafast rules
    Dim rules_of_words_total(sum_total_pfx_and_sfx)
    Dim rules_of_words_position(sum_total_pfx_and_sfx)
    Dim rules_of_words_each$(sum_total_pfx_and_sfx,0)
    Dim rules_of_words_total_direct_memory_access(0)
    rules_counter=0 ; To point to the array, instead of the address
    rules_maximum_number=0 ; To ReDim the array of the rules
    rules_highest_number=0 ; For adding zeros before rules numbers exporting as .CSV
    
    ; Show the progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Optimising .AFF decode cache..."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_LFs)     
    
    AddWindowTimer(#WINDOW_MAIN,800,800)       
    For f=1 To number_of_LFs
      If sum_total_pfx_and_sfx=0 : Break : EndIf
      If sum_pfx_and_sfx=sum_total_pfx_and_sfx : Break : EndIf
        
      t_line$=StringField(t$,f,#LF$)
      pfxsfx$=StringField(t_line$,1," ")
      
      flag=#False
      If pfxsfx$="AF"
        t1+Len(t_line$)+1
        AF_compressed_number=Val(StringField(t_line$,2," "))
        Dim AF_compressed_array$(AF_compressed_number)
        For ff=1 To AF_compressed_number
          t_line$=StringField(t$,f+ff,#LF$)
          t_line$=ReplaceString(t_line$,Chr(9)," ")
          AF_compressed_array$(ff)=StringField(t_line$,2," ")
;           Debug "ff:"+Str(ff)+" AF_compressed_array$(ff)="+AF_compressed_array$(ff)
          If ff<>AF_compressed_number : t1+Len(t_line$)+1 : EndIf
        Next ff
        f+AF_compressed_number
        flag=#True
      EndIf
      
      If (pfxsfx$="PFX" Or pfxsfx$="SFX") And flag=#False
        sum_pfx_and_sfx+1
        yn$=StringField(t_line$,3," ")
        
        If yn$="Y" Or yn$="N"
          
          rules_counter+1
          
          ; For even more ultraspeed - 18/MAY/2018
          ultra_fast_position=Len(ultrafast_aff_search$)+1
          ReDim rules_of_words_total_direct_memory_access(ultra_fast_position)
          rules_of_words_total_direct_memory_access(ultra_fast_position)=rules_counter           
          
          ultrafast_aff_search$+StringField(t_line$,1," ")+" "+StringField(t_line$,2," ")+" "+StringField(t_line$,3," ")+Chr(1) ;+Str(rules_counter)+Chr(1) - 07/MAR/2019
          ultrafast_aff_search_pfx_sfx_line$+pfxsfx$+" "+yn$+" "+StringField(t_line$,2," ")+" "+Str(f)+" "+StringField(t_line$,4," ")+Chr(1)  ; New technique for "AFF Validate" that stores "PFX"/"SUF" + " " + "Y"/"N" + "FLAG" + "LINE_IN_AFF" + "NUMBER_RULES_IF_FLAG" + chr(1) - 7/MAR/2020 + 10/MAR/2020
          If pfxsfx$="PFX" : ultrafast_aff_search_pfx_sfx_line_pfx_counter+1 : EndIf ; New technique for "AFF Validate" that stores number of prefixes in optimising aff - 7/MAR/2020
          If pfxsfx$="SFX" : ultrafast_aff_search_pfx_sfx_line_sfx_counter+1 : EndIf ; New technique for "AFF Validate" that stores number of suffixes in optimising aff - 7/MAR/2020 
          ReDim ultrafast_aff_search_pfx_sfx_line_check_for_dupes$(ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter-1) ; Detect dupes in "AFF VAlidate" - 9/MAR/2020
          ultrafast_aff_search_pfx_sfx_line_check_for_dupes$(ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter-1)=StringField(t_line$,2," ") ; Detect dupes in "AFF VAlidate" - 9/MAR/2020
          rules_of_words_position(rules_counter)=t1
        
          ; Jump (ignore) the inside lines - Now it stores in an array for ultraspeed - 27/APR/2018
          number_of_lines_aff=Val(StringField(t_line$,4," "))
          rules_of_words_total(rules_counter)=number_of_lines_aff
          If number_of_lines_aff>rules_maximum_number
            rules_maximum_number=number_of_lines_aff
            ReDim rules_of_words_each$(sum_total_pfx_and_sfx,rules_maximum_number)
          EndIf
          For ff=1 To number_of_lines_aff
            ttt$=StringField(t$,f+ff,#LF$)
            rules_of_words_each$(rules_counter,ff)=ttt$
            t1+Len(ttt$)+1
          Next ff
          If rules_highest_number<number_of_lines_aff : rules_highest_number=number_of_lines_aff : EndIf
          f+number_of_lines_aff
          
        EndIf
      EndIf
      t1+Len(t_line$)+1
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event=WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Optimising .AFF decode cache..."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_LFs,1))
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf       
            
    Next f
    
    
    ; KEEPCASE support - 1/APR/2019
    keepcaseword$=""
    keepcaseword=#False
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"KEEPCASE")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      keepcaseword$=StringField(t_line$,2," ")
      keepcaseword=#True
    EndIf        
    
    
    ; Get the NOSUGGEST flag
    nosuggestword$=""
    nosuggestword_colour=#Black
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"NOSUGGEST")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      nosuggestword$=StringField(t_line$,2," ")
    EndIf    
    
    
    ; NEEDAFFIX support - 16/APR/2018
    needaffixword$=""
    needaffixword=#False
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"NEEDAFFIX")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      needaffixword$=StringField(t_line$,2," ")
      needaffixword=#True
    EndIf    
    
    ; FORBIDDENWORD support - 14/JUN/2018
    forbiddenword$=""
    forbiddenword=#False
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"FORBIDDENWORD")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      forbiddenword$=StringField(t_line$,2," ")
      forbiddenword=#True
    EndIf        
    
    ; COMPLEXPREFIXES support - 16/JUL/2019
    complexprefixes_code=#False
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"COMPLEXPREFIXES")
    If t>0 : complexprefixes_code=#True : EndIf  
    
    ; CIRCUMFIX support - 29/OCT/2018
    circumfix$=""
    circumfix=#False
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"CIRCUMFIX")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      circumfix$=StringField(t_line$,2," ")
      circumfix=#True
    EndIf            
    

    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    RemoveWindowTimer(#WINDOW_MAIN,800)
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 21/MAR/2014    

  EndIf
  
EndProcedure




; Add the help lines to the .aff aid combobox
;
; V1.0 - 6/JAN/2017
; V1.1 - 19/MAR/2017
;        o PREFS now accept external files with rules and flags
; V1.2 - 19/MAR/2017
;        o The custom Aff aid was showing the first line (language name) in the editor combobox.
; V1.3 - 05/FEB/2019
;        o Added new GB rule to .AFF Aid Language created by Ding-adong.
; V1.4 - 12/FEB/2019
;        o Replaced prefix flag "O" with flag "^" GB rule in .AFF Aid Language by Ding-adong.
; V1.5 - 08/MAR/2019
;        o Fixed description of two GB rules in .AFF Aid Language by Ding-adong.
; V1.6 - 14/MAR/2019
;        o Improved/updated description of flag "P"
; V1.7 - 10/JUL/2019
;        o Added improved GB rules to .AFF Aid Language.
;
Procedure populate_combobox_aff_aid(image,current_aff_aid_language)
  
  t$=""
  flag=image
  
  Select image ;aff_aid_language$
      
    Case 0 ;"n/a"
      ProcedureReturn
      
      
    Case 30
      
      flag=30 ; English flag
      
      t$=""
      t$+"A # re- prefix: to do again"
      t$+#LF$
      t$+"a # mis- prefix"
      t$+#LF$
      t$+"I # in- im- il- ir- prefixes for negation"
      t$+#LF$      
      t$+"c # over- prefix: extends"
      t$+#LF$      
      t$+"U # un- prefix: reverse the action"
      t$+#LF$      
      t$+"C # de- prefix: reverse"
      t$+#LF$
      t$+"E # dis- prefix for negation"
      t$+#LF$
      t$+"F # con- prefix"
      t$+#LF$
      t$+"K # pre- prefix"
      t$+#LF$
      t$+"e # out- prefix"
      t$+#LF$
      t$+"f # under- prefix"
      t$+#LF$
      t$+"^ # non- prefix"
      t$+#LF$
      t$+"4 # trans- prefix"
      t$+#LF$
      t$+"V # -ive endings for verbs"
      t$+#LF$
      t$+"v # -ively endings for verbs"
      t$+#LF$
      t$+"N # -ion and -ion's ends for stem verb creating action noun"
      t$+#LF$
      t$+"n # -ion, -ions and -ion's ends for stem verb creating action noun"
      t$+#LF$
      t$+"X # -ions ends for stem verb creating action noun plural"
      t$+#LF$ 
      t$+"x # -ional, -ionally ends for stem verb"
      t$+#LF$
      t$+"H # adds both -th and -fold suffixes"
      t$+#LF$
      t$+"Y # -ly adverb endings (for adjectives)"
      t$+#LF$
      t$+"G # -ing endings for verbs where the stress falls on the last syllable"
      t$+#LF$
      t$+"J # plural noun form of -ings verbs"
      t$+#LF$
      t$+"k # -ingly endings"
      t$+#LF$
      t$+"D # regular verb past tenses and past participle with -ed"
      t$+#LF$
      t$+"d # regular verb past tenses and past participle with -ed"
      t$+#LF$
      t$+"d # regular adverbial -ing endings."
      t$+#LF$
      t$+"h # -edly to form adverb from thing"
      t$+#LF$
      t$+"T # adjectival comparatives: big bigger biggest"
      t$+#LF$
      t$+"R # -er, -ers, -er's ending for verbs creating thing or person who does"
      t$+#LF$
      t$+"r # -er, -ers, -er's ending for verbs creating thing or person who does"
      t$+#LF$
      t$+"S # noun plurals and verb conjugation (I you he we you they)"
      t$+#LF$
      t$+"m # -man, -men, -man's, -men's suffixes"
      t$+#LF$
      t$+"5 # -woman, -women, -woman's, suffixes"
      t$+#LF$
      t$+"6 # -ful suffix"
      t$+#LF$
      t$+"j # -fully suffix"
      t$+#LF$      
      t$+"p # -less suffix"
      t$+#LF$
      t$+"Q # -ise, -ises, -ised, -ising"
      t$+#LF$ 
      t$+"8 # -ize, -izes, -ized, -izing"
      t$+#LF$
      t$+"q # -isation, -isations and -isation's, suffixes for nouns"
      t$+#LF$
      t$+"- # -ization, -izations and -ization's, suffixes for nouns"
      t$+#LF$
      t$+"s # -iser, -isers, -iser's, suffixes for nouns"
      t$+#LF$
      t$+"9 # -izer, -izers, -izer's, suffixes for nouns"
      t$+#LF$
      t$+"t # -isable, -isability, suffixes for nouns"
      t$+#LF$
      t$+"+ # -izable, -izability, suffixes for nouns"
      t$+#LF$
      t$+"M # possessive form"
      t$+#LF$   
      t$+"B # -able, -ability forms for words"
      t$+#LF$
      t$+"7 # -able, -ability forms for words where stem emphasis is NOT on last syllable"
      t$+#LF$
      t$+"g # -ability form for words where stem emphasis is NOT on last syllable"
      t$+#LF$
      t$+"l # -ably endings"
      t$+#LF$
      t$+"b # -ible endings"
      t$+#LF$
      t$+"L # -ment, -ments, -ment's suffixes"
      t$+#LF$
      t$+"Z # -y, diminutive and adjectival form"
      t$+#LF$
      t$+"P # -ness, -nesses, -ness's suffixes for adjectives"
      t$+#LF$      
      t$+"2 # -iness, -inesses, -iness's, diminutive and adjectival comparison form"
      t$+#LF$
      t$+"u # -iveness endings for verbs"
      t$+#LF$
      t$+"i # -edness,-ednesses,-edness's suffixes"
      t$+#LF$      
      t$+"= # -lessness, -lessnesses, -lessness's suffixes"
      t$+#LF$      
      t$+"z # -ily, adverbial ending for nouns where the adjective form adds y"
      t$+#LF$
      t$+"y # -ry, adjectival and noun forms"
      t$+#LF$
      t$+"O # -al, noun from verb"
      t$+#LF$
      t$+"W # -ic, adjectival ending"
      t$+#LF$
      t$+"o # -ally, adverb form from verb"
      t$+#LF$
      t$+"w # -ical, adjectival ending"
      t$+#LF$      
      t$+"1 # -ically, adverbial form ending"
      t$+#LF$
      t$+"3 # -ist, -ists, -ist's suffixes"
      t$+#LF$
      t$+"~ # -ish, adjectival ending"
      t$+#LF$
      
      
    Default
      t$=aff_aid_language_array_rules$(current_aff_aid_language)
      t$=Mid(t$,FindString(t$,#LF$)+1) ; Fix: 19/MAR/2017
      
          
  EndSelect
  
  
  ; Don't add anything to the combobox
  If t$="" : ProcedureReturn : EndIf
  
  
  ; Add all help to the combobox
  For f=1 To CountString(t$,#LF$)
    If IsImage(flag)
      AddGadgetItem(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,-1,StringField(t$,f,#LF$),ImageID(flag))
    Else
      AddGadgetItem(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,-1,StringField(t$,f,#LF$))
    EndIf
  Next f
  
EndProcedure




; To show the total of items selected
; 
; V1.0 - 22/DEC/2016
;
Procedure update_number_words_selected()
  
  t$="Selected:"
  If number_of_words_total_selected=0 : t$+"None" : EndIf
  If number_of_words_total_selected<>0 And number_of_words_total_selected<>number_of_words : t$+Str(number_of_words_total_selected) : EndIf
  If number_of_words_total_selected=number_of_words And number_of_words<>0 : t$+"All" : EndIf
  
  If GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_SELECTED)<>t$ : SetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_SELECTED,t$) : EndIf
    
EndProcedure




; To select ALL in the array
; 
; V1.0 - 22/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure update_number_words_selected_all()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  If number_of_words>0
    For f=0 To number_of_words-1
      words_selected(f)=#True
    Next f
  EndIf
  
  number_of_words_total_selected=number_of_words
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; To select NONE in the array
; 
; V1.0 - 22/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure update_number_words_selected_none()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  If number_of_words>0
    For f=0 To number_of_words-1
      words_selected(f)=#False
    Next f
  EndIf  
  
  number_of_words_total_selected=0
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; To INVERT in the array
; 
; V1.0 - 2/MAR/2018
;
Procedure update_number_words_selected_invert()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  count=0
  If number_of_words>0
    For f=0 To number_of_words-1
      v=words_selected(f)
      If v=#False
        v=#True
        count+1
      Else
        v=#False
      EndIf
      words_selected(f)=v
    Next f
  EndIf  
  
  number_of_words_total_selected=count
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; Count the number of selected words after a delete and such
;
; V1.0 - 23/DEC/2016
;
Procedure update_number_words_selected_count()
  
  number_of_words_total_selected=0
  
  If number_of_words>0
    For f=0 To number_of_words-1
      If words_selected(f)=#True : number_of_words_total_selected+1 : EndIf
    Next f
  EndIf    
  
EndProcedure




; Delete the number of selected words
;
; V1.0 - 24/DEC/2016
; V1.1 - 1/JAN/2017
;        o Fix deleting selected only when there was one item
; V1.2 - 7/JUN/2017
;        o undo/redo support
; V1.3 - 1/DEC/2017
;        o Message Requesters now have a sign. 
; V1.4 - 18/NOV/2018
;        o Replaced some more "words" with "dictionary words"
;
Procedure update_number_words_selected_delete()
  
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  
  If number_of_words_total_selected=0
    MessageRequester("Warning!","No selected dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  
  t$=""
  If number_of_words_total_selected>1 : t$="s" : EndIf
  If MessageRequester("Warning!","Really delete "+Str(number_of_words_total_selected)+" dictionary word"+t$+"?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No  
    ProcedureReturn
  EndIf
  
  
  dictionary_store_undo(#False) ; 7/JUN/2017
  
  
  Dim buffer$(number_of_words-1)
  
  counter=0
  position=0
  For f=0 To number_of_words-1
    If words_selected(f)=#False
      buffer$(position)=words$(f)
      position+1
    EndIf
    If words_selected(f)=#True
      counter+1
      words_selected(f)=#False
    EndIf
  Next f
  
  CopyArray(buffer$(),words$())
  number_of_words-counter  
  zero_items=#False
  If number_of_words=0
    zero_items=#True
    number_of_words=1
  EndIf
  ReDim words$(number_of_words-1)
  ReDim words_selected(number_of_words-1)
  
  number_of_words_total_selected=0
  
  SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  selected_dictionary_word=-1 ; Reset the current word selected
  
  ; Fix deleting only when there was one item
  If zero_items=#True : number_of_words=0 : EndIf
  
  dictionary_changed=#True
  update_number_words()
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; To clone words from the dictionary
;
; V1.0 - 18/MAR/2017
; V1.1 - 7/JUN/2017
;        o undo/redo support
; V1.2 - 1/DEC/2017
;        o Message Requesters now have a sign. 
; V1.3 - 3/APR/2018
;        o Support for hyphenation
; V1.4 - 18/NOV/2018
;        o Replaced some more "words" with "dictionary words"
;
Procedure clone_word()
  
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    
    t=selected_dictionary_word
    If t=-1
      MessageRequester("Warning!","No dictionary word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf  
    
    
    dictionary_store_undo(#False) ; 7/JUN/2017
    
    
    dictionary_changed=#True
    ReDim words$(number_of_words)        
    ReDim words_selected(number_of_words)        
    words$(number_of_words)=words$(t)
    number_of_words+1
    SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_words()
    global_update_virtual_vertical_scroll_bar(-1)        
    update_vertical_scroll_bar_dictionary_current_item()
    
    ; Hyphenation support - 3/APR/2018
    SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_hyphenations()
    global_update_virtual_vertical_scroll_bar(2)        
    update_vertical_scroll_bar_hyphenation_current_item()    
    
EndProcedure




; Disable undo + redo
;
; V1.0 - 7/JUN/2017
;
Procedure dictionary_disable_undo_redo()
  
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#True)
  Dim dictionary_undo_array(0)
  Dim dictionary_undo_array_total_selected(0)    
  dictionary_undo=#False
  dictionary_undo_number_items=0
  dictionary_undo_selected_item=-1
  dictionary_undo_items_total_selected=0
  dictionary_undo_top_of_listicongadget=0
  
  
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#True)    
  Dim dictionary_redo_array(0)
  Dim dictionary_redo_array_total_selected(0)
  dictionary_redo=#False
  dictionary_redo_number_items=0
  dictionary_redo_selected_item=-1
  dictionary_redo_items_total_selected=0
  dictionary_reddo_top_of_listicongadget=0  
  
EndProcedure




; Store undo
;
; V1.0 - 07/JUN/2017
; V1.1 - 23/MAR/2022
;        o Disable main GUI for input/output operations
;
Procedure dictionary_store_undo(disable_gadgets)
  Debug "STORE UNDO"
  
  ; Enable undo button and disable redo button
  If disable_gadgets=#False
    DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#False)
    DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#True)  
  EndIf  
    
  dictionary_undo=#True
  dictionary_redo=#False
  
  Dim dictionary_undo_array$(ArraySize(words$()))
  CopyArray(words$(),dictionary_undo_array$())
  Dim dictionary_undo_array_total_selected(ArraySize(words_selected()))  
  CopyArray(words_selected(),dictionary_undo_array_total_selected())
  dictionary_undo_number_items=number_of_words
  dictionary_undo_selected_item=selected_dictionary_word
  dictionary_undo_items_total_selected=number_of_words_total_selected
  dictionary_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
EndProcedure




; Restore undo
;
; V1.0 - 7/JUN/2017
; V1.1 - 3/APR/2018
;        o Support for hyphenation
;
Procedure dictionary_restore_undo()
  Debug "RESTORE UNDO"
  
  
  ; Disable undo button and enable redo button
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#True)
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#False)    
  
  dictionary_undo=#False
  dictionary_redo=#True
  
  Dim words$(ArraySize(dictionary_undo_array$()))
  CopyArray(dictionary_undo_array$(),words$())
  Dim words_selected(ArraySize(dictionary_undo_array_total_selected()))  
  CopyArray(dictionary_undo_array_total_selected(),words_selected())
  number_of_words=dictionary_undo_number_items
  selected_dictionary_word=dictionary_undo_selected_item
  number_of_words_total_selected=dictionary_undo_items_total_selected
;   dictionary_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_words_selected_count()
      
      
  ; Update number of selected words
  update_number_words_selected()
      
      
  ; Update number of words
  update_number_words()
  update_number_hyphenations() ; Hyphenation support - 3/APR/2018
      
      
  ; Reset dictionary
  selected_dictionary_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  global_update_virtual_vertical_scroll_bar(-1)
             
  selected_dictionary_word=dictionary_undo_selected_item
  
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,dictionary_undo_top_of_listicongadget)
  global_update_virtual_vertical_scroll_bar(-1)
  
  ; Hyphenation support - 3/APR/2018
  selected_hyphenation_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  global_update_virtual_vertical_scroll_bar(2)  
  
EndProcedure




; Store redo
;
; V1.0 - 7/JUN/2017
;
Procedure dictionary_store_redo()
  Debug "STORE REDO"
  
  ; Enable redo button and disable undo button
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#True)
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#False)  
  
  dictionary_undo=#False
  dictionary_redo=#True
  
  Dim dictionary_redo_array$(ArraySize(words$()))
  CopyArray(words$(),dictionary_redo_array$())
  Dim dictionary_redo_array_total_selected(ArraySize(words_selected()))  
  CopyArray(words_selected(),dictionary_redo_array_total_selected())
  dictionary_redo_number_items=number_of_words
  dictionary_redo_selected_item=selected_dictionary_word
  dictionary_redo_items_total_selected=number_of_words_total_selected
  dictionary_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
EndProcedure




; Restore redo
;
; V1.0 - 7/JUN/2017
; V1.1 - 3/APR/2018
;        o Support for hyphenation
;
Procedure dictionary_restore_redo()
  Debug "RESTORE REDO"
  
  
  ; Disable redo button and enable undo button
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#False)
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#True)    
  
  dictionary_undo=#True
  dictionary_redo=#False
  
  Dim words$(ArraySize(dictionary_redo_array$()))
  CopyArray(dictionary_redo_array$(),words$())
  Dim words_selected(ArraySize(dictionary_redo_array_total_selected()))  
  CopyArray(dictionary_redo_array_total_selected(),words_selected())
  number_of_words=dictionary_redo_number_items
  selected_dictionary_word=dictionary_redo_selected_item
  number_of_words_total_selected=dictionary_redo_items_total_selected
;   dictionary_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_words_selected_count()
      
      
  ; Update number of selected words
  update_number_words_selected()
      
      
  ; Update number of words
  update_number_words()
  update_number_hyphenations() ; Hyphenation support - 3/APR/2018
      
      
  ; Reset dictionary
  selected_dictionary_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  global_update_virtual_vertical_scroll_bar(-1)
  
  selected_dictionary_word=dictionary_redo_selected_item
      
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,dictionary_redo_top_of_listicongadget)
  global_update_virtual_vertical_scroll_bar(-1)
  
  ; Hyphenation support - 3/APR/2018
  selected_hyphenation_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  global_update_virtual_vertical_scroll_bar(2)  
  
EndProcedure




; Import the places/people names into the .dic
;
; Automatise the process to save months of work
;
; V1.0 - 7/AUG/2018
;        8/AUG/2018
; V1.1 - 31/AUG/2018
;        o Added two removal strings to the "Import places/people names adding possessives" (customisable)
; V1.2 - 10/OCT/2018
;        o Import the places/people names now accepts empty dictionaries
; V1.3 - 16/OCT/2018
;        o Linux: Can now close the window in the gadget.
; V1.4 - 4/JUN/2019
;        o GTK3 Ready
; V1.5 - 20/JUN/2019
;        o Preferences allow to choose the font for the language. 
;        o People and Places names import now shows "<ESC> to abort." during scan.
; V1.6 - 03/MAY/2020
;        o Names/Places import window now has a go to bottom button
; V1.7 - 04/MAY/2020
;        o Names/Places import window now has a go to top button
; V1.8 - 18/JUN/2020
;        o "Import places/people names" now has a "Clear" button
; V1.9 - 10/NOV/2020
;        o Clear buttons now use a trashcan image.
; V2.0 - 17/NOV/2020
;        o F1 coded    
;
Procedure import_places_people_names()
      
    If OpenWindow(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,10+get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120+10,"Import places/people names with possessives",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)    
    
    ; EditorGadget in
    EditorGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,10,10,-15+100+400+get_extra_width(50),-10+400+get_extra_height(100))
    GadgetToolTip(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,"Paste here the full text with the names.")
    SetGadgetFont(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,FontID(#font_for_language_id))
    
    ; EditorGadget out
    EditorGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,10+GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+GadgetWidth(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+10,10,-15+100+400+get_extra_width(50),-10+400+get_extra_height(100))
    GadgetToolTip(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,"Output to be placed in log file.")
    SetGadgetFont(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,FontID(#font_for_language_id))
    
    ; CLOSE  button
    x=WindowWidth(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES)-90-10
    y=WindowHeight(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES)-25-10-4
    ButtonGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE,x,y,90,default_gadget_height_button,"Close")
    
    x=GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)
    y=GadgetY(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+GadgetHeight(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+10
    
    ; Process button
    ButtonGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS,x,y,90,default_gadget_height_button,"Process")
    
    ; Clear button
    ButtonImageGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_CLEAR,GadgetX(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS)+GadgetWidth(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS)+16-4-2,y,default_gadget_height_button,default_gadget_height_button,ImageID(101))
    GadgetToolTip(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_CLEAR,"Clear")
  
    ; BOTTOM button IN gadget
    ButtonImageGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM,GadgetX(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_CLEAR)+GadgetWidth(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_CLEAR)+16+16-8,y,default_gadget_height_button,default_gadget_height_button,ImageID(37))    
    GadgetToolTip(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM,"Go to bottom.")
    
    ; TOP button IN gadget
    ButtonImageGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_TOP,GadgetX(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM)+GadgetWidth(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM)+16-4-2,y,default_gadget_height_button,default_gadget_height_button,ImageID(94))
    GadgetToolTip(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_TOP,"Go to top")
    
    
    y+25+5+5+5
    
    ; Remove
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_REMOVE,x,y,100,default_gadget_height_text,"       Remove:")
    GadgetFit(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_REMOVE)
    x+GadgetWidth(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_REMOVE)+5-2-1
    StringGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1,x,y,25,default_gadget_height_string,import_places_people_names_remove1$)
    SetGadgetFont(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1,FontID(#font_for_language_id))
    x+GadgetWidth(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1)+5
    StringGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2,x,y,25,default_gadget_height_string,import_places_people_names_remove2$)
    SetGadgetFont(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2,FontID(#font_for_language_id))

    
    x=GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)
    
    y+25+3+2
    
    y+2
    
    y+5
    
    ; Possessives flag
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,x,y,100,default_gadget_height_text,"Possessives:")
    GadgetFit(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)
    StringGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,GadgetX(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)+GadgetWidth(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)+2,y,40+10,default_gadget_height_string,"/M")
    SetGadgetFont(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,FontID(#font_for_language_id))
    
    y+25+3+2
    
    y+4
    
    y+5
    
    ; Processing X of Y
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING,x,y,200,default_gadget_height_text,"Processing: 0/0")
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_ESC_TO_ABORT,GadgetX(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING),GadgetY(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING)+GadgetHeight(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING)+1,100,default_gadget_height_text,"")       

    
    
    x=GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)
    y=GadgetY(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)+GadgetHeight(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)+5+3
    
    
    ; Number of names added
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL,x,y,200,default_gadget_height_text,"Found:0")
    GadgetFit(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL)
    
    
    ; BOTTOM button OUT gadget
    ButtonImageGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM2,25+30+20+200+GadgetX(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL),GadgetY(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL),default_gadget_height_button,default_gadget_height_button,ImageID(37))        
    GadgetToolTip(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM2,"Go to bottom.")
    
    ; TOP button OUT gadget
    ButtonImageGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_TOP2,GadgetX(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM2)+GadgetWidth(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM2)+16-4-2,GadgetY(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM2),default_gadget_height_button,default_gadget_height_button,ImageID(94))
    GadgetToolTip(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_TOP2,"Go to top")
    
    
    y+20
    
    ; Index starting at
    CheckBoxGadget(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,x,y,100,default_gadget_height_checkbox,"Log Index Starting at:")
    GadgetFit(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)
    SpinGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,x+GadgetWidth(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)+2,y,100,default_gadget_height_spin,1,9999999,#PB_Spin_Numeric)
    SetGadgetText(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,"1")
    GadgetToolTip(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,"For log file.")
    Global index_starting_at=#True
    SetGadgetState(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,index_starting_at)
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,#PB_Shortcut_Escape,1000) 
    AddKeyboardShortcut(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,#PB_Shortcut_F1,1020)
    
    close=#False
    Repeat
      
      event=WaitWindowEvent()
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_ESC_TO_ABORT,"")
        close=#True
      EndIf
      
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And EventMenu()=1020
        Debug "F1 pressed"
        html_help("#5.2.5.3.import_places_names_possessives")
      EndIf
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        close=#True     
      EndIf   
      
      
      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
            Case #CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING                
              index_starting_at=GetGadgetState(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)
              If index_starting_at=#True
                DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#False)
              Else
                DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#True)
              EndIf
              
            Case #STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1
              import_places_people_names_remove1$=GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1)
              
            Case #STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2  
              import_places_people_names_remove2$=GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2)
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE
              close=#True
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS
              DisableGadget(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#True)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS,#True)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE,#True)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,#True)
              DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#True)
              DisableGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,#True)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1,#True)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2,#True)
              ClearGadgetItems(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)
              SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_ESC_TO_ABORT,"<ESC> to abort.")
              import_places_people_names_process()   
              SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_ESC_TO_ABORT,"")
              DisableGadget(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#False)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS,#False)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE,#False)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,#False)
              If index_starting_at=#True : DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#False) : EndIf
              DisableGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,#False)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1,#False)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2,#False)      
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_CLEAR
              SetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,"")
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM
              SetPosBottom(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,1)
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_TOP
              SetPosTop(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,1)              
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_BOTTOM2
              SetPosBottom(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,1)              
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_TOP2
              SetPosTop(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,1)                 
              
          EndSelect
          
      EndSelect
      
    Until close=#True
    
    
    ; Dictionary support
    SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_words()
    global_update_virtual_vertical_scroll_bar(-1)  
        
    ; Hyphenation support
    SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_hyphenations()
    global_update_virtual_vertical_scroll_bar(2)
        

    update_number_words()
    update_number_hyphenations()
    update_vertical_scroll_bar_dictionary_current_item()
    update_vertical_scroll_bar_hyphenation_current_item()
    
    
    CloseWindow(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES)
    DisableWindow(#WINDOW_MAIN,#False)  
    
EndProcedure




; Process the proper/places names
;
; V1.0 - 07/AUG/2018
;        08/AUG/2018
; V1.1 - 31/AUG/2018
;        o Added two removal strings to the "Import places/people names adding possessives" (customisable)
; V1.2 - 03/SEP/2018
;        o Import places/people names now also removes the chrs "+" and "%" and replaces "–" with "-"
; V1.3 - 15/NOV/2018
;        o Import places/people names now also removes the chrs "<", ">", " - ", " − " and " — "
; V1.4 - 13/JUL/2022
;        o Fix: Wordlists importing would cause refresh of GUI
;
Procedure import_places_people_names_process()
  
  places_people_names$=GetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)
  
  places_people_names$=ConvertStringToUnix(places_people_names$)
  
  If Right(places_people_names$,1)<>#LF$ : places_people_names$+#LF$: EndIf ; Always end with a blank line
  
  places_people_names$=ReplaceString(places_people_names$,"+"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"%"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"–","-")
  
  places_people_names$=ReplaceString(places_people_names$,","," ")
  
  places_people_names$=ReplaceString(places_people_names$,"."," ")
  
  places_people_names$=ReplaceString(places_people_names$,"&"," ")
  
  places_people_names$=ReplaceString(places_people_names$,Chr(9)," ")
  
  places_people_names$=ReplaceString(places_people_names$,"("," ")
  
  places_people_names$=ReplaceString(places_people_names$,")"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"["," ")
  
  places_people_names$=ReplaceString(places_people_names$,"]"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"{"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"}"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"/"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"*"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"?"," ")
  
  places_people_names$=ReplaceString(places_people_names$,":"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"!"," ")
  
  places_people_names$=ReplaceString(places_people_names$,";"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"|"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"--","-") ; Fix the parish that has two -'s
  
  places_people_names$=ReplaceString(places_people_names$,"0"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"1"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"2"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"3"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"4"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"5"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"6"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"7"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"8"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"9"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"<"," ")
  
  places_people_names$=ReplaceString(places_people_names$,">"," ")
  
  places_people_names$=ReplaceString(places_people_names$," - "," ")
  
  places_people_names$=ReplaceString(places_people_names$," − "," ")
  
  places_people_names$=ReplaceString(places_people_names$," — "," ")
  
  If import_places_people_names_remove1$<>"" : places_people_names$=ReplaceString(places_people_names$,import_places_people_names_remove1$," ") : EndIf
  
  If import_places_people_names_remove2$<>"" : places_people_names$=ReplaceString(places_people_names$,import_places_people_names_remove2$," ") : EndIf
  
  places_people_names$=ReplaceString(places_people_names$,"  "," ")
  
  If Len(places_people_names$)=0 Or places_people_names$=" " Or places_people_names$=#LF$
    MessageRequester("Warning!","Nothing to process.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
  
  
  possessive_flag$=GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES) ; /M
  
  undo_redo=#False ; Changes to #True if added to undo (new proper name) just used once
  
  counter=0 ; Number of names/places added
  
  editor_gadget_fix_refreshing$=GetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)
  
  AddWindowTimer(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,800,800)
  total_lines=CountString(places_people_names$,#LF$)
  For lines=1 To total_lines
    
    t$=StringField(places_people_names$,lines,#LF$)
    t$+" "
    total_spaces=CountString(t$," ")
    
    
    For import_words=1 To total_spaces
      
      name$=StringField(t$,import_words," ")
      
      If name$<>"" And Left(name$,1)=UCase(Left(name$,1)) And Left(name$,1)<>"-" And Right(name$,1)<>"-"
        
        found=#False ; So that it adds the new proper name + undo if #False
        
        For dictionary_words=0 To number_of_words-1
          
          If FindString(words$(dictionary_words),name$+possessive_flag$,1,#PB_String_CaseSensitive)
            found=#True
            Break  
          EndIf
          
          
          ; <ESC> pressed to abort
          If EventMenu()=1000
            If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
              RemoveWindowTimer(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,800) 
              
              ; redo
              If undo_redo=#True
                dictionary_restore_undo()
                undo_redo=#False
              EndIf              
              
              ; Fix always refreshing - 13/JUL/2022
              SetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,editor_gadget_fix_refreshing$)
              
              ProcedureReturn #False
            EndIf
          EndIf          
          
          
          ; Fix to window stop responding if we click on it while in the loop, plus update the progress bar
          event = WindowEvent()
          If event=#PB_Event_Timer And EventTimer()=800
            SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING,"Processing: "+Str(lines)+"/"+Str(total_lines))
            SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL,"Found:"+Str(counter))
            GadgetFit(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL)
            
            ; Fix always refreshing - 13/JUL/2022
            SetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,editor_gadget_fix_refreshing$)
          EndIf          
          
          
        Next dictionary_words
        
        ; Name not found, add it
        If found=#False
            
          ; undo
          If undo_redo=#False
            dictionary_store_undo(#False)
            undo_redo=#True
          EndIf
            
          ReDim words$(number_of_words)        
          ReDim words_selected(number_of_words)        
          words$(number_of_words)=name$+possessive_flag$
          number_of_words+1            
        
          ; Add to the OUT EditorGadget
          index$=""
          If index_starting_at=#True : index$=Str(GetGadgetState(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)+counter)+") " : EndIf
          editor_gadget_fix_refreshing$+index$+name$+GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)+#LF$
          ; 2 lines below removed to fix refreshing issue - 13/JUL/2022
;           ttt$=GetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)+index$+name$+GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)+#LF$
;           SetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,ttt$)
          counter+1
            
          dictionary_changed=#True
            
        EndIf        
        
      EndIf
      
    Next import_words
    
    
  Next lines
  RemoveWindowTimer(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,800)
  
  ; Fix always refreshing - 13/JUL/2022
  SetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,editor_gadget_fix_refreshing$)
  
  SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING,"Processing: "+Str(lines-1)+"/"+Str(total_lines)+" Done!")
  SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL,"Found:"+Str(counter))  
  GadgetFit(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL)
  
EndProcedure




; Checks inside a rule if it has the CIRCUMFIX FLAG
; Uses the variables: needcircumfix_code_inside_up & needcircumfix_code_inside_down
;
; Remember to remove the variable everywhere: circumfix_twofold_counter
;
; 14/MAY/2020 - Changed the logic of needcircumfix_code_inside:
; needcircumfix_code_inside_up=0 or 1 (up)
; needcircumfix_code_inside_down=0 or 1 (down - recursivity)
;
; Top:
; =0 word doesn't have CIRCUMFIX flag
; =1 word HAS CIRCUMFIX flag
;
; Down;
; =0 word doesn't have CIRCUMFIX flag
; =1 word HAS CIRCUMFIX flag 
;
;
; V1.0 - 20/JAN/2019
; V1.1 - 09/FEB/2019
;        o Now gets each code by calling a procedure for the task
; V1.2 - 27/FEB/2019
;        o Optimisation in affix decoding if not using the CIRCUMFIX flag
; V1.3 - 24/MAR/2020
;        10/APR/2020
;        11/APR/2020
;        o Fixed the CIRCUMFIX flag
; V1.4 - 16/APR/2020
;        o Procedure is_circumfix() now sets circumfix_code2 if CIRCUMFIX flag found
; V1.5 - 18/APR/2020
;        o Better handling of circumfix_twofold_counter
; V1.6 - 29/APR/2020
;        o Better approach for reseting the needcircumfix_code_inside variable
; V1.7 - 30/APR/2020
;        o Now sets circumfix_twofold_counter=0 & needcircumfix_code_inside=0 if only one CIRCUMFIX flag found
; V1.8 - 10/MAY/2020
;        14/MAY/2020
;        o Improved the is_circumfix() procedure to deal with the new logic of CIRCUMFIX
;
Procedure is_circumfix(t$,recursivity)
  
  ; Exits if CIRCUMFIX flag not used - faster - 27/FEB/2019 + 10/MAY/2020
  If circumfix$=""
    needcircumfix_code_inside_up=#False
    needcircumfix_code_inside_down=#False
    ProcedureReturn
  EndIf
  
  
 ; No flags in word - 10/MAY/2020 + 14/MAY/2020
  word_codes$=t$
  If word_codes$=""
    Select recursivity
        
      Case 0
        needcircumfix_code_inside_up=#False
        needcircumfix_code_inside_down=#False
        ProcedureReturn
        
      Case 1        
        needcircumfix_code_inside_down=#False
        ProcedureReturn
        
    EndSelect
  EndIf
  
  
  ; For utf-8/chr,long AND num
  len_word_codes=Len(word_codes$)  
  If flag_char=2
    len_word_codes=Round(Len(word_codes$)/2.0,#PB_Round_Up)
  EndIf
  If flag_char=3
    len_word_codes=CountString(word_codes$,",")+1
  EndIf
  

  ; Try to find a CIRCUMFIX flag
  For f=1 To len_word_codes                      
    
    faster_word_codes$=get_faster_word_codes(word_codes$,f)
        
    ; Found the CIRCUMFIX flag
    ; Improved the code on 10/MAY/2020 + 14/MAY/2020
    If faster_word_codes$=circumfix$
      
      circumfix_code2=#True ; Found CIRCUMFIX code while decoding to turn on the GUI icon
      
      Select recursivity
        
        Case 0
          needcircumfix_code_inside_up=#True
          needcircumfix_code_inside_down=#False
          ProcedureReturn
        
        Case 1      
          needcircumfix_code_inside_down=#True        
          ProcedureReturn
        
      EndSelect      
      
    EndIf  
      
  Next f
  
  
  ; Not found CIRCUMFIX flag  
  Select recursivity
        
    Case 0
      needcircumfix_code_inside_up=#False
      needcircumfix_code_inside_down=#False  
      ProcedureReturn
        
    Case 1        
      needcircumfix_code_inside_down=#False           
      ProcedureReturn
       
  EndSelect
      
EndProcedure




; Checks inside a rule if it has the NEEDAFFIX FLAG
;
; V1.0 - 26/SEP/2019
; V1.1 - 17/MAR/2020
;        o Changed to work with local NEEDAFFIX FLAG
; V1.2 - 17/MAR/2020
;        o Optimised the is_needaffix() function: if there is no NEEDAFFIX FLAG, it returns #False
;
Procedure is_needaffix(t$)
  
  
  ; Exits if NEEDAFFIX flag not used - faster
  If needaffixword$="" : ProcedureReturn #False : EndIf
  
   
  word_codes$=StringField(t$,2,"/")
  If word_codes$="" : ProcedureReturn #False : EndIf
  
  
  ; For utf-8/chr,long AND num
  len_word_codes=Len(word_codes$)  
  If flag_char=2
    len_word_codes=Round(Len(word_codes$)/2.0,#PB_Round_Up)
  EndIf
  If flag_char=3
    len_word_codes=CountString(word_codes$,",")+1
  EndIf
  
  
  ; Try to find a NEEDAFFIX flag
  For f=1 To len_word_codes                      
    
    faster_word_codes$=get_faster_word_codes(word_codes$,f)
        
    ; Found the NEEDAFFIX flag
    If faster_word_codes$=needaffixword$
      ProcedureReturn #True
    EndIf      
    
  Next f
  
  
  ; Not found NEEDAFFIX flag
  ProcedureReturn #False
  
  
EndProcedure




; To simplify getting each code for a word
;
; V1.0 - 09/FEB/2019
;
Procedure.s get_faster_word_codes(word_codes$,position)
  
    If flag_char=1  
      ProcedureReturn Mid(word_codes$,position,1)
    EndIf
    If flag_char=2
      ProcedureReturn Mid(word_codes$,(position-1)*2+1,2)
    EndIf
    If flag_char=3
      ProcedureReturn StringField(word_codes$,position,",")
    EndIf
   
EndProcedure




; "Show/Merge/Delete duplicates .dic"
;
; V1.0 - 13/FEB/2019
;        14/FEB/2019
;        15/FEB/2019
;        21/FEB/2019
;        22/FEB/2019
;        25/FEB/2019
;        20/MAR/2019
; V1.1 - 22/MAR/2019
;        o "Show/Merge/Delete duplicates .dic": Small GUI fix for Linux.
; V1.2 - 28/MAR/2019
;        31/MAR/2019
;        01/APR/2019
;        24/MAY/2019
;        o Continued coding it
; V1.3 - 11/JUN/2019
;        o GTK3 Ready
; V1.4 - 16/JUN/2019
;        o "Show/Merge/Delete duplicates .dic" now shows "<ESC> to abort." during scan.
; V1.5 - 18/JUN/2019
;        o Fixed: "Show/Merge/Delete duplicates .dic" variable in function used by value
; V1.6 - 20/JUN/2019
;        o Preferences allow to choose the font for the language. 
; V1.7 - 16/JUL/2019
;        o Added support for "COMPLEXPREFIXES"
; V1.8 - 15/FEB/2020
;        o All file operations support BOM.
; V1.9 - 21/FEB/2020
;        o All remaining export operations now use UNIX #LF$
; V2.0 - 05/MAY/2020
;        o "Show/Merge/Delete duplicates .dic" now has a remember button
; V2.1 - 18/MAY/2020
;        o In the merge flags in .dics there is now an EditorGadget and buttons
; V2.2 - 12/OCT/2020
;        o Adding of F1 to open the help in the various windows.
; V2.3 - 10/NOV/2020
;        o Clear buttons now use a trashcan image.
; V2.4 - 26/NOV/2020
;        o Clear now restores header text of Editor Gadget
; V2.5 - 12/JAN/2021
;        o "Show/Merge/Delete duplicates .dic" now has the "Process" + "Export" button disabled if results=0
; V2.6 - 25/NOV/2021
;        o Started showing the time taken in timely operations.
; V2.7 - 20/APR/2023
;        21/APR/2023
;        22/APR/2023
;        23/APR/2023
;        24/APR/2023
;        o Support for repeated flags
;        o Shows progress % while scanning/processing
; V2.8 - 02/MAY/2023
;        o Improved the GUI
;
Procedure show_merge_duplicates_dic(show_merge_delete_duplicates_dic_placed_on_undo_)

    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf      
    
    
    show_merge_delete_duplicates_dic_placed_on_undo=show_merge_delete_duplicates_dic_placed_on_undo_
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    ; In this procedure it is used to differenciate from flags merging
    faster_searchable_version_of_aff()    
    
    w=get_extra_width(100)+(1024-800)+(800-640)+640-10
    h=5+get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120
    
    t$="Show/Merge/Delete duplicates .DIC"
    If OpenWindow(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,0,0,w,h,t$,#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore)
    w=WindowWidth(#WINDOW_SHOW_MERGE_DUPLICATES_DIC)
    h=WindowHeight(#WINDOW_SHOW_MERGE_DUPLICATES_DIC)
    DisableWindow(#WINDOW_MAIN,#True)    
    
    
    ; ListIconGadget with the repeated words
    ListIconGadget(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,10+40+5-20-10,200-40-50-20-20+60-20+20+20-40-5-40-40,150-30+5+20+20+30-60-40-40-10-10-10-40-40-40-60-60-40-40+10+20+get_extra_width(60)+3+(1024-800)+(800-640)+10+320+100-30-50+5+155+20+20,-10-20+40+40+60+100+20+5+40-5+get_extra_height(100),"Line #",30+40+10+5+10,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,1,"Word",100+10+20+20+10+get_extra_width(5))        
    AddGadgetColumn(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,2,"🏁",10+20-100-40+100+10+20+20+10+30+20+40+40+40+40+40-40-60-40+get_extra_width(5))      
    AddGadgetColumn(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,3,"Postags/Comments",10+20-100-40+100+10+20+20+10+30+20+40+40+40+40+40-40-60-40+get_extra_width(5))
    AddGadgetColumn(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,4,"Information",10+20-100-40+100+10+20+20+10+30+20+40+40+40+40+40-40-60-40+get_extra_width(5))
    AddGadgetColumn(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,5,"Repeated Flags",10+20-100-40+100+10+20+20+10+30+20+40+40+40+40+40-40-60-40+get_extra_width(5))
    SetGadgetFont(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,FontID(#font_for_language_id))
    
    x=-30+10+40+5+10-5-3
    y=GadgetY(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)+GadgetHeight(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)+5
    
    ; Scan button
    ButtonGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_SCAN,x,y,default_gadget_width_button,default_gadget_height_button,"Scan")       
    
    x+default_gadget_width_button+10
    
    ; Export button
    ButtonGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_EXPORT,x,y,default_gadget_width_button,default_gadget_height_button,"Export")       
    DisableGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_EXPORT,#True)
    
    x+default_gadget_width_button+10
    
    ; Total found text
    TextGadget(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND,x,y,400,default_gadget_height_text,"  # Repeated Words:")
    GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND)
    TextGadget(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER,x+GadgetWidth(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND)+1,y,100,default_gadget_height_text,"0")
    GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER)
    y+default_gadget_height_text+3
    TextGadget(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW,x,y,400,default_gadget_height_text,"# Repeated FlagsW:")
    GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW)
    TextGadget(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER,x+GadgetWidth(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW)+1,y,100,default_gadget_height_text,"0")
    GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER)
    
    x=GadgetX(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)+GadgetWidth(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)-6-default_gadget_width_progress
    y=GadgetY(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)+GadgetHeight(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)+5
    
    ; Progress bar of scan
    ProgressBarGadget(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,x,y,default_gadget_width_progress,default_gadget_height_small_progress,0,number_of_words*2)
    
    y+default_gadget_height_small_progress+1
    
    ; Time processing
    TextGadget(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,x,y,100,default_gadget_height_text,"")
    
    y+default_gadget_height_text+1
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,x,y,100,default_gadget_height_text,"")
    
    
    ; EditorGadget for log
    y=GadgetY(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)
    x=GadgetX(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)+GadgetWidth(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)+5+4
    EditorGadget(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,x,y,-3+100+100+100+get_extra_width(40),60+GadgetHeight(#LISTICON_SHOW_MERGE_DUPLICATES_DIC),#PB_Editor_ReadOnly)
    SetGadgetFont(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,FontID(#font_for_language_id))
    ; Description of option    
    merge_dic_header$="Show/Merge/Delete duplicates in .dic V2.6 - 2023-05-06"+#LF$
    merge_dic_header$+"========================================="+#LF$
    merge_dic_header$+"This option will remove duplicates by"+#LF$
    merge_dic_header$+"merging flags and deleting repeated words."+#LF$
    merge_dic_header$+""+#LF$
    SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,merge_dic_header$)
    
    x=GadgetX(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC)
    y=GadgetY(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC)+GadgetHeight(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC)+5
    
    
    ; Clear button
    ButtonImageGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_CLEAR,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(101))
    GadgetToolTip(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_CLEAR,"Clear")
    
    x+default_gadget_width_button_image_24x24+15-5+16+16-8
    
    ; BOTTOM button EditorGadget gadget
    ButtonImageGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_BOTTOM,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(37))            
    GadgetToolTip(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_BOTTOM,"Go to bottom")       
    
    x+default_gadget_width_button_image_24x24+16-4-2
    
    ; TOP button EditorGadget gadget
    ButtonImageGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_TOP,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(94))
    GadgetToolTip(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_TOP,"Go to top")
    
    
    x=GadgetX(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)
    y=GadgetY(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_SCAN)+GadgetHeight(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_SCAN)+10
    x+3
    y+3
    
    ; Flags
    TextGadget(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FLAGS,x,y,100,default_gadget_height_text,"Flags:")
    GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FLAGS)
    x+GadgetWidth(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FLAGS)+5
    ; KEEP CASE + NEEDAFFIX + FORBIDDENWORD + COMPLEXPREFIXES + CIRCUMFIX - USES THE CONSTANTS FROM THE DICTIONARY EDITOR
    ImageGadget(#IMAGE_KEEPCASE,x,y,53,23,ImageID(80))
    GadgetToolTip(#IMAGE_KEEPCASE,"KEEPCASE Flag")
    ImageGadget(#IMAGE_NEEDAFFIX,GadgetX(#IMAGE_KEEPCASE)+GadgetWidth(#IMAGE_KEEPCASE)+5,y,53,23,ImageID(56))
    GadgetToolTip(#IMAGE_NEEDAFFIX,"NEEDAFFIX Flag")
    ImageGadget(#IMAGE_FORBIDDENWORD,GadgetX(#IMAGE_NEEDAFFIX)+GadgetWidth(#IMAGE_NEEDAFFIX)+5,y,53,23,ImageID(57))
    GadgetToolTip(#IMAGE_FORBIDDENWORD,"FORBIDDENWORD Flag")      
    ImageGadget(#IMAGE_COMPLEXPREFIXES,GadgetX(#IMAGE_FORBIDDENWORD)+GadgetWidth(#IMAGE_FORBIDDENWORD)+5,y,53,23,ImageID(86))
    GadgetToolTip(#IMAGE_COMPLEXPREFIXES,"COMPLEXPREFIXES Flag")         
    ImageGadget(#IMAGE_CIRCUMFIX,GadgetX(#IMAGE_COMPLEXPREFIXES)+GadgetWidth(#IMAGE_COMPLEXPREFIXES)+5,y,53,23,ImageID(78))
    GadgetToolTip(#IMAGE_CIRCUMFIX,"CIRCUMFIX Flag")      
    ; KEEPCASE support
    If keepcaseword$<>"" : SetGadgetState(#IMAGE_KEEPCASE,ImageID(81)) : EndIf
    ; NEEDAFFIX support
    If needaffixword$<>"" : SetGadgetState(#IMAGE_NEEDAFFIX,ImageID(54)) : EndIf 
    ; FORBIDDENWORD support
    If forbiddenword$<>"" : SetGadgetState(#IMAGE_FORBIDDENWORD,ImageID(55)) : EndIf
    ; COMPLEXPREFIXES support
    If complexprefixes_code=#True : SetGadgetState(#IMAGE_COMPLEXPREFIXES,ImageID(85)) : EndIf
    ; CIRCUMFIX support
    If circumfix$<>"" : SetGadgetState(#IMAGE_CIRCUMFIX,ImageID(77)) : EndIf    
    ; FLAG CHR, NUM, LONG
    ImageGadget(#IMAGE_CHR_NUM_LONG,GadgetX(#IMAGE_CIRCUMFIX)+GadgetWidth(#IMAGE_CIRCUMFIX)+5,GadgetY(#IMAGE_CIRCUMFIX),53,23,0)
    Select flag_char
      Case 1
        SetGadgetState(#IMAGE_CHR_NUM_LONG,ImageID(74))
        GadgetToolTip(#IMAGE_CHR_NUM_LONG,"FLAG chr")
        
      Case 2
        SetGadgetState(#IMAGE_CHR_NUM_LONG,ImageID(76))
        GadgetToolTip(#IMAGE_CHR_NUM_LONG,"FLAG long")        
        
      Case 3
        SetGadgetState(#IMAGE_CHR_NUM_LONG,ImageID(75))
        GadgetToolTip(#IMAGE_CHR_NUM_LONG,"FLAG num")                
        
    EndSelect    
    
    
    ;CheckBoxes + Process button + frame
    y+GadgetHeight(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FLAGS)+10+10+5
    y-3
    xx=x
    yy=y
    CheckBoxGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_IGNORE_POS,x,y,90+180+200,default_gadget_height_checkbox,"Ignore Postags/Comments (Postags/Comments Merge)")
    GadgetFit(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_IGNORE_POS)
    SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_IGNORE_POS,show_merge_delete_duplicates_dic_checkbox(1))
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_TWOFOLD,x,y,90+180+300,default_gadget_height_checkbox,"Twofold — Merge only primary and identical affixes (if unsure subaffixes are of same type)")    
    GadgetFit(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_TWOFOLD)
    SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_TWOFOLD,show_merge_delete_duplicates_dic_checkbox(2))
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,x,y,90+180+300,default_gadget_height_checkbox,"Merge only primary and affixes of same type")
    GadgetFit(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME)
    SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,show_merge_delete_duplicates_dic_checkbox(3))
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,x,y,90+180+300,default_gadget_height_checkbox,"With KEEPCASE/NEEDAFFIX/FORBIDDENWORD/COMPLEXPREFIXES/CIRCUMFIX only merge primary and identical affixes")
    GadgetFit(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY)
    SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,show_merge_delete_duplicates_dic_checkbox(4))
    If show_merge_delete_duplicates_dic_checkbox(2)=#True
      SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,#False)
      DisableGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,#True)
      SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,#False)
      DisableGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,#True)                 
    EndIf
    y+default_gadget_height_checkbox+10-3-2-1
    ButtonGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_PROCESS,x,y,default_gadget_width_button,default_gadget_height_button,"Process")   
    DisableGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_PROCESS,#True)
    FrameGadget(#FRAME_SHOW_MERGE_DUPLICATES_DIC,xx-5-5,yy-5-5-2,GadgetX(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY)+GadgetWidth(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY)+10-(xx-5-5),120+16+15-1+5,"")
    
    x=GadgetX(#FRAME_SHOW_MERGE_DUPLICATES_DIC)+GadgetWidth(#FRAME_SHOW_MERGE_DUPLICATES_DIC)-default_gadget_width_button-10
    y=GadgetY(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_PROCESS)
    
    ; REMEMBER button
    ButtonGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_REMEMBER,x,y,default_gadget_width_button,default_gadget_height_button,"Remember")    
    
    x=w-default_gadget_width_button_ok_close_cancel-15
    y=h-10-default_gadget_height_button
    
    ; OK button
    ButtonGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_OKAY,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"OK")       
    
    x-default_gadget_width_button_ok_close_cancel-15
    
    ; Cancel button
    ButtonGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_CANCEL,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"Cancel")       
    
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,#PB_Shortcut_Escape,1000)  
    AddKeyboardShortcut(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,#PB_Shortcut_F1,1004)   
    
    
    ; Store the .dic in the undo if there is a change and set to #True
    ; 24/MAY/2019 - it is now passed in the procedure parameters to be used by the "GB to AU/CA/NZ/ZA"
;     show_merge_delete_duplicates_dic_placed_on_undo=#False
    
    repeated_flagsW_exact_number$=GetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER)
    repeated_flags_exact_number=0
    
    okay=#False
    cancel=#False
    
    
    Repeat
      
      
      
      
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf     
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf     
      
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And EventMenu()=1020
        Debug "F1 pressed"
        html_help("#5.2.5.8.show_duplicates")
      EndIf
      
      
      Select Event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
              
           ; Scan
           Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_SCAN
             Debug "PRESSED SCAN"             
             SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER,"0")            
             GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER)
             SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER,"0")            
             GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER)
             If number_of_words=1          
               MessageRequester("Warning!","No duplicate words found.",#PB_MessageRequester_Warning)
             Else
               SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,"")
               ClearGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)
               show_merge_duplicates_dic_scan(0)
               If CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)=0
                 MessageRequester("Warning!","No duplicate words found.",#PB_MessageRequester_Warning)
               EndIf
             EndIf
             flag=#False
             If CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)=0 : flag=#True : EndIf
             DisableGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_PROCESS,flag)
             DisableGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_EXPORT,flag)

             
             
           ; Export
           Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_EXPORT
             Debug "PRESSED EXPORT"
             If CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)=0
               MessageRequester("Warning!","No duplicate words found.",#PB_MessageRequester_Warning)
             Else
               flag=#False
               file$=SaveFileRequester("Export Duplicates to file","","Text Documents (*.txt)|*.txt",0)
               If file$<>""
                 If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
                 t=ReadFile(1,file$)
                 If t<>0
                   CloseFile(1)
                   If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                       DeleteFile(file$) ; Delete .TXT file                 
                   Else
                       flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                   EndIf  
                 EndIf    
                 
                 If flag=#False
                    ; Saving the .TXT file
                    AddWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800,800)
                    OpenFile(1,file$)
                    WriteStringFormat(1,#PB_UTF8)
                    
                    WriteString(1,"DUPLICATES FOUND:"+FormatNumber(CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC),0,".",number_separator_chr$)+#LF$,#PB_UTF8)
                    WriteString(1,""+#LF$,#PB_UTF8)
                    
                    For f=1 To CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)
                      WriteString(1,GetGadgetItemText(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,f-1,0)+":"+GetGadgetItemText(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,f-1,1)+":"+GetGadgetItemText(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,f-1,2)+GetGadgetItemText(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,f-1,3)+":"+GetGadgetItemText(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,f-1,4)+#LF$,#PB_UTF8)
                      
                      ; Fix to window stop responding if we click on it while exporting
                      event = WindowEvent()
                      If event=#PB_Event_Timer And EventTimer()=800 : EndIf 
                
                    Next f
                    CloseFile(1)
                    RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)          
                 EndIf
                 
               EndIf
               
             EndIf
             
             
           ; Ignore POS
           Case #CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_IGNORE_POS
             show_merge_delete_duplicates_dic_checkbox(1)=GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_IGNORE_POS)
             
             
           ; Twofold
           Case #CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_TWOFOLD
             show_merge_delete_duplicates_dic_checkbox(2)=GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_TWOFOLD)
             Select show_merge_delete_duplicates_dic_checkbox(2)
               Case #False
                 SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,show_merge_delete_duplicates_dic_checkbox(3))
                 DisableGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,#False)
                 SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,show_merge_delete_duplicates_dic_checkbox(4))
                 DisableGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,#False)                 
               Case #True
                 SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,#False)
                 DisableGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME,#True)
                 SetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,#False)
                 DisableGadget(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY,#True)                 
             EndSelect
             
             
           ; Merge only affixes of same type  
           Case #CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME
             show_merge_delete_duplicates_dic_checkbox(3)=GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME)
             
             
           ; With NEEDAFFIX/FORBIDDENWORD/CIRCUMFIX only merge primary
           Case #CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY
             show_merge_delete_duplicates_dic_checkbox(4)=GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY)
             
             
           ; Process
           Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_PROCESS
             If CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)=0
               MessageRequester("Warning!","No duplicate words found.",#PB_MessageRequester_Warning)
             Else
               SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,"")
               
               time_taken=ElapsedMilliseconds()
               
               ; Process merge affixes
               repeated_flagsW_exact_number$=GetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER)
               number_merged_words=0
               show_merge_duplicates_dic_process()
               
               ; scan again the affixes for duplicate flags and for updating the list
               ClearGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)
               If show_merge_duplicates_dic_scan(time_taken)=#False : Continue : EndIf
    
               ; Process repeated flags
               t$=GetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER)
               If t$<>"0"
                 If show_merge_duplicates_flags_process(time_taken)=#False : Continue : EndIf
                 AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Number of merged/deleted FlagsW:"+t$+#LF$+#LF$)
                 SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
;                  If show_merge_duplicates_dic_scan(time_taken)=#False : Continue : EndIf
                 ClearGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)
                 If show_merge_duplicates_dic_scan(time_taken)=#False : Continue : EndIf  
               EndIf
               MessageRequester("Warning!","Merged/deleted words:"+Str(number_merged_words)+" "+#LF$+"Deleted repeated FlagsW:"+repeated_flagsW_exact_number$+#LF$+"Exact number repeated flags:"+Str(repeated_flags_exact_number),#PB_MessageRequester_Info)
               repeated_flags_exact_number=0
             EndIf
             
             flag=#False
             If CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)=0 : flag=#True : EndIf
             DisableGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_PROCESS,flag)
             DisableGadget(#BUTTON_SHOW_MERGE_DUPLICATES_DIC_EXPORT,flag)

             
           ; Press REMEMBER
           Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_REMEMBER           
            ; Delete old .prefs file    
            If FileSize(main_path$+"settings/"+"show_merge_del_dupes.prefs")>=0
              DeleteFile(main_path$+"settings/"+"show_merge_del_dupes.prefs")
            EndIf               
            
            ; Attempt to save checkboxes .PREFS file
            OpenFile(1,main_path$+"settings/"+"show_merge_del_dupes.prefs")    
              WriteString(1,prefs_file_version$+#LF$,#PB_UTF8)
              WriteByte(1,GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_IGNORE_POS))
              WriteByte(1,GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_TWOFOLD))
              WriteByte(1,GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_AFFIXES_SAME))
              WriteByte(1,GetGadgetState(#CHECKBOX_SHOW_MERGE_DUPLICATES_DIC_MERGE_ONLY_PRIMARY))
            CloseFile(1)
            
            ; Clear
            Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_CLEAR
              SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,merge_dic_header$)
              
            ; Go to bottom
            Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_BOTTOM
              SetPosBottom(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,1)
              
            ; Go to top  
            Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_TOP
              SetPosTop(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,1)   
            
           ; Cancel
           Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_CANCEL
             Debug "PRESSED CANCEL"
             cancel=#True
             
             
           ; Ckay
           Case #BUTTON_SHOW_MERGE_DUPLICATES_DIC_OKAY
             Debug "PRESSED OK"
             okay=#True
             
             
          EndSelect
       EndSelect                 
       
       
    Until okay=#True Or cancel=#True
Debug "show_merge_delete_duplicates_dic_placed_on_undo:"+Str(show_merge_delete_duplicates_dic_placed_on_undo)      
    
    ; Pressed "OK"
    If okay=#True
      
      Dim words_buffer$(number_of_words-1)
      CopyArray(words$(),words_buffer$())
      counter=0
      position=0
      
      SetGadgetAttribute(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,#PB_ProgressBar_Maximum,number_of_words-1)
      SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
      
      AddWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800,800)
      FUCKTIME=ElapsedMilliseconds()  

      For f=0 To number_of_words-1
        
        If words_buffer$(f)<>""
          words$(position)=words_buffer$(f)
          counter+1
          position+1
        EndIf
        
        event = WindowEvent()
    
        ; Fix to window stop responding
        If event=#PB_Event_Timer And EventTimer()=800
          SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,f)
        EndIf
 
      Next f
      
      RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)
      Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
      
      FreeArray(words_buffer$())
      
      ReDim words$(counter-1)
      number_of_words=counter
      
      selected_dictionary_word=-1 ; Reset the current word selected
      dictionary_changed=#True    ; Dictionary set to changed   
      update_number_words()

      
      ; New code to handle the vertical scroll bar
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
      SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,0)
      global_update_virtual_vertical_scroll_bar(-1)
      
    EndIf
    
    
    ; If pressed "Cancel" and dictionary changed, undo
    If cancel=#True And show_merge_delete_duplicates_dic_placed_on_undo=#True
      Debug "dictionary_restore_undo()"
      show_merge_delete_duplicates_dic_placed_on_undo=#False
      dictionary_restore_undo()   
    EndIf
    
    
    ; Close the "Show/Merge/Delete duplicates .dic" window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_SHOW_MERGE_DUPLICATES_DIC)     
    
EndProcedure




; "Show/Merge/Delete duplicates .dic" SCAN button
;
; It has 3 steps:
; 1) Copy each original word with formatted characters into a new array
;    1) name + #LF$ + flags + #LF$ + postags + #LF$ + word_position
; 2) Sort the new array
; 3) Scan for duplicates in the new array
;
; The progress bar is number_of_words*2, for 1) and 3)
;
; During 3) it populates the ListIconGadget if a duplicate is found
;
; The variable of the function, use_custom_time_taken, is used to know if the function was called or not from the process procedure.
; If =0 it shows the time from 00:00, if >0 it has the current time.
;
; 20/APR/2023:
; - Support for repeated flags
;
;
; V1.0 - 13/FEB/2019
;        14/FEB/2019
; V1.1 - 15/FEB/2019
;        o Fix: No longer decodes commented dictionary words (starting with "/")
; V1.2 - 20/MAR/2019
;        28/MAR/2019
;        31/MAR/2019
;        01/APR/2019
;        o Continued coding
; V1.3 - 17/APR/2019
;        o Added colours to Suffixes and Prefixes
; V1.4 - 08/MAY/2019
;        o Minor speed up scanning the .dic duplicates
; V1.5 - 16/JUN/2019
;        o "Show/Merge/Delete duplicates .dic" now shows "<ESC> to abort." during scan.
; V1.6 - 13/JAN/2021
;        o "Show/Merge/Delete duplicates .dic" now uses an EditorGadget line buffer to avoid refresh CPU load.
; V1.7 - 25/NOV/2021
;        o Started showing the time taken in timely operations.
; V1.8 - 20/APR/2023
;        21/APR/2023
;        22/APR/2023
;        23/APR/2023
;        24/APR/2023
;        25/APR/2023
;        o Support for repeated flags
;        o Shows progress % while scanning/processing
; V1.9 - 06/MAY/2023
;        o It now shows the number of repeated flags per word
;
Procedure show_merge_duplicates_dic_scan(use_custom_time_taken)
  
  MARCOAGPINTO_LineEditorGadgetBuffer$=GetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC)
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Step 1/3: Scanning the .dic...")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
  
  AddWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800,800)
  
  time_taken=use_custom_time_taken
  If use_custom_time_taken=0 : time_taken=ElapsedMilliseconds() : EndIf
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
  
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"<ESC> to abort.")
  
  ; Step 1/3 - copy formatted words into a target array
  repeated_flags_found=0
  repeated_flags_duplicates_found=0
  Dim show_merge_duplicates_dic_scan_target_array_ultrafast.show_merge_duplicates_dic_scan_target_array_ultrafast(number_of_words-1)
  For f=0 To number_of_words-1
    
    ; Word already changed to "" by "Process" button
    If words$(f)="" : Continue : EndIf
    
    text$=words$(f)+#LF$
    t$=StringField(text$,1,#LF$)    
    If Left(t$,1)<>"/" : t$=ReplaceString(t$,"\/",";") : EndIf ; Support escaped
    word_main$=StringField(t$,1,"/")
    word_flags_main$=StringField(t$,2,"/")
    If Left(t$,1)<>"/" : text$=ReplaceString(text$,";","\/") : EndIf ; Support escaped
    word_pos_main$=StringField(text$,2,#LF$)
    word_line_main$=Str(f+1)
    show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_in_full_old_method.s=word_main$+#LF$+word_flags_main$+#LF$+word_pos_main$+#LF$+word_line_main$ ; Field 0
    show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s=word_main$ ; Field 1 (old code)
    show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_flags_main.s=word_flags_main$ ; Field 2 (old code)
    show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_pos_main.s=word_pos_main$ ; Field 3 (old code)
    show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_line_main.l=f+1 ; Field 4 (old code)
    show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags.l=is_there_repeated_flags_inside(word_flags_main$) ; Field 5 (old code)
    repeated_flags_duplicates_found+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags.l
    If show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags.l=#True
      show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags_counter.l=count_flags_inside(word_flags_main$)-count_flags_inside(remove_repeated_flags(word_flags_main$))
    EndIf
    
    event = WindowEvent()
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,f)
      SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_words-1,1))
      GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN)
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)
        SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
        SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"")
        AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Aborted!")
        AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
        SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
        ProcedureReturn #False
      EndIf
    EndIf
    
    
  Next f
  
  
  ; Step 2/3 - sort the processed array
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Step 2/3: Sorting the processed array...")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
  SortStructuredArray(show_merge_duplicates_dic_scan_target_array_ultrafast(),#PB_Sort_Ascending,OffsetOf(show_merge_duplicates_dic_scan_target_array_ultrafast\word_in_full_old_method.s),TypeOf(show_merge_duplicates_dic_scan_target_array_ultrafast\word_in_full_old_method.s))
  
  
  ; Step 3/3 - scan for duplicates
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Step 3/3: Scanning for duplicates...")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
  ReDim individual_number_duplicates(0)
  individual_number_duplicates_total=0
  individual_number_duplicates_counter=0
  duplicates_found=0
  total_listicongadget_lines=0
  flag=#False ; to add +2 if current word = first duplicate
  For f=0 To number_of_words-1 ;-1
    
    ; Word already changed to "" by "Process" button
;     If show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_in_full_old_method.s="" : Continue : EndIf    
    If words$(f)="" : Continue : EndIf
    
    ; Repeated flags and current word != word+1 and repeated flags
    repeated_flags_with_bug_fix=#False
    If show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags.l=#True
Debug "F:"+Str(f)
Debug "show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s:"+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s

      If f=0 And show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s<>show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_main.s : repeated_flags_with_bug_fix=#True : EndIf
      If f=number_of_words-1 And show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s<>show_merge_duplicates_dic_scan_target_array_ultrafast(f-1)\word_main.s : repeated_flags_with_bug_fix=#True : EndIf
      If f>0 And f<>number_of_words-1 And show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s<>show_merge_duplicates_dic_scan_target_array_ultrafast(f-1)\word_main.s And show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s<>show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_main.s : repeated_flags_with_bug_fix=#True : EndIf      
Debug "repeated_flags_with_bug_fix="+Str(repeated_flags_with_bug_fix)
    EndIf
;     (show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s<>show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_main.s And f=0) And 
    If repeated_flags_with_bug_fix=#True
      t$=Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_line_main.l)+#LF$
      t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s+#LF$
      t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_flags_main.s+#LF$
      t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_pos_main.s+#LF$
      t$+#LF$
      t$+"Yes (+"+Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags_counter.l)+")"
;       t$+#LF$
;       If show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags.l=#True : t$+"Yes" : EndIf
      total_listicongadget_lines+1
      AddGadgetItem(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,-1,t$) 
      SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines-1,#PB_Gadget_FrontColor,#Red,5) ; For duplicate flags
      turn_affixes_to_information_and_colour_and_add_to_listicongadget(-f,total_listicongadget_lines) 
      SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines-1,#PB_Gadget_BackColor,#GreyLight,0) ; For "Line #"
      Continue
    EndIf
    
    ; Fix bug of array out of bounds - 25/APR/2023
    If f=number_of_words-1 : Continue : EndIf
    
    If Left(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s,1)<>"" And show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s=show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_main.s
      If flag=#False
        duplicates_found+1
        individual_number_duplicates_counter+1
        flag=#True
        t$=Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_line_main.l)+#LF$
        t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s+#LF$
        t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_flags_main.s+#LF$
        t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_pos_main.s+#LF$
        t$+#LF$
        If show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags.l=#True : t$+"Yes (+"+Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags_counter.l)+")" : EndIf
        total_listicongadget_lines+1
        AddGadgetItem(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,-1,t$) 
        SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines-1,#PB_Gadget_FrontColor,#Red,5) ; For duplicate flags
        SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines-1,#PB_Gadget_BackColor,#GreyLight,0) ; For "Line #"
        
        ; Add the individual data of a word into the array that will be used to merge/delete
        ReDim show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)
        show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_main=show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s
        show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_flags_main=show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_flags_main.s
        show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_pos_main=show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_pos_main.s
        show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_line_main=Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_line_main.l)
        show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_flags_type=show_merge_duplicates_dic_scan_get_affixes_type(show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_flags_main)       
        
        turn_affixes_to_information_and_colour_and_add_to_listicongadget(duplicates_found,total_listicongadget_lines)        
        
      EndIf
      t$=Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_line_main.l)+#LF$
      t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_main.s+#LF$
      t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_flags_main.s+#LF$
      t$+show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_pos_main.s+#LF$
      t$+#LF$
      If show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_has_repeated_flags.l=#True : t$+"Yes (+"+Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_has_repeated_flags_counter.l)+")" : EndIf
      total_listicongadget_lines+1
      AddGadgetItem(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,-1,t$)      
      SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines-1,#PB_Gadget_FrontColor,#Red,5) ; For duplicate flags
      SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines-1,#PB_Gadget_BackColor,#GreyLight,0) ; For "Line #"
      duplicates_found+1
      individual_number_duplicates_counter+1
;       SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER,Str(duplicates_found))
;       GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER)
      
      ; Add the individual data of a word into the array that will be used to merge/delete
      ReDim show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)
      show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_main=show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_main.s
      show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_flags_main=show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_flags_main.s
      show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_pos_main=show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_pos_main.s
      show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_line_main=Str(show_merge_duplicates_dic_scan_target_array_ultrafast(f+1)\word_line_main.l)
      show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_flags_type=show_merge_duplicates_dic_scan_get_affixes_type(show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_flags_main)        
      
      turn_affixes_to_information_and_colour_and_add_to_listicongadget(duplicates_found,total_listicongadget_lines)      
      
    Else
      flag=#False 
      If individual_number_duplicates_counter>0     
        individual_number_duplicates_total+1
        ReDim individual_number_duplicates(individual_number_duplicates_total)
        individual_number_duplicates(individual_number_duplicates_total)=individual_number_duplicates_counter
        individual_number_duplicates_counter=0
      EndIf
    EndIf
  
    event = WindowEvent()
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_words-1-1,1))
      GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN)
      SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER,Str(duplicates_found))
      GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER)      
      SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,number_of_words+f)
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)
        SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
        SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"")
        AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Aborted!")
        AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"") 
        SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
        ProcedureReturn #False
      EndIf
    EndIf
    
    
  Next f
  
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER,Str(duplicates_found))
  GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER)
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER,Str(repeated_flags_duplicates_found))
  GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER)
  
  If individual_number_duplicates_counter>0
    individual_number_duplicates_total+1
    ReDim individual_number_duplicates(individual_number_duplicates_total)
    individual_number_duplicates(individual_number_duplicates_total)=individual_number_duplicates_counter
  EndIf
  
  SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)  
  RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
   
  ; Below debugs the counter of duplicate lines (remmed) - 28/MAR/2019
If fukkkk=99999999 
   Debug "individual_number_duplicates_total:"+Str(individual_number_duplicates_total)
   For f=1 To individual_number_duplicates_total
     Debug "individual_number_duplicates("+Str(f)+"):"+Str(individual_number_duplicates(f))
   Next f
EndIf


  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"# Repeated Words:"+Str(duplicates_found))
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"# Repeated FlagsW:"+Str(repeated_flags_duplicates_found))
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"🛈:")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"If there are words with postags and postag merge is off,")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"those words won't be merged.")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Duplicate flags removal will still work.")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)

  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"")
  
If fukkkk=99999999 
  Debug "THE DEBUGGING OF THINGS! *START*"
  Debug "duplicates_found:"+Str(duplicates_found)
  For f=0 To duplicates_found-1
    Debug "F="+Str(f)
    Debug show_merge_duplicates_dic_scan_target_array_info(f)\word_main
    Debug show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_main
    Debug show_merge_duplicates_dic_scan_target_array_info(f)\word_pos_main
    Debug show_merge_duplicates_dic_scan_target_array_info(f)\word_line_main
    Debug show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_type
  Next f
  Debug "THE DEBUGGING OF THINGS! *END*"  
EndIf
  
  ProcedureReturn #True
    
EndProcedure




; This Procedures turns the affixes kind to text, adds to the ListIconGadget and sets the colours accordingly
; to avoid repeating the code in the scan Procedure
;
; If the duplicates_found is negative (used "-f") it only adds the type of flags and the colour, since this
; Procedure is more for duplicate words
;
; V1.0 - 23/APR/2023
;
Procedure turn_affixes_to_information_and_colour_and_add_to_listicongadget(duplicates_found, total_listicongadget_lines)
  
  total_listicongadget_lines-1
  
  information$="Unknown"
  If duplicates_found<=0
    duplicates_found=Abs(duplicates_found)
    iii$=show_merge_duplicates_dic_scan_target_array_ultrafast(duplicates_found)\word_flags_main.s
    i$=show_merge_duplicates_dic_scan_get_affixes_type(iii$)
  Else
    i$=show_merge_duplicates_dic_scan_target_array_info(duplicates_found-1)\word_flags_type
  EndIf
  If i$="0" : information$="Primary" : EndIf
  If i$="1" : information$="All suffixes" : EndIf
  If i$="2" : information$="All prefixes" : EndIf
  If i$="3" : information$="All suffixes + prefixes" : EndIf
  If i$="4" : information$="CIRCUMFIX/FORBIDDENWORD/NEEDAFFIX" : EndIf
  If i$="5" : information$="SFX N (only suffixes can be added)" : EndIf
  If i$="6" : information$="AF Compressed" : EndIf
  If i$="7" : information$="Keep Case" : EndIf
  SetGadgetItemText(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines,information$,4)
  If information$="Unknown" : SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines,#PB_Gadget_FrontColor,#Red,4) : EndIf
  If information$="Primary" : SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_primary),4) : EndIf
  If information$="All suffixes" : SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_suffix),4) : EndIf
  If information$="All prefixes" : SetGadgetItemColor(#LISTICON_SHOW_MERGE_DUPLICATES_DIC,total_listicongadget_lines,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),4) : EndIf
  
EndProcedure




; This function decodes the flags of each word in the scan procedure
;
; It returns:
; 0 = Primary - No flags type
; 1 = All suffixes
; 2 = All prefixes
; 3 = All suf + pref
; 4 = CIRCUMFIX+FORBIDDENWORD+NEEDAFFIX (Any of them being used) <- Remmed so that people know the flags type (21/MAR/2019)
; 5 = SFX N (only suffixes can be added)
; 6 = AF Compressed
; 7 = KEEPCASE
;
; The code follows a specific order to find the types
;
; 4/SEP/2021: Noticed that there was a bug since the variables SFX, PFX were global.
;             So I added an underscore to them. It now detects "All Prefixes".
;
; V1.0 - 20/MAR/2019
;        21/MAR/2019
;        01/APR/2019
; V1.1 - 04/SEP/2021
;        o Fix: "Show/Merge/Delete duplicates in .dic" would not detect "All Prefixes"
;
Procedure.s show_merge_duplicates_dic_scan_get_affixes_type(word_codes$)
  
  ; =0
  If word_codes$="" : ProcedureReturn "0" : EndIf
    
  ; =6
  If AF_compressed_number>0 : ProcedureReturn "6" : EndIf
  
  ; =4
;   If needaffixword$<>"" Or forbiddenword$<>"" Or circumfix$<>"" : ProcedureReturn "4" : EndIf
  
  
  ; =1 / =2 / =3 / =5 / =7
  suf_=0
  pfx_=0
  flag_sfx_no_=0 ; To not derivates prefixes for given word: 0=No,1=Yes
  keepcase_=#False
  
  
  ; Count flags inside
  len_word_codes=count_flags_inside(word_codes$)
  

  For f=1 To len_word_codes                        
              
    faster_word_codes$=get_faster_word_codes(word_codes$,f)
    
    If faster_word_codes$=keepcaseword$
      keepcase_=#True
      Continue
    EndIf
              
    t1=FindString(ultrafast_aff_search$,"PFX "+faster_word_codes$+" Y")              
    If t1<>0 And pfx_=0 : pfx_=1 : EndIf  
    t1=FindString(ultrafast_aff_search$,"SFX "+faster_word_codes$+" Y") ; Now gives priority to Y - 15/MAY/2018
    If t1=0
      t1=FindString(ultrafast_aff_search$,"SFX "+faster_word_codes$+" N")
      If t1<>0: flag_sfx_no_=1 : EndIf              
    EndIf          
    If t1<>0 And sfx_=0 : sfx_=1 : EndIf
    
    ; Only suffixes can be added
    If flag_sfx_no_=1 : ProcedureReturn "5" : EndIf
   
    ; Mix of suffixes and prefixes
    If pfx_=1 And flag_sfx_no_=1 : ProcedureReturn "5" : EndIf
            
  Next f
  
  ; Only KEEPCASE
  If keepcase_=#True And sfx_=0 And pfx_=0 : ProcedureReturn "7" : EndIf
  
  ; Only suffixes
  If sfx_=1 And pfx_=0 : ProcedureReturn "1" : EndIf
  
  ; Only prefixes
  If sfx_=0 And pfx_=1 : ProcedureReturn "2" : EndIf 
  
  ;All suf + pref
  If sfx_=1 And pfx_=1 : ProcedureReturn "3" : EndIf
  
EndProcedure  
  



; This procedure processes the duplicates, merging or removing them
;
; Remember to keep the order of the rules, so that they work according to the CheckBoxes values
;
; V1.0 - 28/MAR/2019
;        31/MAR/2019
;        06/APR/2019
;        17/APR/2019
; V1.1 - 16/JUN/2019
;        o "Show/Merge/Delete duplicates .dic" now shows "<ESC> to abort." during scan.
; V1.2 - 18/MAY/2020
;        o In the merge flags in .dics there is now an EditorGadget and buttons
; V1.3 - 24/SEP/2020
;        o Show/Merge/Delete duplicates .dic: Processing of large wordlists no longer causes PTG to stop responding
; V1.4 - 26/NOV/2020
;        o Show/Merge/Delete duplicates .dic: zero changes no longer clears the Editor Gadget
; V1.5 - 13/JAN/2021
;        o "Show/Merge/Delete duplicates .dic" now uses an EditorGadget line buffer to avoid refresh CPU load.
; V1.6 - 25/NOV/2021
;        o Started showing the time taken in timely operations.
;
Procedure show_merge_duplicates_dic_process()
  
  MARCOAGPINTO_LineEditorGadgetBuffer$=GetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC)
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,#LF$+"Processing the .dic...")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
  
  show_merge_duplicates_dic_scan_target_array_position=0
  
  AddWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800,800)

  time_taken=ElapsedMilliseconds()  
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
  
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"<ESC> to abort.")
  
  SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
  
  
  ttt$=""
  
Repeat
  
  changed_global=#False
  
  For current_position=1 To individual_number_duplicates_total
    
    Repeat
      
      changed=#False
      
      For f=show_merge_duplicates_dic_scan_target_array_position To show_merge_duplicates_dic_scan_target_array_position+individual_number_duplicates(current_position)-1
        
        
        word_source_name$=show_merge_duplicates_dic_scan_target_array_info(f)\word_main
        word_source_flags$=show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_main
        word_source_pos$=show_merge_duplicates_dic_scan_target_array_info(f)\word_pos_main
        word_source_line$=show_merge_duplicates_dic_scan_target_array_info(f)\word_line_main
        word_source_flags_type$=show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_type
        If show_merge_delete_duplicates_dic_checkbox(1)=#True : word_source_pos$="" : EndIf
        
; Debug "f:"+Str(f)
; Debug "word_source_name$:"+word_source_name$
; Debug "word_source_flags$:"+word_source_flags$
; Debug "word_source_pos$:"+word_source_pos$
; Debug "word_source_line$:"+word_source_line$
; Debug "word_source_flags_type$:"+word_source_flags_type$
; Debug "changed:"+Str(changed)
        
        
        For ff=show_merge_duplicates_dic_scan_target_array_position To show_merge_duplicates_dic_scan_target_array_position+individual_number_duplicates(current_position)-1
          
          word_target_name$=show_merge_duplicates_dic_scan_target_array_info(ff)\word_main
          word_target_flags$=show_merge_duplicates_dic_scan_target_array_info(ff)\word_flags_main
          word_target_pos$=show_merge_duplicates_dic_scan_target_array_info(ff)\word_pos_main
          word_target_line$=show_merge_duplicates_dic_scan_target_array_info(ff)\word_line_main
          word_target_flags_type$=show_merge_duplicates_dic_scan_target_array_info(ff)\word_flags_type
          If show_merge_delete_duplicates_dic_checkbox(1)=#True : word_target_pos$="" : EndIf
        
          
          If f<>ff And word_source_name$<>""
            
            ; Source = Primary word
            If word_source_flags_type$="0" And word_target_flags_type$<>"999999" And word_source_pos$=word_target_pos$
              
              changed=#True
              changed_global=#True
              
              ; Store the .dic in the undo if there is a change and set to #True
              If changed=#True And show_merge_delete_duplicates_dic_placed_on_undo=#False
                show_merge_delete_duplicates_dic_placed_on_undo=#True
                dictionary_store_undo(#False)
              EndIf
              
              
              ; Add line of merged word into EditorGadget
              ttt$+"#"+Str(number_merged_words+1)+#LF$
              ttt$+"Merged:"+Chr(34)+word_target_name$+Chr(34)+#LF$
              If word_source_flags$="" And word_target_flags$=""
                ttt$+"Primary"+#LF$
              Else                
                ttt$+"Flags:"+Chr(34)+word_source_flags$+Chr(34)+"→"+Chr(34)+word_target_flags$+Chr(34)+#LF$
              EndIf
              ttt$+#LF$
              
              
              show_merge_duplicates_dic_scan_target_array_info(f)\word_main=""
              show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_type="999999" ; To avoid double deletion
              words$(Val(word_source_line$)-1)=""
              
              number_merged_words+1
              Break            
            EndIf
            
            
            ; Source = Target
            If word_target_flags_type$<>"999999" And word_source_name$=word_target_name$ And word_source_flags$=word_target_flags$ And word_source_pos$=word_target_pos$
              
              changed=#True
              changed_global=#True
              
              ; Store the .dic in the undo if there is a change and set to #True
              If changed=#True And show_merge_delete_duplicates_dic_placed_on_undo=#False
                show_merge_delete_duplicates_dic_placed_on_undo=#True
                dictionary_store_undo(#False)
              EndIf
              
              
              ; Add line of merged word into EditorGadget
              ttt$+"#"+Str(number_merged_words+1)+#LF$
              ttt$+"Merged:"+Chr(34)+word_target_name$+Chr(34)+#LF$
              If word_source_flags$="" And word_target_flags$=""
                ttt$+"Primary"+#LF$
              Else                
                ttt$+"Flags:"+Chr(34)+word_source_flags$+Chr(34)+"→"+Chr(34)+word_target_flags$+Chr(34)+#LF$
              EndIf
              ttt$+#LF$
              
              
              show_merge_duplicates_dic_scan_target_array_info(f)\word_main=""
              show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_type="999999" ; To avoid double deletion
              words$(Val(word_source_line$)-1)=""
              
              number_merged_words+1
              Break            
            EndIf            
            
            
            ; Twofold disabled, no need for any check if =#True since the two rules above do the work for #True, anyway
            If show_merge_delete_duplicates_dic_checkbox(2)=#False
              
              ; checkbox 3 code here, ONLY affixes of the same type
              If show_merge_delete_duplicates_dic_checkbox(3)=#True
                  If word_source_flags_type$=word_target_flags_type$ And (word_source_flags_type$="1" Or word_source_flags_type$="2") And word_target_flags_type$<>"999999" And word_source_pos$=word_target_pos$
;                   If ((word_source_flags_type$="1" And word_target_flags_type$="1") Or (word_source_flags_type$="2" And word_target_flags_type$="2"))  And word_target_flags_type$<>"999999" And word_source_flags_type$<>"999999" And word_source_pos$=word_target_pos$
                  Debug "Flag YES of same type"  
                    
                  changed=#True
                  changed_global=#True
              
                    ; Store the .dic in the undo if there is a change and set to #True
                    If changed=#True And show_merge_delete_duplicates_dic_placed_on_undo=#False
                      show_merge_delete_duplicates_dic_placed_on_undo=#True
                      dictionary_store_undo(#False)
                    EndIf
                    
                    
                    ; Add line of merged word into EditorGadget
                    ttt$+"#"+Str(number_merged_words+1)+#LF$
                    ttt$+"Merged:"+Chr(34)+word_target_name$+Chr(34)+#LF$
                    If word_source_flags$="" And word_target_flags$=""
                      ttt$+"Primary"+#LF$
                    Else                
                      ttt$+"Flags:"+Chr(34)+word_source_flags$+Chr(34)+"→"+Chr(34)+word_target_flags$+Chr(34)+#LF$
                    EndIf
                    ttt$+#LF$
                    
                    
                    show_merge_duplicates_dic_scan_target_array_info(f)\word_main=""
                    word_target_flags$=merge_flags(word_source_flags$,word_target_flags$)
                    show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_type="999999" ; To avoid double deletion
;                     words$(Val(word_source_line$)-1)=""
                    t$=""
                    t$+word_target_name$
                    If word_target_flags$<>"" : t$+"/" : EndIf
                    t$+word_target_flags$
                    If word_target_pos$<>""
                      t$+#LF$+word_target_pos$
                    EndIf
;                     words$(Val(word_target_line$)-1)=t$
                    words$(Val(word_target_line$)-1)=""
                    words$(Val(word_source_line$)-1)=t$
                    
                    number_merged_words+1
                    Break            
                  EndIf
                  
              EndIf
                            
              ; checkbox 3 code here, affixes NOT of the same type
              If show_merge_delete_duplicates_dic_checkbox(3)=#False                  
                  If (word_source_flags_type$="1" Or word_source_flags_type$="2" Or word_source_flags_type$="3") And (word_target_flags_type$="1" Or word_target_flags_type$="2" Or word_target_flags_type$="3") And word_target_flags_type$<>"999999" And word_source_pos$=word_target_pos$
;                   If ((word_source_flags_type$="1" And word_target_flags_type$="2") Or (word_source_flags_type$="2" And word_target_flags_type$="1") Or (word_source_flags_type$="1" And word_target_flags_type$="1") Or (word_source_flags_type$="2" And word_target_flags_type$="2") Or (word_source_flags_type$="1" And word_target_flags_type$="3") Or (word_source_flags_type$="2" And word_target_flags_type$="3") Or (word_source_flags_type$="3" And word_target_flags_type$="1") Or (word_source_flags_type$="3" And word_target_flags_type$="2")) And word_target_flags_type$<>"999999" And word_source_flags_type$<>"999999" And word_source_pos$=word_target_pos$
                  Debug "Flag NOT of same type"  
                    
                    changed=#True
                    changed_global=#True
              
                    ; Store the .dic in the undo if there is a change and set to #True
                    If changed=#True And show_merge_delete_duplicates_dic_placed_on_undo=#False
                      show_merge_delete_duplicates_dic_placed_on_undo=#True
                      dictionary_store_undo(#False)
                    EndIf
                    
                    
                    ; Add line of merged word into EditorGadget
                    ttt$+"#"+Str(number_merged_words+1)+#LF$
                    ttt$+"Merged:"+Chr(34)+word_target_name$+Chr(34)+#LF$
                    If word_source_flags$="" And word_target_flags$=""
                      ttt$+"Primary"+#LF$
                    Else                
                      ttt$+"Flags:"+Chr(34)+word_source_flags$+Chr(34)+"→"+Chr(34)+word_target_flags$+Chr(34)+#LF$
                    EndIf
                    ttt$+#LF$
                    
                    
                    show_merge_duplicates_dic_scan_target_array_info(f)\word_main=""
                    word_target_flags$=merge_flags(word_source_flags$,word_target_flags$)
                    show_merge_duplicates_dic_scan_target_array_info(f)\word_flags_type="999999" ; To avoid double deletion
;                     words$(Val(word_source_line$)-1)=""
                    t$=""
                    t$+word_target_name$
                    If word_target_flags$<>"" : t$+"/" : EndIf
                    t$+word_target_flags$
                    If word_target_pos$<>""
                      t$+#LF$+word_target_pos$
                    EndIf
;                     words$(Val(word_target_line$)-1)=t$
                    words$(Val(word_target_line$)-1)=""
                    words$(Val(word_source_line$)-1)=t$
                    
                    number_merged_words+1
                    Break            
                  EndIf
                  
                
              EndIf
              
              
              ; checkbox 4 code here, anyway, it is not used since it is the same as the first two rules
              
            EndIf
               
            
            ; ########## PLACE HERE THE TEST CODE, CHANGING changed=#True if change done
            ; ########## and remember to use break if also if changed=#True then break on the next f
            
          EndIf
          
        Next ff  
        
        If changed=#True : Break : EndIf
        
      Next f
      
;       ; Store the .dic in the undo if there is a change and set to #True
;       If changed=#True And show_merge_delete_duplicates_dic_placed_on_undo=#False
;         show_merge_delete_duplicates_dic_placed_on_undo=#True
;         dictionary_store_undo()
;       EndIf
      
    Until changed=#False

    
    event = WindowEvent()
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,current_position)
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)
        SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
        SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"")
        ProcedureReturn #False
      EndIf
    EndIf
    
    show_merge_duplicates_dic_scan_target_array_position+individual_number_duplicates(current_position)
    
  Next current_position
  
  
  ; Redo the processing (2-pass)
  If changed_global=#True
    show_merge_duplicates_dic_scan_target_array_position=0  
    SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER,"0") 
    GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_COUNTER)
    SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER,"0") 
    GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TOTAL_FOUND_FLAGSW_COUNTER)
    ClearGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)
    show_merge_duplicates_dic_scan(time_taken)
    If CountGadgetItems(#LISTICON_SHOW_MERGE_DUPLICATES_DIC)=0 : changed_global=#False : EndIf    
  EndIf
  
Until changed_global=#False    

  ; Add changes into EditorGadget - No longer refreshes in real time to fix mouse pointer getting into wait state - 24/SEP/2020
  If ttt$<>""
    AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,ttt$)
  EndIf

  ; Add line with number of merged words into EditorGadget
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Number of merged/deleted words:"+Str(number_merged_words))
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)


Debug "show_merge_delete_duplicates_dic_placed_on_undo inside process:"+Str(show_merge_delete_duplicates_dic_placed_on_undo)
  
  SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
  
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"")
  
  RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
  

  ; To see if it scans again the .dic
  If number_merged_words=0
    ProcedureReturn #False
  Else
    ProcedureReturn #True
  EndIf
  
EndProcedure




; Remove the repeated flags from the .DIC
;
; The variable of the function, use_custom_time_taken, is used to know if the function was called or not from the process procedure.
; If =0 it shows the time from 00:00, if >0 it has the current time.
;
; V1.0 - 24/APR/2023
;
Procedure show_merge_duplicates_flags_process(use_custom_time_taken)
  
  MARCOAGPINTO_LineEditorGadgetBuffer$=GetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC)
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,#LF$+"Removing duplicate flags from .dic...")
  AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
  SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
  
  
  AddWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800,800)
  
  time_taken=use_custom_time_taken
  If use_custom_time_taken=0 : time_taken=ElapsedMilliseconds() : EndIf
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
  
  
  SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"<ESC> to abort.")
  
  SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
  

  For f=0 To number_of_words-1
    
    ; Word already changed to "" by "Process" button
    If words$(f)="" : Continue : EndIf
    
    
    ; Get each word information
    text$=words$(f)+#LF$
    t$=StringField(text$,1,#LF$)    
    If Left(t$,1)<>"/" : t$=ReplaceString(t$,"\/",";") : EndIf ; Support escaped
    word_main$=StringField(t$,1,"/")
    word_flags_main$=StringField(t$,2,"/")
    If Left(t$,1)<>"/" : text$=ReplaceString(text$,";","\/") : EndIf ; Support escaped
    word_pos_main$=StringField(text$,2,#LF$)
    
    
    ; Remove repeated flags
    If is_there_repeated_flags_inside(word_flags_main$)
      
      ; Store the .dic in the undo if there is a change and set to #True
      If show_merge_delete_duplicates_dic_placed_on_undo=#False
        show_merge_delete_duplicates_dic_placed_on_undo=#True
        dictionary_store_undo(#False)
      EndIf
      
      repeated_flags_exact_number+count_flags_inside(word_flags_main$)-count_flags_inside(remove_repeated_flags(word_flags_main$))
      t$=word_main$+"/"
      ttt$=remove_repeated_flags(word_flags_main$)
      t$+ttt$
      If word_pos_main$<>"" : t$+#LF$+word_pos_main$ : EndIf
      words$(f)=t$
;       repeated_flags_exact_number-count_flags_inside(ttt$)
    EndIf
    
    
    
    ; Remove repeated flags
;     If show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_has_repeated_flags.l=#True
; Debug "FUCK!!!!!! THERE ARE REPEATED FLAGS!!!!"
;       t$=show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_main.s+"/"
;       t$+remove_repeated_flags(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_flags_main.s)
;       If show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_pos_main.s<>""
;         t$+#TAB$+show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_pos_main.s
;       EndIf
;       words$(show_merge_duplicates_dic_scan_target_array_ultrafast(f)\word_line_main.l)=t$  
;     EndIf
    
    
    
    event = WindowEvent()
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,f)
      SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_words-1,1))
      GadgetFit(#TEXT_SHOW_MERGE_DUPLICATES_DIC_TIME_TAKEN)
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_SHOW_MERGE_DUPLICATES_DIC,800)
        SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,0)
        SetGadgetText(#TEXT_SHOW_MERGE_DUPLICATES_DIC_ESC_TO_ABORT,"")
        AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"Aborted!")
        AddLineEditorGadgetBuffer(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,"")
        SetGadgetText(#EDITORGADGET_SHOW_MERGE_DUPLICATES_DIC,MARCOAGPINTO_LineEditorGadgetBuffer$)
        ProcedureReturn #False
      EndIf
    EndIf
    
    
  Next f
  
  ProcedureReturn #True
  
EndProcedure




; Merges source flags with target flags, returning the combining of both
;
; V1.0 - 06/APR/2019
;        17/APR/2019
;        19/APR/2019
;
Procedure.s merge_flags(source$,target$)
  
  
  new_flag$=target$
  word_codes$=source$
  
  
  ; Count flags inside
  len_word_codes=count_flags_inside(word_codes$)
  
  
  ; Try to find the matching flag
  For f=1 To len_word_codes                      
    
    faster_word_codes$=get_faster_word_codes(word_codes$,f)
        
    ; Not found the matching flag -> merge it
    If is_flag_inside(faster_word_codes$,target$)=#False
      If flag_char=3 : faster_word_codes$+"," : EndIf
      new_flag$=faster_word_codes$+new_flag$      
    EndIf      
    
  Next f
  
  
  ; If flag NUM remove last ","
  If flag_char=3 And Right(new_flag$,1)="," : new_flag$=Left(new_flag$,Len(new_flag$)-1) : EndIf
  
  
  ; Return the combining flags
  Debug "new_flag$:"+new_flag$
  ProcedureReturn new_flag$
  
EndProcedure




; Remove the duplicated flags
;
; V1.0 - 19/APR/2023
;
Procedure.s remove_repeated_flags(word_codes$)

  ; Count flags inside
  len_word_codes=count_flags_inside(word_codes$)
  
  
  ; Redim structured array
  ReDim array_for_dupe_flags_removal(len_word_codes-1)
  
  
  ; Extract each file into the structured array
  For f=1 To len_word_codes                      
    faster_word_codes$=get_faster_word_codes(word_codes$,f)
    array_for_dupe_flags_removal(f-1)\flag_name.s=faster_word_codes$
    array_for_dupe_flags_removal(f-1)\flag_name_order.l=f
  Next f

  
  ; Sort flags text for easier analysis
  SortStructuredArray(array_for_dupe_flags_removal(),#PB_Sort_Ascending,OffsetOf(array_for_dupe_flags_removal\flag_name.s),TypeOf(array_for_dupe_flags_removal\flag_name.s))
 
  
  ; Place duplicates as blank
  For f=0 To len_word_codes-1-1
    If array_for_dupe_flags_removal(f)\flag_name.s=array_for_dupe_flags_removal(f+1)\flag_name.s : array_for_dupe_flags_removal(f)\flag_name.s="" : EndIf
  Next f

  
  ; Sort flags order based on their position for original order 
  SortStructuredArray(array_for_dupe_flags_removal(),#PB_Sort_Ascending,OffsetOf(array_for_dupe_flags_removal\flag_name_order.l),TypeOf(array_for_dupe_flags_removal\flag_name_order.l))
  
  
  ; Place the flags back in the word flag codes
  new_flag$=""
  For f=0 To len_word_codes-1
    If array_for_dupe_flags_removal(f)\flag_name.s<>""
      new_flag$+array_for_dupe_flags_removal(f)\flag_name.s
      If flag_char=3 : new_flag$+"," : EndIf
    EndIf
  Next f
  
  
  ; If flag NUM remove last ","
  If flag_char=3 And Right(new_flag$,1)="," : new_flag$=Left(new_flag$,Len(new_flag$)-1) : EndIf
  
  
  ; Return the combining flags
  Debug "new_flag$:"+new_flag$
  ProcedureReturn new_flag$
  
EndProcedure




; Checks inside a word flag it has the given FLAG
;
; Return #True if Yes and #False if No
;
;
; V1.0 - 06/APR/2019
;
Procedure is_flag_inside(flag_to_check$,word_codes$)
   
  ; Count flags inside
  len_word_codes=count_flags_inside(word_codes$)
  
  
  ; Try to find the matching flag
  For f=1 To len_word_codes                      
    
    faster_word_codes$=get_faster_word_codes(word_codes$,f)
        
    ; Found the matching flag
    If faster_word_codes$=flag_to_check$
      ProcedureReturn #True
    EndIf      
    
  Next f
  
  
  ; Not found matching flag
  ProcedureReturn #False
  
  
EndProcedure




; Counts the total of word flags in a given list of flags
;
; Returns the number of flags found
;
; V1.0 - 18/APR/2023
;
Procedure count_flags_inside(word_codes$)
  
  ; Count flags inside
  ; For utf-8/chr,long AND num
  len_word_codes=Len(word_codes$)  
  If flag_char=2
    len_word_codes=Round(Len(word_codes$)/2.0,#PB_Round_Up)
  EndIf
  If flag_char=3
    len_word_codes=CountString(word_codes$,",")+1
  EndIf
  
  
  ; Return number of flags
  ProcedureReturn len_word_codes
  
EndProcedure




; Checks inside a word flag if there are repeated flags
;
; Return #True if Yes and #False if No
;
;
; V1.0 - 18/APR/2023
;
Procedure is_there_repeated_flags_inside(word_codes$)
  
  ; Count flags inside
  len_word_codes=count_flags_inside(word_codes$)
  
  
  ; Return #False if number of flags <=1 (no repeated)
  If len_word_codes<=1 : ProcedureReturn #False : EndIf
  
  
  ; Check for repeated flags
  Dim repeated_flags$(len_word_codes-1)
 
  For f=1 To len_word_codes                          
    repeated_flags$(f-1)=get_faster_word_codes(word_codes$,f)
  Next f
  
  SortArray(repeated_flags$(),#PB_Sort_Ascending)
  
  For f=0 To len_word_codes-1-1
    If repeated_flags$(f)=repeated_flags$(f+1) : ProcedureReturn #True : EndIf
  Next f
  
  
  ; Not found repeated flags
  ProcedureReturn #False
  
  
EndProcedure




; This procedure adds zeros to rules numbers while extracting as .csv
;
; for example: "1" becomes "01" if it uses two decimal houses to make it easier to sort in Calc/Excel
;
; V1.0 - 18/MAR/2019
;
Procedure.s add_zeros(t$)
  ProcedureReturn Right("0000"+t$,Len(Str(rules_highest_number)))
EndProcedure




; This procedure implements a way of easily creating Australian, Canadian and New Zealand spellers
; based on the GB one
;
; V1.0 - 07/MAY/2019
;        08/MAY/2019
;        09/MAY/2019
;        10/MAY/2019
;        11/MAY/2019
;        15/MAY/2019
;        21/MAY/2019
;        24/MAY/2019
;        27/MAY/2019
; V1.1 - 07/JUN/2019
;        o GTK3 Ready
; V1.2 - 16/JUN/2019
;        o "GB to AU/CA/NZ" now shows "<ESC> to abort." during scan.
; V1.3 - 20/JUN/2019
;        o Preferences allow to choose the font for the language. 
; V1.4 - 29/SEP/2019
;        o Replaced "GB to AU/CA/NZ" with "GB to AU/CA/NZ/ZA" (added also ZA)
; V1.5 - 12/OCT/2020
;        o Adding of F1 to open the help in the various windows.
; V1.6 - 10/NOV/2020
;        o Clear buttons now use a trashcan image.
; V1.7 - 24/NOV/2020
;        o Added "Go to top" and "Go to bottom" buttons
; V1.8 - 17/FEB/2021
;        o Improved "GB to AU/CA/NZ/ZA (Marco Pinto - GB)"
;
Procedure gb_to_au_ca_nz_za()
  
    t$="GB to AU/CA/NZ/ZA (Marco Pinto - GB)"
    If OpenWindow(#WINDOW_GB_TO_AU_CA_NZ_ZA,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,10+5+get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,t$,#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_GB_TO_AU_CA_NZ_ZA,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore)
    DisableWindow(#WINDOW_MAIN,#True)      
    
    x=10
    y=10
    TextGadget(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_1,x,y,80+20+40+40+40+40,default_gadget_height_text,"Step #1: Merge latest GB .dic")
    SetGadgetColor(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_1,#PB_Gadget_FrontColor,#Blue)
    
    y+20
    
    y+5
    
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_1_MERGE_GB_DIC,x,y,90+20,default_gadget_height_button,"Merge GB .dic")
    
    y+20+20
    
    y+5
    
    TextGadget(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_2,x,y,80+20+40+40+40+40,default_gadget_height_text,"Step #2: Merge/Delete duplicates")
    SetGadgetColor(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_2,#PB_Gadget_FrontColor,#Blue)
    
    
    y+20
    
    y+5
    
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_2_MERGE_DELETE_DUPLICATES,x,y,90+20,default_gadget_height_button,"Remove dupes")
    
    
    y+20+20
    
    y+5
    
    TextGadget(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_3,x,y,80+20+40+40+40+40,default_gadget_height_text,"Step #3: Remove -ize/-ise")
    SetGadgetColor(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_3,#PB_Gadget_FrontColor,#Blue)
    
    
    y+20
    
    y+5
    
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_REMOVE_IZE,x,y,90+20,default_gadget_height_button,"Remove -ize")
    
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_REMOVE_ISE,x+GadgetWidth(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_REMOVE_IZE)+15,y,90+20,default_gadget_height_button,"Remove -ise")
    
    y+20+20
    
    TextGadget(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_3_1,x,y,80+20+40+40+40+40,default_gadget_height_text,"Exceptions:")
    
    y+20
    
    EditorGadget(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS,x,y,80+20+40+40+40,-5-20+40+60+40-40-15+get_extra_height(50))
    
    SetGadgetFont(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS,FontID(#font_for_language_id))
    
    SetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS,exceptions_editor_gadget$)
    
    y=GadgetY(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS)+GadgetHeight(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS)
    
    y+5
    
    ButtonImageGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS_CLEAR,x,y,default_gadget_height_button,default_gadget_height_button,ImageID(101))
    GadgetToolTip(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS_CLEAR,"Clear")
    
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS_SAVE,x+GadgetWidth(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS_CLEAR)+15,y,90,default_gadget_height_button,"Save")    
    
    
    y+20
    
    y+5    
    
    y+20
    
    y+5
    
    TextGadget(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_4,x,y,80+20+40+40+40+40,default_gadget_height_text,"Step #4: Remove GB specific words")    
    SetGadgetColor(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_4,#PB_Gadget_FrontColor,#Blue)
    
    
    y+20
    
    y+5
    
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_REMOVE_GB_ONLY,x,y,90+20,default_gadget_height_button,"Remove GB only")
    
    y+5
    
    y+20+20
        
    
    y-10
    
    y+2
    
    EditorGadget(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC,x,y,80+20+40+40+40,-5-20+40+60+40-40-15+get_extra_height(50))
    
    SetGadgetFont(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC,FontID(#font_for_language_id))
    
    SetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC,gb_specific_editor_gadget$)
    
    
    y=GadgetY(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC)+GadgetHeight(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC)
    
    y+5
    
    ButtonImageGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_CLEAR,x,y,default_gadget_height_button,default_gadget_height_button,ImageID(101))
    GadgetToolTip(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_CLEAR,"Clear")
    
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_SAVE,x+GadgetWidth(#BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_CLEAR)+15,y,90,default_gadget_height_button,"Save")    
    
    
    y+20+20
    
    y+5    
    
    TextGadget(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_5,x,y,80+20+40+40+40+40,default_gadget_height_text,"Step #5: Add your specific words")    
    SetGadgetColor(#TEXT_GB_TO_AU_CA_NZ_ZA_STEP_5,#PB_Gadget_FrontColor,#Blue)
    
    
    ; Editor Gadget
    x=200+100
    y=10
    EditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,x,y,600+100+get_extra_width(100),400+50+get_extra_height(100),#PB_Editor_ReadOnly)    
    SetGadgetFont(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,FontID(#font_for_language_id))
    ; Description of option
    gb_to_au_ca_nz_za_header$="GB to AU/CA/NZ/ZA (Marco Pinto - GB) V2.0 - 2021-02-17"+#LF$
    gb_to_au_ca_nz_za_header$+"=========================================="+#LF$
    gb_to_au_ca_nz_za_header$+"This option allows to create variants of the GB dictionary."+#LF$
    gb_to_au_ca_nz_za_header$+""+#LF$
    SetGadgetText(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,gb_to_au_ca_nz_za_header$)
    
    y=GadgetY(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA)+GadgetHeight(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA)+10-3
    
    
    x=GadgetX(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA)
    
    
    ; Clear EditorGadget
    ButtonImageGadget(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_CLEAR,x,y,default_gadget_height_button,default_gadget_height_button,ImageID(101))
    GadgetToolTip(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_CLEAR,"Clear")
    
    ; Bottom
    ButtonImageGadget(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_BOTTOM,GadgetX(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_CLEAR)+GadgetWidth(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_CLEAR)+16+16-8,y,default_gadget_height_button,default_gadget_height_button,ImageID(37))            
    GadgetToolTip(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_BOTTOM,"Go to bottom")    
    
    ; Top
    ButtonImageGadget(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_TOP,GadgetX(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_BOTTOM)+GadgetWidth(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_BOTTOM)+16-4-2,y,default_gadget_height_button,default_gadget_height_button,ImageID(94))
    GadgetToolTip(#BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_TOP,"Go to top")
    
    
    x=GadgetX(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA)+GadgetWidth(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA)-(100+5+60-20-5)-10+3
    
    
    ; Progress bar of scan
    ProgressBarGadget(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,x-20,y,default_gadget_width_progress,default_gadget_height_progress,0,1)
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,GadgetX(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA),GadgetY(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA)+GadgetHeight(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA)+1,100,default_gadget_height_text,"")
 
    ; OK button
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_OKAY,WindowWidth(#WINDOW_GB_TO_AU_CA_NZ_ZA)-100-15,-15+WindowHeight(#WINDOW_GB_TO_AU_CA_NZ_ZA)-20-10,90,default_gadget_height_button,"OK")       
    
    ; Cancel button
    ButtonGadget(#BUTTON_GB_TO_AU_CA_NZ_ZA_CANCEL,5+GadgetX(#BUTTON_GB_TO_AU_CA_NZ_ZA_OKAY)-100-15,-15+WindowHeight(#WINDOW_GB_TO_AU_CA_NZ_ZA)-20-10,90,default_gadget_height_button,"Cancel")         
    
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_GB_TO_AU_CA_NZ_ZA,#PB_Shortcut_Escape,1000)  
    AddKeyboardShortcut(#WINDOW_GB_TO_AU_CA_NZ_ZA,#PB_Shortcut_F1,1004)   
    
    
    ; Store the .dic in the undo if there is a change and set to #True
    show_merge_delete_duplicates_dic_placed_on_undo=#False
    
    
;     total_sum_of_removals=0
        
    okay=#False
    cancel=#False
    
    Repeat
      
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf     
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf   
      
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And EventMenu()=1020
        Debug "F1 pressed"
        html_help("#5.2.5.4.GB_to_AU_CA_NZ_ZA")
      EndIf
      
     
      
      Select Event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
           
           ; Merge GB .dic
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_1_MERGE_GB_DIC
             gb_to_au_ca_nz_za_merge_dic()
             
             
           ; Remove dupes
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_2_MERGE_DELETE_DUPLICATES
             AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Merging/Deleting duplicates...")
             show_merge_duplicates_dic(show_merge_delete_duplicates_dic_placed_on_undo)
             DisableWindow(#WINDOW_MAIN,#True)
             SetActiveWindow(#WINDOW_GB_TO_AU_CA_NZ_ZA)
             
           ; Remove -ize
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_REMOVE_IZE
             
             ; Convert exceptions into array
             gb_to_au_ca_nz_za_convert_exceptions_to_array()             
             
             AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,#LF$+"Removing -ize..."+#LF$)
             gb_to_au_ca_nz_za_remove_ize_ise(1)
             AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")
             
           ; Remove -ise
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_REMOVE_ISE
             
             ; Convert exceptions into array
             gb_to_au_ca_nz_za_convert_exceptions_to_array()                          
             
             AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,#LF$+"Removing -ise..."+#LF$)
             gb_to_au_ca_nz_za_remove_ize_ise(2)             
             AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")
             
           ; Clear -ize/-ise EditorGadget
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS_CLEAR
             SetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS,"")
             
           ; Save exceptions
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS_SAVE
             t$=GetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS)
             t$=ConvertStringToUnix(t$)
             If t$=""
               MessageRequester("Warning!","No exceptions found.",#PB_MessageRequester_Warning)
             Else
               DeleteFile(main_path$+"settings/"+"-ize-ise_exceptions.txt")
               
               ; Save the exceptions file, using UTF-8 plus BOM encoding
               OpenFile(1,main_path$+"settings/"+"-ize-ise_exceptions.txt")
                  WriteStringFormat(1,#PB_UTF8)
                  WriteString(1,t$,#PB_UTF8)
               CloseFile(1)
                  
            EndIf
            
            
           ; Remove GB specific
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_REMOVE_GB_ONLY
             
             ; Convert GB specific into array
             gb_to_au_ca_nz_za_convert_gb_specific_to_array()                          
             
             If gb_specific_array_number>0
                AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,#LF$+"Removing GB specific..."+#LF$)
                gb_to_au_ca_nz_za_remove_gb_specific()             
                AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")
             Else
                AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")
             EndIf
             
             
           ; Clear GB specific EditorGadget
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_CLEAR
             SetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC,"")
             
             
           ; Save GB specific
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC_SAVE
             t$=GetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC)
             t$=ConvertStringToUnix(t$)
             If t$=""
               MessageRequester("Warning!","No GB specific words found.",#PB_MessageRequester_Warning)
             Else
               DeleteFile(main_path$+"settings/"+"gb_specific.txt")
               
               ; Save the GB specific file, using UTF-8 plus BOM encoding
               OpenFile(1,main_path$+"settings/"+"gb_specific.txt")
                  WriteStringFormat(1,#PB_UTF8)
                  WriteString(1,t$,#PB_UTF8)
               CloseFile(1)
                  
            EndIf
             
             
           ; Clear log EditorGadget
           Case #BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_CLEAR
             SetGadgetText(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,gb_to_au_ca_nz_za_header$)
             
           ; Go to BOTTOM  
           Case #BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_BOTTOM
              SetPosBottom(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,1)
              
           ; Go to TOP   
           Case #BUTTON_EDITORGADGET_GB_TO_AU_CA_NZ_ZA_TOP
              SetPosTop(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,1)             
             
           ; OK 
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_OKAY
             okay=#True
             
           ; CANCEL  
           Case #BUTTON_GB_TO_AU_CA_NZ_ZA_CANCEL
             cancel=#True
             
         EndSelect
         
       EndSelect                 
       
       
    Until okay=#True Or cancel=#True
    
    
    ; Remember the exceptions + GB specific
    If okay=#True
      exceptions_editor_gadget$=GetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS)
      gb_specific_editor_gadget$=GetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC)
    EndIf
    
    
    ; Pressed "OK" -> Updated .dic with changes
    If okay=#True And number_of_words>0 And show_merge_delete_duplicates_dic_placed_on_undo=#True
      
      Dim words_buffer$(number_of_words-1)
      CopyArray(words$(),words_buffer$())
      counter=0
      position=0
      
      SetGadgetAttribute(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,#PB_ProgressBar_Maximum,number_of_words-1)
      SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0)
      
      AddWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800,800)
      FUCKTIME=ElapsedMilliseconds()  

      For f=0 To number_of_words-1
        
        If words_buffer$(f)<>""
          words$(position)=words_buffer$(f)
          counter+1
          position+1
        EndIf
        
        event = WindowEvent()
    
        ; Fix to window stop responding
        If event=#PB_Event_Timer And EventTimer()=800
          SetGadgetState(#PROGRESSBAR_WINDOW_SHOW_MERGE_DUPLICATES_DIC_SCAN,f)
        EndIf
 
      Next f
      
      RemoveWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800)
      Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
      
      FreeArray(words_buffer$())
      
      ReDim words$(counter-1)
      number_of_words=counter
      
      selected_dictionary_word=-1 ; Reset the current word selected
      dictionary_changed=#True    ; Dictionary set to changed   
      update_number_words()

      
      ; New code to handle the vertical scroll bar
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
      SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,0)
      global_update_virtual_vertical_scroll_bar(-1)
      
    EndIf

    
    ; If pressed "Cancel" and dictionary changed, undo
    If cancel=#True And show_merge_delete_duplicates_dic_placed_on_undo=#True
      Debug "dictionary_restore_undo()"
      show_merge_delete_duplicates_dic_placed_on_undo=#False
      dictionary_restore_undo()    
    EndIf    
    
    
    ; Close the "Show/Merge/Delete duplicates .dic" window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_GB_TO_AU_CA_NZ_ZA) 
    
EndProcedure




; To merge the GB .dic into the current one
;
; V1.0 - 08/MAY/2019
;        10/MAY/2019
;        24/MAY/2019
; V1.1 - 16/JUN/2019
;        o "GB to AU/CA/NZ" now shows "<ESC> to abort." during scan.
; V1.2 - 15/FEB/2020
;        o All file operations support BOM.
;
Procedure gb_to_au_ca_nz_za_merge_dic()
  
  
    ; Merging GB .dic message
    AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Merging GB .dic...")
  
    ; Select file
    file$=OpenFileRequester("Merge GB Dictionary", "", "Dictionary (*.dic)|*.dic", 0)
    If file$=""
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Aborted merging GB .dic..."+#LF$)
      ProcedureReturn #False
    EndIf

    
    
    ; Check if .DIC file exists
    If FileSize(file$) <= 0
      MessageRequester("Error", ".DIC file not found.",#PB_MessageRequester_Warning)
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"No GB .dic selected."+#LF$)
      ProcedureReturn #False
    EndIf    
    
    
    ; Store the .dic in the undo if there is a change and set to #True
    If show_merge_delete_duplicates_dic_placed_on_undo=#False
      show_merge_delete_duplicates_dic_placed_on_undo=#True
      dictionary_store_undo(#False)
    EndIf
    
    
    selected_dictionary_word=-1 ; Reset the current word selected
    dictionary_changed=#True ; Dictionary set to changed
             
    old_number_of_words=number_of_words
    
    FUCKTIME=ElapsedMilliseconds()
    
    AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Merging GB .dic:"+Chr(34)+file$+Chr(34))
    
    ; Read .DIC file
    If number_of_words>0 : ClearGadgetItems(#PANEL_BOX_DICTIONARY) : EndIf ; To remove the current words from the gadgetlist    
    convert_space_to_tab=-1 ;Convert space to tab (Galician dictionary) -1 = Didn't ask | 0 = Asked and NO | 1 = Asked and YES
    AddWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800,800)      
    ReadFile(1,File$)
    ReadStringFormat(1)
    
    ; Set the maximum of the progress bar
    SetGadgetAttribute(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,#PB_ProgressBar_Maximum,Lof(1))
    SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0)    
    
    SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"<ESC> to abort.")
        
;       number_of_words=0
;       Dim words$(0)
;       Dim words_selected(0)
      number_of_words_total_selected=0
      text$=ReadString(1,#PB_UTF8) ;Just to read the header of the .DIC which contains the number of words
      Repeat
        word$=ReadString(1,#PB_UTF8)
        ; Ask here to convert space to tab *START*
        If FindString(word$,#TAB$)<>0 : convert_space_to_tab=0 : EndIf ; 3/DEC/2014 - Fixed a bug of adding multiple tabs while opening a .DIC with spaces instead of tabs
        If FindString(word$,#TAB$)=0 And (FindString(word$,"[")<>0 Or FindString(word$,"]")<>0 Or FindString(word$,",")<>0 Or FindString(word$,":")<>0) And convert_space_to_tab=-1  
          If MessageRequester("Warning!","This .DIC seems to separate Words from Tags using a space."+#LF$+#LF$+"Use first space as a separator?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            convert_space_to_tab=1 ; YES
          Else
            convert_space_to_tab=0 ; NO
          EndIf      
        EndIf
        ; Ask here to convert space to tab *END*
        If convert_space_to_tab=1
          ReplaceString(word$," ",#LF$,#PB_String_InPlace,1,1)
        Else
          ReplaceString(word$,#TAB$,#LF$,#PB_String_InPlace,1,1)
        EndIf
        ReDim words$(number_of_words)
        ReDim words_selected(number_of_words)
        words$(number_of_words)=word$
        number_of_words+1
        
        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event=WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
           SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,Loc(1)) 
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
         If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            CloseFile(1)
            SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0) 
            RemoveWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800)           
            AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Aborted!"+#LF$)
            SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"")
            ProcedureReturn #False
         EndIf
        EndIf

      Until Eof(1)
    CloseFile(1)  
    AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Merged "+Str(number_of_words-old_number_of_words)+" words of GB .dic..."+#LF$)
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    update_number_words()
    RemoveWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800)
    SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0) ; Reset progress bar
    SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"")
      
    ; New code to handle the vertical scroll bar
    SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(-1)
    
  
EndProcedure




; Used to remove -ize/-ise
;
; remove_ize_ise=1 -IZE   =2 -ISE
;
; V1.0 - 10/MAY/2019
;        24/MAY/2019
; V1.1 - 16/JUN/2019
;        o "GB to AU/CA/NZ" now shows "<ESC> to abort." during scan.
; V1.2 - 17/FEB/2021
;        o "GB to AU/CA/NZ/ZA (Marco Pinto - GB)" now uses an EditorGadget line buffer to avoid refresh CPU load. 
;
Procedure gb_to_au_ca_nz_za_remove_ize_ise(remove_ize_ise)
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf  
  
  
  ; Set the maximum of the progress bar
  SetGadgetAttribute(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,#PB_ProgressBar_Maximum,number_of_words-1)
  SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0)
  
  SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"<ESC> to abort.")
  
  ; Total/current number of -ize/-ise removed
  counter_ize_ise_removed=0
  
  
  ; Faster way to avoid refresh each word
  MARCOAGPINTO_LineEditorGadgetBuffer$=GetGadgetText(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA)
  
  
  FUCKTIME=ElapsedMilliseconds()
  AddWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800,800)
  For f=0 To number_of_words-1
        
    text$=words$(f)
    
    word_full_without_LF$=ReplaceString(text$,#LF$," ")
    
    word_without_pos$=StringField(text$,1,#LF$)
    
    word_without_pos_and_without_slash$=StringField(word_without_pos$,1,"/")+"/"
    
    
    ; Remove -IZE
    If remove_ize_ise=1
      
      If Right(word_without_pos_and_without_slash$,Len("ize/"))="ize/" Or Right(word_without_pos_and_without_slash$,Len("izing/"))="izing/" Or Right(word_without_pos_and_without_slash$,Len("ized/"))="ized/" Or Right(word_without_pos_and_without_slash$,Len("izes/"))="izes/" Or Right(word_without_pos_and_without_slash$,Len("zation/"))="zation/" Or Right(word_without_pos_and_without_slash$,Len("zations/"))="zations/"
        
        If is_exception(word_without_pos_and_without_slash$)=#False
        
          counter_ize_ise_removed+1
          AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Removed #"+Str(counter_ize_ise_removed)+":")
          AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,Chr(34)+word_full_without_LF$+Chr(34)+#LF$)
          
          ; Store the .dic in the undo if there is a change and set to #True
          If show_merge_delete_duplicates_dic_placed_on_undo=#False
            show_merge_delete_duplicates_dic_placed_on_undo=#True
            dictionary_store_undo(#False)
          EndIf          
          
          words$(f)=""          
          
        EndIf
        
      EndIf
      
    EndIf
    
    
    ; Remove -ISE
    If remove_ize_ise=2
      
      If Right(word_without_pos_and_without_slash$,Len("ise/"))="ise/" Or Right(word_without_pos_and_without_slash$,Len("ising/"))="ising/" Or Right(word_without_pos_and_without_slash$,Len("ised/"))="ised/" Or Right(word_without_pos_and_without_slash$,Len("ises/"))="ises/" Or Right(word_without_pos_and_without_slash$,Len("sation/"))="sation/" Or Right(word_without_pos_and_without_slash$,Len("sations/"))="sations/"
        
        If is_exception(word_without_pos_and_without_slash$)=#False
        
          counter_ize_ise_removed+1
          AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Removed #"+Str(counter_ize_ise_removed)+":")
          AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,Chr(34)+word_full_without_LF$+Chr(34)+#LF$)
          
          ; Store the .dic in the undo if there is a change and set to #True
          If show_merge_delete_duplicates_dic_placed_on_undo=#False
            show_merge_delete_duplicates_dic_placed_on_undo=#True
            dictionary_store_undo(#False)
          EndIf                   
          
          words$(f)=""          
          
        EndIf
          
      EndIf
      
    EndIf
    
    
    
    ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
    event=WindowEvent()
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,f) 
    EndIf 
    
    
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0) 
        RemoveWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800)
        If counter_ize_ise_removed=0 : AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"") : EndIf
        AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Aborted!"+#LF$)
        SetGadgetText(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,MARCOAGPINTO_LineEditorGadgetBuffer$)
        SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"")
        ProcedureReturn #False
      EndIf
    EndIf
    
    
  Next f
  
  If counter_ize_ise_removed=0 : AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"") : EndIf
  AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Removed a total of "+Str(counter_ize_ise_removed)+" words.")
  AddLineEditorGadgetBuffer(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")
  SetGadgetText(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,MARCOAGPINTO_LineEditorGadgetBuffer$)
  
;   total_sum_of_removals+counter_ize_ise_removed
  
  Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
  RemoveWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800)
  SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0) ; Reset progress bar 
  
  SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"")
    
EndProcedure




; Used to remove GB specific
;
;
; V1.0 - 27/MAY/2019
; V1.1 - 16/JUN/2019
;        o "GB to AU/CA/NZ" now shows "<ESC> to abort." during scan.
;
Procedure gb_to_au_ca_nz_za_remove_gb_specific()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf  
  
  
  ; Set the maximum of the progress bar
  SetGadgetAttribute(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,#PB_ProgressBar_Maximum,number_of_words-1)
  SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0)
  
  SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"<ESC> to abort.")
  
  ; Total/current number of -ize/-ise removed
  counter_gb_specific_removed=0
  
  
  FUCKTIME=ElapsedMilliseconds()
  AddWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800,800)
  For f=0 To number_of_words-1
        
    text$=words$(f)
    
    word_full_without_LF$=ReplaceString(text$,#LF$," ")
    
    word_without_pos$=StringField(text$,1,#LF$)
    
    word_without_pos_and_without_slash$=StringField(word_without_pos$,1,"/")+"/"
    
    
    ; Remove GB specific
    If is_gb_specific(word_without_pos_and_without_slash$)=#True
        
      counter_gb_specific_removed+1
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Removed #"+Str(counter_gb_specific_removed)+":")
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,Chr(34)+word_full_without_LF$+Chr(34)+#LF$)
          
      ; Store the .dic in the undo If there is a change And set To #True
      If show_merge_delete_duplicates_dic_placed_on_undo=#False
        show_merge_delete_duplicates_dic_placed_on_undo=#True
        dictionary_store_undo(#False)
      EndIf          
         
      words$(f)=""          
          
    EndIf
            

    ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
    event=WindowEvent()
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,f) 
    EndIf 
    
    
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0) 
        RemoveWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800)
        If counter_gb_specific_removed=0 : AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"") : EndIf
        AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Aborted!"+#LF$)
        SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"")
        ProcedureReturn #False
      EndIf
    EndIf
    
    
  Next f
  
  If counter_gb_specific_removed=0 : AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"") : EndIf
  AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Removed a total of "+Str(counter_gb_specific_removed)+" words.")
  AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")
  
;   total_sum_of_removals+counter_gb_specific_removed
  
  Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
  RemoveWindowTimer(#WINDOW_GB_TO_AU_CA_NZ_ZA,800)
  SetGadgetState(#PROGRESSBAR_GB_TO_AU_CA_NZ_ZA,0) ; Reset progress bar 
  
  SetGadgetText(#TEXT_GB_TO_AU_CA_NZ_ZA_ESC_TO_ABORT,"")
  
    
EndProcedure




; This procedure will convert the exceptions into an array for faster processing
;
; V1.0 - 13/MAY/2019
;
Procedure gb_to_au_ca_nz_za_convert_exceptions_to_array()
  
  
  AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Converting exceptions into arrays for faster processing...")  
 
  t$=GetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_3_1_EXCEPTIONS)
  
  t$=ConvertStringToUnix(t$)
  
  If t$=""
    AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"No exceptions found.")
    ProcedureReturn #False    
  EndIf
  
  If Right(t$,1)<>#LF$ : t$+#LF$ : EndIf
  
  If t$=old_exceptions_editor_gadget$
    AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"No exceptions changed since last conversion.")
    ProcedureReturn #False
  EndIf
  
  exceptions_array_number=0  
  
  t=CountString(t$,#LF$)
  
  If t=0 : ProcedureReturn #False : EndIf
  
  ReDim exceptions_array$(t-1)
  
  For f=1 To t
    
    current_exception$=StringField(t$,f,#LF$)
    
    If current_exception$=""
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"There is a blank exception.")
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Aborted!") 
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")   
      exceptions_array_number=0  
      ProcedureReturn #False      
    EndIf
  
    exceptions_array$(f-1)=current_exception$+"/"
    exceptions_array_number+1
    
  Next f
  
  AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Found "+Str(exceptions_array_number)+" exceptions.") 
  
  old_exceptions_editor_gadget$=t$
  
  ProcedureReturn #True  
    
EndProcedure




; This procedure will convert the GB specific into an array for faster processing
;
; V1.0 - 27/MAY/2019
;
Procedure gb_to_au_ca_nz_za_convert_gb_specific_to_array()
  
  
  AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Converting GB specific into arrays for faster processing...")  
 
  t$=GetGadgetText(#EDITOR_GB_TO_AU_CA_NZ_ZA_STEP_4_SPECIFIC)
  
  t$=ConvertStringToUnix(t$)
  
  If t$=""
    AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"No GB specific found.")
    ProcedureReturn #False    
  EndIf
  
  If Right(t$,1)<>#LF$ : t$+#LF$ : EndIf
  
  If t$=old_gb_specific_editor_gadget$
    AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"No GB specific changed since last conversion.")
    ProcedureReturn #False
  EndIf
  
  gb_specific_array_number=0  
  
  t=CountString(t$,#LF$)
  
  If t=0 : ProcedureReturn #False : EndIf
  
  ReDim gb_specific_array$(t-1)
  
  For f=1 To t
    
    current_gb_specific$=StringField(t$,f,#LF$)
    
    If current_gb_specific$=""
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"There is a blank GB specific.")
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Aborted!") 
      AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"")   
      gb_specific_array_number=0  
      ProcedureReturn #False      
    EndIf
  
    gb_specific_array$(f-1)=current_gb_specific$+"/"
    gb_specific_array_number+1
    
  Next f
  
  AddLineEditorGadget(#EDITORGADGET_GB_TO_AU_CA_NZ_ZA,"Found "+Str(gb_specific_array_number)+" GB specific.") 
  
  old_gb_specific_editor_gadget$=t$
  
  ProcedureReturn #True  
    
EndProcedure




; To check if a word is an exception
;
; V1.0 - 13/MAY/2019
;
Procedure is_exception(t$) 
  
  ; No match found
  If exceptions_array_number=0 : ProcedureReturn #False : EndIf
  
  
  For f=1 To exceptions_array_number
    
    ; Found match
    If exceptions_array$(f-1)=t$ : ProcedureReturn #True : EndIf
    
  Next f
  
  
  ; No match found
  ProcedureReturn #False
  
EndProcedure




; To check if a word is GB specific
;
; V1.0 - 27/MAY/2019
;
Procedure is_gb_specific(t$) 
  
  ; No match found
  If gb_specific_array_number=0 : ProcedureReturn #False : EndIf
  
  
  For f=1 To gb_specific_array_number
    
    ; Found match
    If gb_specific_array$(f-1)=t$ : ProcedureReturn #True : EndIf
    
  Next f
  
  
  ; No match found
  ProcedureReturn #False
  
EndProcedure




; To create the default exceptions
;
; V1.0 - 21/MAY/2019
;        22/MAY/2019
;        24/MAY/2019
;        27/MAY/2019
; V1.1 - 15/FEB/2020
;        o All file operations support BOM.
;
Procedure load_exceptions()
  
  
  ; Check if exceptions file exists
  If FileSize(main_path$+"settings/"+"-ize-ise_exceptions.txt")>0
    ReadFile(1,main_path$+"settings/"+"-ize-ise_exceptions.txt")
      ReadStringFormat(1)
      exceptions_editor_gadget$=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)    
    CloseFile(1)
    ProcedureReturn #True
  EndIf      
  
  
  old_exceptions_editor_gadget$=""
  exceptions_editor_gadget$=""
  exceptions_editor_gadget$+"accusation"+#LF$
  exceptions_editor_gadget$+"Acrise"+#LF$
  exceptions_editor_gadget$+"adversation"+#LF$
  exceptions_editor_gadget$+"anglepoise"+#LF$
  exceptions_editor_gadget$+"anglewise"+#LF$
  exceptions_editor_gadget$+"anise"+#LF$
  exceptions_editor_gadget$+"ansation"+#LF$
  exceptions_editor_gadget$+"anticlockwise"+#LF$
  exceptions_editor_gadget$+"anywise"+#LF$
  exceptions_editor_gadget$+"appraise"+#LF$
  exceptions_editor_gadget$+"appraisingly"+#LF$
  exceptions_editor_gadget$+"arise"+#LF$
  exceptions_editor_gadget$+"assize"+#LF$
  exceptions_editor_gadget$+"Auglaize"+#LF$
  exceptions_editor_gadget$+"Avise"+#LF$
  exceptions_editor_gadget$+"Bagpuize"+#LF$
  exceptions_editor_gadget$+"baize"+#LF$
  exceptions_editor_gadget$+"Belize"+#LF$
  exceptions_editor_gadget$+"bêtise"+#LF$
  exceptions_editor_gadget$+"bitwise"+#LF$
  exceptions_editor_gadget$+"Blaise"+#LF$
  exceptions_editor_gadget$+"Boise"+#LF$
  exceptions_editor_gadget$+"bolognaise"+#LF$
  exceptions_editor_gadget$+"bookwise"+#LF$
  exceptions_editor_gadget$+"breadthwise"+#LF$
  exceptions_editor_gadget$+"Brize"+#LF$
  exceptions_editor_gadget$+"bruise"+#LF$
  exceptions_editor_gadget$+"capsize"+#LF$
  exceptions_editor_gadget$+"Cassation"+#LF$
  exceptions_editor_gadget$+"causation"+#LF$
  exceptions_editor_gadget$+"cerise"+#LF$
  exceptions_editor_gadget$+"cessation"+#LF$
  exceptions_editor_gadget$+"chaise"+#LF$
  exceptions_editor_gadget$+"Charlize"+#LF$
  exceptions_editor_gadget$+"chemise"+#LF$
  exceptions_editor_gadget$+"circumcise"+#LF$
  exceptions_editor_gadget$+"clockwise"+#LF$
  exceptions_editor_gadget$+"Cochise"+#LF$
  exceptions_editor_gadget$+"compensation"+#LF$
  exceptions_editor_gadget$+"compromise"+#LF$
  exceptions_editor_gadget$+"concise"+#LF$
  exceptions_editor_gadget$+"condensation"+#LF$
  exceptions_editor_gadget$+"contrariwise"+#LF$
  exceptions_editor_gadget$+"conversation"+#LF$
  exceptions_editor_gadget$+"cornerwise"+#LF$
  exceptions_editor_gadget$+"cosupervise"+#LF$
  exceptions_editor_gadget$+"counter-clockwise"+#LF$
  exceptions_editor_gadget$+"counterpoise"+#LF$
  exceptions_editor_gadget$+"crabwise"+#LF$
  exceptions_editor_gadget$+"crises"+#LF$
  exceptions_editor_gadget$+"crosswise"+#LF$
  exceptions_editor_gadget$+"cruise"+#LF$
  exceptions_editor_gadget$+"Cruise"+#LF$
  exceptions_editor_gadget$+"daemonize"+#LF$
  exceptions_editor_gadget$+"dauphinoise"+#LF$
  exceptions_editor_gadget$+"decompensation"+#LF$
  exceptions_editor_gadget$+"demise"+#LF$
  exceptions_editor_gadget$+"Denise"+#LF$
  exceptions_editor_gadget$+"despise"+#LF$
  exceptions_editor_gadget$+"devise"+#LF$
  exceptions_editor_gadget$+"Devizes"+#LF$
  exceptions_editor_gadget$+"disguise"+#LF$
  exceptions_editor_gadget$+"dualise"+#LF$
  exceptions_editor_gadget$+"edgewise"+#LF$
  exceptions_editor_gadget$+"Elise"+#LF$
  exceptions_editor_gadget$+"Eloise"+#LF$
  exceptions_editor_gadget$+"Enterprise"+#LF$
  exceptions_editor_gadget$+"enterprise"+#LF$
  exceptions_editor_gadget$+"equipoise"+#LF$
  exceptions_editor_gadget$+"euthanise"+#LF$
  exceptions_editor_gadget$+"excise"+#LF$
  exceptions_editor_gadget$+"exercise"+#LF$
  exceptions_editor_gadget$+"filesize"+#LF$
  exceptions_editor_gadget$+"foodwise"+#LF$
  exceptions_editor_gadget$+"franchise"+#LF$
  exceptions_editor_gadget$+"Françoise"+#LF$
  exceptions_editor_gadget$+"fundraise"+#LF$
  exceptions_editor_gadget$+"fund-raising"+#LF$
  exceptions_editor_gadget$+"Guise"+#LF$
  exceptions_editor_gadget$+"hair-raising"+#LF$
  exceptions_editor_gadget$+"high-rise"+#LF$
  exceptions_editor_gadget$+"Héloise"+#LF$
  exceptions_editor_gadget$+"Héloisehigh-rise"+#LF$
  exceptions_editor_gadget$+"hollandaise"+#LF$
  exceptions_editor_gadget$+"ill-advised"+#LF$
  exceptions_editor_gadget$+"incise"+#LF$
  exceptions_editor_gadget$+"industrial-size"+#LF$
  exceptions_editor_gadget$+"ise"+#LF$
  exceptions_editor_gadget$+"king-size"+#LF$
  exceptions_editor_gadget$+"leastwise"+#LF$
  exceptions_editor_gadget$+"lengthwise"+#LF$
  exceptions_editor_gadget$+"life-size"+#LF$
  exceptions_editor_gadget$+"likewise"+#LF$
  exceptions_editor_gadget$+"longwise"+#LF$
  exceptions_editor_gadget$+"Louise"+#LF$
  exceptions_editor_gadget$+"low-noise"+#LF$
  exceptions_editor_gadget$+"maize"+#LF$
  exceptions_editor_gadget$+"malaise"+#LF$
  exceptions_editor_gadget$+"man-sized"+#LF$
  exceptions_editor_gadget$+"Maoized"+#LF$
  exceptions_editor_gadget$+"marquise"+#LF$
  exceptions_editor_gadget$+"Marseillaise"+#LF$
  exceptions_editor_gadget$+"mayonnaise"+#LF$
  exceptions_editor_gadget$+"medium-sized"+#LF$
  exceptions_editor_gadget$+"microenterprise"+#LF$
  exceptions_editor_gadget$+"middle-sized"+#LF$
  exceptions_editor_gadget$+"mise"+#LF$
  exceptions_editor_gadget$+"Mises"+#LF$
  exceptions_editor_gadget$+"moonrise"+#LF$
  exceptions_editor_gadget$+"mortise"+#LF$
  exceptions_editor_gadget$+"noise"+#LF$
  exceptions_editor_gadget$+"otherwise"+#LF$
  exceptions_editor_gadget$+"outnoise"+#LF$
  exceptions_editor_gadget$+"outsize"+#LF$
  exceptions_editor_gadget$+"overexercise"+#LF$
  exceptions_editor_gadget$+"oversize"+#LF$
  exceptions_editor_gadget$+"pairwise"+#LF$
  exceptions_editor_gadget$+"Paradise"+#LF$
  exceptions_editor_gadget$+"paradise"+#LF$
  exceptions_editor_gadget$+"parallelwise"+#LF$
  exceptions_editor_gadget$+"piecewise"+#LF$
  exceptions_editor_gadget$+"plotwise"+#LF$
  exceptions_editor_gadget$+"pointwise"+#LF$
  exceptions_editor_gadget$+"poise"+#LF$
  exceptions_editor_gadget$+"polonaise"+#LF$
  exceptions_editor_gadget$+"porpoise"+#LF$
  exceptions_editor_gadget$+"Portlaoighise"+#LF$
  exceptions_editor_gadget$+"Portlaoise"+#LF$
  exceptions_editor_gadget$+"positivize"+#LF$
  exceptions_editor_gadget$+"practise"+#LF$
  exceptions_editor_gadget$+"praise"+#LF$
  exceptions_editor_gadget$+"praisingly"+#LF$
  exceptions_editor_gadget$+"preadvise"+#LF$
  exceptions_editor_gadget$+"premise"+#LF$
  exceptions_editor_gadget$+"prize"+#LF$
  exceptions_editor_gadget$+"pulsation"+#LF$
  exceptions_editor_gadget$+"raise"+#LF$
  exceptions_editor_gadget$+"recompensation"+#LF$
  exceptions_editor_gadget$+"repromise"+#LF$
  exceptions_editor_gadget$+"revise"+#LF$
  exceptions_editor_gadget$+"Rise"+#LF$
  exceptions_editor_gadget$+"rise"+#LF$
  exceptions_editor_gadget$+"Rising"+#LF$
  exceptions_editor_gadget$+"Sanskritise"+#LF$
  exceptions_editor_gadget$+"self-raising"+#LF$
  exceptions_editor_gadget$+"sensation"+#LF$
  exceptions_editor_gadget$+"sexercise"+#LF$
  exceptions_editor_gadget$+"sidewise"+#LF$
  exceptions_editor_gadget$+"signal-to-noise"+#LF$
  exceptions_editor_gadget$+"size"+#LF$
  exceptions_editor_gadget$+"sized"+#LF$
  exceptions_editor_gadget$+"slantwise"+#LF$
  exceptions_editor_gadget$+"statewise"+#LF$
  exceptions_editor_gadget$+"stepwise"+#LF$
  exceptions_editor_gadget$+"tortoise"+#LF$
  exceptions_editor_gadget$+"TransferWise"+#LF$
  exceptions_editor_gadget$+"streetwise"+#LF$
  exceptions_editor_gadget$+"subsensation"+#LF$
  exceptions_editor_gadget$+"Sunrise"+#LF$
  exceptions_editor_gadget$+"sunrise"+#LF$
  exceptions_editor_gadget$+"supersize"+#LF$
  exceptions_editor_gadget$+"supervise"+#LF$
  exceptions_editor_gadget$+"surmise"+#LF$
  exceptions_editor_gadget$+"surprise"+#LF$
  exceptions_editor_gadget$+"Surprise"+#LF$
  exceptions_editor_gadget$+"televise"+#LF$
  exceptions_editor_gadget$+"treatise"+#LF$
  exceptions_editor_gadget$+"turquoise"+#LF$
  exceptions_editor_gadget$+"uncompromised"+#LF$
  exceptions_editor_gadget$+"undersize"+#LF$
  exceptions_editor_gadget$+"undisguised"+#LF$
  exceptions_editor_gadget$+"unenterprising"+#LF$
  exceptions_editor_gadget$+"unpractised"+#LF$
  exceptions_editor_gadget$+"unrevised"+#LF$
  exceptions_editor_gadget$+"unsized"+#LF$
  exceptions_editor_gadget$+"unsupervised"+#LF$
  exceptions_editor_gadget$+"unwise"+#LF$
  exceptions_editor_gadget$+"upraise"+#LF$
  exceptions_editor_gadget$+"uprise"+#LF$
  exceptions_editor_gadget$+"uprising"+#LF$
  exceptions_editor_gadget$+"upsize"+#LF$
  exceptions_editor_gadget$+"valise"+#LF$
  exceptions_editor_gadget$+"vichyssoise"+#LF$
  exceptions_editor_gadget$+"wholewise"+#LF$
  exceptions_editor_gadget$+"widthwise"+#LF$
  exceptions_editor_gadget$+"wise"+#LF$
  exceptions_editor_gadget$+"Wise"+#LF$
  exceptions_editor_gadget$+"Woodrising"+#LF$
  exceptions_editor_gadget$+"workwise"+#LF$
  exceptions_editor_gadget$+"worldly-wise"+#LF$
  
  
;   ; Save the exceptions file, using UTF-8 plus BOM encoding
;   OpenFile(1,main_path$+"settings/"+"-ize-ise_exceptions.txt")
;     WriteStringFormat(1,#PB_UTF8)
;     WriteString(1,exceptions_editor_gadget$,#PB_UTF8)
;   CloseFile(1)
        
  
EndProcedure




; To create the default GB Specific
;
; V1.0 - 21/MAY/2019
; V1.1 - 15/FEB/2020
;        o All file operations support BOM.
;
Procedure load_gb_specific()
  
  ; Check if exceptions file exists
  If FileSize(main_path$+"settings/"+"gb_specific.txt")>0
    ReadFile(1,main_path$+"settings/"+"gb_specific.txt")
      ReadStringFormat(1)
      gb_specific_editor_gadget$=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)    
    CloseFile(1)
    ProcedureReturn #True
  EndIf     
  
EndProcedure




; To offer basic validation to the .AFF rules
;
; Based on the procedure of "Bulk Import"
; 
; V1.0 - 01/MAR/2020
;        02/MAR/2020
;        03/MAR/2020
;        04/MAR/2020
;        05/MAR/2020
;        07/MAR/2020
;        08/MAR/2020
;        10/MAR/2020
;        11/MAR/2020
;        13/MAR/2020
;        14/MAR/2020
;        16/MAR/2020
;        17/MAR/2020
;        22/MAR/2020
; V1.1 - 28/MAR/2020
;        29/MAR/2020
;        31/MAR/2020
;        o Enhancements
; V1.2 - 04/MAY/2020
;        o AFF Validate window now has a go to bottom button
;        o AFF Validate window now has a go to top button
; V1.3 - 12/OCT/2020
;        o Adding of F1 to open the help in the various windows.
; V1.4 - 10/NOV/2020
;        o Clear buttons now use a trashcan image.
; V1.5 - 20/NOV/2020
;        23/NOV/2020
;        24/NOV/2020
;        25/NOV/2020
;        o Major improvements
; V1.6 - 02/DEC/2020
;        o "AFF Validate": Now trying to refresh with no entries it says there is nothing to process
; V1.7 - 30/NOV/2021
;        o Started showing the time taken in timely operations.
; V1.8 - 28/DEC/2022
;        o Rewrote the ".AFF Validate"
; V1.9 - 04/JAN/2023
;        05/JAN/2023
;        09/JAN/2023
;        o ".AFF Validate": coding of "undo" and "redo"
;        o It is now possible to abort the processing of an .AFF
; V2.0 - 21/MAR/2023
;        o "AFF Validate": coding of * (changed)
;
Procedure aff_validate()
 
    If OpenWindow(#WINDOW_AFF_VALIDATE,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,"AFF Validate",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_AFF_VALIDATE,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)    
    
    ; EditorGadget in
    EditorGadget(#EDITOR_AFF_VALIDATE_IN,10,10,-5-15-20-20-30-30-15+100+400+get_extra_width(50),-10+400+get_extra_height(100)+25+3+2-8,#PB_Editor_ReadOnly)
    SetGadgetFont(#EDITOR_AFF_VALIDATE_IN,FontID(#font_for_language_id))
    
    ; EditorGadget out (ListIconGadget)
    ListIconGadget(#LISTICONGADGET_AFF_VALIDATE_OUT,10+GadgetX(#EDITOR_AFF_VALIDATE_IN)+GadgetWidth(#EDITOR_AFF_VALIDATE_IN)+10,10,5+15+20+20+30+30-15+100+400+get_extra_width(50),-default_gadget_height_text-5-10+400+get_extra_height(100)-8,"Line #",30+40+10+5,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection|#PB_ListIcon_CheckBoxes)
    AddGadgetColumn(#LISTICONGADGET_AFF_VALIDATE_OUT,1,"Text",GadgetWidth(#LISTICONGADGET_AFF_VALIDATE_OUT)-10-10-15-(30+40+10+5))
    SetGadgetFont(#LISTICONGADGET_AFF_VALIDATE_OUT,FontID(#font_for_language_id))
   
    ; OK button
    x=WindowWidth(#WINDOW_AFF_VALIDATE)-default_gadget_width_button_ok_close_cancel-10
    y=WindowHeight(#WINDOW_AFF_VALIDATE)-default_gadget_height_button-10
    ButtonGadget(#BUTTON_AFF_VALIDATE_OKAY,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"OK")
    
    ; CANCEL button
    x=GadgetX(#BUTTON_AFF_VALIDATE_OKAY)-10-5-default_gadget_width_button_ok_close_cancel
    ButtonGadget(#BUTTON_AFF_VALIDATE_CANCEL,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"Cancel")
    
    x=GadgetX(#EDITOR_AFF_VALIDATE_IN)
    y=GadgetY(#EDITOR_AFF_VALIDATE_IN)+GadgetHeight(#EDITOR_AFF_VALIDATE_IN)+10
    
    ; Process button
    ButtonGadget(#BUTTON_AFF_VALIDATE_IN_PROCESS,x,y,default_gadget_width_button,default_gadget_height_button,"Process")
    
    
    x+default_gadget_width_button+15-5
    
    ; Clear button
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_IN_CLEAR,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(101))
    GadgetToolTip(#BUTTON_AFF_VALIDATE_IN_CLEAR,"Clear")
    
    x+default_gadget_width_button_image_24x24+15-5   
    
    ; Refresh button
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_IN_REFRESH,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(108))
    GadgetToolTip(#BUTTON_AFF_VALIDATE_IN_REFRESH,"Refresh")
    
    x+default_gadget_width_button_image_24x24+16+16-8
    
    ; BOTTOM button IN gadget
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_BOTTOM,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(37))            
    GadgetToolTip(#BUTTON_AFF_VALIDATE_BOTTOM,"Go to bottom")       
    
    x+default_gadget_width_button_image_24x24+16-4-2  
    
    ; TOP button IN gadget
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_TOP,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(94))
    GadgetToolTip(#BUTTON_AFF_VALIDATE_TOP,"Go to top")
    
    
    y=GadgetY(#BUTTON_AFF_VALIDATE_TOP)+default_gadget_height_button_image_24x24+4
    x=GadgetX(#EDITOR_AFF_VALIDATE_IN)
    
    
    ; Processing X of Y
    TextGadget(#TEXT_AFF_VALIDATE_IN_PROCESSING,x,y,200,default_gadget_height_text,"Processing: 0/0")
    GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
    
    ; % of progress
    TextGadget(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,x,y+default_gadget_height_text+2,10,default_gadget_height_text,"")
    
    
    ; Progress bar of "Processing X of Y"
    ProgressBarGadget(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,x+GadgetWidth(#TEXT_AFF_VALIDATE_IN_PROCESSING)+10+5+5+10+10,y,default_gadget_width_progress,default_gadget_height_small_progress,0,1)
    HideGadget(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#True)
    
    x=GadgetX(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING)
    y+default_gadget_height_small_progress+2
    
    ; Time processing
    TextGadget(#TEXT_AFF_VALIDATE_TIME_TAKEN,x,y,100,default_gadget_height_text,"")
    
    y+default_gadget_height_text+2
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_AFF_VALIDATE_ESC_TO_ABORT,x,y,100,default_gadget_height_text,"")
    
    
    x=GadgetX(#LISTICONGADGET_AFF_VALIDATE_OUT)
    y=GadgetY(#LISTICONGADGET_AFF_VALIDATE_OUT)+GadgetHeight(#LISTICONGADGET_AFF_VALIDATE_OUT)+5+3
    
    ; Number of lines added
    TextGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,x,y,200,default_gadget_height_text,"Found:0")
    GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)
    x+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2
    TextGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,x,y,default_gadget_height_text,default_gadget_height_text,"*")
    GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
    SetGadgetColor(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,#PB_Gadget_FrontColor,#Red)
    SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,"")
    
    
    x+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+30+default_gadget_width_progress+10+20+20+15+5
    
    ; CLEAR button OUT gadget
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_CLEAR2,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(101))        
    GadgetToolTip(#BUTTON_AFF_VALIDATE_CLEAR2,"Clear")
    
    x+default_gadget_width_button_image_24x24+16+16-8
    
    ; BOTTOM button OUT gadget
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_BOTTOM2,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(37))        
    GadgetToolTip(#BUTTON_AFF_VALIDATE_BOTTOM2,"Go to bottom")
    
    x+default_gadget_width_button_image_24x24+16-4-2
    
    ; TOP button OUT gadget
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_TOP2,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(94))
    GadgetToolTip(#BUTTON_AFF_VALIDATE_TOP2,"Go to top")
    
    x=GadgetX(#BUTTON_AFF_VALIDATE_CLEAR2)-default_gadget_width_progress-10
    
    ; Progress bar for .AFF scan
    ProgressBarGadget(#PROGRESSBAR_AFF_VALIDATE2,x,y,default_gadget_width_progress,default_gadget_height_small_progress,0,1)
    HideGadget(#PROGRESSBAR_AFF_VALIDATE2,#True)
    
    ; Time below progress bar
    TextGadget(#TEXT_AFF_VALIDATE2,x,y+default_gadget_height_small_progress+3,10,default_gadget_height_text,"")
    
    y+default_gadget_height_text+3
    
    ; <ESC>> to abort below time
    TextGadget(#TEXT_AFF_VALIDATE2_ESC_TO_ABORT,x,y+default_gadget_height_small_progress+3,10,default_gadget_height_text,"")
    
    
    x=GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)
;     y+default_gadget_height_text+3
    
    
    ; Selected
    TextGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,x,y,200,default_gadget_height_text,"Selected:None")
    GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)
    
    
    x=GadgetX(#LISTICONGADGET_AFF_VALIDATE_OUT)+GadgetWidth(#LISTICONGADGET_AFF_VALIDATE_OUT)-default_gadget_width_button_image_24x24
    y=GadgetY(#LISTICONGADGET_AFF_VALIDATE_OUT)+GadgetHeight(#LISTICONGADGET_AFF_VALIDATE_OUT)+5+3
    
    ; Undo/Redo
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_IN_REDO,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(33))
    GadgetToolTip(#BUTTON_AFF_VALIDATE_IN_REDO,"Redo")
    x-default_gadget_height_button_image_24x24-10
    ButtonImageGadget(#BUTTON_AFF_VALIDATE_IN_UNDO,x,y,default_gadget_width_button_image_24x24,default_gadget_height_button_image_24x24,ImageID(34))
    GadgetToolTip(#BUTTON_AFF_VALIDATE_IN_UNDO,"Undo")
    aff_validate_disable_undo_redo()
        
    
    x-default_gadget_width_button-10
    
    ; Goto
    ButtonGadget(#BUTTON_AFF_VALIDATE_IN_GOTO,x,y,default_gadget_width_button,default_gadget_height_button,"Goto")
    
    
    ; Create Popup menus
    CreatePopupImageMenu(#POPUPMENU_AFF_VALIDATE,#PB_Menu_ModernLook)
    MenuItem(#POPUPMENU_AFF_VALIDATE_ADD_ABOVE,"Add Above",ImageID(90))    
    MenuItem(#POPUPMENU_AFF_VALIDATE_ADD_BELOW,"Add Below",ImageID(91))    
    MenuItem(#POPUPMENU_AFF_VALIDATE_EDIT,"Edit",ImageID(38))
    MenuItem(#POPUPMENU_AFF_VALIDATE_CLONE_BELOW,"Clone Below",ImageID(92))    
    MenuBar()
    MenuItem(#POPUPMENU_AFF_VALIDATE_SELECT_ALL,"Select All",ImageID(42))
    MenuItem(#POPUPMENU_AFF_VALIDATE_SELECT_NONE,"Select None",ImageID(43))
    MenuItem(#POPUPMENU_AFF_VALIDATE_SELECT_INVERT,"Invert Selection",ImageID(47))
    MenuBar()
    MenuItem(#POPUPMENU_AFF_VALIDATE_DELETE_SELECTED,"Delete Selected",ImageID(44))
    MenuItem(#POPUPMENU_AFF_VALIDATE_DELETE,"Delete",ImageID(39))
    

    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_AFF_VALIDATE,#PB_Shortcut_Escape,1000)    
    AddKeyboardShortcut(#WINDOW_AFF_VALIDATE,#PB_Shortcut_Control|#PB_Shortcut_G,1017)
    AddKeyboardShortcut(#WINDOW_AFF_VALIDATE,#PB_Shortcut_Control|#PB_Shortcut_Z,1018)
    AddKeyboardShortcut(#WINDOW_AFF_VALIDATE,#PB_Shortcut_Control|#PB_Shortcut_Y,1019)
    AddKeyboardShortcut(#WINDOW_AFF_VALIDATE,#PB_Shortcut_F1,1020)
    
    
    ; Description of option
    aff_validate_header$="AFF Validate V2.2 - 2023-03-21"+#LF$
    aff_validate_header$+"========================"+#LF$
    aff_validate_header$+"This option provides SMART validation of the AFF."+#LF$
    aff_validate_header$+""+#LF$
    aff_validate_header$+"After making changes in the panel in the right,"+#LF$
    aff_validate_header$+"if you wish to press the PROCESS button, you"+#LF$
    aff_validate_header$+"will have to press the REFRESH button first."+#LF$
    aff_validate_header$+""+#LF$
    SetGadgetText(#EDITOR_AFF_VALIDATE_IN,aff_validate_header$)
    
    
    ; Get .AFF lines and populate them in the ListIconGadget
    AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Processing number of lines...")
    aff_validate_disable_gadgets_on_off(#True)
    aff_validate_get_aff_lines_and_populate_them()
    aff_validate_disable_gadgets_on_off(#False)
    AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
    
    
    okay=#False
    cancel=#False
    Repeat
      
      event=WaitWindowEvent()
      event_gadget=EventGadget()
      event_type=EventType()
      
      
      ; Clicked on a checkbox of a line
      If event_type=#PB_EventType_Change And event_gadget=#LISTICONGADGET_AFF_VALIDATE_OUT And CountGadgetItems(#LISTICONGADGET_AFF_VALIDATE_OUT)>0
        Debug "check/uncheck item"
        
        aff_validate_number_lines_selected=0
        
        For source_line=1 To aff_validate_number_lines
          
          If GetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT, source_line-1) & #PB_ListIcon_Checked
            aff_validate_individual_line(source_line)\line_selected=#True
            aff_validate_number_lines_selected+1
          Else
            aff_validate_individual_line(source_line)\line_selected=#False 
          EndIf
          
        Next source_line 
        t$=Str(aff_validate_number_lines_selected)+"/"+Str(aff_validate_number_lines)
        If aff_validate_number_lines_selected=0 : t$="None" : EndIf
        If aff_validate_number_lines_selected=aff_validate_number_lines : t$="All" : EndIf
        SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:"+t$)
        GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)
       
      EndIf  
      
      
      ; POP-UP MENU
      If EventGadget()=#LISTICONGADGET_AFF_VALIDATE_OUT And EventType()=#PB_EventType_RightClick
        DisplayPopupMenu(#POPUPMENU_AFF_VALIDATE, WindowID(#WINDOW_AFF_VALIDATE))
      EndIf  
      
      Select event
  
      Case #PB_Event_Menu
      
        Select EventMenu()  ; To see which popup menu has been selected

          Case #POPUPMENU_AFF_VALIDATE_EDIT
            aff_validate_edit()
            popup_menu_gadget=#False
            
          Case #POPUPMENU_AFF_VALIDATE_ADD_ABOVE
            flag=#False
            
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf            
            
            t=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
            If aff_validate_number_lines>0 And t=-1 And flag=#False
              MessageRequester("Warning!","No line selected.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf                  
            
            ; Item selected is the last
            If flag=#False
  
              ; Selected last line in the ListIconGadget
              If aff_validate_number_lines-t=1
                
                ; Store undo
                aff_validate_store_undo()
                
                aff_validate_number_lines+1
                ReDim aff_validate_individual_line(aff_validate_number_lines)
                aff_validate_individual_line(aff_validate_number_lines)\line_text=aff_validate_individual_line(aff_validate_number_lines-1)\line_text
                aff_validate_individual_line(aff_validate_number_lines)\line_colour=aff_validate_individual_line(aff_validate_number_lines-1)\line_colour
                aff_validate_individual_line(aff_validate_number_lines)\line_selected=aff_validate_individual_line(aff_validate_number_lines-1)\line_selected
                aff_validate_individual_line(aff_validate_number_lines-1)\line_text=""
                aff_validate_individual_line(aff_validate_number_lines-1)\line_colour=#Black
                aff_validate_individual_line(aff_validate_number_lines-1)\line_selected=#False                
                AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,t,Str(aff_validate_number_lines-1)+#LF$+"")
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
                ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
                
                ; Update the number of each line in the gadget
                For f=0 To aff_validate_number_lines-1
                  SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f,Str(f+1),0)
                Next f                      
                
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)             
              
                flag=#True
              EndIf
            EndIf       
            
            ; Item selected is NOT the last
            If flag=#False
  
              ; Selected NOT last line in the ListIconGadget
              If aff_validate_number_lines-t<>1
                
                ; Store undo
                aff_validate_store_undo()
                
                aff_validate_number_lines+1
                ReDim aff_validate_individual_line(aff_validate_number_lines)
                For f=aff_validate_number_lines To t+1+1-1 Step -1
                  aff_validate_individual_line(f)\line_text=aff_validate_individual_line(f-1)\line_text
                  aff_validate_individual_line(f)\line_colour=aff_validate_individual_line(f-1)\line_colour
                  aff_validate_individual_line(f)\line_selected=aff_validate_individual_line(f-1)\line_selected
                Next f
                aff_validate_individual_line(t+1+1-1)\line_text=""
                aff_validate_individual_line(t+1+1-1)\line_colour=#Black
                aff_validate_individual_line(t+1+1-1)\line_selected=#False
                
                AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,t+1-1,Str(aff_validate_number_lines)+#LF$+"")
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL) 
                ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
                
                ; Update the number of each line in the gadget
                For f=0 To aff_validate_number_lines-1
                  SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f,Str(f+1),0)
                Next f               
                
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)             
              
                flag=#True
              EndIf
            EndIf             
            
            popup_menu_gadget=#False
            
                        
          Case #POPUPMENU_AFF_VALIDATE_ADD_BELOW
            flag=#False
            
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf            
            
            t=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
            If aff_validate_number_lines>0 And t=-1 And flag=#False
              MessageRequester("Warning!","No line selected.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf                  
            
            ; Item selected is the last
            If flag=#False
  
              ; Selected last line in the ListIconGadget
              If aff_validate_number_lines-t=1
                
                ; Store undo
                aff_validate_store_undo()
                
                aff_validate_number_lines+1
                ReDim aff_validate_individual_line(aff_validate_number_lines)
                aff_validate_individual_line(aff_validate_number_lines)\line_text=""
                aff_validate_individual_line(aff_validate_number_lines)\line_colour=#Black
                aff_validate_individual_line(aff_validate_number_lines)\line_selected=#False
                AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,-1,Str(aff_validate_number_lines)+#LF$+aff_validate_individual_line(aff_validate_number_lines)\line_text)
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)    
                ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
              
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)             
              
                flag=#True
              EndIf
            EndIf       
            
            ; Item selected is NOT the last
            If flag=#False
  
              ; Selected NOT last line in the ListIconGadget
              If aff_validate_number_lines-t<>1
                
                ; Store undo
                aff_validate_store_undo()
                
                aff_validate_number_lines+1
                ReDim aff_validate_individual_line(aff_validate_number_lines)
                For f=aff_validate_number_lines To t+1+1 Step -1
                  aff_validate_individual_line(f)\line_text=aff_validate_individual_line(f-1)\line_text
                  aff_validate_individual_line(f)\line_colour=aff_validate_individual_line(f-1)\line_colour
                  aff_validate_individual_line(f)\line_selected=aff_validate_individual_line(f-1)\line_selected
                Next f
                aff_validate_individual_line(t+1+1)\line_text=""
                aff_validate_individual_line(t+1+1)\line_colour=#Black
                aff_validate_individual_line(t+1+1)\line_selected=#False
                
                AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,t+1,Str(aff_validate_number_lines)+#LF$+aff_validate_individual_line(t+1+1)\line_text)
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
                ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
                
                ; Update the number of each line in the gadget
                For f=0 To aff_validate_number_lines-1
                  SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f,Str(f+1),0)
                Next f               
                
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)             
              
                flag=#True
              EndIf
            EndIf             
            
            popup_menu_gadget=#False
            
            
          Case #POPUPMENU_AFF_VALIDATE_CLONE_BELOW
            flag=#False
            
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf            
            
            t=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
            If aff_validate_number_lines>0 And t=-1 And flag=#False
              MessageRequester("Warning!","No line selected.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf                  
            
            ; Item selected is the last
            If flag=#False
  
              ; Selected last line in the ListIconGadget
              If aff_validate_number_lines-t=1
                
                ; Store undo
                aff_validate_store_undo()
                
                aff_validate_number_lines+1
                ReDim aff_validate_individual_line(aff_validate_number_lines)
                aff_validate_individual_line(aff_validate_number_lines)\line_text=aff_validate_individual_line(aff_validate_number_lines-1)\line_text
                aff_validate_individual_line(aff_validate_number_lines)\line_colour=#Black
                aff_validate_individual_line(aff_validate_number_lines)\line_selected=#False
                AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,-1,Str(aff_validate_number_lines)+#LF$+aff_validate_individual_line(aff_validate_number_lines)\line_text)
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
                ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
              
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)             
              
                flag=#True
              EndIf
            EndIf       
            
            ; Item selected is NOT the last
            If flag=#False
  
              ; Selected NOT last line in the ListIconGadget
              If aff_validate_number_lines-t<>1
                
                ; Store undo
                aff_validate_store_undo()
                
                aff_validate_number_lines+1
                ReDim aff_validate_individual_line(aff_validate_number_lines)
                For f=aff_validate_number_lines To t+1+1 Step -1
                  aff_validate_individual_line(f)\line_text=aff_validate_individual_line(f-1)\line_text
                  aff_validate_individual_line(f)\line_colour=aff_validate_individual_line(f-1)\line_colour
                  aff_validate_individual_line(f)\line_selected=aff_validate_individual_line(f-1)\line_selected
                Next f
                AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,t+1,Str(aff_validate_number_lines)+#LF$+aff_validate_individual_line(t+1+1)\line_text)
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
                ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
                
                ; Update the number of each line in the gadget
                For f=0 To aff_validate_number_lines-1
                  SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f,Str(f+1),0)
                Next f               
                
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)             
              
                flag=#True
              EndIf
            EndIf             
            
            popup_menu_gadget=#False
            
          Case #POPUPMENU_AFF_VALIDATE_SELECT_ALL  
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
            Else
              For f=1 To aff_validate_number_lines
                aff_validate_individual_line(f)\line_selected=#True
                SetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,#PB_ListIcon_Checked)
              Next f
              aff_validate_number_lines_selected=aff_validate_number_lines
              SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:All")
              GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)
            EndIf
            popup_menu_gadget=#False
            
          Case #POPUPMENU_AFF_VALIDATE_SELECT_NONE
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
            Else
              For f=1 To aff_validate_number_lines
                aff_validate_individual_line(f)\line_selected=#False
                SetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,#False)
              Next f
              aff_validate_number_lines_selected=0
              SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:None")
              GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)
            EndIf
            popup_menu_gadget=#False
            
          Case #POPUPMENU_AFF_VALIDATE_SELECT_INVERT
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
            Else
              counter=0
              For f=1 To aff_validate_number_lines
                If aff_validate_individual_line(f)\line_selected=#False
                  aff_validate_individual_line(f)\line_selected=#True
                  SetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,#PB_ListIcon_Checked)
                  counter+1
                Else
                  aff_validate_individual_line(f)\line_selected=#False
                  SetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,#False)         
                EndIf
              Next f
              aff_validate_number_lines_selected=counter
              t$=Str(aff_validate_number_lines_selected)
              If aff_validate_number_lines_selected=aff_validate_number_lines : t$="All" : EndIf
              If aff_validate_number_lines_selected=0 : t$="None" : EndIf
              SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:"+t$)
              GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)
            EndIf
            popup_menu_gadget=#False

          Case #POPUPMENU_AFF_VALIDATE_DELETE_SELECTED
            flag=#False
            
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf            
            
            If aff_validate_number_lines_selected=0 And flag=#False
              MessageRequester("Warning!","No selected lines found.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf                  
            
            If flag=#False
              If MessageRequester("Warning!","Really delete selected lines?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                
                ; Store undo
                aff_validate_store_undo()
                
                
                ; Only one line or all of them selected (to become=0)
                If aff_validate_number_lines_selected=aff_validate_number_lines
                  aff_validate_number_lines_selected=0
                  aff_validate_number_lines=0
                  ReDim aff_validate_individual_line(0)
                  ClearGadgetItems(#LISTICONGADGET_AFF_VALIDATE_OUT)
                  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:None")
                  GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)            
                  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:0")
                  GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
                  ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                  GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
                  Continue
                EndIf
Debug "KARALHO1:aff_validate_number_lines:"+aff_validate_number_lines 
Debug "KARALHO2:aff_validate_number_lines_selected:"+aff_validate_number_lines_selected
                
                ; Resize buffer array
                ReDim aff_validate_individual_line_buffer(aff_validate_number_lines-aff_validate_number_lines_selected)
                
                
                ; Copy non-selected lines to buffer array
                t=aff_validate_number_lines-aff_validate_number_lines_selected
                For f=aff_validate_number_lines To 1 Step -1
                  If aff_validate_individual_line(f)\line_selected=#True
                    aff_validate_individual_line_buffer(t)\line_text=aff_validate_individual_line(f)\line_text
                    aff_validate_individual_line_buffer(t)\line_colour=aff_validate_individual_line(f)\line_colour
                    aff_validate_individual_line_buffer(t)\line_selected=#False
                    aff_validate_individual_line(f)\line_selected=#False ; Fix: 17/MAR/2023
                    RemoveGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1)
                  EndIf
                  t-1
                Next f
                
                
                ; Updated total lines plus reset total selected lines
                aff_validate_number_lines-aff_validate_number_lines_selected
                aff_validate_number_lines_selected=0
                
; Below was causing bug in redo
 If fuk=32525               
                ; Swap buffer array with main
                ReDim aff_validate_individual_line(aff_validate_number_lines)
                CopyArray(aff_validate_individual_line_buffer(),aff_validate_individual_line())
 EndIf               
                
                ; Update the number of each line in the gadget
                For f=0 To aff_validate_number_lines-1
                  SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f,Str(f+1),0)
                Next f
                
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0) 
  
;               EndIf
              t$=Str(aff_validate_number_lines_selected)+"/"+Str(aff_validate_number_lines)
              If aff_validate_number_lines_selected=aff_validate_number_lines : t$="All" : EndIf
              If aff_validate_number_lines_selected=0 : t$="None" : EndIf
              SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:"+t$)
              GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)            
              SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
              GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
              ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
              GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
EndIf   
            EndIf
            flag=#True
            popup_menu_gadget=#False
            
          Case #POPUPMENU_AFF_VALIDATE_DELETE
            flag=#False
            
            If aff_validate_number_lines=0
              MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf            
            
            t=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
            If aff_validate_number_lines>0 And t=-1 And flag=#False
              MessageRequester("Warning!","No line selected.",#PB_MessageRequester_Warning)
              flag=#True
            EndIf                  
            
            If flag=#False
              If MessageRequester("Warning!","Really delete line?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                
                ; Store undo
                aff_validate_store_undo()
                
                t=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
                RemoveGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,t)
                t+1
                If aff_validate_individual_line(t)\line_selected=#True : aff_validate_number_lines_selected-1 : EndIf
                t$=Str(aff_validate_number_lines_selected)+"/"+Str(aff_validate_number_lines-1)
                If aff_validate_number_lines_selected=aff_validate_number_lines-1 : t$="All" : EndIf
                If aff_validate_number_lines_selected=0 : t$="None" : EndIf
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:"+t$)
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)            
                SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines-1))
                GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
                ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
                GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
                
                If t=aff_validate_number_lines
                  aff_validate_number_lines-1
                  ReDim aff_validate_individual_line(aff_validate_number_lines)
                Else
                  For f=t+1 To aff_validate_number_lines
                    aff_validate_individual_line(t)\line_text=aff_validate_individual_line(f)\line_text
                    aff_validate_individual_line(t)\line_colour=aff_validate_individual_line(f)\line_colour
                    aff_validate_individual_line(t)\line_selected=aff_validate_individual_line(f)\line_selected
                    t+1
                  Next f
                  aff_validate_number_lines-1
                  ReDim aff_validate_individual_line(aff_validate_number_lines)
                EndIf
                
                For f=0 To aff_validate_number_lines-1
                  SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f,Str(f+1),0)
                Next f
                
                ; Colour of first column grey
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0) 
  
              EndIf
            EndIf
            flag=#True
            popup_menu_gadget=#False
            
            
        EndSelect    
      EndSelect        
    
        
        
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        SetGadgetText(#TEXT_AFF_VALIDATE_ESC_TO_ABORT,"")
        cancel=#True
      EndIf
      
      ; Shortcut for CTR+G
      If event=#PB_Event_Menu And EventMenu()=1017
        Debug "CTR+G pressed"
        goto_position(0,0,0,0,1)
      EndIf
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And EventMenu()=1020
        Debug "F1 pressed"
        html_help("#5.2.5.1.aff_validate")
      EndIf
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf      
      
      
      ; Double-click on ListIconGadget
      If event_gadget=#LISTICONGADGET_AFF_VALIDATE_OUT And event_type=#PB_EventType_LeftDoubleClick
        aff_validate_edit()
      EndIf
      

      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
            Case  #BUTTON_AFF_VALIDATE_IN_GOTO
              goto_position(0,0,0,0,1)
              
            Case #BUTTON_AFF_VALIDATE_CANCEL
              cancel=#True              
              
            Case #BUTTON_AFF_VALIDATE_OKAY
              okay=#True
              
            Case #BUTTON_AFF_VALIDATE_IN_PROCESS
              If aff_validate_number_lines=0
                MessageRequester("Warning!","Nothing to process.",#PB_MessageRequester_Warning)
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Nothing to process.")
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
              Else
                
                ; Store undo
                aff_validate_store_undo()
                
                aff_validate_disable_gadgets_on_off(#True)
                SetGadgetText(#TEXT_AFF_VALIDATE_ESC_TO_ABORT,"<ESC> to abort.")
                SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,0)
                HideGadget(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#False)  
                aff_validate_process()
                SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,0)
                HideGadget(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#True)                  
                aff_validate_disable_gadgets_on_off(#False)
                SetGadgetText(#TEXT_AFF_VALIDATE_ESC_TO_ABORT,"")
                SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,"")
              EndIf
              
            Case #BUTTON_AFF_VALIDATE_IN_CLEAR  
              SetGadgetText(#EDITOR_AFF_VALIDATE_IN,aff_validate_header$)
              
            Case #BUTTON_AFF_VALIDATE_IN_REFRESH
              
              aff_validate_disable_gadgets_on_off(#True)
              
              ; Remmed lines below since the contents of the ListIconGadget don't change with a refresh - 24/NOV/2020
;               ; Clear the ListIconGadget AFF
;               ClearGadgetItems(#LISTICONGADGET_AFF_VALIDATE_OUT)
              
              AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Converting the right panel to .AFF text")              
              
              ; Nothing to process
              If aff_validate_number_lines=0
                MessageRequester("Warning!","Nothing to process.",#PB_MessageRequester_Warning)
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Nothing to process.")
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
              EndIf
              
              
              ; Get the new .AFF text gadget
              t$=""
              If aff_validate_number_lines>0
                
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")                
                
                ; Add timer to avoid freezing
                AddWindowTimer(#WINDOW_AFF_VALIDATE,800,800)
  
                ; Progress bar gadget
                SetGadgetAttribute(#PROGRESSBAR_AFF_VALIDATE2,#PB_ProgressBar_Maximum,aff_validate_number_lines)
                SetGadgetState(#PROGRESSBAR_AFF_VALIDATE2,0)
                HideGadget(#PROGRESSBAR_AFF_VALIDATE2,#False)
                
                For f=1 To aff_validate_number_lines
                  t$+aff_validate_individual_line(f)\line_text+#LF$
                  
                  ; Fix to window stop responding if we click on it while in the loop, plus update the progress bar
                  event = WindowEvent()
                  If event=#PB_Event_Timer And EventTimer()=800
                    ; Update progress bar
                    SetGadgetState(#PROGRESSBAR_AFF_VALIDATE2,f)
                  EndIf
                  
                Next f
                
                ; Remove timer
                RemoveWindowTimer(#WINDOW_AFF_VALIDATE,800)                  
                
                HideGadget(#PROGRESSBAR_AFF_VALIDATE2,#True)
                
                
                ; Replacing .AFF gadget
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Replacing the original .AFF gadget")
                t1$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
                SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,t$)
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
              
                ; Processing the .AFF gadget
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Processing the .AFF gadget")
                derived_aff$="" 
                faster_searchable_version_of_aff()              
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
              
              ; Remmed lines below since the contents of the ListIconGadget don't change with a refresh - 24/NOV/2020
;               ; Get .AFF lines and populate them in the ListIconGadget
;               AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Processing number of lines...")   
;               aff_validate_get_aff_lines_and_populate_them()
;               AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
              
                ; Restoring text in .AFF gadget
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Restoring back the original .AFF gadget")
                SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,t1$)
                AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
                derived_aff$=""
              
              EndIf              
              
              aff_validate_disable_gadgets_on_off(#False)
              
            Case #BUTTON_AFF_VALIDATE_BOTTOM
              SetPosBottom(#EDITOR_AFF_VALIDATE_IN,1)
              
            Case #BUTTON_AFF_VALIDATE_TOP
              SetPosTop(#EDITOR_AFF_VALIDATE_IN,1)              
              
            Case #BUTTON_AFF_VALIDATE_CLEAR2
              aff_validate_clear_aff()
              
            Case #BUTTON_AFF_VALIDATE_BOTTOM2
              SetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT,CountGadgetItems(#LISTICONGADGET_AFF_VALIDATE_OUT)-1)  
              
            Case #BUTTON_AFF_VALIDATE_TOP2
              SetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT,0)                
              
              
            Case #BUTTON_AFF_VALIDATE_IN_UNDO
              aff_validate_store_redo()
              aff_validate_restore_undo()
              
            Case #BUTTON_AFF_VALIDATE_IN_REDO
              aff_validate_store_undo()
              aff_validate_restore_redo()
              
          EndSelect
          
      EndSelect
      
    Until okay=#True Or cancel=#True
    
    If okay=#True
      t$=""
      If aff_validate_number_lines>0
        For f=1 To aff_validate_number_lines
          t$+aff_validate_individual_line(f)\line_text+#LF$
        Next f
      EndIf
      
      If t$<>aff_validate_aff_original$
        SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,t$)
        dictionary_changed=#True
      EndIf
      
    EndIf
    
    
    CloseWindow(#WINDOW_AFF_VALIDATE)
    DisableWindow(#WINDOW_MAIN,#False)  
    
EndProcedure




; Gets the lines of the .AFF and inserts them in the ListIconGadget
;
; V1.0 - 01/MAR/2020
;        02/MAR/2020
;        20/NOV/2020
;        28/DEC/2022
;        05/JAN/2023
;
Procedure aff_validate_get_aff_lines_and_populate_them()

  aff_validate_aff_original$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
  aff_validate_aff_original$=ConvertStringToUnix(aff_validate_aff_original$)
  aff_validate_number_lines=0  
  t$=aff_validate_aff_original$
  
   ; AFF is blank
  If t$=""
    MessageRequester("Warning!","Nothing to process.",#PB_MessageRequester_Warning)
    AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Nothing to process.")
    ProcedureReturn
  EndIf
  
  ; Add a #LF$ so that we have at least one line
  If Right(t$,1)<>#LF$ : t$+#LF$ : EndIf
  
  ; Create the array to store the lines in
  aff_validate_number_lines=CountString(t$,#LF$)
  Dim aff_validate_individual_line(aff_validate_number_lines)
  
  ; Add timer to avoid freezing
  AddWindowTimer(#WINDOW_AFF_VALIDATE,800,800)
  
  ; Progress bar gadget
  SetGadgetAttribute(#PROGRESSBAR_AFF_VALIDATE2,#PB_ProgressBar_Maximum,aff_validate_number_lines*2)
  SetGadgetState(#PROGRESSBAR_AFF_VALIDATE2,0)
  HideGadget(#PROGRESSBAR_AFF_VALIDATE2,#False)
  aff_validate_number_lines_total=0
  
  time_taken=ElapsedMilliseconds()
  SetGadgetText(#TEXT_AFF_VALIDATE2,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  GadgetFit(#TEXT_AFF_VALIDATE2)
  
  SetGadgetText(#TEXT_AFF_VALIDATE2_ESC_TO_ABORT,"<ESC> to abort.")
  GadgetFit(#TEXT_AFF_VALIDATE2_ESC_TO_ABORT)
  
  ; Copy the .AFF lines into the array
  For f=1 To aff_validate_number_lines
    
    aff_validate_individual_line(f)\line_text=StringField(t$,f,#LF$)
    aff_validate_individual_line(f)\line_colour=#Black
    aff_validate_individual_line(f)\line_selected=#False
    
    
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_AFF_VALIDATE,800)
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE2,0)
        SetGadgetText(#TEXT_AFF_VALIDATE2_ESC_TO_ABORT,"")
        SetGadgetText(#TEXT_AFF_VALIDATE2,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        GadgetFit(#TEXT_AFF_VALIDATE2)
        ProcedureReturn
      EndIf
    EndIf
    
    
    ; Fix to window stop responding if we click on it while in the loop, plus update the progress bar
    event = WindowEvent()
    If event=#PB_Event_Timer And EventTimer()=800
      
      SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(f)+ConvertProgressToPercentage(f+aff_validate_number_lines_total,aff_validate_number_lines*2,1)) 
      GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)
      
      ; Update progress bar
      SetGadgetState(#PROGRESSBAR_AFF_VALIDATE2,f+aff_validate_number_lines_total)    
      
      ; Update time taken
      SetGadgetText(#TEXT_AFF_VALIDATE2,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
      GadgetFit(#TEXT_AFF_VALIDATE2)
      
    EndIf
    
  Next f
  
  
  ; So that progress bar should all this procedure code (multiplied*2)
  aff_validate_number_lines_total=f
  
  
  ; Number of lines processed
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Number of lines processed:"+Str(aff_validate_number_lines))
  

  ; Populate the ListIconGadget
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Populating gadget with the lines...")
  For f=1 To aff_validate_number_lines
    
    AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,-1,Str(f)+#LF$+aff_validate_individual_line(f)\line_text)
    
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_AFF_VALIDATE,800)
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE2,0)
        SetGadgetText(#TEXT_AFF_VALIDATE2_ESC_TO_ABORT,"")
        SetGadgetText(#TEXT_AFF_VALIDATE2,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        GadgetFit(#TEXT_AFF_VALIDATE2)
        ProcedureReturn
      EndIf
    EndIf
    
    ; Fix to window stop responding if we click on it while in the loop, plus update the progress bar
    event = WindowEvent()
    If event=#PB_Event_Timer And EventTimer()=800
      
      SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(f)+ConvertProgressToPercentage(f+aff_validate_number_lines_total,aff_validate_number_lines*2,1)) 
      GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)
      
      ; Update progress bar
      SetGadgetState(#PROGRESSBAR_AFF_VALIDATE2,f+aff_validate_number_lines_total)
      
      ; Update time taken
      SetGadgetText(#TEXT_AFF_VALIDATE2,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
      GadgetFit(#TEXT_AFF_VALIDATE2)
      
    EndIf
        
  Next f
  
  ; Colour of first column grey
  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)    
  
  ; Lines found
  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))  
  GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)
  ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
  GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
  
  ; Remove timer
  RemoveWindowTimer(#WINDOW_AFF_VALIDATE,800)  
  
  ; Populating done
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Populating done.")
  
  ; Hide progress bar gadget
  HideGadget(#PROGRESSBAR_AFF_VALIDATE2,#True)
  
  ; Update time
  SetGadgetText(#TEXT_AFF_VALIDATE2,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  GadgetFit(#TEXT_AFF_VALIDATE2)
  
  ; Remove "<ESC> to abort."
  SetGadgetText(#TEXT_AFF_VALIDATE2_ESC_TO_ABORT,"")
  
EndProcedure




; Uses the rules to validate the .AFF and spot possible errors
;
; V1.0 - 02/MAR/2020
;        04/MAR/2020
;        07/MAR/2020
;        09/MAR/2020
;        10/MAR/2020
;        11/MAR/2020
;        14/MAR/2020
;        16/MAR/2020
;        17/MAR/2020
;        22/MAR/2020
; V1.1 - 28/MAR/2020
;        29/MAR/2020
;        31/MAR/2020
;        24/NOV/2020
;        25/NOV/2020
;        o Enhancements
; V1.2 - 30/NOV/2021
;        o Started showing the time taken in timely operations.
; V1.3 - 04/MAR/2023
;        o Minor speed up processing the .AFF after checking for excessive spaces
;
Procedure aff_validate_process()
  
  current_process_number=0
  total_process_number=6
  
  
  ; Turn lines to black
  For f=1 To aff_validate_number_lines
    aff_validate_individual_line(f)\line_colour=#Black
  Next f
  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,1)
  
  
  ; Global Error variable, so that at the end of processing it informs the user if any errors were found
  global_error_variable=#False
  
  
  time_taken=ElapsedMilliseconds()
  
  ; Add timer to avoid freezing
  AddWindowTimer(#WINDOW_AFF_VALIDATE,900,900)
  
  
  ; Step 1 - searching for "SET UTF-8"
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  aff_validate_process_current_text_buffer_in_steps$=""
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  current_process_number+1
  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:"+Str(current_process_number)+"/"+Str(total_process_number))
  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Step "+Str(current_process_number)+"/"+Str(total_process_number)+" - Searching For "+Chr(34)+"SET UTF-8"+Chr(34))
  counter=CountString(aff_validate_aff_original$,"SET UTF-8")
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  ; Progressbar gadget
  SetGadgetAttribute(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#PB_ProgressBar_Maximum,aff_validate_number_lines*2)
  SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,0)
  
  ; Check if UTF-8 is in Remmed lines
  If counter>0
    
    setutf8_not_at_start=#False ; 30/DEC/2022
    counter_real=0 ; So that it bypasses remmed lines with multiple SET UTF-8
    For f=1 To aff_validate_number_lines
      t$=Trim(aff_validate_individual_line(f)\line_text)
      If Left(t$,1)<>"#" And CountString(t$,"SET UTF-8")>0 : counter_real+CountString(t$,"SET UTF-8") : EndIf
      
      ; 30/DEC/2022 - Check if SET UTF-8 is not the first command
      If setutf8_not_at_start=#False And counter_real=0 And Left(t$,1)<>"#" And t$<>""
        setutf8_not_at_start=#True
      EndIf
      
      ; 30/DEC/2022
      remmed$=""
      If CountString(t$,"SET UTF-8")>0
        If Left(t$,1)="#" : remmed$=" (remmed)" : EndIf
        If CountString(t$,"SET UTF-8")>1 : remmed$+" (multiple)" : EndIf
        AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,Chr(34)+"SET UTF-8"+Chr(34)+" found at line:"+Str(f)+remmed$)
      EndIf
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,aff_validate_number_lines*2,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
    Next f
    counter=counter_real
    
  EndIf  
  
  ; Turn lines to Red
  If counter>1
    
    For f=1 To aff_validate_number_lines
      t$=Trim(aff_validate_individual_line(f)\line_text)
      If Left(t$,1)<>"#" And CountString(t$,"SET UTF-8")>0
        SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,#PB_Gadget_FrontColor,#Red,1)
        aff_validate_individual_line(f)\line_colour=#Red
;         AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,Chr(34)+"SET UTF-8"+Chr(34)+" found at line:"+Str(f))      
        global_error_variable=#True
      EndIf
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f+aff_validate_number_lines)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f+aff_validate_number_lines,aff_validate_number_lines*2,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
    Next f    
    
  EndIf  
  
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Error: Not found in the .AFF")
    global_error_variable=#True
  EndIf  
  If counter>1
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Error: Found "+Str(counter)+" occurences in the .AFF")
    global_error_variable=#True
  EndIf
  If counter=1 And setutf8_not_at_start=#False
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"OK! Found one entry in the .AFF")
  EndIf
  If setutf8_not_at_start=#True
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Error: "+Chr(34)+"SET UTF-8"+Chr(34)+" is not the first command in the .AFF.")
    global_error_variable=#True
  EndIf
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$+#LF$)
  
  
  
  ; Step 2 - Searching and validating reserved flags
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  aff_validate_process_current_text_buffer_in_steps$=""
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  current_process_number+1
  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:"+Str(current_process_number)+"/"+Str(total_process_number))
  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Step "+Str(current_process_number)+"/"+Str(total_process_number)+" - Searching and validating reserved flags")
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Not coded yet!")
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  

  
  ; Step 3 - Listing flags in .AFF
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  aff_validate_process_current_text_buffer_in_steps$=""
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  current_process_number+1
  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:"+Str(current_process_number)+"/"+Str(total_process_number))
  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Step "+Str(current_process_number)+"/"+Str(total_process_number)+" - Listing flags in .AFF")
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  ; Progressbar gadget
  SetGadgetAttribute(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#PB_ProgressBar_Maximum,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter)
  SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,0)  
  
  ; Prefixes
  total_of_rules_found_pfx=0
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Showing Prefixes...")  
  counter=ultrafast_aff_search_pfx_sfx_line_pfx_counter
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No Prefixes found!"+#LF$)
  Else
    t$="entries"
    If counter=1 : t$="entry" : EndIf
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Found "+Str(counter)+" "+t$+":")
    counter_in_string=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))
    For f=1  To counter_in_string
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
      t$=StringField(ultrafast_aff_search_pfx_sfx_line$,f,Chr(1))
      If StringField(t$,1," ")="PFX"
        AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Flag "+Chr(34)+StringField(t$,3," ")+Chr(34)+" count "+StringField(t$,5," ")+" at line:"+StringField(t$,4," "))
        counter-1
        total_of_rules_found_pfx+Val(StringField(t$,5," "))
      EndIf
      If counter=0 : Break : EndIf
    Next f
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  EndIf
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  
  ; Suffixes
  total_of_rules_found_sfx=0
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Showing Suffixes...")  
  counter=ultrafast_aff_search_pfx_sfx_line_sfx_counter
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No Suffixes found!"+#LF$)
  Else
    t$="entries"
    If counter=1 : t$="entry" : EndIf    
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Found "+Str(counter)+" "+t$+":")
    counter_in_string=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))
    For f=1  To counter_in_string
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,ultrafast_aff_search_pfx_sfx_line_pfx_counter+f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(ultrafast_aff_search_pfx_sfx_line_pfx_counter+f,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
      t$=StringField(ultrafast_aff_search_pfx_sfx_line$,f,Chr(1))
      If StringField(t$,1," ")="SFX"
        AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Flag "+Chr(34)+StringField(t$,3," ")+Chr(34)+" count "+StringField(t$,5," ")+" at line:"+StringField(t$,4," "))
        counter-1
        total_of_rules_found_sfx+Val(StringField(t$,5," "))
      EndIf
      If counter=0 : Break : EndIf
    Next f
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  EndIf
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  
  ; Total Prefixes + Suffixes found + rules of each
  aff_validate_process_current_text_buffer_in_steps$=""
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Total of rules Prefixes ("+Str(total_of_rules_found_pfx)+") + Suffixes ("+Str(total_of_rules_found_sfx)+") found:"+Str(total_of_rules_found_pfx+total_of_rules_found_sfx))
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Total of Prefixes ("+Str(ultrafast_aff_search_pfx_sfx_line_pfx_counter)+") + Suffixes ("+Str(ultrafast_aff_search_pfx_sfx_line_sfx_counter)+") found:"+Str(ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter))
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$+#LF$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  
  
  ; Step 4 - Searching for duplicate flags
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  aff_validate_process_current_text_buffer_in_steps$=""
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  current_process_number+1
  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:"+Str(current_process_number)+"/"+Str(total_process_number))
  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Step "+Str(current_process_number)+"/"+Str(total_process_number)+" - Searching for duplicate flags")
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  ; Progressbar gadget
  SetGadgetAttribute(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#PB_ProgressBar_Maximum,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter)
  SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,0)    
  
  counter=ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No flags found!"+#LF$) 
  ElseIf counter>1
    found_duplicates=#False ; So that it shows at the end if no duplicates are found
    SortArray(ultrafast_aff_search_pfx_sfx_line_check_for_dupes$(),#PB_Sort_Ascending)
    For f=0 To counter-1
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
      t$=ultrafast_aff_search_pfx_sfx_line_check_for_dupes$(f)
      occurrences=0
      lines$=""
      
      For ff=f To counter-1            
          
        If t$=ultrafast_aff_search_pfx_sfx_line_check_for_dupes$(ff)
          occurrences+1          
        Else
          Break
        EndIf
                             
      Next ff
      
      f=ff-1
      
      ; Add duplicate word to  Editor gadget and red the ListIconGadget lines
      If occurrences>1
        found_duplicates=#True
        AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Flag:"+t$+"("+Str(occurrences)+")")
        counter_in_string=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))
        For fff=1 To counter_in_string
          
          
          ; <ESC> pressed to abort
          If EventMenu()=1000
            If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
              SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
              GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
              RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
              ProcedureReturn #False
            EndIf      
          EndIf

       
          ; Fix to window stop responding if we click on it while in the loop
          event = WindowEvent()
          If event=#PB_Event_Timer And EventTimer()=900
            SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
            SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
            ; Add % of progress
            SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter,1))
            GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
          EndIf        
          
          t1$=StringField(ultrafast_aff_search_pfx_sfx_line$,fff,Chr(1))
          If StringField(t1$,3," ")=t$
            AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"  Line:"+StringField(t1$,4," "))
            SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t1$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
            aff_validate_individual_line(Val(StringField(t1$,4," ")))\line_colour=#Red            
            global_error_variable=#True
          EndIf
        Next fff
      EndIf
      
    Next f  
    
  EndIf
  If counter=1 Or found_duplicates=#False
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No duplicate flags found!")
  EndIf
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$+#LF$)
  
  
  
  ; Step 5 - Counting number of rules per flag
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  aff_validate_process_current_text_buffer_in_steps$=""
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  current_process_number+1
  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:"+Str(current_process_number)+"/"+Str(total_process_number))
  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Step "+Str(current_process_number)+"/"+Str(total_process_number)+" - Counting number of rules per flag")
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  ; Progressbar gadget
  SetGadgetAttribute(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#PB_ProgressBar_Maximum,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter)
  SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,0)    
  
  ; Prefixes
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Analysing Prefixes...")  
  counter=ultrafast_aff_search_pfx_sfx_line_pfx_counter
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No Prefixes found!"+#LF$)
  Else
    counter_in_string=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))
    errors_found_in_rules=#False ; To show if it found any invalid count of rules. #False=No errors, #True=Errors found
    For f=1  To counter_in_string
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
      t$=StringField(ultrafast_aff_search_pfx_sfx_line$,f,Chr(1))
      rules_found_in_flag_counter=0
      rules_expected_in_flag_counter=Val(StringField(t$,5," "))
      If StringField(t$,1," ")="PFX"
        For ff=Val(StringField(t$,4," "))+1 To aff_validate_number_lines
          current_line_in_aff$=Trim(aff_validate_individual_line(ff)\line_text)
          current_line_to_find_on_the_left$=StringField(t$,1," ")+" "+StringField(t$,3," ")+" "
          If Left(current_line_in_aff$,Len(current_line_to_find_on_the_left$))=current_line_to_find_on_the_left$
            rules_found_in_flag_counter+1
          Else
            Break
          EndIf
        Next ff
        If rules_found_in_flag_counter<>rules_expected_in_flag_counter
          errors_found_in_rules=#True
          AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Flag "+Chr(34)+StringField(t$,3," ")+Chr(34)+" expected "+Str(rules_expected_in_flag_counter)+" found "+Str(rules_found_in_flag_counter)+" at line:"+StringField(t$,4," "))
          SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
          aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red   
          global_error_variable=#True
        EndIf
      EndIf
      If counter=0 : Break : EndIf
    Next f
    If errors_found_in_rules=#False
      AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No errors found!")
    EndIf
  EndIf
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$+#LF$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  ; Suffixes
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Analysing Suffixes...")  
  counter=ultrafast_aff_search_pfx_sfx_line_sfx_counter
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No Suffixes found!"+#LF$)
  Else
    counter_in_string=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))
    errors_found_in_rules=#False ; To show if it found any invalid count of rules. #False=No errors, #True=Errors found
    For f=1  To counter_in_string
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,ultrafast_aff_search_pfx_sfx_line_pfx_counter+f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(ultrafast_aff_search_pfx_sfx_line_pfx_counter+f,ultrafast_aff_search_pfx_sfx_line_pfx_counter+ultrafast_aff_search_pfx_sfx_line_sfx_counter,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
      t$=StringField(ultrafast_aff_search_pfx_sfx_line$,f,Chr(1))
      rules_found_in_flag_counter=0
      rules_expected_in_flag_counter=Val(StringField(t$,5," "))
      If StringField(t$,1," ")="SFX"
        For ff=Val(StringField(t$,4," "))+1 To aff_validate_number_lines
          current_line_in_aff$=Trim(aff_validate_individual_line(ff)\line_text)
          current_line_to_find_on_the_left$=StringField(t$,1," ")+" "+StringField(t$,3," ")+" "
          If Left(current_line_in_aff$,Len(current_line_to_find_on_the_left$))=current_line_to_find_on_the_left$
            rules_found_in_flag_counter+1
          Else
            Break
          EndIf
        Next ff
        If rules_found_in_flag_counter<>rules_expected_in_flag_counter
          errors_found_in_rules=#True
          AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Flag "+Chr(34)+StringField(t$,3," ")+Chr(34)+" expected "+Str(rules_expected_in_flag_counter)+" found "+Str(rules_found_in_flag_counter)+" at line:"+StringField(t$,4," "))
          SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
          aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red   
          global_error_variable=#True
        EndIf
      EndIf
      If counter=0 : Break : EndIf
    Next f
    If errors_found_in_rules=#False
      AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No errors found!")
    EndIf
  EndIf    
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$+#LF$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  
  
  ; Step 6 - Searching for repeated/invalid rules in flags
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  aff_validate_process_current_text_buffer_in_steps$=""
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"")
  current_process_number+1
  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:"+Str(current_process_number)+"/"+Str(total_process_number))
  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Step "+Str(current_process_number)+"/"+Str(total_process_number)+" - Searching for repeated/invalid rules in flags")
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  ; Progressbar gadget
  count_string_max_progress_bar=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))*2
  SetGadgetAttribute(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,#PB_ProgressBar_Maximum,count_string_max_progress_bar)
  SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,0)    
  
  ; Prefixes
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Analysing Prefixes...")  
  counter=ultrafast_aff_search_pfx_sfx_line_pfx_counter
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No Prefixes found!"+#LF$)
  Else
    counter_in_string=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))
    counter_in_string_sum_for_progress_bar=counter_in_string
    errors_found_in_rules=#False ; To show if it found any invalid count of rules. #False=No errors, #True=Errors found    
    For f=1  To counter_in_string      
      SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))    
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf      
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,count_string_max_progress_bar,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf           
      
      Dim rule_per_line$(0) ; To store each line of rules + #LF$ + line number
      t$=StringField(ultrafast_aff_search_pfx_sfx_line$,f,Chr(1))
      rules_found_in_flag_counter=0
      rules_expected_in_flag_counter=Val(StringField(t$,5," "))
      If StringField(t$,1," ")="PFX" : AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Flag "+Chr(34)+StringField(t$,3," ")+Chr(34)) : EndIf
      If StringField(t$,1," ")="PFX"
        For ff=Val(StringField(t$,4," "))+1 To aff_validate_number_lines
          
          ; <ESC> pressed to abort
          If EventMenu()=1000
            If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
              SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
              GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
              RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
              ProcedureReturn #False
            EndIf      
          EndIf
          
          ; Fix to window stop responding if we click on it while in the loop
          event = WindowEvent()
          If event=#PB_Event_Timer And EventTimer()=900
            SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
            SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
            ; Add % of progress
            SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,count_string_max_progress_bar,1))
            GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
          EndIf      
          
          
          current_line_in_aff$=Trim(aff_validate_individual_line(ff)\line_text)
          current_line_to_find_on_the_left$=StringField(t$,1," ")+" "+StringField(t$,3," ")+" "
          If Left(current_line_in_aff$,Len(current_line_to_find_on_the_left$))=current_line_to_find_on_the_left$
            rules_found_in_flag_counter+1
            ReDim rule_per_line$(rules_found_in_flag_counter-1)
            rule_per_line$(rules_found_in_flag_counter-1)=TrimStringAll(current_line_in_aff$)+#LF$+Str(ff)
            If StringField(current_line_in_aff$,5," ")=""
              AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Not 5 fields at line "+Str(ff)+": "+Chr(34)+current_line_in_aff$+Chr(34)+" (in blue)")
              SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
              aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red 
              SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,ff-1,#PB_Gadget_FrontColor,#Blue,1)
              aff_validate_individual_line(ff)\line_colour=#Blue                             
              errors_found_in_rules=#True
              global_error_variable=#True
            EndIf
            
            ; Excessive spaces *START*     
            t_excessive_spaces$=current_line_in_aff$
            t=FindString(t_excessive_spaces$,"  ")
            If t
              position_of_rem=FindString(t_excessive_spaces$+"#","#")
;               If position_of_rem=0 : position_of_rem=999999 : EndIf
              If t<position_of_rem
                AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Excessive spaces at line "+Str(ff)+": "+Chr(34)+current_line_in_aff$+Chr(34))
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red 
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,ff-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(ff)\line_colour=#Red                             
                errors_found_in_rules=#True
                global_error_variable=#True
              EndIf
            EndIf
            ; Excessive spaces *END*
            
            
            ; Minor speed up processing the .AFF after checking for excessive spaces - 4/MAR/2023
            current_line_in_aff$=TrimStringAll(current_line_in_aff$)
            
            
            ; Mistatch of brackets/^ *START*
            t_brackets_carrets$=StringField(current_line_in_aff$,5," ") 
            If t_brackets_carrets$<>"" And (FindString(t_brackets_carrets$,"[") Or FindString(t_brackets_carrets$,"]") Or FindString(t_brackets_carrets$,"^"))              
              mismatch_of_brackets_and_carrets=#False
              If CountString(t_brackets_carrets$,"^")<>CountString(t_brackets_carrets$,"[^")
                mismatch_of_brackets_and_carrets=#True                
              EndIf
              If CountString(t_brackets_carrets$,"[]")
                mismatch_of_brackets_and_carrets=#True                
              EndIf              
              If CountString(t_brackets_carrets$,"[^]")
                mismatch_of_brackets_and_carrets=#True                
              EndIf               
              If mismatch_of_brackets_and_carrets=#False ; And (FindString(t_brackets_carrets$,"[") Or FindString(t_brackets_carrets$,"]"))
                open_bracket_left=0
                For brackets=1 To Len(t_brackets_carrets$)
                  
                  ; <ESC> pressed to abort
                  If EventMenu()=1000
                    If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                      SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
                      GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
                      RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
                      ProcedureReturn #False
                    EndIf      
                  EndIf
                  
                  ; Fix to window stop responding if we click on it while in the loop
                  event = WindowEvent()
                  If event=#PB_Event_Timer And EventTimer()=900
                    SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
                    SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
                    ; Add % of progress
                    SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,count_string_max_progress_bar,1))
                    GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
                  EndIf   
                  
                  If Mid(t_brackets_carrets$,brackets,1)="["
                    open_bracket_left+1
                    If open_bracket_left>1
                      mismatch_of_brackets_and_carrets=#True
                      Break
                    EndIf
                  EndIf
                  If Mid(t_brackets_carrets$,brackets,1)="]"
                    open_bracket_left-1
                    If open_bracket_left<0
                      mismatch_of_brackets_and_carrets=#True
                      Break
                    EndIf
                  EndIf 
                Next brackets
                If open_bracket_left<>0 : mismatch_of_brackets_and_carrets=#True : EndIf
              EndIf
              If mismatch_of_brackets_and_carrets=#True
                AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Mistmatch of brackets/^ at line "+Str(ff)+": "+Chr(34)+current_line_in_aff$+Chr(34))
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red 
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,ff-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(ff)\line_colour=#Red                             
                mismatch_of_brackets_and_carrets=#False
                errors_found_in_rules=#True
                global_error_variable=#True
              EndIf       
            EndIf
            ; Mistatch of brackets/^ *END*
          Else
            Break
          EndIf
          
          ; To avoid constant refresh
          time_avoid_refresh$=TimeToString("⌚",ElapsedMilliseconds()-time_taken)
          If GetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN)<>time_avoid_refresh$
            SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,time_avoid_refresh$)          
          EndIf
        
        Next ff
        If rules_found_in_flag_counter>0
          SortArray(rule_per_line$(),#PB_Sort_Ascending)
          For ff=0 To rules_found_in_flag_counter-1
            ttt$=rule_per_line$(ff)
            occurrences=0
        
              For fff=ff To rules_found_in_flag_counter-1            
                
                ; <ESC> pressed to abort
                If EventMenu()=1000
                  If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                    SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
                    GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
                    RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
                    ProcedureReturn #False
                  EndIf      
                EndIf                
                
                ; Fix to window stop responding if we click on it while in the loop
                event = WindowEvent()
                If event=#PB_Event_Timer And EventTimer()=900
                  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
                  SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
                  ; Add % of progress
                  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(f,count_string_max_progress_bar,1))
                  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
                EndIf      
                
                
                If StringField(ttt$,1,#LF$)=StringField(rule_per_line$(fff),1,#LF$)
                occurrences+1    
                Else
                  Break
                EndIf
                             
              Next fff
      
              ff=fff-1
      
              ; Add duplicate word to  Editor gadget and red the ListIconGadget lines
              If occurrences>1
                errors_found_in_rules=#True
                For fff=ff-occurrences+1 To ff
                  
                  ; Fix to window stop responding if we click on it while in the loop
                  event = WindowEvent()
                  If event=#PB_Event_Timer And EventTimer()=900
                    SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
                    SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,f)
                  EndIf      
                  
                  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Found duplicate rule at line "+StringField(rule_per_line$(fff),2,#LF$)+": "+Chr(34)+aff_validate_individual_line(Val(StringField(rule_per_line$(fff),2,#LF$)))\line_text+Chr(34))
                  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(rule_per_line$(fff),2,#LF$))-1,#PB_Gadget_FrontColor,#Red,1)
                  aff_validate_individual_line(Val(StringField(rule_per_line$(fff),2,#LF$)))\line_colour=#Red                     
                  global_error_variable=#True
                Next fff
              EndIf  
              
            ; To avoid constant refresh
            time_avoid_refresh$=TimeToString("⌚",ElapsedMilliseconds()-time_taken)
            If GetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN)<>time_avoid_refresh$
              SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,time_avoid_refresh$)          
            EndIf
          Next ff
          
        EndIf
        
      EndIf
      
    Next f
    If errors_found_in_rules=#False
      AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No errors found!")
    EndIf
  EndIf
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$+#LF$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  
  ; Suffixes
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Analysing Suffixes...")  
  counter=ultrafast_aff_search_pfx_sfx_line_sfx_counter
  If counter=0
    AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No Suffixes found!"+#LF$)
  Else
    counter_in_string=CountString(ultrafast_aff_search_pfx_sfx_line$,Chr(1))
    errors_found_in_rules=#False ; To show if it found any invalid count of rules. #False=No errors, #True=Errors found    
    For f=1  To counter_in_string  
      SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))    
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
          GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
          RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
          ProcedureReturn #False
        EndIf  
      EndIf
      
      ; Fix to window stop responding if we click on it while in the loop
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=900
        SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,counter_in_string_sum_for_progress_bar+f)
        ; Add % of progress
        SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(counter_in_string_sum_for_progress_bar+f,count_string_max_progress_bar,1))
        GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
      EndIf      
      
      Dim rule_per_line$(0) ; To store each line of rules + #LF$ + line number
      t$=StringField(ultrafast_aff_search_pfx_sfx_line$,f,Chr(1))
      rules_found_in_flag_counter=0
      rules_expected_in_flag_counter=Val(StringField(t$,5," "))
      If StringField(t$,1," ")="SFX" : AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"Flag "+Chr(34)+StringField(t$,3," ")+Chr(34)) : EndIf
      If StringField(t$,1," ")="SFX"
        For ff=Val(StringField(t$,4," "))+1 To aff_validate_number_lines
          
          ; <ESC> pressed to abort
          If EventMenu()=1000
            If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
              SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
              GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
              RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
              ProcedureReturn #False
            EndIf
          EndIf
          
          ; Fix to window stop responding if we click on it while in the loop
          event = WindowEvent()
          If event=#PB_Event_Timer And EventTimer()=900
            SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
            SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,counter_in_string_sum_for_progress_bar+f)
            ; Add % of progress
            SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(counter_in_string_sum_for_progress_bar+f,count_string_max_progress_bar,1))
            GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
          EndIf      
          
          current_line_in_aff$=Trim(aff_validate_individual_line(ff)\line_text)
          current_line_to_find_on_the_left$=StringField(t$,1," ")+" "+StringField(t$,3," ")+" "
          If Left(current_line_in_aff$,Len(current_line_to_find_on_the_left$))=current_line_to_find_on_the_left$
            rules_found_in_flag_counter+1
            ReDim rule_per_line$(rules_found_in_flag_counter-1)
            rule_per_line$(rules_found_in_flag_counter-1)=TrimStringAll(current_line_in_aff$)+#LF$+Str(ff)
            If StringField(current_line_in_aff$,5," ")=""
              AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Not 5 fields at line "+Str(ff)+": "+Chr(34)+current_line_in_aff$+Chr(34)+" (in blue)")
              SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
              aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red 
              SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,ff-1,#PB_Gadget_FrontColor,#Blue,1)
              aff_validate_individual_line(ff)\line_colour=#Blue                             
              errors_found_in_rules=#True
              global_error_variable=#True
            EndIf
            
            ; Excessive spaces *START*     
            t_excessive_spaces$=current_line_in_aff$
            t=FindString(t_excessive_spaces$,"  ")
            If t
              position_of_rem=FindString(t_excessive_spaces$+"#","#")
;               If position_of_rem=0 : position_of_rem=999999 : EndIf
              If t<position_of_rem
                AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Excessive spaces at line "+Str(ff)+": "+Chr(34)+current_line_in_aff$+Chr(34))
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red 
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,ff-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(ff)\line_colour=#Red                             
                errors_found_in_rules=#True
                global_error_variable=#True
              EndIf
            EndIf
            ; Excessive spaces *END*
            
            
            ; Minor speed up processing the .AFF after checking for excessive spaces - 4/MAR/2023
            current_line_in_aff$=TrimStringAll(current_line_in_aff$)
            
            
            ; Mistatch of brackets/^ *START*
            t_brackets_carrets$=StringField(current_line_in_aff$,5," ") 
            If t_brackets_carrets$<>"" And (FindString(t_brackets_carrets$,"[") Or FindString(t_brackets_carrets$,"]") Or FindString(t_brackets_carrets$,"^"))              
              mismatch_of_brackets_and_carrets=#False
              If CountString(t_brackets_carrets$,"^")<>CountString(t_brackets_carrets$,"[^")
                mismatch_of_brackets_and_carrets=#True                
              EndIf
              If CountString(t_brackets_carrets$,"[]")
                mismatch_of_brackets_and_carrets=#True                
              EndIf              
              If CountString(t_brackets_carrets$,"[^]")
                mismatch_of_brackets_and_carrets=#True                
              EndIf                             
              If mismatch_of_brackets_and_carrets=#False ; And (FindString(t_brackets_carrets$,"[") Or FindString(t_brackets_carrets$,"]"))
                open_bracket_left=0
                For brackets=1 To Len(t_brackets_carrets$)
                  
                  ; <ESC> pressed to abort
                  If EventMenu()=1000
                    If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                      SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
                      GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
                      RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
                      ProcedureReturn #False
                    EndIf
                  EndIf
                  
                  ; Fix to window stop responding if we click on it while in the loop
                  event = WindowEvent()
                  If event=#PB_Event_Timer And EventTimer()=900
                    SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
                    SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,counter_in_string_sum_for_progress_bar+f)
                    ; Add % of progress
                    SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(counter_in_string_sum_for_progress_bar+f,count_string_max_progress_bar,1))
                    GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
                  EndIf      
                  
                  If Mid(t_brackets_carrets$,brackets,1)="["
                    open_bracket_left+1
                    If open_bracket_left>1
                      mismatch_of_brackets_and_carrets=#True
                      Break
                    EndIf
                  EndIf
                  If Mid(t_brackets_carrets$,brackets,1)="]"
                    open_bracket_left-1
                    If open_bracket_left<0
                      mismatch_of_brackets_and_carrets=#True
                      Break
                    EndIf
                  EndIf 
                Next brackets
                If open_bracket_left<>0 : mismatch_of_brackets_and_carrets=#True : EndIf
              EndIf
              If mismatch_of_brackets_and_carrets=#True
                AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Mistmatch of brackets/^ at line "+Str(ff)+": "+Chr(34)+current_line_in_aff$+Chr(34))
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(t$,4," "))-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(Val(StringField(t$,4," ")))\line_colour=#Red 
                SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,ff-1,#PB_Gadget_FrontColor,#Red,1)
                aff_validate_individual_line(ff)\line_colour=#Red                             
                mismatch_of_brackets_and_carrets=#False
                errors_found_in_rules=#True
                global_error_variable=#True
              EndIf       
            EndIf
            ; Mistatch of brackets/^ *END*            
          Else
            Break
          EndIf
          
          ; To avoid constant refresh
          time_avoid_refresh$=TimeToString("⌚",ElapsedMilliseconds()-time_taken)
          If GetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN)<>time_avoid_refresh$
            SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,time_avoid_refresh$)          
          EndIf
        
        Next ff
        If rules_found_in_flag_counter>0
          SortArray(rule_per_line$(),#PB_Sort_Ascending)
          For ff=0 To rules_found_in_flag_counter-1
            
            ; <ESC> pressed to abort
            If EventMenu()=1000
              If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
                GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
                RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
                ProcedureReturn #False
              EndIf
            EndIf            
            
            ; Fix to window stop responding if we click on it while in the loop
            event = WindowEvent()
            If event=#PB_Event_Timer And EventTimer()=900
              SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
              SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,counter_in_string_sum_for_progress_bar+f)
              ; Add % of progress
              SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(counter_in_string_sum_for_progress_bar+f,count_string_max_progress_bar,1))
              GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
            EndIf      
            
            ttt$=rule_per_line$(ff)
            occurrences=0
        
              For fff=ff To rules_found_in_flag_counter-1            
          
                If StringField(ttt$,1,#LF$)=StringField(rule_per_line$(fff),1,#LF$)
                occurrences+1    
                Else
                  Break
                EndIf
                             
              Next fff
      
              ff=fff-1
      
              ; Add duplicate word to  Editor gadget and red the ListIconGadget lines
              If occurrences>1
                errors_found_in_rules=#True
                For fff=ff-occurrences+1 To ff
                  
                  ; <ESC> pressed to abort
                  If EventMenu()=1000
                    If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                      SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:Aborted.")
                      GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)
                      RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)
                      ProcedureReturn #False
                    EndIf
                  EndIf                              
                  
                  ; Fix to window stop responding if we click on it while in the loop
                  event = WindowEvent()
                  If event=#PB_Event_Timer And EventTimer()=900
                    SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
                    SetGadgetState(#PROGRESSBAR_AFF_VALIDATE_IN_PROCESSING,counter_in_string_sum_for_progress_bar+f)
                    ; Add % of progress
                    SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE,ConvertProgressToPercentage(counter_in_string_sum_for_progress_bar+f,count_string_max_progress_bar,1))
                    GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING_PERCENTAGE)
                  EndIf      
                  
                  AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN," Found duplicate rule at line "+StringField(rule_per_line$(fff),2,#LF$)+": "+Chr(34)+aff_validate_individual_line(Val(StringField(rule_per_line$(fff),2,#LF$)))\line_text+Chr(34))
                  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,Val(StringField(rule_per_line$(fff),2,#LF$))-1,#PB_Gadget_FrontColor,#Red,1)
                  aff_validate_individual_line(Val(StringField(rule_per_line$(fff),2,#LF$)))\line_colour=#Red                     
                  global_error_variable=#True
                Next fff
              EndIf  
              
              ; To avoid constant refresh
              time_avoid_refresh$=TimeToString("⌚",ElapsedMilliseconds()-time_taken)
              If GetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN)<>time_avoid_refresh$
                SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,time_avoid_refresh$)          
              EndIf    
              
          Next ff
          
        EndIf
        
      EndIf
      
    Next f
    If errors_found_in_rules=#False
      AddLineEditorGadgetBuffer_old(#EDITOR_AFF_VALIDATE_IN,"No errors found!")
    EndIf
  EndIf
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,aff_validate_process_current_text_buffer_in_steps$+#LF$)
  aff_validate_process_current_text_buffer_in_steps$=""
  
  
  ; Done!
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
  If global_error_variable=#True
    AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Errors found in validation!")
  Else
    AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"No errors found in validation!")
  EndIf  
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"Done!")
  
  
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,"")
  AddLineEditorGadget(#EDITOR_AFF_VALIDATE_IN,""+#LF$)
  
  ; Done!
  SetGadgetText(#TEXT_AFF_VALIDATE_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  SetGadgetText(#TEXT_AFF_VALIDATE_IN_PROCESSING,"Processing:"+Str(current_process_number)+"/"+Str(total_process_number)+" Done!")
  GadgetFit(#TEXT_AFF_VALIDATE_IN_PROCESSING)  
  
  ; Remove timer
  RemoveWindowTimer(#WINDOW_AFF_VALIDATE,900)  
  
EndProcedure




; EDIT AFF VALIDATE LINES for pop-up menu
;
; Based on the Autocorrect code
;
; V1.0 - 13/MAR/2020
;        31/MAR/2020
;        30/DEC/2022
;
Procedure aff_validate_edit()
    
  If aff_validate_number_lines=0
    MessageRequester("Warning!","No lines found.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf    
    
  t=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
  If t=-1
    MessageRequester("Warning!","No line selected.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
    
         
  If OpenWindow(#WINDOW_AFF_VALIDATE_EDIT,0,0,20+(800-640)+640-10-100,10+20+2+3-10-20-40-50-200+200-50-20-20+100+100+200-30-30-30+10,"Edit an AFF Validate line",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
  DisableWindow(#WINDOW_AFF_VALIDATE,#True)
        
  ; Incorrect field
  ImageGadget(#IMAGE_AFF_VALIDATE_EDIT_INCORRECT,0+6+3+2,1+1+1+2+0+6+2+1,16,16,ImageID(83))
  t$=aff_validate_individual_line(t+1)\line_text
  StringGadget(#STRING_AFF_VALIDATE_EDIT_INCORRECT,-25+10+0+6+100-15-15-20,6+0+6,20+320-10-20-40+20+30+30+60+70+70+70+20,default_gadget_height_string,t$,#PB_String_ReadOnly)
  SetGadgetFont(#STRING_AFF_VALIDATE_EDIT_INCORRECT,FontID(#font_for_language_id))
    
  ; Correct field
  ImageGadget(#IMAGE_AFF_VALIDATE_EDIT_CORRECT,0+6+3+2,1+1+1+2+6+0+6+2+1+20+5+2+5+3,16,16,ImageID(84))
  StringGadget(#STRING_AFF_VALIDATE_EDIT_CORRECT,-25+10+0+6+100-15-15-20,2+3+6+0+6+2+1+20+5+5+3,20+320-10-20-40+20+30+30+60+70+70+70+20,default_gadget_height_string,t$)      
  SetGadgetFont(#STRING_AFF_VALIDATE_EDIT_CORRECT,FontID(#font_for_language_id))
  SetActiveGadget(#STRING_AFF_VALIDATE_EDIT_CORRECT)
  
  y=WindowHeight(#WINDOW_AFF_VALIDATE_EDIT)-10-default_gadget_height_button
  
  ; Add the OK and CANCEL buttons
  ButtonGadget(#BUTTON_AFF_VALIDATE_EDIT_OKAY,0+6+100-15,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"OK")
  ButtonGadget(#BUTTON_AFF_VALIDATE_EDIT_CANCEL,0+6+100-15+100,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"Cancel")
            
  ; Add keyboard shortcuts
  AddKeyboardShortcut(#WINDOW_AFF_VALIDATE_EDIT,#PB_Shortcut_Escape,1000)    
  AddKeyboardShortcut(#WINDOW_AFF_VALIDATE_EDIT,#PB_Shortcut_Return,1003)    
  AddKeyboardShortcut(#WINDOW_AFF_VALIDATE_EDIT,#PB_Shortcut_F1,1004)
    
  okay=#False
  cancel=#False
  Repeat
        
    event=WaitWindowEvent()
      
      
    ; Pressed the close window gadget (Ubuntu)
    If event=#PB_Event_CloseWindow
      Debug "Close window pressed (Ubuntu)"
      cancel=#True     
    EndIf     
      
      
    ; Check if <ESC> was pressed
    If event=#PB_Event_Menu And EventMenu()=1000
      Debug "<ESC> was pressed"
      cancel=#True
    EndIf        
      
      
    ; Check if <ENTER> was pressed
    If event=#PB_Event_Menu And EventMenu()=1003
      Debug "OK was pressed"
      PostEvent(#PB_Event_Gadget, #WINDOW_AFF_VALIDATE_EDIT, #BUTTON_AFF_VALIDATE_EDIT_OKAY)
    EndIf      
      
      
    ; Shortcut for F1 (help)
    If event=#PB_Event_Menu And EventMenu()=1004
      html_help("")
    EndIf         
    
    
    Select Event 
      Case #PB_Event_Gadget
        Select EventGadget()
                                         
         ; Press OK
         Case #BUTTON_AFF_VALIDATE_EDIT_OKAY
           okay=#True
           
         ; Press CANCEL  
         Case #BUTTON_AFF_VALIDATE_EDIT_CANCEL
           cancel=#True             
             
        EndSelect
     EndSelect                 
                 
      
  Until okay=#True Or cancel=#True
   
  ; OK button pressed. It will now add or update an autocorrect word    
  If okay=#True      
    t$=GetGadgetText(#STRING_AFF_VALIDATE_EDIT_CORRECT)
    
    ; store undo
    If aff_validate_individual_line(t+1)\line_text<>t$
      aff_validate_store_undo()
    EndIf
    
    aff_validate_individual_line(t+1)\line_text=t$
    SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,t,t$,1)
    SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,t,#PB_Gadget_FrontColor,aff_validate_individual_line(t+1)\line_colour,1)
  EndIf
    
    
  ; Close the Add/Edit window and activate the main window
  DisableWindow(#WINDOW_AFF_VALIDATE,#False)
  CloseWindow(#WINDOW_AFF_VALIDATE_EDIT) 
    
EndProcedure 




  ; Clears the EditorGadget of the aff but inside the "AFF Validate"
  ;
  ; V1.0 - 20/NOV/2020
  ;
  Procedure aff_validate_clear_aff()
    
    ; Confirm if to erase the Hyphenation if the file in memory has been changed
    If aff_validate_number_lines<>0
      If MessageRequester("Warning!","Really delete all lines?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf
    
    ; Store undo
    aff_validate_store_undo()

    aff_validate_number_lines_selected=0
    aff_validate_number_lines=0
    ReDim aff_validate_individual_line(0)
    ClearGadgetItems(#LISTICONGADGET_AFF_VALIDATE_OUT)
    SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:None")
    GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)            
    SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:0")
    GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL) 
    ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
    GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
    
  EndProcedure
  
  
  
  
  ; Disables the gadgets before processing and enables after
  ;
  ; V1.0 - 20/NOV/2020
  ;        04/JAN/2023
  ;        09/JAN/2023
  ;
  Procedure aff_validate_disable_gadgets_on_off(is_disable)

    DisableGadget(#BUTTON_AFF_VALIDATE_OKAY,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_CANCEL,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_IN_PROCESS,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_IN_CLEAR,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_IN_REFRESH,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_BOTTOM,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_TOP,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_CLEAR2,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_BOTTOM2,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_TOP2,is_disable)
    DisableGadget(#BUTTON_AFF_VALIDATE_IN_GOTO,is_disable)
    DisableGadget(#LISTICONGADGET_AFF_VALIDATE_OUT,is_disable)
    
    
    ; Enable/disable undo/redo buttons
    If is_disable=#False
      flag=#True
      If aff_validate_undo=#True : flag=#False : EndIf
      DisableGadget(#BUTTON_AFF_VALIDATE_IN_UNDO,flag)
      flag=#True
      If aff_validate_redo=#True : flag=#False : EndIf
      DisableGadget(#BUTTON_AFF_VALIDATE_IN_REDO,flag)  
    Else
      DisableGadget(#BUTTON_AFF_VALIDATE_IN_UNDO,#True)
      DisableGadget(#BUTTON_AFF_VALIDATE_IN_REDO,#True)
    EndIf
    
  EndProcedure
  
  
  
  
  ; Add text to buffer string to avoid refreshing the EditorGadget and cause a large CPU load
  ;
  ; V1.0 - 24/NOV/2020
  ;
  Procedure AddLineEditorGadgetBuffer_old(gadget,text$)
    aff_validate_process_current_text_buffer_in_steps$+text$+#LF$
  EndProcedure
  
  
  
  
; Disable undo + redo
;
; V1.0 - 28/MAR/2020
;
Procedure aff_validate_disable_undo_redo()
  
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_UNDO,#True)
  Dim aff_validate_undo_array(0)
  Dim aff_validate_undo_array_total_selected(0)
  aff_validate_undo=#False
  aff_validate_undo_number_items=0
  aff_validate_undo_selected_item=-1
  aff_validate_undo_items_total_selected=0
  
  
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_REDO,#True)    
  Dim aff_validate_redo_array(0)
  Dim aff_validate_redo_array_total_selected(0)
  aff_validate_redo=#False
  aff_validate_redo_number_items=0
  aff_validate_redo_selected_item=-1
  aff_validate_redo_items_total_selected=0
  
EndProcedure




; Aff validate Store undo
;
; V1.0 - 04/JAN/2022
;
Procedure aff_validate_store_undo()
  
  Debug "STORE UNDO"
  
  ; Enable undo button and disable redo button
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_UNDO,#False)
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_REDO,#True)  
    
  aff_validate_undo=#True
  aff_validate_redo=#False
  
  ReDim aff_validate_undo_array(ArraySize(aff_validate_individual_line()))
  CopyArray(aff_validate_individual_line(),aff_validate_undo_array())
  aff_validate_undo_number_items=aff_validate_number_lines
  aff_validate_undo_selected_item=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
  aff_validate_undo_items_total_selected=aff_validate_number_lines_selected
  
  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,"*")
  
EndProcedure




; Restore undo
;
; V1.0 - 09/JAN/2023
;
Procedure aff_validate_restore_undo()
  
  Debug "RESTORE UNDO"
  
  ; Disable undo button and enable redo button
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_UNDO,#True)
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_REDO,#False)   
  
  
  aff_validate_undo=#False
  aff_validate_redo=#True
  
  
  ReDim aff_validate_individual_line(ArraySize(aff_validate_undo_array()))
  CopyArray(aff_validate_undo_array(),aff_validate_individual_line())
;   aff_validate_number_lines=aff_validate_undo_number_items
;   SetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT,aff_validate_undo_selected_item)
;   aff_validate_number_lines_selected=aff_validate_undo_items_total_selected
  
  
  ; Add blank lines if the undo has more lines
  counter=aff_validate_undo_number_items-aff_validate_number_lines
  If counter>0
    For f=1 To counter
      AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,-1,"")
    Next f
  EndIf
  
  
  ; Remove lines if the undo has less lines
  counter=aff_validate_number_lines-aff_validate_undo_number_items
  If counter>0
    For f=aff_validate_number_lines-1 To aff_validate_number_lines-1-counter+1 Step -1
      RemoveGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,f)
    Next f
  EndIf  
  
  
  ; Update the total of variables
  aff_validate_number_lines=aff_validate_undo_number_items
  SetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT,aff_validate_undo_selected_item)
  aff_validate_number_lines_selected=aff_validate_undo_items_total_selected
  
  
  ; Update the ListIconGadget items
  counter=aff_validate_number_lines
  If counter>0
;     For f=0 To counter-1
;       t$=Str(f+1)+#LF$+aff_validate_individual_line(f)\line_text
    For f=1 To counter
      t$=Str(f)+#LF$+aff_validate_individual_line(f)\line_text
    
      SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,t$)
      SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,#PB_Gadget_FrontColor,aff_validate_individual_line(f)\line_colour)  
      t=aff_validate_individual_line(f)\line_selected
      If t=#True : t=#PB_ListIcon_Checked : EndIf
      SetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,t)
    Next f
  EndIf
  
  
  ; Colour of first column grey
  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)  

  
  ; Set the selected line again
  SetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT,aff_validate_undo_selected_item)
  

  ; Set the numbers in their places
  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
  GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)               
  ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
  GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK) 
  t$=Str(aff_validate_number_lines_selected)+"/"+Str(aff_validate_number_lines)
  If aff_validate_number_lines_selected=0 : t$="None" : EndIf
  If aff_validate_number_lines_selected=aff_validate_number_lines : t$="All" : EndIf
  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:"+t$)
  GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)
        
EndProcedure




; Store redo
;
; V1.0 - 4/MAR/2023
;
Procedure aff_validate_store_redo()
  Debug "STORE REDO"
  Debug "FUCKER STORE REDO"

  ReDim aff_validate_redo_array(ArraySize(aff_validate_individual_line()))
  CopyArray(aff_validate_individual_line(),aff_validate_redo_array())

  aff_validate_redo_number_items=aff_validate_number_lines
  aff_validate_redo_selected_item=GetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT)
  aff_validate_redo_items_total_selected=aff_validate_number_lines_selected

If fuk=2523523525
  aff_validate_redo_number_items=aff_validate_undo_number_items
  aff_validate_redo_selected_item=aff_validate_undo_selected_item
  aff_validate_redo_items_total_selected=aff_validate_undo_items_total_selected
EndIf

EndProcedure




; Restore redo
;
; V1.0 - 4/MAR/2023
;
Procedure aff_validate_restore_redo()
  Debug "RESTORE REDO"
  Debug "FUCKER RESTORE REDO"

  
  ; Disable undo button and enable redo button
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_UNDO,#False)
  DisableGadget(#BUTTON_AFF_VALIDATE_IN_REDO,#True)    
  
  aff_validate_undo=#True
  aff_validate_redo=#False
  
  ReDim aff_validate_individual_line(ArraySize(aff_validate_redo_array()))
  CopyArray(aff_validate_redo_array(),aff_validate_individual_line())

  
  ; Add blank lines if the redo has more lines
  counter=aff_validate_redo_number_items-aff_validate_number_lines
  If counter>0
    For f=1 To counter
      AddGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,-1,"")
    Next f
  EndIf
  
  
  ; Remove lines if the redo has less lines
  counter=aff_validate_number_lines-aff_validate_redo_number_items
  If counter>0
    For f=aff_validate_number_lines-1 To aff_validate_number_lines-1-counter+1 Step -1
      RemoveGadgetItem(#LISTICONGADGET_AFF_VALIDATE_OUT,f)
    Next f
  EndIf  
  
  
  ; Update the total of variables
  aff_validate_number_lines=aff_validate_redo_number_items
  SetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT,aff_validate_redo_selected_item)
  aff_validate_number_lines_selected=aff_validate_redo_items_total_selected
  
  
  ; Update the ListIconGadget items
  counter=aff_validate_number_lines
  If counter>0
;     For f=0 To counter-1
;       t$=Str(f+1)+#LF$+aff_validate_individual_line(f)\line_text
    For f=1 To counter
      t$=Str(f)+#LF$+aff_validate_individual_line(f)\line_text
    
      SetGadgetItemText(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,t$)
      SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,#PB_Gadget_FrontColor,aff_validate_individual_line(f)\line_colour)      
;       SetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,aff_validate_individual_line(f)\line_selected)
      t=aff_validate_individual_line(f)\line_selected
      If t=#True : t=#PB_ListIcon_Checked : EndIf
      SetGadgetItemState(#LISTICONGADGET_AFF_VALIDATE_OUT,f-1,t)
    Next f
  EndIf
  
  
  ; Colour of first column grey
  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
  SetGadgetItemColor(#LISTICONGADGET_AFF_VALIDATE_OUT,#PB_All,#PB_Gadget_FrontColor,#Black,0)  

  
  ; Set the selected line again
  SetGadgetState(#LISTICONGADGET_AFF_VALIDATE_OUT,aff_validate_redo_selected_item)
  

  ; Set the numbers in their places
  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL,"Found:"+Str(aff_validate_number_lines))
  GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)  
  ResizeGadget(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK,GadgetX(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL)+2,GadgetY(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL),GadgetWidth(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK),GadgetHeight(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK))
  GadgetRefresh(#TEXT_AFF_VALIDATE_OUT_INDEX_TOTAL_ASTERISK)
  t$=Str(aff_validate_number_lines_selected)+"/"+Str(aff_validate_number_lines)
  If aff_validate_number_lines_selected=0 : t$="None" : EndIf
  If aff_validate_number_lines_selected=aff_validate_number_lines : t$="All" : EndIf
  SetGadgetText(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED,"Selected:"+t$)
  GadgetFit(#TEXT_AFF_VALIDATE_OUT_INDEX_SELECTED)
       
  
  
EndProcedure




; Sort the flags
;
; V1.0 - 24/SEP/2020
; V1.1 - 25/SEP/2020
;        o Fix: Coded Sort the flags (refresh + non-casesensitive)
; V1.2 - 20/NOV/2020
;        o Fix: Sorting of .dic flags now sets dictionary as changed
; V1.3 - 30/NOV/2021
;        o Started showing the time taken in timely operations.
; V1.4 - 19/APR/2023
;        o Removal of duplicates in the sort of flags
;
Procedure dictionary_sort_flags()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf      
    
    
  ; To create a faster searchable version of the AFF gadgettext with just single spaces
  ; In this procedure it is used to differenciate from flags merging
  faster_searchable_version_of_aff() 
    
  
  counter=0
  counter_flags_duplicated_removed=0
  
  sort_words_flags_stored_undo=#False
  
  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting flags of words... <ESC> to abort.")
  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
  
  ; Array in which to store each flag, for sorting
  Dim dictionary_flags_to_sort$(0)
  
  
  ; Step 1/1 - Check and sort word by word
  AddWindowTimer(#WINDOW_MAIN,800,800)
  time_taken=ElapsedMilliseconds()    
  For f=0 To number_of_words-1
    
    ; Word already changed to "" by "Process" button
    If words$(f)="" : Continue : EndIf
    
    text$=words$(f)+#LF$
    t$=StringField(text$,1,#LF$)    
    If Left(t$,1)<>"/" : t$=ReplaceString(t$,"\/",";") : EndIf ; Support escaped
    word_main$=StringField(t$,1,"/")
    word_flags_main$=StringField(t$,2,"/")
    word_flags_main_original$=StringField(t$,2,"/")
    If Left(t$,1)<>"/" : text$=ReplaceString(text$,";","\/") : EndIf ; Support escaped
    word_pos_main$=StringField(text$,2,#LF$)
    
    event = WindowEvent()
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting flags of words...... <ESC> to abort. Current:"+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f,number_of_words-1,1))
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_MAIN,800)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf
    EndIf
    
    
    ; Extract and sort the flags of each word
    If word_flags_main$<>""
      
      word_codes$=word_flags_main$ ; because I reused code from the processing words, so better to keep it as identical as possible to avoid screwing up
      
      ; Count flags inside
      len_word_codes=count_flags_inside(word_codes$)
      
      ReDim dictionary_flags_to_sort$(len_word_codes-1)
      
      For flags_in_word=1 To len_word_codes
        
        faster_word_codes$=get_faster_word_codes(word_codes$,flags_in_word)
        
        ; For flags using numbers - added zeros on the left for sorting to work properly
        If flag_char=3
          faster_word_codes$=Right("0000000000"+faster_word_codes$,10)
        EndIf
        
        ; Copy each flag into the array
        dictionary_flags_to_sort$(flags_in_word-1)=faster_word_codes$
        
      Next flags_in_word
      
      ; Sort the flags array
      SortArray(dictionary_flags_to_sort$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
      
      ; Get the sorted flags
      word_flags_main$=""
      For flags_in_word=1-1 To len_word_codes-1
        If flag_char<>3
          word_flags_main$+dictionary_flags_to_sort$(flags_in_word)
        Else
          word_flags_main$+","+Str(Val(dictionary_flags_to_sort$(flags_in_word))) ; Removed the zeros added on the left for sorting to work properly
        EndIf
      Next flags_in_word
      
      ; Remove starting "," if flag type =number
      If flag_char=3 : LTrim(word_flags_main$,",") : EndIf
      
    EndIf


    ; Replace the current word with the sorted flags if flags sorted
    repeated_flags_inside=is_there_repeated_flags_inside(word_flags_main$)
    If word_flags_main$<>word_flags_main_original$ Or repeated_flags_inside=#True
      counter+1
      If sort_words_flags_stored_undo=#False
        sort_words_flags_stored_undo=#True
        dictionary_store_undo(#False)
      EndIf
      If repeated_flags_inside=#True
        word_flags_main$=remove_repeated_flags(word_flags_main$)
        counter_flags_duplicated_removed+(len_word_codes-count_flags_inside(word_flags_main$))
      EndIf
      t$=word_main$+"/"+word_flags_main$
      If word_pos_main$<>"" : t$+#LF$+word_pos_main$ : EndIf
      words$(f)=t$
    EndIf
    
    
  Next f
  RemoveWindowTimer(#WINDOW_MAIN,800)
  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar
  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. Words with flags sorted:"+FormatNumber(counter,0,".",number_separator_chr$)+", flags removed:"+FormatNumber(counter_flags_duplicated_removed,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))    
  
  
  ; Refresh the ListIconGadget
  global_update_virtual_vertical_scroll_bar(-1)
  
  
  ; Show number of words where flags were sorted  
  t$="s"
  If counter=1 : t$="" : EndIf
  MessageRequester("Warning!","Sorted the flags in  "+FormatNumber(counter,0,".",number_separator_chr$)+" word"+t$+"."+#LF$+#LF$+"Duplicate flags removed:"+FormatNumber(counter_flags_duplicated_removed,0,".",number_separator_chr$),#PB_MessageRequester_Info)
  
  
  ; Mark .dic as changed
  If counter>0 : dictionary_changed=#True : EndIf
  
EndProcedure




; To sort the tags in the .aff file, suggested by Shantanu Oak
;
; V1.0 - 13/NOV/2022
;        14/NOV/2022
; V1.1 - 15/NOV/2022
;        o Improved code slightly
;
Procedure aff_sort_tags()
  
  ; Attempt to check if .AFF is empty
  t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
  t$=Trim(t$)
  If t$=""
    MessageRequester("Warning!",".AFF is empty.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
  
  
  ; Ask if the user wants to proceed (no "undo" possible in .aff)
  If MessageRequester("Warning!","Sorting the tags in the Aff."+#LF$+#LF$+"This operation can't be undone."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
    ProcedureReturn #False
  EndIf
  
  
  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting tags in .aff... <ESC> to abort.")
  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)   
  
  
  ; To make it easier for future changes
  total_of_steps_current=1
  total_of_steps_total=3
  
  
  ; Get the number of lines in .aff
  t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
  t$=ConvertStringToUnix(t$)
  If Right(t$,1)<>#LF$ : t$+#LF$ : EndIf
  total_of_lines_in_aff=CountString(t$,#LF$)
  
  
  ; Create the structure array
  Structure array_for_aff_tags_sorting Align #PB_Structure_AlignC
    real_line.s
    line_without_spaces.s
    line_rule_fourth_and_fifth_columns.s
  EndStructure
  Global Dim array_for_aff_tags_sorting.array_for_aff_tags_sorting(total_of_lines_in_aff)  
  
  
  ; If there are tags found in the first step to avoid doing other steps in #False
  tags_found_in_aff=#False
  
  
  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_of_lines_in_aff*total_of_steps_total)
  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
  
  
  ; STEP #1 - Copying lines to array & removing spaces
  AddWindowTimer(#WINDOW_MAIN,800,800)
  time_taken=ElapsedMilliseconds()    
  For f=0 To total_of_lines_in_aff-1
    
    event = WindowEvent()
    
    current_line$=StringField(t$,f+1,#LF$)
    current_line_without_spaces$=TrimStringAll(current_line$)
    array_for_aff_tags_sorting(f)\real_line.s=current_line$
    array_for_aff_tags_sorting(f)\line_without_spaces.s=current_line_without_spaces$
    If Left(current_line_without_spaces$,4)="SFX " Or Left(current_line_without_spaces$,4)="PFX "
      array_for_aff_tags_sorting(f)\line_rule_fourth_and_fifth_columns.s=StringField(current_line_without_spaces$,4," ")+" "+StringField(current_line_without_spaces$,5," ")
      If tags_found_in_aff=#False : tags_found_in_aff=#True : EndIf  
    EndIf
    
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Copying & processing lines to array:"+Str(total_of_steps_current)+"/"+Str(total_of_steps_total)+"... <ESC> to abort. Current:"+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_of_lines_in_aff,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f+(total_of_steps_current-1)*total_of_lines_in_aff,total_of_lines_in_aff*total_of_steps_total,1))
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,total_of_lines_in_aff*(total_of_steps_current-1)+f,#PB_StatusBar_Raised)  
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_MAIN,800)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        ProcedureReturn #False
      EndIf
    EndIf
    
  Next f
  RemoveWindowTimer(#WINDOW_MAIN,800)
  
  
  ; Exit if no tags found
  If tags_found_in_aff=#False
    MessageRequester("Warning!","No tags found in .AFF.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
  
  
  total_of_steps_current+1
  
  
  ; STEP #2 - Sorting tags
  AddWindowTimer(#WINDOW_MAIN,800,800)
  For f=0 To total_of_lines_in_aff-1
    
    event = WindowEvent()

    current_line_without_spaces$=array_for_aff_tags_sorting(f)\line_without_spaces.s
    If Left(current_line_without_spaces$,4)="SFX " Or Left(current_line_without_spaces$,4)="PFX "
      counter=Val(StringField(current_line_without_spaces$,4," "))
      SortStructuredArray(array_for_aff_tags_sorting(),#PB_Sort_Ascending,OffsetOf(array_for_aff_tags_sorting\line_rule_fourth_and_fifth_columns.s),TypeOf(array_for_aff_tags_sorting\line_rule_fourth_and_fifth_columns.s),f+1,f+counter)
      f+counter
      Continue
    EndIf
    
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting tags in array:"+Str(total_of_steps_current)+"/"+Str(total_of_steps_total)+"... <ESC> to abort. Current:"+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_of_lines_in_aff,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f+(total_of_steps_current-1)*total_of_lines_in_aff,total_of_lines_in_aff*total_of_steps_total,1))
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,total_of_lines_in_aff*(total_of_steps_current-1)+f,#PB_StatusBar_Raised)  
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_MAIN,800)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        ProcedureReturn #False
      EndIf
    EndIf
    
  Next f
  RemoveWindowTimer(#WINDOW_MAIN,800)
  
  
  total_of_steps_current+1
  
  back_to_aff$=""
  
  
  ; STEP #3 - Copying lines back to AFF
  AddWindowTimer(#WINDOW_MAIN,800,800)
  For f=0 To total_of_lines_in_aff-1
    
    event = WindowEvent()
    
    back_to_aff$+array_for_aff_tags_sorting(f)\real_line.s+#LF$
    
    
    ; Fix to window stop responding
    If event=#PB_Event_Timer And EventTimer()=800
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Copying lines back to AFF:"+Str(total_of_steps_current)+"/"+Str(total_of_steps_total)+"... <ESC> to abort. Current:"+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_of_lines_in_aff,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(f+(total_of_steps_current-1)*total_of_lines_in_aff,total_of_lines_in_aff*total_of_steps_total,1))
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,total_of_lines_in_aff*(total_of_steps_current-1)+f,#PB_StatusBar_Raised)  
    EndIf
        
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        RemoveWindowTimer(#WINDOW_MAIN,800)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        ProcedureReturn #False
      EndIf
    EndIf
    
  Next f
  SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,back_to_aff$)
  RemoveWindowTimer(#WINDOW_MAIN,800)

  
  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))   
  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
  dictionary_changed=#True
  
EndProcedure




; Coding of words missing in master wordlist
;
; Allow to eXclusive delete words so that we know which words are missing in a main wordlist.
; The main wordlist is loaded twice so that all its entries are deleted.
; 
; To be used with Kevin Atkinson's wordlists
;
; V1.0 - 08/OCT/2020
;        09/OCT/2020
; V1.1 - 12/OCT/2020
;        o Adding of F1 to open the help in the various windows.
; V1.2 - 18/NOV/2020
;        24/NOV/2020
;        o Now disables gadgets during processing
; V1.3 - 02/DEC/2021
;        o Started showing the time taken in timely operations.
; V1.4 - 09/DEC/2021
;        o "Words missing in Master Wordlist" shows filesizes in ListIconGadget
; V1.5 - 13/JUL/2022
;        o The Master wordlist extraction now has a button to extract uppercase-only words.
; V1.6 - 26/DEC/2022
;        o If words missing=0 it popups a requester saying so.
;        o Fix: if line was ticked and selected, it wouldn't recognize it for delete
; V1.7 - 25/MAR/2023
;        o Improved code
;
Procedure words_missing_in_master_wordlist()
  
    ; The variable that will allow SAVE AS to work properly
    sort_arrays_in_latin_counter=0 ; Found # of missing words
  
  
    t$="Words missing in Master Wordlist"
    If OpenWindow(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,t$,#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)   
    
    ; IMPORT MASTER  button
    x=10
    y=10
    ButtonGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_MASTER,x,y,default_gadget_width_button+30,default_gadget_height_button,"Select Master")
    
    
    y+default_gadget_height_button+5+3
    
    ; IMPORT OTHER  button
    ButtonGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_OTHER,x,y,default_gadget_width_button+30,default_gadget_height_button,"Select Others")    
    
    y+default_gadget_height_button+5+3
    
    ; PROCESS  button
    ButtonGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_PROCESS,x,y,default_gadget_width_button+30,default_gadget_height_button,"Import && Process")    
    
    
    ; CLOSE  button
    x=WindowWidth(#WINDOW_WORDS_MISSING_MASTER_WORDLIST)-default_gadget_width_button_ok_close_cancel-10
    y=WindowHeight(#WINDOW_WORDS_MISSING_MASTER_WORDLIST)-default_gadget_height_button-10
    ButtonGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_CLOSE,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"Close")    
    
    
    ; SAVE  button
    x-10-10-default_gadget_width_button-10
    ButtonGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE,x,y,default_gadget_width_button+10,default_gadget_height_button,"Save as")      
    
    
    ; SAVE UPPERCASE button
    x-10-default_gadget_width_button-10
    ButtonGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE_UCASE,x,y,default_gadget_width_button+10,default_gadget_height_button,"UCase Save as")      
    
    
    ; ListIconGadget with the files
    y=GadgetY(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_MASTER)
    x=GadgetX(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_MASTER)+GadgetWidth(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_MASTER)+30-10
    ListIconGadget(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,x,y,10-20+15+10+40+40+30+15+600-20+10+5+4+4+4-300-10-5-5-2+200+150+30+30+get_extra_width(100),100-20+300-30+5+40+get_extra_height(100),"#",30+40+10+5,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection|#PB_ListIcon_CheckBoxes)    
    AddGadgetColumn(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,1,"Type",150+5+15)
    AddGadgetColumn(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,2,"File",-20-15-15+100+10+20+20+10+300+get_extra_width(100))
    AddGadgetColumn(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,3,"Filesize",150-20-10)
    SetGadgetFont(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,FontID(#font_for_language_id))
    
    
    x=GadgetX(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
    y=GadgetY(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)+GadgetHeight(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)+10
    
    ; ALL ListIconGadget
    ButtonGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_ALL,x,y,default_gadget_width_button_all_none,default_gadget_height_button,"All")  
    
    x+default_gadget_width_button_all_none+10
    
    ; NONE ListIconGadget
    ButtonGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_NONE,x,y,default_gadget_width_button_all_none,default_gadget_height_button,"None")  
    
    x+default_gadget_width_button_all_none+10
    
    ; DELETE ListIconGadget
    x+10-5
    ButtonGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_DELETE,x,y,default_gadget_width_button,default_gadget_height_button,"Delete") 
    
    
    ; Processing status
    x+default_gadget_width_button+20
    TextGadget(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,x,y,100,default_gadget_height_text,"Processing:0/0")
    GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
    
    
    ; Total words + Total Missing Words
    x=GadgetX(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)+10
    y=GadgetY(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_ALL)+GadgetHeight(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_ALL)+10
    TextGadget(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS,x,y,100,default_gadget_height_text,"Total words:N/A")
    y+default_gadget_height_text+6
    TextGadget(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS,x,y,100,default_gadget_height_text,"Missing words:N/A")
    GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS)
    GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS)
    
    
    ; Progress bar
    x=GadgetX(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)+GadgetWidth(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)-5
    y=GadgetY(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)+GadgetHeight(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)+5+2
    ProgressBarGadget(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,x-default_gadget_width_progress,y,default_gadget_width_progress,default_gadget_height_small_progress,0,0)
    

    ; Time processing
    TextGadget(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,GadgetX(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST),GadgetY(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST)+default_gadget_height_small_progress+1,100,default_gadget_height_text,"")
    
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,GadgetX(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN),GadgetY(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN)+default_gadget_height_text+2,100,default_gadget_height_text,"")
    
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,#PB_Shortcut_Escape,1000)    
    AddKeyboardShortcut(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,#PB_Shortcut_F1,1020)
    
    close=#False
    Repeat
      
      event=WaitWindowEvent()
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        close=#True
      EndIf
      
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And EventMenu()=1020
        Debug "F1 pressed"
        html_help("#5.2.5.11.words_missing_master_wordlist")
      EndIf      
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        close=#True     
      EndIf     
      
      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
            Case #BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_MASTER
              t$=words_missing_in_master_wordlist_select("Master")
              If t$<>""
                counter=1+CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
                AddGadgetItem(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,-1,Str(counter)+#LF$+"Master"+#LF$+t$)
                SetGadgetItemColor(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,counter-1,#PB_Gadget_FrontColor,#GreenDark,1)
                SetGadgetItemColor(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,counter-1,#PB_Gadget_BackColor,#GreyLight,0)
              EndIf
              
            Case #BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_OTHER
              t$=words_missing_in_master_wordlist_select("Other")
              If t$<>""
                counter=1+CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
                AddGadgetItem(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,-1,Str(counter)+#LF$+"Other"+#LF$+t$)
                SetGadgetItemColor(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,counter-1,#PB_Gadget_BackColor,#GreyLight,0)
              EndIf              
              
            Case #BUTTON_WORDS_MISSING_MASTER_WORDLIST_PROCESS
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_MASTER,#True)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_OTHER,#True)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_PROCESS,#True)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_CLOSE,#True)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE,#True)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE_UCASE,#True)
              DisableGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_ALL,#True)
              DisableGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_NONE,#True)
              DisableGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_DELETE,#True)
              DisableGadget(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,#True)              
              words_missing_in_master_wordlist_import_and_process()
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_MASTER,#False)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_IMPORT_OTHER,#False)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_PROCESS,#False)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_CLOSE,#False)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE,#False)
              DisableGadget(#BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE_UCASE,#False)
              DisableGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_ALL,#False)
              DisableGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_NONE,#False)
              DisableGadget(#WINDOW_WORDS_MISSING_MASTER_WORDLIST_DELETE,#False)
              DisableGadget(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,#False)
              
            Case #WINDOW_WORDS_MISSING_MASTER_WORDLIST_ALL
              counter=CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
              If counter>0
                For f=0 To counter-1
                  SetGadgetItemState(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,#PB_ListIcon_Checked)
                Next f
              EndIf
              
            Case #WINDOW_WORDS_MISSING_MASTER_WORDLIST_NONE
              counter=CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
              If counter>0
                For f=0 To counter-1
                  SetGadgetItemState(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,#False)
                Next f
              EndIf        
              
            Case #WINDOW_WORDS_MISSING_MASTER_WORDLIST_DELETE
              counter=CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
              
              If counter=0
                MessageRequester("Warning!", "Nothing to delete.",#PB_MessageRequester_Warning)
              EndIf
              
              If counter>0
                flag=#False
                For f=0 To counter-1
                  If GetGadgetItemState(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f)=#PB_ListIcon_Checked
                    If MessageRequester("Warning!", "Delete selected entries?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
                      counter=0
                    EndIf
                    Break
                  EndIf
                Next f
              EndIf
              
              If counter>0
                flag=#False
                For f=counter-1 To 0 Step -1
                  ; Line below =3 means checkbox ticked PLUS line selected
                  If GetGadgetItemState(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f)=#PB_ListIcon_Checked Or GetGadgetItemState(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f)=3
                    flag=#True
                    RemoveGadgetItem(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f)
                  EndIf
                Next f
                If flag=#True
                  counter=CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
                  For f=0 To counter-1
                    SetGadgetItemText(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,Str(f+1),0)
                  Next f  
                EndIf
                If flag=#False
                  MessageRequester("Warning!", "Nothing to delete.",#PB_MessageRequester_Warning)
                EndIf
              EndIf
              
            Case #BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE
              words_missing_in_master_wordlist_saveas(#False)  
              
            Case #BUTTON_WORDS_MISSING_MASTER_WORDLIST_SAVE_UCASE
              words_missing_in_master_wordlist_saveas(#True)  
              
            Case #BUTTON_WORDS_MISSING_MASTER_WORDLIST_CLOSE
              close=#True
              
          EndSelect
      EndSelect
      
      
    Until close=#True
      
    CloseWindow(#WINDOW_WORDS_MISSING_MASTER_WORDLIST)
    DisableWindow(#WINDOW_MAIN,#False)  
      
EndProcedure




; Gets a .txt file path + name for importing.
;
; Returns a string, so it can be used for Master and Other
;
; V1.0 - 08/OCT/2020
;
Procedure.s words_missing_in_master_wordlist_select(t$)
  
  ; Get the existing number of files in the listicongadget
  counter=CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
  
  
  ; There is a maximum number of files
  If counter=10
    MessageRequester("Error", "Number of files is limited to 10.",#PB_MessageRequester_Warning)
    ProcedureReturn ""    
  EndIf
  
  
  ; There is a maximum of just ONE main file
  If counter>0 And t$="Master"    
    For f=0 To counter-1
      If GetGadgetItemText(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,1)="Master"
        MessageRequester("Error", "You can only have one Master file.",#PB_MessageRequester_Warning)
        ProcedureReturn ""    
      EndIf
    Next f
  EndIf
  
  
  ; Select the text filename
  file$=OpenFileRequester("Select "+t$+" Text File", "", "Text File (*.txt)|*.txt", 0)
  If file$=""
    ProcedureReturn ""
  EndIf      
    
    
  ; Check if .TXT file exists
  If FileSize(file$) <= 0
    MessageRequester("Error", "File not found.",#PB_MessageRequester_Warning)
    ProcedureReturn ""
  EndIf      
  
  
  ; Get filesize
  OpenFile(1,file$)
    t$=ShowFileSizeFormatted("",Lof(1),#False)
  CloseFile(1)
  
  
  ; .TXT file valid
  ProcedureReturn file$+#LF$+t$
  
EndProcedure




; Export the words that are missing in the Master wordlist
;
; V1.0 - 09/OCT/2020
; V1.1 - 02/DEC/2021
;        o Started showing the time taken in timely operations.
; V1.2 - 13/JUL/2022
;        o The Master wordlist extraction now has a button to extract uppercase-only words.
;
Procedure words_missing_in_master_wordlist_saveas(save_as_ucase)
  
  ; No missing words
  If sort_arrays_in_latin_counter=0
    MessageRequester("Warning!", "Nothing to save.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
  
  
  ; Get Save .TXT filename
  file$=SaveFileRequester("Save Text File", "", "Text File (*.txt)|*.txt", 0)
  If file$=""
    ProcedureReturn #False
  EndIf
  If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
  
  
  ; Check if file exists   
  If FileSize(file$)>0
    If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
      ProcedureReturn #False
    EndIf      
  EndIf      
  DeleteFile(file$) ; Delete .TXT file
  
  
  ; Set/Reset the progressbar gadget
  SetGadgetAttribute(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,#PB_ProgressBar_Maximum,sort_arrays_in_latin_number_of_lines)
  SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
  
  counter_ucase=0
  
  ; Save the .TXT file, using UTF-8 encoding
  AddWindowTimer(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,800,800)
  OpenFile(1,file$)
    WriteStringFormat(1,#PB_UTF8)    
    time_taken=ElapsedMilliseconds()
    SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
    For f=0 To sort_arrays_in_latin_number_of_lines-1
      text$=sort_arrays_in_latin(f)\in_ascii
      If save_as_ucase=#True And text$<>""
        t1$=Left(text$,1)
        If UCase(t1$)=t1$ And UCase(t1$)<>LCase(t1$)
          counter_ucase+1
        Else
          text$=""
        EndIf
          
      EndIf
      If text$<>"" : WriteString(1,text$+#LF$,#PB_UTF8) : EndIf
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,f)
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
          CloseFile(1)
          RemoveWindowTimer(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

      Next f
      SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
      RemoveWindowTimer(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,800)      
      SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
      CloseFile(1)
      
      If save_as_ucase=#True : MessageRequester("Warning!","UCase words extracted: "+Str(counter_ucase),#PB_MessageRequester_Info) : EndIf
      
 EndProcedure
 

 
 
; This procedure will import each file with wordlists, sort them, and eXclusive delete the duplicates
; The "Master Wordlist" is loaded twice to assure it gets deleted
;
; V1.0 - 09/OCT/2020
; V1.1 - 02/DEC/2021
;        o Started showing the time taken in timely operations. 
;
Procedure words_missing_in_master_wordlist_import_and_process()
  
  ; Check if the number of files and files are valid
  counter=CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
  
  If counter=0
    MessageRequester("Warning!", "Nothing to process.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
  
  If counter=1
    MessageRequester("Warning!", "You need at least two wordlists.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
  
  If counter>1
    flag=#False
    For f=0 To counter-1
      If GetGadgetItemText(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,1)="Master"
        master_wordlist_filename$=GetGadgetItemText(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,2) ; Now that we are at it, get the filename for STEP 1
        flag=#True
        Break
      EndIf
    Next f
    If flag=#False
      MessageRequester("Warning!", "You need at least one Master wordlist.",#PB_MessageRequester_Warning)
      ProcedureReturn #False      
    EndIf
  EndIf
  
  
  
  ; STEP 0
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,"")
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"<ESC> to abort.")
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS,"Total words:N/A")
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS,"Missing words:N/A")
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:0/0")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS)
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS)
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  sort_arrays_in_latin_number_of_lines_in_master_wordlist=0
  sort_arrays_in_latin_number_of_lines=0
  sort_arrays_in_latin_counter=0
  ReDim sort_arrays_in_latin(0)
  
  ; To allow an easier way to show the progress
  steps_current=0
  steps_total=6
  
  ; Timer flag to avoid no response, plus to get the time taken
  AddWindowTimer(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,800,800) 
  time_taken=ElapsedMilliseconds()
  
  
  
  ; STEP 1 - LOAD MASTER WORDLIST TWICE
  steps_current+1
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Loading Master wordlist twice...")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  
  ; Check if Master wordlist file exists
  file$=master_wordlist_filename$
  ReadFile(1,file$)
  If FileSize(file$) <= 0
    MessageRequester("Error", "File not found.",#PB_MessageRequester_Warning)
    CloseFile(1)
    SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Aborted.")
    GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
    SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")
    ProcedureReturn #False
  EndIf      
  
  ; Set the progress gadget to twice the size of the master wordlist filesize
  SetGadgetAttribute(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,#PB_ProgressBar_Maximum,Lof(1)*2)
  SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
  
  For f=1 To 2
    
    ReadFile(1,file$)
    ReadStringFormat(1)
    
    Repeat
      
      word$=ReadString(1,#PB_UTF8)
      sort_arrays_in_latin_number_of_lines+1
      ReDim sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)
      sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)\in_latin=RemoveAccentsString(word$)
      sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)\in_ascii=word$
      sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)\master_wordlist=#True
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event=WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS,"Total words:"+FormatNumber(sort_arrays_in_latin_number_of_lines-sort_arrays_in_latin_number_of_lines_in_master_wordlist,0,".",number_separator_chr$)+" (All files)")
        GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS)        
        loc_of_file=Loc(1)
        If f=2 : loc_of_file+Lof(1) : EndIf
        SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,loc_of_file)
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Aborted.")
          GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
          SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
          SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")
          CloseFile(1)
          RemoveWindowTimer(#WINDOW_MAIN,800)           
          ProcedureReturn #False
        EndIf
      EndIf      
      
    Until Eof(1)
    
    ; Get number of lines in Master Wordlist to properly show the total number of lines
    If f=1 : sort_arrays_in_latin_number_of_lines_in_master_wordlist=sort_arrays_in_latin_number_of_lines : EndIf
    
  Next f
  
  ; Show total number of words found but remove second Master Wordlist entries
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS,"Total words:"+FormatNumber(sort_arrays_in_latin_number_of_lines-sort_arrays_in_latin_number_of_lines_in_master_wordlist,0,".",number_separator_chr$)+" (All files)")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS)
  
  
  
  ; STEP 2 - LOAD OTHER WORDLISTS
  steps_current+1
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Loading Other wordlists... 1 of "+Str(counter-1))
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  
  counter=CountGadgetItems(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES)
  counter_other=0 ; So that it properly displays "file X of COUNTER-1"
  
  For f=1-1 To counter-1
    
    If GetGadgetItemText(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,1)="Other" : counter_other+1 : EndIf ; So that it properly displays "file X of COUNTER-1"
    
    SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Loading Other wordlists... "+Str(counter_other)+" of "+Str(counter-1))
    GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)    
    
    ; Check if wordlist file exists
    file$=GetGadgetItemText(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,2)
    If GetGadgetItemText(#LISTICONGADGET_WORDS_MISSING_MASTER_WORDLIST_FILES,f,1)="Master" : Continue : EndIf ; To avoid importing Master again
    ReadFile(1,file$)
    If FileSize(file$) <= 0
      MessageRequester("Error", "File not found.",#PB_MessageRequester_Warning)
      CloseFile(1)
      SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Aborted.")
      GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
      SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")
      ProcedureReturn #False
    EndIf      
    
    ; Set the progress gadget to twice the size of the master wordlist filesize
    SetGadgetAttribute(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,#PB_ProgressBar_Maximum,Lof(1))
    SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
    SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  
    ReadFile(1,file$)
    ReadStringFormat(1)
    
    Repeat
      
      word$=ReadString(1,#PB_UTF8)
      sort_arrays_in_latin_number_of_lines+1
      ReDim sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)
      sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)\in_latin=RemoveAccentsString(word$)
      sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)\in_ascii=word$   
      sort_arrays_in_latin(sort_arrays_in_latin_number_of_lines-1)\master_wordlist=#False
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event=WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS,"Total words:"+FormatNumber(sort_arrays_in_latin_number_of_lines-sort_arrays_in_latin_number_of_lines_in_master_wordlist,0,".",number_separator_chr$)+" (All files)")
        GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS)        
        SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,Loc(1))
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Aborted.")
          GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
          SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
          SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")
          CloseFile(1)
          RemoveWindowTimer(#WINDOW_MAIN,800)           
          ProcedureReturn #False
        EndIf
      EndIf      
      
    Until Eof(1)
    
  Next f
  
  ; Show total number of words found but remove second Master Wordlist entries
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS,"Total words:"+FormatNumber(sort_arrays_in_latin_number_of_lines-sort_arrays_in_latin_number_of_lines_in_master_wordlist,0,".",number_separator_chr$)+" (All files)")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_WORDS)
  
  
  
  ; STEP 3 - SORT WORDLISTS IN ASCII
  steps_current+1
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Sorting wordlists in ASCII...")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  
  ; Sort in ASCII
  SortStructuredArray(sort_arrays_in_latin(),#PB_Sort_Ascending,OffsetOf(sort_arrays_in_latin\in_ascii.s),TypeOf(sort_arrays_in_latin\in_ascii.s))  
  
  
  
  ; STEP 4 - EXCLUSIVE DELETE - GET UNIQUE WORDS
  steps_current+1
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": eXclusive deleting words... 1 of 2")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  
  ; Set the progress gadget to the total number of words in the wordlists
  SetGadgetAttribute(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,#PB_ProgressBar_Maximum,sort_arrays_in_latin_number_of_lines)
  SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)  
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  
  For f=0 To sort_arrays_in_latin_number_of_lines-1-1
               
    If sort_arrays_in_latin(f)\in_ascii=sort_arrays_in_latin(f+1)\in_ascii
      t$=sort_arrays_in_latin(f)\in_ascii
      If f+1<=sort_arrays_in_latin_number_of_lines-1
        For ff=f+1 To sort_arrays_in_latin_number_of_lines-1
          If sort_arrays_in_latin(ff)\in_ascii=t$ And t$<>""
            If sort_arrays_in_latin(ff)\master_wordlist=#True : sort_arrays_in_latin(f)\master_wordlist=#True : EndIf
            sort_arrays_in_latin(ff)\in_ascii=""
          Else
            Break
          EndIf
        Next ff
        
      EndIf  
    EndIf
     
    ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
    event=WindowEvent()
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,f)
      SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
    EndIf 
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Aborted.")
        GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
        SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")    
        RemoveWindowTimer(#WINDOW_MAIN,800)           
        ProcedureReturn #False
      EndIf
    EndIf 
    
  Next f
  
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": eXclusive deleting words... 2 of 2")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  
  ; Set the progress gadget to the total number of words in the wordlists
  SetGadgetAttribute(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,#PB_ProgressBar_Maximum,sort_arrays_in_latin_number_of_lines)
  SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  
  For f=0 To sort_arrays_in_latin_number_of_lines-1
    
    If sort_arrays_in_latin(f)\master_wordlist=#True : sort_arrays_in_latin(f)\in_ascii="" : EndIf
     
    ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
    event=WindowEvent()
    If event=#PB_Event_Timer And EventTimer()=800
      SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,f)
      SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
    EndIf 
        
    ; <ESC> pressed to abort
    If EventMenu()=1000
      If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Aborted.")
        GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
        SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")    
        RemoveWindowTimer(#WINDOW_MAIN,800)           
        ProcedureReturn #False
      EndIf
    EndIf 
    
  Next f
  
  
  
  
  ; STEP 5 - SORT WORDLISTS IN LATIN
  steps_current+1
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Sorting wordlists in Latin...")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  
  ; Sort in Latin
  SortStructuredArray(sort_arrays_in_latin(),#PB_Sort_Ascending,OffsetOf(sort_arrays_in_latin\in_latin.s),TypeOf(sort_arrays_in_latin\in_latin.s))
  
  
  
  ; STEP 6 - COUNT NUMBER OF ENTRIES AFTER EXCLUSIVE DELETING DUPLICATES
  steps_current+1
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Counting number of unique words...")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  
  ; Set the progress gadget to the total number of words in the wordlists
  SetGadgetAttribute(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,#PB_ProgressBar_Maximum,sort_arrays_in_latin_number_of_lines)
  SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  
  For f=1-1 To sort_arrays_in_latin_number_of_lines-1
      
    word$=sort_arrays_in_latin(f)\in_ascii
    If word$<>"" : sort_arrays_in_latin_counter+1 : EndIf
    
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event=WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS,"Missing words:"+FormatNumber(sort_arrays_in_latin_counter,0,".",number_separator_chr$))
        GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS)        
        SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,f)
        SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Aborted.")
          GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
          SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
          SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")
          RemoveWindowTimer(#WINDOW_MAIN,800)           
          ProcedureReturn #False
        EndIf
      EndIf      
      
  Next f
  
  ; Show total number of words found but remove second Master Wordlist entries
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS,"Missing words:"+FormatNumber(sort_arrays_in_latin_counter,0,".",number_separator_chr$))
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TOTAL_MISSING_WORDS)
  
  
  ; THE END OF STEPS
  RemoveWindowTimer(#WINDOW_WORDS_MISSING_MASTER_WORDLIST,800)      
  SetGadgetState(#PROGRESSBAR_WORDS_MISSING_MASTER_WORDLIST,0)
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_ESCAPE_TO_ABORT,"")
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS,"Processing:"+Str(steps_current)+"/"+Str(steps_total)+": Done!")
  GadgetFit(#TEXT_WORDS_MISSING_MASTER_WORDLIST_PROCESSING_TEXT_STEPS)
  SetGadgetText(#TEXT_WORDS_MISSING_MASTER_WORDLIST_TIME_TAKEN,TimeToString("⌚",ElapsedMilliseconds()-time_taken))
  CloseFile(1)  
  
  
  ; "No words are missing in Master wordlist." - 26/DEC/2022
  If sort_arrays_in_latin_counter=0 : MessageRequester("Warning!", "No words are missing in Master wordlist.",#PB_MessageRequester_Warning) : EndIf
  
EndProcedure




  ; "Crunch words with Flags", to be used with the wordlists of the Oxford Professors, so that it adds flags to words which
  ; can be duplicate deleted with the other option
  ;
  ; V1.0 - 11/NOV/2020
  ;        12/NOV/2020
  ;        13/NOV/2020
  ;        16/NOV/2020
  ;        17/NOV/2020
  ; V1.1 - 25/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ; V1.2 - 12/JUN/2022
  ;        o Added "Characters match" tab
  ;        o GTK3 ready
  ; V1.3 - 10/OCT/2022
  ;        28/OCT/2022
  ;        29/OCT/2022
  ;        o Rewrote "Crunch words with Flags"
  ; V1.4 - 15/NOV/2022
  ;        25/DEZ/2022
  ;        02/JAN/2023
  ;        02/FEB/2023
  ;        o Improved "Crunch words with Flags"
  ;
  Procedure crunch_words_with_flags()
          
    If number_of_words=0 
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    w=15+20+20+20-5+60+60+320+100-30-20-20+20+20+20+120+200-20-15-10+30+25+90+20+25
    h=3+200-50-20-20+100+100+25+10+20+2+20+20+20+15+15+15
    
 
    If OpenWindow(#WINDOW_CRUNCH_WORDS,0,0,w,h,"Crunch words with flags (Marco Pinto - GB)",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    DisableWindow(#WINDOW_MAIN,#True)
    
    
    ; Frame gadget with the checkboxes of the flags to check
    x=10
    y=10
    FrameGadget(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS,x,y,90+190-20-20+30+25+90+20+25,h-10-20,"Flags to crunch (SFX with 1 derivate only)")
    x+10
    y+20+5
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES,x,y,90+180-15,default_gadget_height_checkbox,"X's → /M (Possessive -'s)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES,crunch_words_with_flags_flag_M)            
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_PLURALS,x,y,90+180,default_gadget_height_checkbox,"Xs → /S (Plural/Verb -s)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_PLURALS)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_PLURALS,crunch_words_with_flags_flag_S)    
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_ING,x,y,90+180,default_gadget_height_checkbox,"Xing → /G (Verb -ing)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_ING)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ING,crunch_words_with_flags_flag_G)
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_ED,x,y,90+180,default_gadget_height_checkbox,"Xed → /D (Verb -ed)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_ED)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ED,crunch_words_with_flags_flag_D)
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_LESS,x,y,90+180,default_gadget_height_checkbox,"Xless → /p (Adjective -less)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_LESS)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_LESS,crunch_words_with_flags_flag_p)
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_AL_O,x,y,90+180,default_gadget_height_checkbox,"Xal → /O (Adjective -al)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_AL_O)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_AL_O,crunch_words_with_flags_flag_O)     
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_AL_w,x,y,90+180,default_gadget_height_checkbox,"Xal → /w (Adjective -al)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_AL_w)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_AL_w,crunch_words_with_flags_flag_w_lowercase)  
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_IC,x,y,90+180,default_gadget_height_checkbox,"Xic → /W (Adjective -ic)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_IC)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_IC,crunch_words_with_flags_flag_W)  
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_LY,x,y,90+180,default_gadget_height_checkbox,"Xly → /Y (Adj/Adverb -ly)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_LY)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_LY,crunch_words_with_flags_flag_Y)  
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_CALLY,x,y,90+180,default_gadget_height_checkbox,"Xically → /1 (Adverb -ically)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_CALLY)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_CALLY,crunch_words_with_flags_flag_1) 
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_INGLY,x,y,90+180,default_gadget_height_checkbox,"Xingly → /k (Adverb -ingly)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_INGLY)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_INGLY,crunch_words_with_flags_flag_k)  
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_FUL,x,y,90+180,default_gadget_height_checkbox,"Xful → /6 (Adjective -ful)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_FUL)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_FUL,crunch_words_with_flags_flag_6) 
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_FULLY,x,y,90+180,default_gadget_height_checkbox,"Xfully → /j (Adverb -fully)")
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_FULLY)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_FULLY,crunch_words_with_flags_flag_j_lowercase) 
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_EDLY,x,y,90+180,default_gadget_height_checkbox,"Xedly → /h (Adverb -edly)")   
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_EDLY)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_EDLY,crunch_words_with_flags_flag_h) 
    
    ; second column
    y=GadgetY(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES)
    x=GadgetX(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES)+GadgetWidth(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES)+30+25
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_INGS,x,y,90+180,default_gadget_height_checkbox,"Xings → /J (Plural noun -ings)")   
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_INGS)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_INGS,crunch_words_with_flags_flag_J)
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_ALLY,x,y,90+180,default_gadget_height_checkbox,"Xally → /o (Adverb -ally)")   
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_ALLY)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ALLY,crunch_words_with_flags_flag_o_lowercase)    
    y+default_gadget_height_checkbox+3
    CheckBoxGadget(#CHECKBOX_WINDOW_CRUNCH_TILDE,x,y,90+180,default_gadget_height_checkbox,"Xish → /~ (Adjective -ish)")   
    GadgetFit(#CHECKBOX_WINDOW_CRUNCH_TILDE)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_TILDE,crunch_words_with_flags_flag_tilde) 
    
    
    x=GadgetX(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS)+5
    y=GadgetY(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS)+GadgetHeight(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS)-5-default_gadget_height_button
    ButtonGadget(#BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_ALL,x,y,default_gadget_width_button_all_none,default_gadget_height_button,"All")
    x+default_gadget_width_button_all_none+10
    ButtonGadget(#BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_NONE,x,y,default_gadget_width_button_all_none,default_gadget_height_button,"None")
    
    
    ; ListIconGadget plus labels
    y=10
    x=GadgetX(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS)+GadgetWidth(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS)+20
    ListIconGadget(#PANEL_BOX_CRUNCH_WORDS,x,y,15+20+20+20+60+60+320+100-30-50+5+20+20+10,2-2+100+20+5+10+20-3+30+20+20+20,"Line #",30+40+10+5+5,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#PANEL_BOX_CRUNCH_WORDS,1,"Words",100+10+20+20+10+40+40+5)
    AddGadgetColumn(#PANEL_BOX_CRUNCH_WORDS,2,"Characters match",100+10+20+20+10+40+40-20-25)
    SetGadgetFont(#PANEL_BOX_CRUNCH_WORDS,FontID(#font_for_language_id))
    x=GadgetX(#PANEL_BOX_CRUNCH_WORDS)+5
    y=GadgetY(#PANEL_BOX_CRUNCH_WORDS)+GadgetHeight(#PANEL_BOX_CRUNCH_WORDS)+5+2
    TextGadget(#TEXT_WINDOW_CRUNCH_WORDS,x,y,100,default_gadget_height_text,"Found:0"+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$))
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS)
    y=GadgetY(#TEXT_WINDOW_CRUNCH_WORDS)+GadgetHeight(#TEXT_WINDOW_CRUNCH_WORDS)+3
    x=GadgetX(#TEXT_WINDOW_CRUNCH_WORDS)
    TextGadget(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,x,y,100,default_gadget_height_text,"Processing:0/0")
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)

    
    ; Progress bar
    t=number_of_words
    x=GadgetX(#PANEL_BOX_CRUNCH_WORDS)+GadgetWidth(#PANEL_BOX_CRUNCH_WORDS)-5-default_gadget_width_progress
    y=GadgetY(#PANEL_BOX_CRUNCH_WORDS)+GadgetHeight(#PANEL_BOX_CRUNCH_WORDS)+5+2
    ProgressBarGadget(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,x,y,default_gadget_width_progress,default_gadget_height_small_progress,0,t,#PB_ProgressBar_Smooth)
    SetGadgetColor(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,#PB_Gadget_FrontColor,#Blue) ; Blue progress bar
    
    y+default_gadget_height_small_progress+1
    
    ; Time processing
    TextGadget(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,x,y,100,default_gadget_height_text,"")
    
    y+default_gadget_height_text+2
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_ABORT,x,y,100,default_gadget_height_text,"")
    
    
    y=GadgetY(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_ABORT)+20+5+5
    x=GadgetX(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS)+GadgetWidth(#FRAME_WINDOW_CRUNCH_WORDS_FLAGS)+20
    TextGadget(#TEXT_WINDOW_CRUNCH_TRACK_CHRS,x,y,100,default_gadget_height_text,"Match Left Characters:")
    GadgetFit(#TEXT_WINDOW_CRUNCH_TRACK_CHRS)
    y+default_gadget_height_text+3+2
    TrackBarGadget(#TRACK_WINDOW_CRUNCH_TRACK,X,Y,default_gadget_width_track+100+100,default_gadget_height_track,1,3,#PB_TrackBar_Ticks)
    SetGadgetState(#TRACK_WINDOW_CRUNCH_TRACK,crunch_words_with_flags_number_of_chrs_match_left)
    y+default_gadget_height_track+3+2
    t=crunch_words_with_flags_number_of_chrs_match_left
    t$="1 chr: Best (Slowest)"
    If t=2 : t$="2 chrs: Very Good (Faster) (recommended)": EndIf
    If t=3 : t$="3 chrs: Good (Fastest)" : EndIf
    TextGadget(#TEXT_WINDOW_CRUNCH_TRACK_SPEED,x,y,100,default_gadget_height_text,t$)
    GadgetFit(#TEXT_WINDOW_CRUNCH_TRACK_SPEED)  
    
    
    ; Flags used
    y=GadgetY(#TRACK_WINDOW_CRUNCH_TRACK)
    x=GadgetX(#TRACK_WINDOW_CRUNCH_TRACK)+GadgetWidth(#TRACK_WINDOW_CRUNCH_TRACK)+10+10+10+15
    TextGadget(#TEXT_WINDOW_CRUNCH_FLAGS_USED,x,y,100,default_gadget_height_text,"🏁🏁 used:")
    GadgetFit(#TEXT_WINDOW_CRUNCH_FLAGS_USED)
    TextGadget(#TEXT_WINDOW_CRUNCH_FLAGS_USED_n_OF_N,x,y+default_gadget_height_text+2,10,default_gadget_height_text,"")
    x+GadgetWidth(#TEXT_WINDOW_CRUNCH_FLAGS_USED)+10-5
    StringGadget(#STRING_WINDOW_CRUNCH_FLAGS_USED,x,y,100+200-25-5-10-15,default_gadget_height_string,"",#PB_String_ReadOnly)
    
    
    x=w-default_gadget_width_button_ok_close_cancel-5-5
    y=h-10-default_gadget_height_button
    ButtonGadget(#BUTTON_WINDOW_CRUNCH_WORDS_CLOSE,x,y,default_gadget_width_button_ok_close_cancel,default_gadget_height_button,"Close")    
    x-default_gadget_width_button-10-5
    ButtonGadget(#BUTTON_WINDOW_CRUNCH_WORDS_PROCESS,x,y,default_gadget_width_button,default_gadget_height_button,"Process")

    
    ; Undo/Redo support
    crunch_words_with_flags_changed=#False
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_CRUNCH_WORDS,#PB_Shortcut_Escape,1000)
    AddKeyboardShortcut(#WINDOW_CRUNCH_WORDS,#PB_Shortcut_Return,1003)
    AddKeyboardShortcut(#WINDOW_CRUNCH_WORDS,#PB_Shortcut_F1,1020)
    
    
    ; Update flags used
    crunch_words_with_flags_update_flags_used()
    
    cancel=#False
    Repeat
      
      event=WaitWindowEvent()
      event_menu=EventMenu()
      

      ;Change the settings of the find text box
      t=GetGadgetState(#TRACK_WINDOW_CRUNCH_TRACK)
      If crunch_words_with_flags_number_of_chrs_match_left<>t
        crunch_words_with_flags_number_of_chrs_match_left=t
        t$="1 chr: Best (Slowest)"
        If t=2 : t$="2 chrs: Very Good (Faster) (recommended)": EndIf
        If t=3 : t$="3 chrs: Good (Fastest)" : EndIf
        SetGadgetText(#TEXT_WINDOW_CRUNCH_TRACK_SPEED,t$)
        GadgetFit(#TEXT_WINDOW_CRUNCH_TRACK_SPEED)  
      EndIf
            
      
      ; Check if <ENTER> was pressed in the text box or in the OK button
      If event=#PB_Event_Menu And event_menu=1003 And GetActiveGadget()=#BUTTON_WINDOW_CRUNCH_WORDS_CLOSE
        Debug "CLOSE was pressed"
        cancel=#True
      EndIf
     
   
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And event_menu=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      
      ; Shortcut for F1
      If event=#PB_Event_Menu And event_menu=1020
        Debug "F1 pressed"
        html_help("#5.2.5.13.crunch_words_with_flags")
      EndIf      
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
      

      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
           Case #CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES
             crunch_words_with_flags_flag_M=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES)
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()
             
             
           Case #CHECKBOX_WINDOW_CRUNCH_PLURALS
             crunch_words_with_flags_flag_S=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_PLURALS)
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_ING
             crunch_words_with_flags_flag_G=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ING)
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_ED
             crunch_words_with_flags_flag_D=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ED)
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_LESS
             crunch_words_with_flags_flag_p=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_LESS)            
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_AL_O
             crunch_words_with_flags_flag_O=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_AL_O)            
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_AL_w
             crunch_words_with_flags_flag_w_lowercase=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_AL_w)   
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_IC
             crunch_words_with_flags_flag_W=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_IC)   
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_LY
             crunch_words_with_flags_flag_Y=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_LY)  
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_CALLY
             crunch_words_with_flags_flag_1=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_CALLY)  
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()

             
           Case #CHECKBOX_WINDOW_CRUNCH_INGLY
             crunch_words_with_flags_flag_k=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_INGLY)  
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()
             
             
           Case #CHECKBOX_WINDOW_CRUNCH_FUL
             crunch_words_with_flags_flag_6=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_FUL)  
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()
             
             
           Case #CHECKBOX_WINDOW_CRUNCH_FULLY
             crunch_words_with_flags_flag_j_lowercase=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_FULLY)  
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()            
             
             
           Case #CHECKBOX_WINDOW_CRUNCH_EDLY
             crunch_words_with_flags_flag_h=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_EDLY)  
           
             ; Update flags used
             crunch_words_with_flags_update_flags_used()           
             
             
           Case #CHECKBOX_WINDOW_CRUNCH_INGS
             crunch_words_with_flags_flag_J=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_INGS)  
           
             ; Update flags used
             crunch_words_with_flags_update_flags_used() 
             
             
           Case #CHECKBOX_WINDOW_CRUNCH_ALLY
             crunch_words_with_flags_flag_o_lowercase=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ALLY)  
           
             ; Update flags used
             crunch_words_with_flags_update_flags_used() 
             
             
           Case #CHECKBOX_WINDOW_CRUNCH_TILDE
             crunch_words_with_flags_flag_tilde=GetGadgetState(#CHECKBOX_WINDOW_CRUNCH_TILDE)  
           
             ; Update flags used
             crunch_words_with_flags_update_flags_used() 
             
                 
           Case #BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_ALL
             crunch_words_with_flags_update_all_none(#True)
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()       
             
             
           Case #BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_NONE
             crunch_words_with_flags_update_all_none(#False)
             
             ; Update flags used
             crunch_words_with_flags_update_flags_used()               
             
             
           Case #BUTTON_WINDOW_CRUNCH_WORDS_PROCESS
             
             ; Clear the ListIconGadget
             ClearGadgetItems(#PANEL_BOX_CRUNCH_WORDS)
             
             ; # Items on ListIconGadget for grey column
             counter_total_crunch_words=0
             
             ; Disable gadgets
             DisableGadget(#TRACK_WINDOW_CRUNCH_TRACK,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_PLURALS,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_ING,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_ED,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_LESS,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_AL_O,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_AL_w,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_IC,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_LY,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_CALLY,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_INGLY,#True)       
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_FUL,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_FULLY,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_EDLY,#True)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_ALL,#True)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_NONE,#True)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_PROCESS,#True)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_CLOSE,#True)   
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_INGS,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_ALLY,#True)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_TILDE,#True)
             
             SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,"")
             
             SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_ABORT,"<ESC> to abort.")
             
             crunch_words_with_flags_process()
             
             global_update_virtual_vertical_scroll_bar(-1) 
             
             SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_ABORT,"")
             
             ; Sets entire first column to grey background, since the colour was removed in one of the calls above
             SetGadgetItemColor(#PANEL_BOX_CRUNCH_WORDS,#PB_All,#PB_Gadget_BackColor,#GreyLight,0) ; For "Line #"
             
             ; Enable gadgets
             DisableGadget(#TRACK_WINDOW_CRUNCH_TRACK,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_PLURALS,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_ING,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_ED,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_LESS,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_AL_O,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_AL_w,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_IC,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_LY,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_CALLY,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_INGLY,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_FUL,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_FULLY,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_EDLY,#False)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_ALL,#False)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_FLAGS_NONE,#False)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_PROCESS,#False)
             DisableGadget(#BUTTON_WINDOW_CRUNCH_WORDS_CLOSE,#False)    
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_INGS,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_ALLY,#False)
             DisableGadget(#CHECKBOX_WINDOW_CRUNCH_TILDE,#False)
             
             
           Case #BUTTON_WINDOW_CRUNCH_WORDS_CLOSE
              Debug "CLOSE was pressed"
              cancel=#True             
             
         EndSelect

      EndSelect
      
      
    Until cancel=#True
    
    
    ; Close the Find window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_CRUNCH_WORDS)
    
  EndProcedure
  
  
  
  
  ; To select all/none the flags to be used in the processing of the crunch
  ;
  ; V1.0 - 29/OCT/2022
  ;
  Procedure crunch_words_with_flags_update_all_none(flag)
    
    crunch_words_with_flags_flag_M=flag
    crunch_words_with_flags_flag_S=flag
    crunch_words_with_flags_flag_G=flag
    crunch_words_with_flags_flag_D=flag
    crunch_words_with_flags_flag_p=flag
    crunch_words_with_flags_flag_O=flag
    crunch_words_with_flags_flag_w_lowercase=flag
    crunch_words_with_flags_flag_W=flag
    crunch_words_with_flags_flag_Y=flag
    crunch_words_with_flags_flag_1=flag
    crunch_words_with_flags_flag_k=flag
    crunch_words_with_flags_flag_6=flag
    crunch_words_with_flags_flag_j_lowercase=flag   
    crunch_words_with_flags_flag_h=flag  
    crunch_words_with_flags_flag_J=flag
    crunch_words_with_flags_flag_o_lowercase=flag
    crunch_words_with_flags_flag_tilde=flag
    
    
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_WORDS_POSSESSIVES,crunch_words_with_flags_flag_M)   
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_PLURALS,crunch_words_with_flags_flag_S) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ING,crunch_words_with_flags_flag_G)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ED,crunch_words_with_flags_flag_D)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_LESS,crunch_words_with_flags_flag_p)
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_AL_O,crunch_words_with_flags_flag_O)  
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_AL_w,crunch_words_with_flags_flag_w_lowercase)  
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_IC,crunch_words_with_flags_flag_W) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_LY,crunch_words_with_flags_flag_Y) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_CALLY,crunch_words_with_flags_flag_1) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_INGLY,crunch_words_with_flags_flag_k) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_FUL,crunch_words_with_flags_flag_6) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_FULLY,crunch_words_with_flags_flag_j_lowercase) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_EDLY,crunch_words_with_flags_flag_h) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_INGS,crunch_words_with_flags_flag_J) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_ALLY,crunch_words_with_flags_flag_o_lowercase) 
    SetGadgetState(#CHECKBOX_WINDOW_CRUNCH_TILDE,crunch_words_with_flags_flag_tilde) 
    
  EndProcedure
  
  
        
  
  ; To show the flags to be used in the processing of the crunch
  ;
  ; V1.0 - 28/OCT/2022
  ;        29/OCT/2022
  ;        25/DEZ/2022
  ;
  Procedure crunch_words_with_flags_update_flags_used()
    
    counter_current=0
    counter_max=0
    
    t$=""
    
    ; M
    If crunch_words_with_flags_flag_M=#True
      t$+"M"
      counter_current+1
    EndIf
    counter_max+1
    
    ; S
    If crunch_words_with_flags_flag_S=#True
      t$+"S"
      counter_current+1
    EndIf
    counter_max+1
    
    ; G
    If crunch_words_with_flags_flag_G=#True
      t$+"G"
      counter_current+1
    EndIf
    counter_max+1
    
    ; D
    If crunch_words_with_flags_flag_D=#True
      t$+"D"
      counter_current+1
    EndIf
    counter_max+1
    
    ; p
    If crunch_words_with_flags_flag_p=#True
      t$+"p"
      counter_current+1
    EndIf
    counter_max+1
    
    ; O
    If crunch_words_with_flags_flag_O=#True
      t$+"O"
      counter_current+1
    EndIf
    counter_max+1
    
    ; w
    If crunch_words_with_flags_flag_w_lowercase=#True
      t$+"w"
      counter_current+1
    EndIf
    counter_max+1
    
    ; W
    If crunch_words_with_flags_flag_W=#True
      t$+"W"
      counter_current+1
    EndIf
    counter_max+1
    
    ; Y
    If crunch_words_with_flags_flag_Y=#True
      t$+"Y"
      counter_current+1
    EndIf
    counter_max+1
    
    ; 1
    If crunch_words_with_flags_flag_1=#True
      t$+"1"
      counter_current+1
    EndIf
    counter_max+1
    
    ; k
    If crunch_words_with_flags_flag_k=#True
      t$+"k"
      counter_current+1
    EndIf
    counter_max+1
    
    ; 6
    If crunch_words_with_flags_flag_6=#True
      t$+"6"
      counter_current+1
    EndIf
    counter_max+1
    
    ; j
    If crunch_words_with_flags_flag_j_lowercase=#True
      t$+"j"
      counter_current+1
    EndIf
    counter_max+1
    
    ; h
    If crunch_words_with_flags_flag_h=#True
      t$+"h"
      counter_current+1
    EndIf
    counter_max+1
    
    ; J
    If crunch_words_with_flags_flag_J=#True
      t$+"J"
      counter_current+1
    EndIf
    counter_max+1
    
    ; o
    If crunch_words_with_flags_flag_o_lowercase=#True
      t$+"o"
      counter_current+1
    EndIf
    counter_max+1
    
    ; ~
    If crunch_words_with_flags_flag_tilde=#True
      t$+"~"
      counter_current+1
    EndIf
    counter_max+1
    
    SetGadgetText(#STRING_WINDOW_CRUNCH_FLAGS_USED,t$)
    SetGadgetText(#TEXT_WINDOW_CRUNCH_FLAGS_USED_n_OF_N,"("+Str(counter_current)+"/"+Str(counter_max)+")")
    GadgetFit(#TEXT_WINDOW_CRUNCH_FLAGS_USED_n_OF_N)
    
  EndProcedure
  
  
  
  
  ; To process the "crunch words with flags"
  ;
  ; V1.0 - 12/NOV/2020
  ;        13/NOV/2020
  ;        15/NOV/2020
  ;        16/NOV/2020
  ; V1.1 - 27/NOV/2020
  ;        o Fix not processing if (f) had flags
  ; V1.2 - 25/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ; V1.3 - 27/JUN/2022
  ;        o Fixed: It now recognises the POS information
  ;        o Major speed gain: around twice the speed
  ; V1.4 - 10/OCT/2022
  ;        28/OCT/2022
  ;        29/OCT/2022
  ;        o Rewrote "Crunch words with Flags"
  ; V1.5 - 21/NOV/2022
  ;        27/DEC/2022
  ;        o Improved "Crunch words with Flags": also check backwards to get more hits.
  ;
  Procedure crunch_words_with_flags_process()
    
    ; Check if all checkboxes are unticked
    If crunch_words_with_flags_flag_M=#False And crunch_words_with_flags_flag_S=#False And crunch_words_with_flags_flag_G=#False And crunch_words_with_flags_flag_D=#False And crunch_words_with_flags_flag_p=#False And crunch_words_with_flags_flag_O=#False And crunch_words_with_flags_flag_w_lowercase=#False And crunch_words_with_flags_flag_W=#False And crunch_words_with_flags_flag_Y=#False And crunch_words_with_flags_flag_1=#False And crunch_words_with_flags_flag_k=#False And crunch_words_with_flags_flag_6=#False And crunch_words_with_flags_flag_j_lowercase=#False And crunch_words_with_flags_flag_h=#False And crunch_words_with_flags_flag_J=#False And crunch_words_with_flags_flag_o_lowercase=#False And crunch_words_with_flags_flag_tilde=#False
      MessageRequester("Warning!","No rules selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
     
    ; If number of words =1, no changes are made
    If number_of_words=1
      MessageRequester("Warning!","No flags added.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()    
    
    
    ; Working array
    ReDim array_for_crunch_words_flags.array_for_crunch_words_flags(number_of_words-1)    
    
    
    
    ; Step 1/4
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:1/4... Populating array...")
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
    
    time_taken=ElapsedMilliseconds()
    AddWindowTimer(#WINDOW_CRUNCH_WORDS,800,800)
    
    SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
    
    For f=0 To number_of_words-1
      
      array_for_crunch_words_flags(f)\in_ascii=words$(f)
      array_for_crunch_words_flags(f)\real_line=f
      t$=words$(f)+#LF$
      t1$=StringField(t$,1,#LF$)
      array_for_crunch_words_flags(f)\word_without_flags=StringField(t1$+"/",1,"/")
      array_for_crunch_words_flags(f)\word_changed_without_flags=StringField(t1$+"/",1,"/")
      array_for_crunch_words_flags(f)\word_flags=StringField(t1$+"/",2,"/")
      Debug "array_for_crunch_words_flags(f)\word_flags="+array_for_crunch_words_flags(f)\word_flags
      array_for_crunch_words_flags(f)\word_pos=StringField(t$+#LF$,2,#LF$)
      array_for_crunch_words_flags(f)\changed=#False ; If #True it populates the ListIconGadget in the end
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,f)
        SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:1/4... Populating array..."+FormatNumber(f+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$)+ConvertProgressToPercentage(f+1,number_of_words,1))
        GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
          SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:Aborted.")
          RemoveWindowTimer(#WINDOW_CRUNCH_WORDS,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f
    
    
    
    ; Step 2/4
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:2/4... Sorting array...")
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
    SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
    SortStructuredArray(array_for_crunch_words_flags(),#PB_Sort_Ascending,OffsetOf(array_for_crunch_words_flags\word_without_flags.s),TypeOf(array_for_crunch_words_flags\word_without_flags.s))
    SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,number_of_words)
    
    
    
    ; Step 3/4
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:3/4... Processing flags...")
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS,"Found:0"+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$))
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS)    

    SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
    
    counter=0 ; Number of words changed
    
    For f=0 To number_of_words-1
      
      ; Small speed up since strings are very slow
      left_word_in_f$=Left(array_for_crunch_words_flags(f)\word_without_flags,crunch_words_with_flags_number_of_chrs_match_left)
      
      ; Check backwards - 21/NOV/2022
      f_backwards=f
      If f>0
        For f_backwards=f To 1 Step -1
          If left_word_in_f$<>Left(array_for_crunch_words_flags(f_backwards-1)\word_without_flags,crunch_words_with_flags_number_of_chrs_match_left) : Break : EndIf
        Next f_backwards
      EndIf
      
      ; Decode the words in the range
      ; Removed the old code since there is no need to check for words before F because the words are sorted (no hits before F), thus a 2x speed gain - 27/JUN/2022
      For ff=f_backwards To number_of_words-1
        
        If f=ff : Continue : EndIf ; word=current word
        If left_word_in_f$<>Left(array_for_crunch_words_flags(ff)\word_without_flags,crunch_words_with_flags_number_of_chrs_match_left) : Break : EndIf
        If array_for_crunch_words_flags(f)\word_pos<>array_for_crunch_words_flags(ff)\word_pos : Continue : EndIf ; POS doesn't match
        If array_for_crunch_words_flags(f)\changed=#True : Break : EndIf                                          ; Word already changed
;         If array_for_crunch_words_flags(f)\word_flags<>"" : Break : EndIf                                         ; Word already had flags (F) ; Removed on 27/NOV/2020 to fix not processing if (f) had flags
        If array_for_crunch_words_flags(ff)\word_flags<>"" : Continue : EndIf                                         ; Word already had flags (FF)
;         If array_for_crunch_words_flags(ff)\word_changed_without_flags<>array_for_crunch_words_flags(ff)\word_changed_without_flags : Continue : EndIf ; Word already changed
        
        ; "X's → /M (Possessive)"
        If crunch_words_with_flags_flag_M=#True : counter + crunch_words_with_flags_process_flag(f,ff,"M") : EndIf
        
        ; "Xs → /S (Plural/Verb -s)"
        If crunch_words_with_flags_flag_S=#True
          t$=array_for_crunch_words_flags(f)\word_without_flags
          ; below line is a fix for words both ending with "s" and "x" - 27/NOV/2020
          If Right(t$,2)<>"au" And Right(t$,2)<>"eu" 
            counter + crunch_words_with_flags_process_flag(f,ff,"S")
          EndIf
        EndIf
        
        ; "Xing → /G (Verb -ing)"
        If crunch_words_with_flags_flag_G=#True : counter + crunch_words_with_flags_process_flag(f,ff,"G") : EndIf
        
        ; "Xed → /D (Verb -ed)"
        If crunch_words_with_flags_flag_D=#True : counter + crunch_words_with_flags_process_flag(f,ff,"D") : EndIf        
        
        ; "Xless → /p (Adjective -less)"
        If crunch_words_with_flags_flag_p=#True : counter + crunch_words_with_flags_process_flag(f,ff,"p") : EndIf        
        
        ; "Xal → /O (Adjective -al)"
        If crunch_words_with_flags_flag_O=#True : counter + crunch_words_with_flags_process_flag(f,ff,"O") : EndIf        

        ; "Xal → /w (Adjective -al)"
        If crunch_words_with_flags_flag_w_lowercase=#True : counter + crunch_words_with_flags_process_flag(f,ff,"w") : EndIf        
        
        ; "Xic → /W (Adjective -ic)"
        If crunch_words_with_flags_flag_W=#True : counter + crunch_words_with_flags_process_flag(f,ff,"W") : EndIf        
        
        ; "Xly → /Y (Adj/Adverb -ly)"
        If crunch_words_with_flags_flag_Y=#True : counter + crunch_words_with_flags_process_flag(f,ff,"Y") : EndIf  
     
        ; "Xically → /1 (Adverb -ically)"
        If crunch_words_with_flags_flag_1=#True : counter + crunch_words_with_flags_process_flag(f,ff,"1") : EndIf     

        ; "Xingly → /k (Adverb -ingly)"
        If crunch_words_with_flags_flag_k=#True : counter + crunch_words_with_flags_process_flag(f,ff,"k") : EndIf     

        ; "Xful → /6 (Adjective -ful)"
        If crunch_words_with_flags_flag_6=#True : counter + crunch_words_with_flags_process_flag(f,ff,"6") : EndIf  
        
        ; "Xfully → /j (Adverb -fully)"
        If crunch_words_with_flags_flag_j_lowercase=#True : counter + crunch_words_with_flags_process_flag(f,ff,"j") : EndIf   
        
        ; "Xedly → /h (Adverb -edly)"
        If crunch_words_with_flags_flag_h=#True : counter + crunch_words_with_flags_process_flag(f,ff,"h") : EndIf   

        ; "Xings → /J (Plural noun -ings)"
        If crunch_words_with_flags_flag_J=#True And Right(t$,5)<>"ing's" : counter + crunch_words_with_flags_process_flag(f,ff,"J") : EndIf   
        
        ; "Xally → /o (Adverb -ally)"
        If crunch_words_with_flags_flag_o_lowercase=#True : counter + crunch_words_with_flags_process_flag(f,ff,"o") : EndIf   
        
        ; "Xish → /~ (Adjective -ish)"
        If crunch_words_with_flags_flag_tilde=#True : counter + crunch_words_with_flags_process_flag(f,ff,"~") : EndIf 
        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,f)
          SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:3/4... Processing flags..."+FormatNumber(f+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$)+ConvertProgressToPercentage(f+1,number_of_words,1))
          GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
          SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS,"Found:"+FormatNumber(counter,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$))
          GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS)
        EndIf       
 
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
            SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:Aborted.")
            RemoveWindowTimer(#WINDOW_CRUNCH_WORDS,800)
            ProcedureReturn #False
          EndIf
        EndIf      
        
      Next ff

      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,f)
        SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:3/4... Processing flags..."+FormatNumber(f+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$)+ConvertProgressToPercentage(f+1,number_of_words,1))
        GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
        SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS,"Found:"+FormatNumber(counter,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$))
        GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS)        
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
          SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:Aborted.")
          RemoveWindowTimer(#WINDOW_CRUNCH_WORDS,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f    
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS,"Found:"+FormatNumber(counter,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$))
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS)
    
    
    
    ; Step 4/4
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:4/4... Sorting and populating list...")
    GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
    SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)    
    SortStructuredArray(array_for_crunch_words_flags(),#PB_Sort_Ascending,OffsetOf(array_for_crunch_words_flags\real_line.l),TypeOf(array_for_crunch_words_flags\real_line.l))
    
    counter_new=0
    
    ; To redisplay sorted
    ClearGadgetItems(#PANEL_BOX_CRUNCH_WORDS)
    
    For f=0 To number_of_words-1
      
      ; If it already populated the only changes done
      If counter_new>=counter : Break : EndIf
      
      ; Populate
      If array_for_crunch_words_flags(f)\changed=#True
        t$=Str(array_for_crunch_words_flags(f)\real_line+1)+#LF$+array_for_crunch_words_flags(f)\word_without_flags+"→"+array_for_crunch_words_flags(f)\word_changed_without_flags+"/"+array_for_crunch_words_flags(f)\word_flags+#LF$+Str(Len(array_for_crunch_words_flags(f)\word_changed_without_flags))
        AddGadgetItem(#PANEL_BOX_CRUNCH_WORDS,-1,t$)
        counter_new+1
      EndIf
      
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,f)
        SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:4/4... Sorting and populating list..."+FormatNumber(f+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$)+ConvertProgressToPercentage(f+1,number_of_words,1))
        GadgetFit(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING)
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
          SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:Aborted.")
          RemoveWindowTimer(#WINDOW_CRUNCH_WORDS,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f    
    
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_TIME_TAKEN,TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    RemoveWindowTimer(#WINDOW_CRUNCH_WORDS,800)     
    SetGadgetState(#PROGRESSBAR_WINDOW_CRUNCH_WORDS,0)
    SetGadgetText(#TEXT_WINDOW_CRUNCH_WORDS_ESC_TO_PROCESSING,"Processing:Done!")
    
    If counter=0
      MessageRequester("Warning!","No changes done.",#PB_MessageRequester_Warning)
    EndIf
    
  EndProcedure
  
  
  
  
  ; This procedures checks individual flags in order to replace words
  ; It retuns 0 or 1 to counter, in case it suceeds or fails
  ;
  ; V1.0 - 15/NOV/2020
  ; V1.1 - 12/JUN/2022
  ;        o Fix: Cruching words with flags (GB) use wrong flag for 1 chr words for non-rerivable words
  ;
  Procedure crunch_words_with_flags_process_flag(f_pos,ff_pos,flag$)

  ; Apply the flag and store the result in the variable: procedure_returned_crunch_word$
  process_pfx_sfx(100,array_for_crunch_words_flags(f_pos)\word_without_flags+"/"+flag$,#False,#False,#False,"","","",#False,#False)

  ; Check if applied flag matches word
  If array_for_crunch_words_flags(ff_pos)\word_without_flags<>procedure_returned_crunch_word$ : ProcedureReturn 0 : EndIf
  If array_for_crunch_words_flags(ff_pos)\word_changed_without_flags=array_for_crunch_words_flags(f_pos)\word_without_flags: ProcedureReturn 0 : EndIf ; Fix the 1 chr lookup replace with "M" instead of "S" - 2022-06-12
  
  ; Store the .dic in the undo if there is a change and set to #True
  If crunch_words_with_flags_changed=#False
    crunch_words_with_flags_changed=#True
    dictionary_store_undo(#False)
    dictionary_changed=#True
  EndIf
  
  ; Word changed
  array_for_crunch_words_flags(ff_pos)\changed=#True
  
  ; Apply the new word + flag to the structured array
  array_for_crunch_words_flags(ff_pos)\word_changed_without_flags=array_for_crunch_words_flags(f_pos)\word_without_flags
  array_for_crunch_words_flags(ff_pos)\word_flags=flag$
  
  ; Apply to the .dic array
  t$=""
  t$+"/"+flag$
  If array_for_crunch_words_flags(ff_pos)\word_pos<>"" : t$+#LF$+array_for_crunch_words_flags(ff_pos)\word_pos : EndIf
  t$=array_for_crunch_words_flags(ff_pos)\word_changed_without_flags+t$
  words$(array_for_crunch_words_flags(ff_pos)\real_line)=t$
  
  ; Insert changes in ListIconGadget in real time
  t$=Str(array_for_crunch_words_flags(ff_pos)\real_line+1)+#LF$+array_for_crunch_words_flags(ff_pos)\word_without_flags+"→"+array_for_crunch_words_flags(ff_pos)\word_changed_without_flags+"/"+array_for_crunch_words_flags(ff_pos)\word_flags+#LF$+Str(Len(array_for_crunch_words_flags(ff_pos)\word_changed_without_flags))
  AddGadgetItem(#PANEL_BOX_CRUNCH_WORDS,-1,t$)  
  counter_total_crunch_words+1
  SetGadgetItemColor(#PANEL_BOX_CRUNCH_WORDS,counter_total_crunch_words-1,#PB_Gadget_BackColor,#GreyLight,0) ; For "Line #"
  
  ; Increase the counter
  ProcedureReturn 1
  
  EndProcedure
  


  
  ; Clears the EditorGadget of the aff
  ;
  ; V1.0 - 19/NOV/2020
  ;
  Procedure clear_aff()
    
    ; Confirm if to erase the AFF if the file in memory has been changed
    If Len(GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF))<>0
      If MessageRequester("Warning!","The .AFF has contents."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf
    
    SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,"")
    
    dictionary_changed=#True
    
  EndProcedure

; IDE Options = PureBasic 6.04 LTS (Windows - x64)
; CursorPosition = 462
; FirstLine = 438
; Folding = -------z+--------
; Markers = 3092,3093,3181,3197,3621,3628
; EnableXP
; DisableDebugger
; CompileSourceDirectory
; EnableUnicode